﻿IF OBJECT_ID('TR__ActionGroupActions__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__ActionGroupActions__ValidateSequence]
GO

CREATE TRIGGER [TR__ActionGroupActions__ValidateSequence]
       ON [ActionGroupActions] FOR INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [ActionGroupActions] AGA
				   INNER JOIN (SELECT [ActionGroup_Id], [Action_Sequence],
									  [Actual_Action_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [ActionGroup_Id]
										   ORDER BY [ActionGroup_Id], [Action_Sequence])
									  FROM [ActionGroupActions]) AGA_ACTUAL
						   ON AGA.[ActionGroup_Id] = AGA_ACTUAL.[ActionGroup_Id]
						  AND AGA.[Action_Sequence] = AGA_ACTUAL.[Action_Sequence]
			 WHERE AGA.[Action_Sequence] <> AGA_ACTUAL.[Actual_Action_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE AGA.[ActionGroup_Id] = [ActionGroup_Id])
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE AGA.[ActionGroup_Id] = [ActionGroup_Id])
				   )
		) 
	BEGIN

		RAISERROR('An invalid Action_Sequence was provided for the ActionGroup.', 16, 1)
		ROLLBACK

	END

END
GO


GO
IF OBJECT_ID('TR__ApplicationGroups__CircularParentCheck') IS NOT NULL
DROP TRIGGER [TR__ApplicationGroups__CircularParentCheck]
GO

CREATE TRIGGER [TR__ApplicationGroups__CircularParentCheck]
ON [ApplicationGroups] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @ApplicationGroups TABLE(
		[Base_ApplicationGroup_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[ApplicationGroup_Id] INT NOT NULL, 
		[Parent_ApplicationGroup_Id] INT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @ApplicationGroups 
	  ([Base_ApplicationGroup_Id], [Resolution_Depth], [ApplicationGroup_Id], [Parent_ApplicationGroup_Id])
	SELECT 
	   [ApplicationGroup_Id], 1, [ApplicationGroup_Id], [Parent_ApplicationGroup_Id]
	  FROM [inserted] 
     

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's parents from the actual table
		INSERT INTO @ApplicationGroups 
		  ([Base_ApplicationGroup_Id], [Resolution_Depth], [ApplicationGroup_Id], [Parent_ApplicationGroup_Id])
		SELECT
		  [Base_ApplicationGroup_Id], @depth, AG.[Parent_ApplicationGroup_Id], PAG.[Parent_ApplicationGroup_Id]
		FROM @ApplicationGroups AG
	         INNER JOIN [ApplicationGroups] PAG
		             ON PAG.[ApplicationGroup_Id] = AG.[Parent_ApplicationGroup_Id]
		WHERE AG.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if an application group is an ancestor of itself then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @ApplicationGroups AG
		                          ON AG.[Resolution_Depth] > 1
		                         AND I.[ApplicationGroup_Id] = AG.[Base_ApplicationGroup_Id]
		                         AND I.[ApplicationGroup_Id] = AG.[Parent_ApplicationGroup_Id] )

		BEGIN 
				RAISERROR('The specified parent application group ID will cause a cycle at depth %i, which is not supported.',16,2, @depth)
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO


GO
IF OBJECT_ID('TR__DeployableDependencies__CircularDependencyCheck') IS NOT NULL
DROP TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
GO

CREATE TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
ON [DeployableDependencies] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @ResolvedDependencies TABLE(
		[Base_Deployable_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Dependant_Deployable_Id] INT NOT NULL, 
		[DependsOn_Deployable_Id] INT NOT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @ResolvedDependencies 
	  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
	SELECT 
	   [Deployable_Id], 1, [Deployable_Id], [DependsOn_Deployable_Id]
	  FROM [inserted] 

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's dependecies from the actual table
		INSERT INTO @ResolvedDependencies 
		  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
		SELECT
		  [Base_Deployable_Id], @depth, RD.[DependsOn_Deployable_Id], DD.[DependsOn_Deployable_Id]
		FROM @ResolvedDependencies RD
	         INNER JOIN [DeployableDependencies] DD
		             ON RD.[DependsOn_Deployable_Id] = DD.[Deployable_Id]
		WHERE RD.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if a inserted deployable depends on itself (somewhere down the line) then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @ResolvedDependencies RD
		                          ON RD.[Resolution_Depth] > 1
		                         AND I.[Deployable_Id] = RD.[Base_Deployable_Id]
		                         AND I.[Deployable_Id] = RD.[Dependant_Deployable_Id] )

		BEGIN 
				RAISERROR('This insertion will cause a circular dependecy at depth %i, which is not supported.',16,2, @depth) 
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO


GO
IF OBJECT_ID('[TR__DeployableDependencies__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__DeployableDependencies__ValidateDeployableId]
GO

CREATE TRIGGER [TR__DeployableDependencies__ValidateDeployableId] 
       ON [DeployableDependencies] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS(SELECT *
				FROM [inserted] i
					 INNER JOIN [Deployables] D1 ON D1.[Deployable_Id] = i.[Deployable_Id]
					 INNER JOIN [Deployables] D2 ON D2.[Deployable_Id] = i.[DependsOn_Deployable_Id]
			   WHERE D1.[Application_Id] <> D2.[Application_Id])
		BEGIN
			RAISERROR('Cross-application deployables are not supported.', 16, 1)
			ROLLBACK
		END
END
GO


GO
IF OBJECT_ID('[TR__DeploymentPlanActionGroups_ValidateGoto]') IS NOT NULL DROP TRIGGER [TR__DeploymentPlanActionGroups_ValidateGoto]
GO

CREATE TRIGGER [TR__DeploymentPlanActionGroups_ValidateGoto] 
       ON [DeploymentPlanActionGroups] FOR UPDATE, INSERT
AS 
BEGIN
	IF EXISTS(SELECT *
				FROM [inserted] i
					 LEFT JOIN [DeploymentPlanActionGroups] DPAG_Success
					        ON DPAG_Success.[DeploymentPlanActionGroup_Id] = i.[OnSuccess_DeploymentPlanActionGroup_Id]
			   WHERE DPAG_Success.[OnSuccess_DeploymentPlanActionGroup_Id] IS NOT NULL
			     AND DPAG_Success.[DeploymentPlan_Id] <> i.[DeploymentPlan_Id])
		BEGIN
			RAISERROR('OnSuccess_DeploymentPlanActionGroup must refer to a group in the same plan.', 16, 1)
			ROLLBACK
		END

	IF EXISTS(SELECT *
				FROM [inserted] i
					 LEFT JOIN [DeploymentPlanActionGroups] DPAG_Failure
					        ON DPAG_Failure.[DeploymentPlanActionGroup_Id] = i.[OnFailure_DeploymentPlanActionGroup_Id]
			   WHERE DPAG_Failure.[OnFailure_DeploymentPlanActionGroup_Id] IS NOT NULL
			     AND DPAG_Failure.[DeploymentPlan_Id] <> i.[DeploymentPlan_Id])
		BEGIN
			RAISERROR('OnFailure_DeploymentPlanActionGroup must refer to a group in the same plan.', 16, 1)
			ROLLBACK
		END
END
GO


GO
IF OBJECT_ID('TR__DeploymentPlanActionGroups__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__DeploymentPlanActionGroups__ValidateSequence]
GO

CREATE TRIGGER [TR__DeploymentPlanActionGroups__ValidateSequence]
       ON [DeploymentPlanActionGroups] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [DeploymentPlanActionGroups] DPAG 
				   INNER JOIN (SELECT [DeploymentPlanActionGroup_Id], [DeploymentPlanActionGroup_Sequence],
									  [Actual_PlanActionGroup_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [DeploymentPlan_Id]
										   ORDER BY [DeploymentPlan_Id], [DeploymentPlanActionGroup_Sequence])
									  FROM [DeploymentPlanActionGroups]) DPAGS
						   ON DPAG.[DeploymentPlanActionGroup_Id] = DPAGS.[DeploymentPlanActionGroup_Id]
			 WHERE DPAG.[DeploymentPlanActionGroup_Sequence] <> DPAGS.[Actual_PlanActionGroup_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE DPAG.[DeploymentPlan_Id] = [DeploymentPlan_Id] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE DPAG.[DeploymentPlan_Id] = [DeploymentPlan_Id] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid DeploymentPlanActionGroup_Sequence was provided for the DeploymentPlanActionGroup.', 16, 1)
		ROLLBACK

	END

END
GO


GO
IF OBJECT_ID('TR__Environments__CircularParentCheck') IS NOT NULL
DROP TRIGGER [TR__Environments__CircularParentCheck]
GO

CREATE TRIGGER [TR__Environments__CircularParentCheck]
ON [Environments] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Environments TABLE(
		[Base_Environment_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Environment_Id] INT NOT NULL, 
		[Parent_Environment_Id] INT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @Environments 
	  ([Base_Environment_Id], [Resolution_Depth], [Environment_Id], [Parent_Environment_Id])
	SELECT 
	   [Environment_Id], 1, [Environment_Id], [Parent_Environment_Id]
	  FROM [inserted] 
     

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's parents from the actual table
		INSERT INTO @Environments 
		  ([Base_Environment_Id], [Resolution_Depth], [Environment_Id], [Parent_Environment_Id])
		SELECT
		  [Base_Environment_Id], @depth, AG.[Parent_Environment_Id], PAG.[Parent_Environment_Id]
		FROM @Environments AG
	         INNER JOIN [Environments] PAG
		             ON PAG.[Environment_Id] = AG.[Parent_Environment_Id]
		WHERE AG.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if an environment is an ancestor of itself then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @Environments AG
		                          ON AG.[Resolution_Depth] > 1
		                         AND I.[Environment_Id] = AG.[Base_Environment_Id]
		                         AND I.[Environment_Id] = AG.[Parent_Environment_Id] )

		BEGIN 
				RAISERROR('The specified parent environment ID will cause a cycle at depth %i, which is not supported.',16,2, @depth)
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO


GO
IF OBJECT_ID('[TR__EventOccurenceDetails__ValidateDetailValue]') IS NOT NULL DROP TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue]
GO

CREATE TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue] 
       ON [EventOccurenceDetails] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS
	(
		SELECT *
		  FROM inserted /*[EventOccurenceDetails]*/ EOD
			   INNER JOIN [EventTypeDetails] ETD
					   ON ETD.[Event_Code] = EOD.[Event_Code]
					  AND ETD.[Detail_Name] = EOD.[Detail_Name]
		 WHERE ETD.[Detail_Type] <> sql_variant_property(EOD.[Detail_Value], 'BaseType')
	)
		BEGIN
			RAISERROR('Event detail value/type mismatch.', 16, 1)
			ROLLBACK
		END
END
GO




GO
IF OBJECT_ID('[TR__ReleaseDeployables__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__ReleaseDeployables__ValidateDeployableId]
GO



GO
IF OBJECT_ID('TR__Releases__ValidateReleaseNumber') IS NOT NULL DROP TRIGGER [TR__Releases__ValidateReleaseNumber]
GO


GO
IF OBJECT_ID('TR__Releases__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__Releases__ValidateSequence]
GO

CREATE TRIGGER [TR__Releases__ValidateSequence]
       ON [Releases] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [Releases] R 
				   INNER JOIN (SELECT [Application_Id], 
                                      [Release_Number],
									  [Actual_Release_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Application_Id]
										        ORDER BY [Application_Id], [Release_Sequence])
									  FROM [Releases]) R_ACTUAL
						   ON R.[Application_Id] = R_ACTUAL.[Application_Id]
                          AND R.[Release_Number] = R_ACTUAL.[Release_Number]
			 WHERE R.[Release_Sequence] <> R_ACTUAL.[Actual_Release_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid Release_Sequence was provided for the Release.', 16, 1)
		ROLLBACK
	
	END
		
END
GO


GO
IF OBJECT_ID('TR__WorkflowSteps__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__WorkflowSteps__ValidateSequence]
GO



GO
IF OBJECT_ID('ABS_MONTH_NUMBER') IS NOT NULL DROP FUNCTION ABS_MONTH_NUMBER
GO
CREATE FUNCTION ABS_MONTH_NUMBER
(
  @Date DATETIME
) RETURNS INT AS BEGIN
  
  RETURN
    /* Months Past Since Day 0 */
    ((DATEPART(yy,@Date) - 1900) * 12)
    /* Plus Months this year */
    + DATEPART(m,@Date)
END
GO


GO
IF OBJECT_ID('ABS_WEEK_NUMBER') IS NOT NULL DROP FUNCTION ABS_WEEK_NUMBER
GO
CREATE FUNCTION ABS_WEEK_NUMBER
(
  @DateTime DATETIME
) RETURNS INT AS BEGIN

  DECLARE @Dec DECIMAL(9,2)
  /* Convert to Days since 1900-01-01 */
  SET @Dec = CAST(@DateTime AS DECIMAL(9,2))
  /* Since 1900-01-01 is a Monday and Week start on Sunday, Add 1 */
  SET @Dec = @Dec + 1
  /* Floor of /7 is week # */
  RETURN FLOOR(@Dec/7)
END
GO


GO
IF OBJECT_ID('ApplicationGroups_GetAncestorGroups') IS NOT NULL DROP FUNCTION ApplicationGroups_GetAncestorGroups
GO

CREATE FUNCTION [ApplicationGroups_GetAncestorGroups]
(
	@ApplicationGroup_Id INT
) RETURNS @groups TABLE ([Level] INT NOT NULL, [ApplicationGroup_Id] INT NOT NULL)
BEGIN

    DECLARE @Depth INT
    SET @Depth = 0

    DECLARE @ParentId INT
    DECLARE @TmpParentId INT
	DECLARE @TmpGroupId INT

    SELECT @TmpGroupId = [ApplicationGroup_Id], 
		   @ParentId = [Parent_ApplicationGroup_Id] 
      FROM [ApplicationGroups] 
     WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

	 -- no application group with specified ID
	 IF @TmpGroupId IS NULL
        RETURN

    -- add specified group at depth 0
    INSERT INTO @groups ([Level], [ApplicationGroup_Id])
        VALUES (0, @ApplicationGroup_Id)

    WHILE (@ParentId IS NOT NULL) 
    BEGIN
        SET @Depth = @Depth + 1

        INSERT INTO @groups ([Level], [ApplicationGroup_Id])
        VALUES (@Depth, @ParentId)
        
        SELECT @TmpParentId = [Parent_ApplicationGroup_Id] FROM [ApplicationGroups] WHERE [ApplicationGroup_Id] = @ParentId
        SET @ParentId = @TmpParentId

        -- sanity check
        IF @Depth > 1000
            BREAK

    END
  
  RETURN
END
GO


GO
IF OBJECT_ID('CompareBuildNumber') IS NOT NULL DROP FUNCTION CompareBuildNumber
GO


GO
IF OBJECT_ID('CompareReleaseNumber') IS NOT NULL DROP FUNCTION CompareReleaseNumber
GO


GO

IF OBJECT_ID('CsvToTable') IS NOT NULL DROP FUNCTION CsvToTable
GO

CREATE FUNCTION CsvToTable (@CSVString VARCHAR(MAX), @Delimiter	VARCHAR(10))
  RETURNS @tbl TABLE ([Value_Sequence] INT, [Value_Text] VARCHAR(MAX))

AS BEGIN 

    DECLARE @i INT, @j INT, @ctr INT
	SET @i = 1 

	WHILE @i <= len(@CSVString)
	BEGIN
		SET @j = CHARINDEX(@Delimiter, @CSVString, @i)
        SET @ctr = COALESCE(@ctr,0) + 1
		IF @j = 0 SET @j = LEN(@CSVString) + 1
		INSERT @tbl SELECT @ctr, SUBSTRING(@CSVString, @i, @j - @i)
		SELECT @i = @j + LEN(@Delimiter)
	END
	
    RETURN

END
GO




GO
IF OBJECT_ID('Environments_GetAncestorEnvironments') IS NOT NULL DROP FUNCTION Environments_GetAncestorEnvironments
GO

CREATE FUNCTION [Environments_GetAncestorEnvironments]
(
	@Environment_Id INT
) RETURNS @environments TABLE ([Level] INT NOT NULL, [Environment_Id] INT NOT NULL)
BEGIN

    DECLARE @Depth INT
    SET @Depth = 0

    DECLARE @ParentId INT
    DECLARE @TmpParentId INT
	DECLARE @TmpEnvId INT

    SELECT @TmpEnvId = [Environment_Id], 
		   @ParentId = [Parent_Environment_Id] 
      FROM [Environments] 
     WHERE [Environment_Id] = @Environment_Id

	 -- no environment with specified ID
	 IF @TmpEnvId IS NULL
        RETURN

    -- add specified environment at depth 0
    INSERT INTO @environments ([Level], [Environment_Id])
        VALUES (0, @Environment_Id)

    WHILE (@ParentId IS NOT NULL) 
    BEGIN
        SET @Depth = @Depth + 1

        INSERT INTO @environments ([Level], [Environment_Id])
        VALUES (@Depth, @ParentId)
        
        SELECT @TmpParentId = [Parent_Environment_Id] FROM [Environments] WHERE [Environment_Id] = @ParentId
        SET @ParentId = @TmpParentId

        -- sanity check
        IF @Depth > 1000
            BREAK

    END
  
  RETURN
END
GO


GO
IF OBJECT_ID('[GetSortableReleaseNumberWithValidation]') IS NOT NULL DROP FUNCTION [GetSortableReleaseNumberWithValidation]
GO



GO
IF OBJECT_ID('IsAuthorizedForTask') IS NOT NULL DROP FUNCTION [IsAuthorizedForTask]
GO


GO
IF OBJECT_ID('IsBuildNumberValid') IS NOT NULL DROP FUNCTION IsBuildNumberValid
GO



GO
IF OBJECT_ID('[IsPromotionApproved]') IS NOT NULL DROP FUNCTION [IsPromotionApproved]
GO



GO
IF OBJECT_ID('IsReleaseNumberValid') IS NOT NULL DROP FUNCTION IsReleaseNumberValid
GO



GO
IF OBJECT_ID('MAXOF') IS NOT NULL DROP FUNCTION MAXOF
GO
CREATE FUNCTION MAXOF
( 
  @a SQL_VARIANT, 
  @b SQL_VARIANT
) RETURNS SQL_VARIANT AS BEGIN 

  IF @a > @b RETURN @a ELSE RETURN @b
  RETURN NULL 
END
GO


GO

IF OBJECT_ID('Session_GetUserName') IS NOT NULL DROP FUNCTION Session_GetUserName
GO

CREATE FUNCTION Session_GetUserName () RETURNS USERNAME
AS BEGIN

	DECLARE @Context_Info VARCHAR(MAX)
	SET @Context_Info = CAST(CONTEXT_INFO() AS VARCHAR(MAX))

	IF CHARINDEX(CHAR(0), @Context_Info) > 0
		SET @Context_Info = SUBSTRING(@Context_Info, 0, CHARINDEX(CHAR(0), @Context_Info))
	RETURN COALESCE(@Context_Info, 'SYSTEM')

END
GO


GO
IF OBJECT_ID('SortableBuildNumber') IS NOT NULL DROP FUNCTION SortableBuildNumber
GO



GO
IF OBJECT_ID('Variables_CascadedValuesByApplication') IS NOT NULL DROP VIEW Variables_CascadedValuesByApplication
GO
IF OBJECT_ID('Variables_CascadedValuesByExecution') IS NOT NULL DROP VIEW Variables_CascadedValuesByExecution
GO
IF OBJECT_ID('Variables_GetValuesForExecution') IS NOT NULL DROP FUNCTION Variables_GetValuesForExecution
GO


CREATE FUNCTION [Variables_GetValuesForExecution]
(
	@ExecutionId INT,
	@Server_Id INT,
    @ServerRole_Id INT,
	@Deployable_Id INT
) RETURNS @vvals TABLE ([Variable_Name] VARCHAR(50), [Value_Text] VARCHAR(MAX), [Scope_Code] CHAR(1), [Sensitive_Indicator] CHAR(1))
BEGIN

	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
	DECLARE @Environment_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER
	DECLARE @Release_Id INT
	DECLARE @Build_Id INT
	DECLARE @Promotion_Id INT
	DECLARE @PipelineStage_Name VARCHAR(50)

	SELECT @Application_Id = A.[Application_Id],
	       @Environment_Id = E.[Environment_Id],
		   @Release_Number = R.[Release_Number],
		   @Build_Number = B.[Build_Number],
		   @ApplicationGroup_Id = A.[ApplicationGroup_Id],
		   @Release_Id = R.[Release_Id],
		   @Build_Id = B.[Build_Id],
		   @Promotion_Id = BP.[Promotion_Id],
		   @PipelineStage_Name = BP.[PipelineStage_Name]
	  FROM [BuildExecutions] BE
	       INNER JOIN [BuildPromotions] BP
		           ON BP.[Promotion_Id] = BE.[Promotion_Id]
		   INNER JOIN [Builds] B
		           ON B.[Build_Id] = BP.[Build_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = R.[Application_Id]
		    LEFT JOIN [Environments] E
			       ON E.[Environment_Id] = BE.[Environment_Id]
     WHERE BE.[Execution_Id] = @ExecutionId

	DECLARE @VDEC_Ids TABLE ([VariableDeclaration_Id] INT)
	DECLARE @VDEF_Ids TABLE ([VariableDefinition_Id] INT, [AppGroupDepth] INT)

	INSERT @VDEF_Ids
	SELECT VDEF.[VariableDefinition_Id], 0
	  FROM [VariableDefinitions] VDEF
	 WHERE (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
	   AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = @ApplicationGroup_Id)
	   AND (VDEF.[Release_Id] IS NULL OR VDEF.[Release_Id] = @Release_Id)
	   AND (VDEF.[Build_Id] IS NULL OR VDEF.[Build_Id] = @Build_Id)
	   AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
	   AND (VDEF.[Execution_Id] IS NULL OR VDEF.[Execution_Id] = @ExecutionId)
	   AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
       AND (VDEF.[ServerRole_Id] IS NULL OR VDEF.[ServerRole_Id] = @ServerRole_Id)
	   AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)
	   AND (VDEF.[Promotion_Id] IS NULL OR VDEF.[Promotion_Id] = @Promotion_Id)

    -- hack for application group groups
    INSERT @VDEF_Ids
    SELECT VDEF.[VariableDefinition_Id], AGA.[Level]
      FROM [VariableDefinitions] VDEF
     INNER JOIN dbo.ApplicationGroups_GetAncestorGroups(@ApplicationGroup_Id) AGA
             ON @ApplicationGroup_Id IS NOT NULL
            AND AGA.[Level] > 0
            AND AGA.[ApplicationGroup_Id] = VDEF.[ApplicationGroup_Id]
       

	INSERT @VDEC_Ids
	SELECT VDEC.[VariableDeclaration_Id]
	  FROM [VariableDeclarations] VDEC
	 WHERE (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
	   AND (VDEC.[Scoped_Environment_Id] IS NULL OR VDEC.[Scoped_Environment_Id] = @Environment_Id)
	   AND (VDEC.[Scoped_PipelineStage_Name] IS NULL OR VDEC.[Scoped_PipelineStage_Name] = @PipelineStage_Name)
	   AND ((@Server_Id IS NULL AND VDEC.[Scope_Code] <> 'V') OR @Server_Id IS NOT NULL)
	   AND ((@Deployable_Id IS NULL AND VDEC.[Scope_Code] <> 'D') OR @Deployable_Id IS NOT NULL)
	   AND NOT EXISTS(SELECT * FROM [VariableDefinitions] _VDEF
					   WHERE _VDEF.[VariableDefinition_Id] IN (SELECT [VariableDefinition_Id] FROM @VDEF_Ids)
						 AND _VDEF.[Variable_Name] = VDEC.[Variable_Name]
						 AND _VDEF.[Scope_Code] = VDEC.[Scope_Code])

	INSERT INTO @vvals ([Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator])
	SELECT [Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator]
	  FROM (SELECT *, [Rank] = RANK() OVER (PARTITION BY [Variable_Name] ORDER BY CHARINDEX([Scope_Code], 'SELVGADRBPX') DESC, [AppGroupDepth] ASC)
			  FROM (SELECT [Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator], NULL AS [Scoped_Environment_Id], VD1.[AppGroupDepth]
                        FROM @VDEF_Ids VD1
                        INNER JOIN [VariableDefinitions] VD2
                                ON VD1.[VariableDefinition_Id] = VD2.[VariableDefinition_Id]
						   UNION ALL
					SELECT [Variable_Name], [DefaultValue_Text], [Scope_Code], [Sensitive_Indicator], [Scoped_Environment_Id], 0 AS [AppGroupDepth]
					  FROM [VariableDeclarations]
					 WHERE [VariableDeclaration_Id] IN (SELECT [VariableDeclaration_Id] FROM @VDEC_Ids))_)_
	 WHERE [Rank] = 1
  
  RETURN
END
GO


GO
IF OBJECT_ID('GetNextBuildNumber') IS NOT NULL DROP FUNCTION GetNextBuildNumber
GO



GO
IF OBJECT_ID('GetNextReleaseNumber') IS NOT NULL DROP FUNCTION GetNextReleaseNumber
GO



GO
IF OBJECT_ID('[GetSortableReleaseNumber]') IS NOT NULL DROP FUNCTION [GetSortableReleaseNumber]
GO



GO
IF OBJECT_ID('[ActionGroupActions_Extended]') IS NOT NULL DROP VIEW [ActionGroupActions_Extended]
GO

CREATE VIEW [ActionGroupActions_Extended] AS

    SELECT AGA.[ActionGroup_Id],
           AGA.[Action_Sequence],
           AGA.[Server_Id],
           AGA.[Long_Description],
           AGA.[ResumeNextOnFailure_Indicator],
           AGA.[Action_Configuration],
           AGA.[Short_Description],
           AGA.[Active_Indicator],
           AGA.[Retry_Count],
           AGA.[LogFailureAsWarning_Indicator],
	       AGA.[Server_Variable_Name],
	       AGA.[Target_Server_Id],
	       AGA.[Target_Server_Variable_Name],
	       AGA.[ExtensionConfiguration_Id],
           [Configuration_Profile_Name] = E.[Profile_Name],
           SRC_S.[Server_Name],
           [Target_Server_Name] = TRG_S.[Server_Name],
           SRC_S.[ServerType_Code],
           [Target_ServerType_Code] = TRG_S.[ServerType_Code]

      FROM [ActionGroupActions] AGA
INNER JOIN [ActionGroups] AG
        ON AG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
 LEFT JOIN [ExtensionConfigurations] E
        ON E.[ExtensionConfiguration_Id] = AGA.[ExtensionConfiguration_Id]
 LEFT JOIN [Servers] SRC_S
        ON SRC_S.[Server_Id] = AGA.[Server_Id]
 LEFT JOIN [Servers] TRG_S
        ON TRG_S.[Server_Id] = AGA.[Target_Server_Id]

GO



GO
IF OBJECT_ID('[ActionGroupUsage_Slim]') IS NOT NULL DROP VIEW [ActionGroupUsage_Slim]
GO

CREATE VIEW [ActionGroupUsage_Slim]
AS
SELECT [ActionGroup_Id] = DPAG.[ActionGroup_Id],
       DPAG.[DeploymentPlanActionGroup_Id],
       DP.[DeploymentPlan_Name],
       A.[Application_Id],
       A.[Application_Name],
       AG.[ApplicationGroup_Id],
       AG.[ApplicationGroup_Name],
       E.[Environment_Id],
       E.[Environment_Name]

  FROM [DeploymentPlanActionGroups] DPAG
       INNER JOIN [DeploymentPlans] DP
               ON DP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
		LEFT JOIN [WorkflowSteps] WS
		       ON WS.[DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
		LEFT JOIN [ApplicationWorkflows] AW
		       ON AW.[Workflow_Id] = WS.[Workflow_Id]
        LEFT JOIN [Applications] A
               ON A.[Application_Id] = AW.[Application_Id]
        LEFT JOIN [ApplicationGroups] AG
               ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
        LEFT JOIN [Environments] E
               ON E.[Environment_Id] = WS.[Environment_Id]

GO


GO
IF OBJECT_ID('[ApplicationDeploymentPlans]') IS NOT NULL
	DROP VIEW [ApplicationDeploymentPlans]
GO

CREATE VIEW [ApplicationDeploymentPlans]
AS

SELECT DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   E.[Environment_Id],
	   E.[Environment_Name],
       [Environment_Display_Sequence] = E.[Display_Sequence],
	   W.[Workflow_Id],
	   W.[Workflow_Name],
	   [Workflow_Step_Sequence] = WS.[Step_Sequence],
       [BuildStep_Indicator] = 'N',
	   DP.[AllowLocalChanges_Indicator],
       DP.[OtterScript_Bytes],
       [DeploymentPlan_Shared_Indicator] = CASE WHEN 
                                                    DP.[DeploymentPlan_Id] IN 
                                                         (SELECT [PlanId] FROM 
                                                            (SELECT [PlanId] = [DeploymentPlan_Id] FROM [WorkflowSteps]
                                                              UNION ALL 
                                                             SELECT [PlanId] = [Build_DeploymentPlan_Id] FROM [Workflows]) AS [AllPlanIds]
                                                        GROUP BY [PlanId]
                                                        HAVING COUNT([PlanId]) > 1)
                                                
                                                THEN 'Y'
                                                ELSE 'N'
                                            END
  FROM [ApplicationWorkflows] AW
       INNER JOIN [Workflows] W
	           ON W.[Workflow_Id] = AW.[Workflow_Id]
       INNER JOIN [WorkflowSteps] WS
	           ON WS.[Workflow_Id] = AW.[Workflow_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = AW.[Application_Id]
	   INNER JOIN [Environments] E
	           ON E.[Environment_Id] = WS.[Environment_Id]
	   INNER JOIN [DeploymentPlans] DP
	           ON DP.[DeploymentPlan_Id] = WS.[DeploymentPlan_Id]
        LEFT JOIN [ApplicationGroups] AG
		       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
UNION ALL 
SELECT 
	   DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   NULL,
	   NULL,
       NULL,
	   W.[Workflow_Id],
	   W.[Workflow_Name],
	   NULL,
       [BuildStep_Indicator] = 'Y',
	   DP.[AllowLocalChanges_Indicator],
       DP.[OtterScript_Bytes],
       [DeploymentPlan_Shared_Indicator] = CASE WHEN 
                                                    DP.[DeploymentPlan_Id] IN 
                                                         (SELECT [PlanId] FROM 
                                                            (SELECT [PlanId] = [DeploymentPlan_Id] FROM [WorkflowSteps]
                                                              UNION ALL 
                                                             SELECT [PlanId] = [Build_DeploymentPlan_Id] FROM [Workflows]) AS [AllPlanIds]
                                                        GROUP BY [PlanId]
                                                        HAVING COUNT([PlanId]) > 1)
                                                
                                                THEN 'Y'
                                                ELSE 'N'
                                            END
  FROM [ApplicationWorkflows] AW
       INNER JOIN [Workflows] W
	           ON W.[Workflow_Id] = AW.[Workflow_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = AW.[Application_Id]
	   INNER JOIN [DeploymentPlans] DP
	           ON DP.[DeploymentPlan_Id] = W.[Build_DeploymentPlan_Id]
        LEFT JOIN [ApplicationGroups] AG
		       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
GO


GO
IF OBJECT_ID('[ApplicationDeploymentPlans_Extended]') IS NOT NULL
	DROP VIEW [ApplicationDeploymentPlans_Extended]
GO


GO
IF OBJECT_ID('[ApplicationEnvironments_Merged]') IS NOT NULL
	DROP VIEW [ApplicationEnvironments_Merged]
GO


GO
IF OBJECT_ID('[Applications_Extended]') IS NOT NULL DROP VIEW [Applications_Extended]
GO

CREATE VIEW [Applications_Extended]
AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
		   A.[Application_Description],
		   A.[ReleaseNumber_Scheme_Name],
		   A.[BuildNumber_Scheme_Name],
		   A.[AllowMultipleActiveBuilds_Indicator],
		   A.[IssueTracking_Provider_Id],
		   A.[IssueTracking_CategoryIdList_Text],
		   A.[IssueTracking_Filter_Configuration],
           A.[IssueTracking_AllowBuildMasterIssues_Indicator],
		   A.[CreatedBy_User_Name],
		   A.[CreatedOn_Date],
		   A.[ModifiedBy_User_Name],
		   A.[ModifiedOn_Date],
		   A.[Dashboard_Text],
		   A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
		   AG.[ApplicationGroup_Description],
		   [Active_Releases_Count] = (SELECT COUNT(*) 
		                                FROM [Releases] 
		                               WHERE [Application_Id] = A.[Application_Id] 
		                                 AND [ReleaseStatus_Name] = 'Active'),
		   A.[VariableSupport_Code],
           [Issues_LastSynchronized_Date] = (SELECT MAX([IssueTracker_LastSync_Date]) 
                                               FROM [Issues] 
                                              WHERE [Application_Id] = A.[Application_Id]),
           [ShowDeployables_Indicator] = CASE 
                                            WHEN EXISTS(SELECT *
                                                       FROM [Deployables] D 
                                                      WHERE D.[Application_Id] = A.[Application_Id])
                                                 OR EXISTS(SELECT * 
                                                             FROM [DeployablesImported] DI 
                                                            WHERE DI.[Application_Id] = A.[Application_Id])
                                                 THEN 'Y' ELSE 'N'
                                         END,
           A.[Active_Indicator]

	  FROM [Applications] A
		   LEFT JOIN [ApplicationGroups] AG
				  ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]


GO
IF OBJECT_ID('[Applications_Health]') IS NOT NULL
	DROP VIEW [Applications_Health]
GO

CREATE VIEW [Applications_Health]
AS

	SELECT 
        A.[Application_Id],
        A.[Active_Indicator],
        [HasDeployable_Indicator] = CASE WHEN D.[HasDeployable_Indicator] IS NULL THEN 'N' ELSE 'Y' END,
        [HasRelease_Indicator] = CASE WHEN R.[HasRelease_Indicator] IS NULL THEN 'N' ELSE 'Y' END,
        [HasActiveWorkflow_Indicator] = CASE WHEN AW.[HasActiveWorkflow_Indicator] IS NULL THEN 'N' ELSE 'Y' END,
        [HasDeploymentPlan_Indicator] = CASE WHEN ADP.[HasDeploymentPlan_Indicator] IS NULL THEN 'N' ELSE 'Y' END,
        [ShowHealthPage_Indicator] = CASE WHEN
                                        D.[HasDeployable_Indicator] IS NULL OR 
                                        R.[HasRelease_Indicator] IS NULL OR
                                        AW.[HasActiveWorkflow_Indicator] IS NULL OR
                                        ADP.[HasDeploymentPlan_Indicator] IS NULL
                                     THEN 'Y' ELSE 'N' END

	  FROM [Applications] A

      OUTER APPLY(
        SELECT TOP 1 [HasDeployable_Indicator] = 'Y' FROM [Deployables] D WHERE D.[Application_Id] = A.[Application_Id]
      ) D

      OUTER APPLY(
        SELECT TOP 1 [HasRelease_Indicator] = 'Y' FROM [Releases] R WHERE R.[Application_Id] = A.[Application_Id]
      ) R

      OUTER APPLY(
        SELECT TOP 1 [HasActiveWorkflow_Indicator] = 'Y' FROM [ApplicationWorkflows] AW WHERE AW.[Application_Id] = A.[Application_Id]
      ) AW

      OUTER APPLY(
        SELECT TOP 1 [HasDeploymentPlan_Indicator] = 'Y' FROM [ApplicationDeploymentPlans] ADP WHERE ADP.[Application_Id] = A.[Application_Id]
      ) ADP
 


GO
IF OBJECT_ID('[Environments_LatestDeployedReleases]') IS NOT NULL DROP VIEW [Environments_LatestDeployedReleases]
GO

IF OBJECT_ID('[Applications_LatestDeployedReleases]') IS NOT NULL DROP VIEW [Applications_LatestDeployedReleases]
GO

CREATE VIEW [Applications_LatestDeployedReleases] AS

SELECT A.[Application_Id]
	  ,A.[ApplicationGroup_Id]
	  ,A.[Application_Name]
	  ,AG.[ApplicationGroup_Name]
	  ,BP.[Promotion_Id]
	  ,BP.[PipelineStage_Name]
	  ,R.[Release_Id]
	  ,R.[Release_Number]
	  ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  ,B.[Build_Id]
	  ,B.[Build_Number]
	  ,BE.[Execution_Id]
      ,[ExecutionStart_Date] = X.[Start_Date]
	  ,X.[ExecutionStatus_Code]
	  ,X.[ExecutionRunState_Code]

  FROM (SELECT [Build_Id],
               [Release_Id],
			   [Build_Number]
		  FROM [Builds]
		 WHERE [BuildStatus_Name] = 'Active') B

	   INNER JOIN [Releases] R
	           ON R.[Release_Id] = B.[Release_Id]

	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]

		LEFT JOIN [ApplicationGroups] AG
		       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]

	   INNER JOIN [BuildPromotions] BP
	           ON BP.[Build_Id] = B.[Build_Id]

       INNER JOIN (SELECT [Promotion_Id],
	                      [Execution_Id] = MAX([Execution_Id])
	                 FROM [BuildExecutions] _BE
					GROUP BY _BE.[Promotion_Id]) BE
			   ON BE.[Promotion_Id] = BP.[Promotion_Id]

	   INNER JOIN [Executions] X
	           ON X.[Execution_Id] = BE.[Execution_Id]

 WHERE A.[Active_Indicator] = 'Y'
   AND R.[ReleaseStatus_Name] <> 'Cancelled'

GO


GO
IF OBJECT_ID('[Applications_LatestEnvironmentExecutions]') IS NOT NULL DROP VIEW [Applications_LatestEnvironmentExecutions]
GO

CREATE VIEW [Applications_LatestEnvironmentExecutions] AS

SELECT 
        A.[Application_Id]
       ,A.[Application_Name]
       ,AG.[ApplicationGroup_Id]
       ,AG.[ApplicationGroup_Name]
       ,R.[Release_Id]
       ,R.[Release_Number]
       ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
       ,P.[Promotion_Id]
       ,P.[PromotionStatus_Name]
       ,B.[Build_Id]
       ,B.[Build_Number]
       ,B.[BuildStatus_Name]
       ,E.[Environment_Id]
       ,E.[Environment_Name]
       ,EX.[Execution_Id]
       ,[ExecutionStart_Date] = EX.[Start_Date]
       ,EX.[ExecutionStatus_Code]
       ,EX.[ExecutionRunState_Code]
      FROM (
            SELECT R.[Application_Id], BEX.[Environment_Id], [Latest_Execution_Id] = MAX(BEX.[Execution_Id])
              FROM [BuildPromotions] P    
            INNER JOIN [Builds] B ON B.[Build_Id] = P.[Build_Id]    
            INNER JOIN [Releases] R ON R.[Release_Id] = B.[Release_Id]    
             LEFT JOIN [BuildExecutions] BEX
                    ON BEX.[Promotion_Id] = P.[Promotion_Id]
            GROUP BY R.[Application_Id], BEX.[Environment_Id]
     ) APP_ENV_EXEC
     INNER JOIN [Executions] EX ON EX.[Execution_Id] = APP_ENV_EXEC.[Latest_Execution_Id]
     INNER JOIN [BuildExecutions] BEX ON BEX.[Execution_Id] = EX.[Execution_Id]
     INNER JOIN [Environments] E ON E.[Environment_Id] = BEX.[Environment_Id]
     INNER JOIN [BuildPromotions] P ON P.[Promotion_Id] = BEX.[Promotion_Id]
     INNER JOIN [Applications] A ON A.[Application_Id] = APP_ENV_EXEC.[Application_Id]
     INNER JOIN [Builds] B ON B.[Build_Id] = P.[Build_Id]
     INNER JOIN [Releases] R ON R.[Release_Id] = B.[Release_Id]
      LEFT JOIN [ApplicationGroups] AG ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

     WHERE A.[Active_Indicator] = 'Y'
       AND E.[Active_Indicator] = 'Y'

GO


GO
IF OBJECT_ID('[Applications_Slim]') IS NOT NULL
	DROP VIEW [Applications_Slim]
GO

CREATE VIEW [Applications_Slim]
AS

	SELECT A.[Application_Id],
		   A.[Application_Name],
           A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
           A.[Active_Indicator],
           AG.[Parent_ApplicationGroup_Id]

	  FROM [Applications] A
 LEFT JOIN [ApplicationGroups] AG
        ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]


GO
IF OBJECT_ID('[BuildApprovalsReceived_Extended]') IS NOT NULL DROP VIEW [BuildApprovalsReceived_Extended]
GO

CREATE VIEW [BuildApprovalsReceived_Extended] AS

SELECT BAR.[Approval_Id],
	   A.[Application_Id],
       A.[Application_Name],
	   R.[Release_Id],
       R.[Release_Number],
	   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
	   B.[Build_Id],
       B.[Build_Number],
	   BAR.[Original_Requirement_Id],
       BAR.[User_Name],
       BAR.[Comments_Text],
       BAR.[Received_Date],
	   BAR.[RequirementType_Code],
	   BAR.[Requirement_Description],
	   R.[Pipeline_Id],
	   BAR.[PipelineStage_Name]

  FROM [BuildApprovalsReceived] BAR
       INNER JOIN [Builds] B
	           ON B.[Build_Id] = BAR.[Build_Id]
	   INNER JOIN [Releases] R
	           ON R.[Release_Id] = B.[Release_Id]
       INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]



GO
IF OBJECT_ID('[BuildApprovals_Extended]') IS NOT NULL DROP VIEW [BuildApprovals_Extended]
GO


GO
IF OBJECT_ID('[BuildArtifacts_Extended]') IS NOT NULL DROP VIEW [BuildArtifacts_Extended]
GO

CREATE VIEW [BuildArtifacts_Extended] AS

	SELECT A.[Application_Id]
          ,A.[Application_Name]
		  ,R.[Release_Id]
		  ,R.[Release_Number]
		  ,B.[Build_Id]
		  ,B.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,D.[Deployable_Name]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,BA.[Artifact_Path]

	  FROM [BuildArtifacts] BA
	       INNER JOIN [Builds] B
		           ON B.[Build_Id] = BA.[Build_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
		   INNER JOIN [Applications] A
		           ON A.[Application_Id] = R.[Application_Id]
		    LEFT JOIN [Deployables] D
				   ON BA.[Deployable_Id] = D.[Deployable_Id]

GO



GO
IF OBJECT_ID('[BuildArtifacts_Slim]') IS NOT NULL DROP VIEW [BuildArtifacts_Slim]
GO

CREATE VIEW [BuildArtifacts_Slim] AS

	SELECT A.[Application_Id]
	      ,R.[Release_Id]
		  ,R.[Release_Number]
		  ,B.[Build_Id]
		  ,B.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,B.[BuildStatus_Name]
		  ,BA.[Artifact_Path]

	  FROM [BuildArtifacts] BA
		   INNER JOIN [Builds] B
				   ON BA.[Build_Id] = B.[Build_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
		   INNER JOIN [Applications] A
		           ON A.[Application_Id] = R.[Application_Id]

GO



GO
IF OBJECT_ID('[BuildExecutions_ActionsInProgress]') IS NOT NULL DROP VIEW [BuildExecutions_ActionsInProgress]
GO


GO
IF OBJECT_ID('[BuildExecutions_Extended]') IS NOT NULL DROP VIEW [BuildExecutions_Extended]
GO

CREATE VIEW [BuildExecutions_Extended] AS

	SELECT BE.[Execution_Id]
		  ,A.[Application_Id]
		  ,R.[Release_Id]
		  ,R.[Release_Number]
		  ,B.[Build_Id]
		  ,B.[Build_Number]
		  ,B.[BuildStatus_Name]
		  ,R.[Pipeline_Id]
		  ,P.[Pipeline_Name]
		  ,P.[Pipeline_Color]
		  ,ENV.[Environment_Id]
		  ,ENV.[Environment_Name]
		  ,BP.[Promotion_Id]
		  ,BP.[PipelineStage_Name]
		  ,BE.[Plan_Name]
		  ,[ExecutionStart_Date] = E.[Start_Date]
		  ,[ExecutionEnded_Date] = E.[End_Date]
		  ,E.[ExecutionRunState_Code]
		  ,E.[ExecutionStatus_Code]
		  ,[ExecutionStatus_Name] =
		   CASE
				WHEN E.[ExecutionRunState_Code] = 'P' THEN 'Pending'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NULL THEN 'Executing'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NOT NULL THEN 'Canceled'
				WHEN E.[ExecutionStatus_Code] = 'E' THEN 'Failed'
				ELSE 'Succeeded'
		   END
		  ,E.[CreatedBy_User_Name]
		  ,E.[CreatedOn_Date]
		  ,E.[CanceledBy_User_Name]
		  ,E.[CanceledOn_Date]
		  ,BE.[LastActionReexecutionOf_Execution_Id]
		  ,A.[Application_Name]
		  ,BP.[PromotionStatus_Name]
		  ,BP.[Promoted_Date]
		  ,BP.[PromotedBy_Name]
		  ,[WarningLogEntry_Indicator] = CASE WHEN E.[ExecutionStatus_Code] = 'W' THEN 'Y' ELSE 'N' END
          ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
		  ,R.[ReleaseStatus_Name]
		  ,BE.[Initial_BuildExecution_DeploymentPlan_Id]
		  ,[Initial_BuildExecution_DeploymentPlan_Name] = BEDP.[Original_DeploymentPlan_Name]
		  ,A.[VariableSupport_Code]
		  ,A.[ApplicationGroup_Id]
		  ,R.[Release_Sequence]
		  ,BE.[ReexecutionState_Xml]
          ,BE.[RunContextHash_Bytes]
		  ,B.[BuildImporter_Configuration]
		  ,E.[DependentUpon_Execution_Id]
          ,BE.[PipelineStageTarget_Index]

	  FROM [BuildExecutions] BE
	       INNER JOIN [Executions] E
		          ON E.[Execution_Id] = BE.[Execution_Id]
		   INNER JOIN [BuildPromotions] BP
		           ON BP.[Promotion_Id] = BE.[Promotion_Id]
		   INNER JOIN [Builds] B
				  ON B.[Build_Id] = BP.[Build_Id]
           INNER JOIN [Releases] R
				  ON R.[Release_Id] = B.[Release_Id]
		   INNER JOIN [Applications] A
				  ON A.[Application_Id] = R.[Application_Id]
			LEFT JOIN [BuildExecution_DeploymentPlans] BEDP
			       ON BEDP.[BuildExecution_DeploymentPlan_Id] = BE.[Initial_BuildExecution_DeploymentPlan_Id]
		    LEFT JOIN [Environments] ENV
			       ON ENV.[Environment_Id] = BE.[Environment_Id]
			LEFT JOIN [Pipelines] P
				   ON P.[Pipeline_Id] = R.[Pipeline_Id]

GO



GO
IF OBJECT_ID('[BuildExecutions_PendingCancellation]') IS NOT NULL DROP VIEW [BuildExecutions_PendingCancellation]
GO


GO
IF OBJECT_ID('[BuildExecutions_PipelineStages]') IS NOT NULL DROP VIEW [BuildExecutions_PipelineStages]
GO

CREATE VIEW [BuildExecutions_PipelineStages] AS

	SELECT 
          BP.[PipelineStage_Name],
          [PipelineStage_Execution_Count] = COUNT([Execution_Id])

	  FROM [BuildExecutions] BX
	  JOIN [BuildPromotions] BP ON BX.[Promotion_Id] = BP.[Promotion_Id]
      GROUP BY BP.[PipelineStage_Name]
                     
GO



GO
IF OBJECT_ID('[BuildExecutions_ActionGroupActions_Extended]') IS NOT NULL DROP VIEW [BuildExecutions_ActionGroupActions_Extended]
GO

CREATE VIEW [BuildExecutions_ActionGroupActions_Extended] AS

SELECT BE.[Execution_Id],
		  [BuildExecutionStatus_Name] =
		   CASE
				WHEN E.[ExecutionRunState_Code] = 'P' THEN 'Pending'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NULL THEN 'Executing'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NOT NULL THEN 'Canceled'
				WHEN E.[ExecutionStatus_Code] = 'E' THEN 'Failed'
				ELSE 'Succeeded'
		   END,
	   BP.[Promotion_Id],
	   BP.[PipelineStage_Name],

       A.[Application_Id],
	   R.[Release_Id],
       R.[Release_Number],
	   B.[Build_Id],
       B.[Build_Number],

       BEAGA.[Original_Action_Sequence],
       BEAGA.[ExecutionAction_Description],

       BEDPAG.[Original_DeploymentPlanActionGroup_Sequence],
	   BEDPAG.[Original_DeploymentPlanActionGroup_Name],

       BEAGA.[ExecutionStarted_Date],
       BEAGA.[ExecutionEnded_Date],
       [ExecutionStatus_Name] = COALESCE(BEAGA.[ExecutionStatus_Name], 'Pending'),

       BEDPAG.[Original_Deployable_Name],
       BEDPAG.[Original_Deployable_Application_Name],

	   BEAGA.[WarningLogEntry_Indicator],
	   A.[VariableSupport_Code]

  FROM [BuildExecutions] BE
       INNER JOIN [Executions] E
	           ON E.[Execution_Id] = BE.[Execution_Id]
       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
               ON BEDP.[Execution_Id] = BE.[Execution_Id]
       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
               ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
       INNER JOIN [BuildExecution_DeploymentPlanActionGroups] BEDPAG
               ON BEDPAG.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
              AND BEDPAG.[Original_DeploymentPlanActionGroup_Sequence] = BEAGA.[Original_DeploymentPlanActionGroup_Sequence]
	   INNER JOIN [BuildPromotions] BP
	           ON BP.[Promotion_Id] = BE.[Promotion_Id]
       INNER JOIN [Builds] B
	           ON B.[Build_Id] = BP.[Build_Id]
	   INNER JOIN [Releases] R
	           ON R.[Release_Id] = B.[Release_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]
GO



GO
IF OBJECT_ID('[BuildMaster_UsageInfo]') IS NOT NULL
	DROP VIEW [BuildMaster_UsageInfo]
GO

CREATE VIEW [BuildMaster_UsageInfo]
AS

	SELECT 
            [SqlServerVersion_Text] = @@VERSION,
            [ChangeScripts_Count] = (SELECT COUNT(*)
                                          FROM [DatabaseChangeScripts]
                                         WHERE [Active_Indicator] = 'Y'),
            [AutomatedApprovals_Count] = (SELECT COUNT(*)
                                              FROM [BuildPromotionRequirements] 
                                             WHERE [Requirement_Configuration] IS NULL),
            [ManualApprovals_Count] = (SELECT COUNT(*)
                                          FROM [BuildPromotionRequirements] 
                                         WHERE [Requirement_Configuration] IS NOT NULL),
            [Deployables_Count] = (SELECT COUNT(*) FROM [Deployables]),
            [DeployablesImported_Count] = (SELECT COUNT(*) FROM [DeployablesImported]),
            [DeployableDependencies_Count] = (SELECT COUNT(*) FROM [DeployableDependencies]),
            [ApplicationsWithIssueTrackers_Count] = (SELECT COUNT(*)
                                                       FROM [Applications]
                                                      WHERE [IssueTracking_Provider_Id] IS NOT NULL)



GO
IF OBJECT_ID('[BuildOutputs_Extended]') IS NOT NULL DROP VIEW [BuildOutputs_Extended]
GO

CREATE VIEW [BuildOutputs_Extended] AS
    SELECT  BO.[Execution_Id]
           ,BO.[Output_Sequence]
           ,BO.[Output_Name]
           ,BO.[Output_Bytes]
           ,BO.[OutputType_Code]
           ,R.[Application_Id]
		   ,R.[Release_Id]
           ,R.[Release_Number]
		   ,B.[Build_Id]
           ,B.[Build_Number]
           ,BP.[Promotion_Id]
		   ,BP.[PipelineStage_Name]
		   ,ENV.[Environment_Id]
		   ,ENV.[Environment_Name]
           ,[ExecutionStart_Date] = E.[Start_Date]
           ,[ExecutionStatus_Name] =
		   CASE
				WHEN E.[ExecutionRunState_Code] = 'P' THEN 'Pending'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NULL THEN 'Executing'
				WHEN E.[ExecutionRunState_Code] = 'X' AND E.[CanceledOn_Date] IS NOT NULL THEN 'Canceled'
				WHEN E.[ExecutionStatus_Code] = 'E' THEN 'Failed'
				ELSE 'Succeeded'
		   END

      FROM [BuildOutputs] BO
INNER JOIN [BuildExecutions] BE
        ON BE.[Execution_Id] = BO.[Execution_Id]
INNER JOIN [Executions] E
        ON E.[Execution_Id] = BO.[Execution_Id]
INNER JOIN [BuildPromotions] BP
        ON BP.[Promotion_Id] = BE.[Promotion_Id]
INNER JOIN [Builds] B
        ON B.[Build_Id] = BP.[Build_Id]
INNER JOIN [Releases] R
        ON R.[Release_Id] = B.[Release_Id]
 LEFT JOIN [Environments] ENV
        ON ENV.[Environment_Id] = BE.[Environment_Id]

GO



GO
IF OBJECT_ID('[BuildPromotionRequirements_Extended]') IS NOT NULL DROP VIEW [BuildPromotionRequirements_Extended]
GO



GO
IF OBJECT_ID('[BuildPromotionRequirements_VerificationRequired]') IS NOT NULL DROP VIEW [BuildPromotionRequirements_VerificationRequired]
GO



GO
IF OBJECT_ID('[BuildPromotionsScheduled_Extended]') IS NOT NULL DROP VIEW [BuildPromotionsScheduled_Extended]
GO

CREATE VIEW [BuildPromotionsScheduled_Extended] AS

SELECT A.[Application_Id],
       A.[Application_Name],
	   R.[Release_Id],
	   R.[Release_Number],
	   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
	   B.[Build_Id],
       B.[Build_Number],
       BPS.[PipelineStage_Name],
	   BPS.[PromoteOn_Date]
  FROM [BuildPromotionsScheduled] BPS
       INNER JOIN [Builds] B
	           ON B.[Build_Id] = BPS.[Build_Id]
	   INNER JOIN [Releases] R
	           ON R.[Release_Id] = B.[Release_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]

GO



GO
IF OBJECT_ID('[BuildPromotions_Extended]') IS NOT NULL DROP VIEW [BuildPromotions_Extended]
GO

CREATE VIEW [BuildPromotions_Extended] AS

SELECT A.[Application_Id]
      ,R.[Release_Id]
      ,R.[Release_Number]
	  ,B.[Build_Id]
      ,B.[Build_Number]
	  ,BP.[Promotion_Id]
      ,BP.[Promoted_Date]
      ,BP.[PromotedBy_Name]
      ,BP.[Comments_Text]
      ,BP.[PromotionStatus_Name]
	  ,BP.[StatusOverriddenBy_User_Name]
	  ,BP.[StatusOverriddenReason_Text]
	  ,BP.[StatusOverridden_Date]
	  ,BP.[Forced_Indicator]

      ,A.[Application_Name]

      ,B.[BuildStatus_Name]

      ,R.[Pipeline_Id]
      ,R.[Release_Sequence]

	  ,BE.[Execution_Id] AS [Last_Execution_Id]
	  ,X.[Start_Date] AS [Last_ExecutionStart_Date]
	  ,[Last_ExecutionStatus_Name] =
		   CASE
				WHEN X.[ExecutionRunState_Code] = 'P' THEN 'Pending'
				WHEN X.[ExecutionRunState_Code] = 'X' AND X.[CanceledOn_Date] IS NULL THEN 'Executing'
				WHEN X.[ExecutionRunState_Code] = 'X' AND X.[CanceledOn_Date] IS NOT NULL THEN 'Canceled'
				WHEN X.[ExecutionStatus_Code] = 'E' THEN 'Failed'
				ELSE 'Succeeded'
		   END

      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  ,BP.[PipelineStage_Name]

  FROM [BuildPromotions] BP
       INNER JOIN [Builds] B
               ON B.[Build_Id] = BP.[Build_Id]
       INNER JOIN [Releases] R
               ON R.[Release_Id] = B.[Release_Id]
       INNER JOIN [Applications] A
               ON A.[Application_Id] = R.[Application_Id]
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = 
	             (
	              SELECT TOP 1 _BE.[Execution_Id]
	                FROM [BuildExecutions] _BE
					     INNER JOIN [Executions] _X
						         ON _X.[Execution_Id] = _BE.[Execution_Id]
	               WHERE _BE.[Promotion_Id] = BP.[Promotion_Id]
	               ORDER BY _X.[Execution_Id] DESC
	             )
		LEFT JOIN [Executions] X
		       ON X.[Execution_Id] = BE.[Execution_Id]
GO



GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovals]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovals]
GO


GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovalsByUser]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovalsByUser]
GO


GO

IF OBJECT_ID('[BuildSchedules_Extended]') IS NOT NULL DROP VIEW [BuildSchedules_Extended]
GO

CREATE VIEW [BuildSchedules_Extended] AS

    SELECT
           BS.[Schedule_Id]
          ,BS.[Schedule_Name]
          ,BS.[Application_Id]
          ,A.[Application_Name]
          ,BS.[Active_Indicator]
          ,BS.[CreatedBy_User_Name]
          ,BS.[CreatedOn_Date]
          ,BS.[ModifiedBy_User_Name]
          ,BS.[ModifiedOn_Date]
          ,BS.[Schedule_Type]
          ,BS.[StartTime_Minutes]
          ,BS.[EndTime_Minutes]
          ,BS.[DailyInterval_Minutes]
          ,BS.[Recurrence_Type]
          ,BS.[RecurrenceFrequency_Value]
          ,BS.[RecurrenceWeeklyDay_Mask]
          ,BS.[RecurrenceMonthlyDay_Value]
          ,BS.[RecurrenceMonthlyRelativity_Type]
          ,BS.[RecurrenceMonthlyRelativityDay_Type]
          ,BS.[RecurrenceMonthlyRelativityWeek_Type]
          ,BS.[RecurrenceMonthlyRelativityWeekDay_Value]
          ,BS.[Url_AllowedIP_CSV]
          ,BS.[Url_Username_Text]
          ,BS.[Url_Password_Text]
          ,BS.[SourceControl_Path_Name]
          ,BS.[SourceControl_Provider_Id]
          ,[SourceControl_Provider_Name] = P.[Provider_Name]
          ,BS.[SourceControl_QuietPeriod_Minutes]
          ,BS.ReleaseFilter_ReleaseType_Code
          ,BS.Releasefilter_Workflow_Id
          ,BS.[BuildImporter_Configuration]
		  ,BS.[LastExecution_Date]
          ,BS.[ReleaseFilter_Pipeline_Id]

      FROM [BuildSchedules] BS

INNER JOIN [Applications] A
        ON A.[Application_Id] = BS.[Application_Id]

LEFT JOIN [Providers] P
        ON P.[Provider_Id] = BS.[SourceControl_Provider_Id]

GO



GO
IF OBJECT_ID('[BuildScheduleVariableDefinitions_Extended]') IS NOT NULL DROP VIEW [BuildScheduleVariableDefinitions_Extended]
GO

CREATE VIEW [BuildScheduleVariableDefinitions_Extended] AS

SELECT BSVD.[Schedule_Id],
       BSVD.[VariableDeclaration_Id],
       VDEC.[Variable_Name],
       VDEC.[Scope_Code],
       VDEC.[Required_Indicator],
       VDEC.[DefaultValue_Text],
       VDEC.[Variable_Configuration],
       BSVD.[Value_Text],
       [Application_Id] = VDEC.[Scoped_Application_Id]
  FROM [BuildScheduleVariableDefinitions] BSVD
       INNER JOIN [VariableDeclarations] VDEC
               ON BSVD.[VariableDeclaration_Id] = VDEC.[VariableDeclaration_Id]
GO


GO
IF OBJECT_ID('[Builds_Extended]') IS NOT NULL DROP VIEW [Builds_Extended]
GO

CREATE VIEW [Builds_Extended] AS

SELECT R.[Release_Id]
	  ,R.[Release_Number]
	  ,B.[Build_Id]
      ,B.[Build_Number]
      ,B.[BuildStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
      ,R.[Target_Date]
	  ,B.[Build_Sequence]
      ,R.[Pipeline_Id]
	  ,P.[Pipeline_Name]
	  ,P.[Pipeline_Color]
      ,R.[ReleaseStatus_Name]
      ,R.[Release_Sequence]
      ,[Release_Notes_Text] = R.[Notes_Text]
      ,B.[Rejection_Notes]
      ,B.[RejectedBy_User_Name]
      ,B.[Rejected_Date]
      ,B.[CreatedBy_User_Name]
      ,B.[CreatedOn_Date]
      ,B.[ModifiedBy_User_Name]
      ,B.[ModifiedOn_Date]
	  ,B.[BuildImporter_Configuration]

	  ,A.[Application_Id]
	  ,A.[Application_Name]

	  ,[Latest_Promotion_Id] = BP.[Promotion_Id]
	  ,[Latest_Promotion_Date] = BP.[Promoted_Date]
	  ,[Latest_Promotion_Status] = BP.[PromotionStatus_Name]
	  ,[Latest_PipelineStage_Name] = BP.[PipelineStage_Name]
	  ,B.[Furthest_PipelineStage_Name]

	  ,[Latest_Execution_Id] = BE.[Execution_Id]
	  --,[Latest_Execution_PipelineStageTarget_Name] = BE.[PipelineStageTarget_Name]
	  ,[Latest_Execution_Environment_Id] = BE.[Environment_Id]
	  ,[Latest_Execution_Environment_Name] = ENV.[Environment_Name]
	  ,[Latest_Execution_RunState_Code] = E.[ExecutionRunState_Code]
	  ,[Latest_Execution_Status_Code] = E.[ExecutionStatus_Code]

  FROM [Builds] B
       INNER JOIN [Releases] R
               ON B.[Release_Id] = R.[Release_Id]
       INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
        OUTER APPLY(
            SELECT TOP 1 * FROM [BuildPromotions] BP WHERE BP.[Build_Id] = B.[Build_Id] ORDER BY BP.[Promoted_Date] DESC
        ) BP
        OUTER APPLY(
            SELECT TOP 1 * FROM [BuildExecutions] BE WHERE BE.[Promotion_Id] = BP.[Promotion_Id] ORDER BY [Execution_Id] DESC
        ) BE
		LEFT JOIN [Executions] E
		       ON E.[Execution_Id] = BE.[Execution_Id]
		LEFT JOIN [Environments] ENV
		       ON ENV.[Environment_Id] = BE.[Environment_Id]
		LEFT JOIN [Pipelines] P
		       ON P.[Pipeline_Id] = R.[Pipeline_Id]
GO



GO
IF OBJECT_ID('[Builds_RequiredPromotions]') IS NOT NULL DROP VIEW [Builds_RequiredPromotions]
GO



GO
IF OBJECT_ID('[BuildTestResults_AllGroups]') IS NOT NULL DROP VIEW [BuildTestResults_AllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [BuildTestResults_AllGroups] WITH SCHEMABINDING AS

SELECT BTR.[Group_Name],
       [Group_Count] = COUNT_BIG(*)
  FROM dbo.[BuildTestResults] BTR
 GROUP BY BTR.[Group_Name]

GO

CREATE UNIQUE CLUSTERED INDEX [IX__BuildTestResults_AllGroups]
	ON [BuildTestResults_AllGroups] ([Group_Name])

GO


GO
IF OBJECT_ID('[BuildTestResults_Extended]') IS NOT NULL DROP VIEW [BuildTestResults_Extended]
GO

CREATE VIEW [BuildTestResults_Extended] AS

SELECT BTR.[Execution_Id]
      ,BTR.[Group_Name]
      ,BTR.[Test_Sequence]
      ,BTR.[Test_Name]
      ,BTR.[TestStatus_Code]
      ,BTR.[TestResult_Text]
      ,BTR.[TestStarted_Date]
      ,BTR.[TestEnded_Date]
      ,A.[Application_Id]
      ,A.[Application_Name]
	  ,R.[Release_Id]
      ,R.[Release_Number]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])

  FROM [BuildTestResults] BTR
	   INNER JOIN [BuildExecutions] BE
	           ON BTR.[Execution_Id] = BE.[Execution_Id]
	   INNER JOIN [BuildPromotions] BP
	           ON BP.[Promotion_Id] = BE.[Promotion_Id]
	   INNER JOIN [Builds] B
	           ON B.[Build_Id] = BP.[Build_Id]
	   INNER JOIN [Releases] R
	           ON R.[Release_Id] = B.[Release_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]
GO



GO
IF OBJECT_ID('[BuildTestResults_TestGroupResults]') IS NOT NULL DROP VIEW [BuildTestResults_TestGroupResults]
GO

CREATE VIEW [BuildTestResults_TestGroupResults] AS

SELECT TR.[Execution_Id],
       TR.[Group_Name],
	   BP.[Build_Id],
       COUNT(*) AS [Total_Tests], 
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'F'), 'I')) AS [Passed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'I')) AS [Failed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'F')) AS [Inconclusive_Tests]
  FROM [BuildTestResults] TR
       INNER JOIN [BuildExecutions] BE
               ON BE.[Execution_Id] = TR.[Execution_Id]
	   INNER JOIN [BuildPromotions] BP
	           ON BP.[Promotion_Id] = BE.[Promotion_Id]
 GROUP BY 
    TR.[Execution_Id], 
    TR.[Group_Name],
	BP.[Build_Id]



GO
IF OBJECT_ID('[ChangeControlEnvironments_Extended]') IS NOT NULL DROP VIEW [ChangeControlEnvironments_Extended]
GO

CREATE VIEW [ChangeControlEnvironments_Extended] AS

	SELECT CCE.[ChangeControl_Id],
	       CCE.[Environment_Id],
	       CC.[ChangeControl_Name],
	       E.[Environment_Name]
	  FROM [ChangeControlEnvironments] CCE
INNER JOIN [Environments] E
	    ON E.[Environment_Id] = CCE.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]

GO


GO
IF OBJECT_ID('[ChangeControlPerformances_Extended]') IS NOT NULL DROP VIEW [ChangeControlPerformances_Extended]
GO

CREATE VIEW [ChangeControlPerformances_Extended] AS

	SELECT CCP.[ChangeControl_Id],
	       CCP.[Environment_Id],
	       CCP.[PerformedBy_User_Name],
	       CCP.[Performed_Date],
	       CCP.[Notes_Text],
	       E.[Environment_Name],
	       R.[Application_Id],
		   R.[Release_Id],
	       R.[Release_Number],
	       CC.[Group_Name],
	       CC.[Description_Text],
	       CC.[ChangeControl_Name],
	       CC.[Principal_Name],
	       CC.[PrincipalType_Code],
	       CC.[CreatedBy_User_Name],
	       CC.[ModifiedBy_User_Name],
	       CC.[CreatedOn_Date],
	       CC.[ModifiedOn_Date],
	       CC.[PerformanceType_Code],
	       CC.[Active_Indicator],
           [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  FROM [ChangeControlPerformances] CCP
INNER JOIN [Environments] E
	    ON  CCP.[Environment_Id] = E.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
INNER JOIN [Releases] R
        ON R.[Release_Id] = CC.[Release_Id]
GO


GO
IF OBJECT_ID('[ChangeControlPerformances_ReleaseSummary]') IS NOT NULL DROP VIEW [ChangeControlPerformances_ReleaseSummary]
GO

CREATE VIEW [ChangeControlPerformances_ReleaseSummary] AS

	    SELECT  CC.[ChangeControl_Id],
	     E.[Environment_Name],
	     E.[Environment_Id],
       CCP.[ChangeControlPerformance_Id],
       CCP.[Notes_Text],
       CCP.[PerformedBy_User_Name],
       CCP.[Performed_Date],
           [ChangeControlPerformed_Indicator] = CASE
                WHEN [ChangeControlPerformance_Id] IS NULL
                    THEN 'N'
                    ELSE 'Y'
                END
          FROM [ChangeControlEnvironments] CCE
    INNER JOIN [ChangeControls] CC
            ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]
    LEFT JOIN  [ChangeControlPerformances] CCP
            ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
            AND CCP.[Environment_Id] = CCE.[Environment_Id]
    INNER JOIN [Environments] E
            ON E.[Environment_Id] = CCE.[Environment_Id]

GO


GO
IF OBJECT_ID('[ChangeControls_Extended]') IS NOT NULL DROP VIEW [ChangeControls_Extended]
GO

CREATE VIEW [ChangeControls_Extended]
AS
SELECT CC.[ChangeControl_Id],
       CC.[Active_Indicator],
	   CC.[ChangeControl_Name],
	   CC.[CreatedBy_User_Name],
	   CC.[CreatedOn_Date],
	   CC.[Description_Text],
	   CC.[Group_Name],
	   CC.[ModifiedBy_User_Name],
	   CC.[ModifiedOn_Date],
	   CC.[PerformanceType_Code],
	   CC.[Principal_Name],
	   CC.[PrincipalType_Code],
	   CC.[Release_Id],
	   R.[Release_Number],
	   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
	   A.[Application_Id],
	   A.[Application_Name]
  FROM [ChangeControls] CC
       INNER JOIN [Releases] R
	           ON R.[Release_Id] = CC.[Release_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]
GO


GO
IF OBJECT_ID('[ConfigurationFileDeployments_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileDeployments_Extended]
GO

CREATE VIEW [ConfigurationFileDeployments_Extended] AS

SELECT CFD.[ConfigurationFileDeployment_Id]
      ,CFD.[Execution_Id]
      ,CFD.[ConfigurationFile_Id]
	  ,CF.[ConfigurationFile_Name]
	  ,CF.[FilePath_Text]
      ,CFD.[Instance_Name]
      ,CFD.[Version_Number]
      ,CFD.[Server_Id]
      ,CFD.[DeployedTo_Path]
      ,CFD.[DeployedBy_User_Name]
      ,CFD.[DeployedOn_Date]

      ,CFI.[Environment_Id]
	  ,E.[Environment_Name]

      ,CF.[Deployable_Id]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,D.[Deployable_Name]
      ,D.[Application_Id]

      ,S.[Server_Name]

      ,R.[Release_Id]
	  ,R.[Release_Number]
	  ,[Release_Name] = COALESCE(R.[Release_Number], R.[Release_Name])
	  ,B.[Build_Id]
	  ,B.[Build_Number]
	  ,BP.[Promotion_Id]
	  ,BP.[PipelineStage_Name]

 FROM [ConfigurationFileDeployments] CFD
      INNER JOIN [ConfigurationFileInstances] CFI
              ON CFI.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFI.[Instance_Name] = CFD.[Instance_Name]
       LEFT JOIN [Environments] E
              ON E.[Environment_Id] = CFI.[Environment_Id]
      INNER JOIN [ConfigurationFiles] CF
              ON CF.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
      INNER JOIN [ConfigurationFileVersions] CFV
              ON CFV.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFV.[Instance_Name] = CFD.[Instance_Name]
             AND CFV.[Version_Number] = CFD.[Version_Number]
       LEFT JOIN [Deployables] D
              ON D.[Deployable_Id] = CF.[Deployable_Id]
      INNER JOIN [Servers] S
              ON S.[Server_Id] = CFD.[Server_Id]
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = CFD.[Execution_Id]
	   LEFT JOIN [BuildPromotions] BP
	          ON BP.[Promotion_Id] = BE.[Promotion_Id]
	   LEFT JOIN [Builds] B
	          ON B.[Build_Id] = BP.[Build_Id]
	   LEFT JOIN [Releases] R
	          ON R.[Release_Id] = B.[Release_Id]

GO


GO
IF OBJECT_ID('[ConfigurationFileInstances_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileInstances_Extended]
GO

CREATE VIEW [ConfigurationFileInstances_Extended] AS

SELECT CFI.[ConfigurationFile_Id]
      ,CFI.[Instance_Name]
      ,CFI.[Active_Indicator]
      ,CFI.[Environment_Id]
      ,E.[Environment_Name]
	  ,E.[Display_Sequence]

      ,CF.[Deployable_Id]
      ,CF.[FilePath_Text]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,CF.[Application_Id]

      ,[LatestVersion_Version_Number] = CFV.[Version_Number]
      ,[LatestVersion_VersionNotes_Text] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[VersionNotes_Text] ELSE CFV_NonNull.[VersionNotes_Text] END
      ,[LatestVersion_File_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Date] ELSE CFV_NonNull.[File_Date] END
      ,[LatestVersion_File_Bytes] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Bytes] ELSE CFV_NonNull.[File_Bytes] END
      ,[LatestVersion_CreatedOn_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedOn_Date] ELSE CFV_NonNull.[CreatedOn_Date] END
      ,[LatestVersion_CreatedByUser_Name] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedBy_User_Name] ELSE CFV_NonNull.[CreatedBy_User_Name] END

      ,CFI.[Template_Indicator]
      ,CFI.[Template_Instance_Name]
      ,[Template_Instance_Count] = (    
                             SELECT CASE WHEN CFI.[Template_Indicator] = 'Y' THEN COUNT(*) ELSE NULL END
                             FROM [ConfigurationFileInstances] 
                             WHERE [ConfigurationFile_Id] = CFI.[ConfigurationFile_Id] 
                               AND [Active_Indicator] = 'Y'
                               AND [Template_Instance_Name] = CFI.[Instance_Name]
                               )
	  ,CFI.[TransformType_Code]

  FROM [ConfigurationFileInstances] CFI
       INNER JOIN [ConfigurationFiles] CF
               ON CFI.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]
        LEFT JOIN [Deployables] D
               ON D.[Deployable_Id] = CF.[Deployable_Id]

	    LEFT JOIN (SELECT [ConfigurationFile_Id],
		                  [Instance_Name],
						  [Latest_Version_Number] = MAX(CFV_Inner.[Version_Number]),
						  [Last_NonNull_Version] = (SELECT MAX(CFV_Inner_Inner.[Version_Number])
						                              FROM [ConfigurationFileVersions] CFV_Inner_Inner
													 WHERE CFV_Inner_Inner.[ConfigurationFile_Id] = CFV_Inner.[ConfigurationFile_Id]
													   AND CFV_Inner_Inner.[Instance_Name] = CFV_Inner.[Instance_Name]
													   AND CFV_Inner_Inner.[Version_Number] < MAX(CFV_Inner.[Version_Number])
													   AND CFV_Inner_Inner.[Null_Indicator] = 'N')
		             FROM [ConfigurationFileVersions] CFV_Inner
					GROUP BY [ConfigurationFile_Id], [Instance_Name]) CFV_Latest
			   ON CFV_Latest.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV_Latest.[Instance_Name] = CFI.[Instance_Name]

		LEFT JOIN [ConfigurationFileVersions] CFV
		       ON CFV.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV.[Instance_Name] = CFI.[Instance_Name]
			  AND CFV.[Version_Number] = CFV_Latest.[Latest_Version_Number]

		LEFT JOIN [ConfigurationFileVersions] CFV_NonNull
		       ON CFV.[Null_Indicator] = 'Y'
			  AND CFV_NonNull.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV_NonNull.[Instance_Name] = CFI.[Instance_Name]
			  AND CFV_NonNull.[Version_Number] = CFV_Latest.[Last_NonNull_Version]

        LEFT JOIN [Environments] E
               ON E.[Environment_Id] = CFI.[Environment_Id]
GO



GO
IF OBJECT_ID('[ConfigurationFiles_Extended]') IS NOT NULL DROP VIEW [ConfigurationFiles_Extended]
GO

CREATE VIEW [ConfigurationFiles_Extended] AS

SELECT C.[ConfigurationFile_Id]

      ,C.[Deployable_Id]
      ,C.[FilePath_Text]
	  ,C.[ConfigurationFile_Name]
	  ,C.[Description_Text]
      ,C.[Application_Id]
      ,C.[Active_Indicator]
      ,C.[CreatedBy_User_Name]
      ,C.[CreatedOn_Date]
      ,C.[ModifiedBy_User_Name]
      ,C.[ModifiedOn_Date]

      ,D.[Deployable_Name]

      ,[LatestDeployment_ConfigurationFileDeployment_Id] = CFD.[ConfigurationFileDeployment_Id]
      ,[LatestDeployment_Execution_Id] = CFD.[Execution_Id]
      ,[LatestDeployment_Instance_Name] = CFD.[Instance_Name]
      ,[LatestDeployment_Version_Number] = CFD.[Version_Number]
      ,[LatestDeployment_Server_Id] = CFD.[Server_Id]
      ,[LatestDeployment_DeployedTo_Path]= CFD.[DeployedTo_Path]
      ,[LatestDeployment_DeployedBy_User_Name] = CFD.[DeployedBy_User_Name]
      ,[LatestDeployment_DeployedOn_Date] = CFD.[DeployedOn_Date]
      ,[LatestDeployment_Server_Name] = S_CFD.[Server_Name]
  
      ,CFV.[Instance_Name]  [LatestVersion_Instance_Name]
      ,CFV.[Version_Number]  [LatestVersion_Version_Number]
      ,CFV.[VersionNotes_Text]  [LatestVersion_VersionNotes_Text]
      ,CFV.[File_Date]  [LatestVersion_File_Date]
      ,CFV.[CreatedBy_User_Name]  [LatestVersionCreatedBy_User_Name]
      ,CFV.[CreatedOn_Date]  [LatestVersionCreatedOn_Date]

 FROM [ConfigurationFiles] C
        LEFT JOIN [Deployables] D
               ON C.[Deployable_Id] = D.[Deployable_Id]
        LEFT JOIN (SELECT [ConfigurationFile_Id]
                         ,MAX([ConfigurationFileDeployment_Id]) [ConfigurationFileDeployment_Id]
                     FROM [ConfigurationFileDeployments] _CFD
                    GROUP BY [ConfigurationFile_Id]) CFD_MAX
               ON CFD_MAX.[ConfigurationFile_Id]  = C.[ConfigurationFile_Id] 
        LEFT JOIN [ConfigurationFileDeployments] CFD
               ON CFD.[ConfigurationFileDeployment_Id]= CFD_MAX.[ConfigurationFileDeployment_Id]
        LEFT JOIN [Servers] S_CFD
               ON S_CFD.[Server_Id] = CFD.[Server_Id]
  
        LEFT JOIN (SELECT ROW_NUMBER() OVER (PARTITION BY [ConfigurationFile_Id] 
                                                 ORDER BY [File_Date] DESC) [Row_Number],
                          [ConfigurationFile_Id], [Instance_Name], [Version_Number]
                     FROM [ConfigurationFileVersions]) CFV_MAX
               ON CFV_MAX.[ConfigurationFile_Id] = C.[ConfigurationFile_Id]
              AND CFV_MAX.[Row_Number] = 1
        LEFT JOIN [ConfigurationFileVersions] CFV
               ON CFV.[ConfigurationFile_Id] = CFV_MAX.[ConfigurationFile_Id]
              AND CFV.[Instance_Name] = CFV_MAX.[Instance_Name]
              AND CFV.[Version_Number] = CFV_MAX.[Version_Number]
GO



GO
IF OBJECT_ID('[ConfigurationFileVersions_Combined]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_Combined]
GO

CREATE VIEW [ConfigurationFileVersions_Combined] AS

SELECT CF.[ConfigurationFile_Id],
       CFV.[Version_Number],
	   CF.[ConfigurationFile_Name],
	   CF.[FilePath_Text],
	   [Version_CreatedOn_Date] = CFV.[CreatedOn_Date],
	   [Version_CreatedBy_User_Name] = CFV.[CreatedBy_User_Name]
  FROM [ConfigurationFiles] CF
       INNER JOIN (SELECT [ConfigurationFile_Id],
                          [Version_Number],
	                      [CreatedOn_Date] = MAX([CreatedOn_Date]),
						  [CreatedBy_User_Name] = MAX([CreatedBy_User_Name])
                     FROM [ConfigurationFileVersions]
                    GROUP BY [ConfigurationFile_Id], [Version_Number]) CFV
			   ON CFV.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]

GO



GO
IF OBJECT_ID('[ConfigurationFileVersions_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_Extended]
GO

CREATE VIEW [ConfigurationFileVersions_Extended] AS

SELECT CFV.[ConfigurationFile_Id],
       CFV.[Instance_Name],
       CFV.[Version_Number],
	   CFI.[Environment_Id],
	   E.[Environment_Name],
	   CFI.[Active_Indicator],
	   D.[Deployable_Id],
	   D.[Deployable_Name],
	   CFI.[Template_Indicator],
	   CFI.[Template_Instance_Name],
	   CFI.[TransformType_Code],
	   CF.[FilePath_Text],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   [VersionNotes_Text] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[VersionNotes_Text] ELSE CFV_Last.[VersionNotes_Text] END,
	   [File_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Date] ELSE CFV_Last.[File_Date] END,
	   [File_Bytes] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Bytes] ELSE CFV_Last.[File_Bytes] END,
	   [CreatedBy_User_Name] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedBy_User_Name] ELSE CFV_Last.[CreatedBy_User_Name] END,
	   [CreatedOn_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedOn_Date] ELSE CFV_Last.[CreatedOn_Date] END,
	   CFV.[Null_Indicator]

  FROM [ConfigurationFileVersions] CFV
       INNER JOIN [ConfigurationFiles] CF
	           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
	   INNER JOIN [ConfigurationFileInstances] CFI
	           ON CFI.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFI.[Instance_Name] = CFV.[Instance_Name]
	    LEFT JOIN [Environments] E
	           ON E.[Environment_Id] = CFI.[Environment_Id]
	    LEFT JOIN [Deployables] D
	           ON D.[Deployable_Id] = CF.[Deployable_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = CF.[Application_Id]
	    LEFT JOIN [ApplicationGroups] AG
	           ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
	    LEFT JOIN (SELECT [ConfigurationFile_Id],
		                  [Instance_Name],
						  [Version_Number],
						  [Last_NonNull_Version] = (SELECT MAX(CFV_Inner_Inner.[Version_Number])
						                              FROM [ConfigurationFileVersions] CFV_Inner_Inner
													 WHERE CFV_Inner_Inner.[ConfigurationFile_Id] = CFV_Inner.[ConfigurationFile_Id]
													   AND CFV_Inner_Inner.[Instance_Name] = CFV_Inner.[Instance_Name]
													   AND CFV_Inner_Inner.[Version_Number] < CFV_Inner.[Version_Number]
													   AND CFV_Inner_Inner.[Null_Indicator] = 'N')
		             FROM [ConfigurationFileVersions] CFV_Inner
					WHERE [Null_Indicator] = 'Y') CFV2
			   ON CFV.[Null_Indicator] = 'Y'
			  AND CFV2.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFV2.[Instance_Name] = CFV.[Instance_Name]
			  AND CFV2.[Version_Number] = CFV.[Version_Number]
		LEFT JOIN [ConfigurationFileVersions] CFV_Last
		       ON CFV.[Null_Indicator] = 'Y'
			  AND CFV_Last.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFV_Last.[Instance_Name] = CFV.[Instance_Name]
			  AND CFV_Last.[Version_Number] = CFV2.[Last_NonNull_Version]
GO



GO
IF OBJECT_ID('[ConfigurationFileVersions_ModifiedInstances]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_ModifiedInstances]
GO

CREATE VIEW [ConfigurationFileVersions_ModifiedInstances] AS

SELECT CFV.[ConfigurationFile_Id],
       CFV.[Version_Number],
	   CFV.[Instance_Name]
  FROM [ConfigurationFileVersions] CFV
 WHERE CFV.[Null_Indicator] = 'N'

GO



GO
IF OBJECT_ID('[Credentials_Extended]') IS NOT NULL DROP VIEW [Credentials_Extended]
GO

CREATE VIEW [Credentials_Extended]
AS

	SELECT  
        C.[Credential_Id],
        C.[Environment_Id],
        C.[Credential_Name],
        C.[CredentialType_Name],
		C.[Configuration_Xml],
        C.[AllowFunctionAccess_Indicator],
        E.[Environment_Name]

	  FROM [Credentials] C

      LEFT JOIN [Environments] E
             ON E.[Environment_Id] = C.[Environment_Id]

GO



GO
IF OBJECT_ID('[DatabaseChangeScriptExecutions_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScriptExecutions_Extended]
GO

CREATE VIEW [DatabaseChangeScriptExecutions_Extended]
AS
SELECT DCSE.[Execution_Id],
       DCSE.[DatabaseConnection_Id],
	   DCSE.[ScriptIds_Csv],
	   X.[Start_Date],
	   X.[End_Date],
	   X.[CreatedBy_User_Name],
	   X.[CreatedOn_Date],
	   X.[ExecutionStatus_Code],
	   X.[ExecutionRunState_Code],
	   DC.[Application_Id],
	   A.[Application_Name],
	   DC.[Environment_Id],
	   E.[Environment_Name],
	   [Server_Id] = DC.[ExecuteOn_Server_Id],
	   S.[Server_Name]

  FROM [DatabaseChangeScriptExecutions] DCSE
       INNER JOIN [Executions] X
	           ON X.[Execution_Id] = DCSE.[Execution_Id]
	   INNER JOIN [DatabaseConnections] DC
	           ON DC.[DatabaseConnection_Id] = DCSE.[DatabaseConnection_Id]
	   INNER JOIN [Applications] A
	           ON DC.[Application_Id] = A.[Application_Id]
	   INNER JOIN [Environments] E
	           ON DC.[Environment_Id] = E.[Environment_Id]
	    LEFT JOIN [Servers] S
	           ON DC.[ExecuteOn_Server_Id] = S.[Server_Id]



GO
IF OBJECT_ID('[DatabaseChangeScripts_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScripts_Extended]
GO

CREATE VIEW [DatabaseChangeScripts_Extended] AS

	SELECT DCS.[Script_Id]
		  ,A.[Application_Id]
		  ,R.[Release_Id]
		  ,R.[Release_Number]
		  ,DCS.[Deployable_Id]
		  ,DCS.[Script_Name]
		  ,DCS.[Script_Text]
		  ,DCS.[Active_Indicator]
		  ,DCS.[CreatedBy_User_Name]
		  ,DCS.[CreatedOn_Date]
		  ,DCS.[ModifiedBy_User_Name]
		  ,DCS.[ModifiedOn_Date]

          ,A.[Application_Name]
          ,[Numeric_Release_Number] = R.[Release_Sequence]

		  ,D.[Deployable_Name]
          ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])

		  ,DCS.[Script_Guid]

	  FROM [DatabaseChangeScripts] DCS
           INNER JOIN [Releases] R
                   ON R.[Release_Id] = DCS.[Release_Id]
	       INNER JOIN [Applications] A
	               ON R.[Application_Id] = A.[Application_Id]
	        LEFT JOIN [Deployables] D
	               ON DCS.[Deployable_Id] = D.[Deployable_Id]
GO



GO
IF OBJECT_ID('[DatabaseConnections_Extended]') IS NOT NULL DROP VIEW [DatabaseConnections_Extended]
GO

CREATE VIEW [DatabaseConnections_Extended]
AS
SELECT DC.[DatabaseConnection_Id],
       DC.[DatabaseConnection_Name],
	   DC.[DatabaseConnection_Description],
	   DC.[DatabaseConnection_Configuration],
	   DC.[ExecuteOn_Server_Id],
	   [ExecuteOn_Server_Name] = S.[Server_Name],
	   DC.[ModifiedBy_User_Name],
	   DC.[ModifiedOn_Date],
	   DC.[Application_Id],
	   A.[Application_Name],
	   DC.[Environment_Id],
	   E.[Environment_Name]
  FROM [DatabaseConnections] DC
       INNER JOIN [Applications] A
	           ON A.[Application_Id] = DC.[Application_Id]
	   INNER JOIN [Environments] E
	           ON E.[Environment_Id] = DC.[Environment_Id]
	   INNER JOIN [Servers] S
	           ON S.[Server_Id] = DC.[ExecuteOn_Server_Id]

GO



GO

IF OBJECT_ID('[DeployablesImported_Extended]') IS NOT NULL DROP VIEW [DeployablesImported_Extended]
GO

CREATE VIEW [DeployablesImported_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]
	
      ,A.[Application_Id]
      ,A.[Application_Name]
      
      ,[ImportedFrom_Application_Id] = DA.[Application_Id]
      ,[ImportedFrom_Application_Name] = DA.[Application_Name]

FROM [DeployablesImported] DI
INNER JOIN [Applications] A
  ON A.[Application_Id] = DI.[Application_Id]
INNER JOIN [Deployables] D
  ON D.[Deployable_Id] = DI.[Deployable_Id]
INNER JOIN [Applications] DA
  ON DA.[Application_Id] = D.[Application_Id]
  
GO



GO
IF OBJECT_ID('[Deployables_Extended]') IS NOT NULL DROP VIEW [Deployables_Extended]
GO

CREATE VIEW [Deployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])

FROM [Deployables] D
JOIN [Applications] A
  ON A.[Application_Id] = D.[Application_Id]
GO



GO
IF OBJECT_ID('[DeploymentPlanActionGroups_Extended]') IS NOT NULL DROP VIEW [DeploymentPlanActionGroups_Extended]
GO

CREATE VIEW [DeploymentPlanActionGroups_Extended] AS

	SELECT DPAG.[DeploymentPlanActionGroup_Id],
	       DPAG.[DeploymentPlanActionGroup_Sequence],
	       DPAG.[DeploymentPlan_Id],
	       DPAG.[Deployable_Name],
	       [Active_Indicator] = DPAG.[Active_Indicator],
	       [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
	       [ActionGroup_Description] = COALESCE(DPAG.[ActionGroup_Description], AG.[ActionGroup_Description]),
	       [Predicate_Configuration] = COALESCE(DPAG.[Predicate_Configuration], AG.[Predicate_Configuration]),
	       [Server_Id] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN COALESCE(DPAG.[Server_Id], AG.[Server_Id]) ELSE NULL END,
	       [Server_Variable_Name] = CASE WHEN DPAG.[Server_Id] IS NULL THEN COALESCE(DPAG.[Server_Variable_Name], AG.[Server_Variable_Name]) ELSE NULL END,
	       [ServerType_Code] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN S.[ServerType_Code] ELSE NULL END,
	       [Server_Name] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN S.[Server_Name] ELSE NULL END,
	       DPAG.[Iterate_Servers_Indicator],
	       DPAG.[Iterate_Deployables_Indicator],
	       DPAG.[Parallel_Indicator],
	       DPAG.[OnSuccess_DeploymentPlanActionGroup_Id],
		   [OnSuccess_DeploymentPlanActionGroup_Sequence] = S_DPAG.[DeploymentPlanActionGroup_Sequence],
		   [OnSuccess_DeploymentPlanActionGroup_Name] = COALESCE(S_DPAG.[ActionGroup_Name], S_AG.[ActionGroup_Name]),
	       DPAG.[OnFailure_DeploymentPlanActionGroup_Id],
		   [OnFailure_DeploymentPlanActionGroup_Sequence] = F_DPAG.[DeploymentPlanActionGroup_Sequence],
		   [OnFailure_DeploymentPlanActionGroup_Name] = COALESCE(F_DPAG.[ActionGroup_Name], F_AG.[ActionGroup_Name]),
	       DPAG.[OnFailure_Continue_Indicator],

	       [DeploymentPlanActionGroup_ActionGroup_Name] = DPAG.[ActionGroup_Name],
	       [DeploymentPlanActionGroup_ActionGroup_Description] = DPAG.[ActionGroup_Description],
	       [DeploymentPlanActionGroup_Predicate_Configuration] = DPAG.[Predicate_Configuration],
	       [DeploymentPlanActionGroup_ModifiedBy_User_Name] = DPAG.[ModifiedBy_User_Name],
	       [DeploymentPlanActionGroup_ModifiedOn_Date] = DPAG.[ModifiedOn_Date],
	       [DeploymentPlanActionGroup_Server_Id] = DPAG.[Server_Id],
	       [DeploymentPlanActionGroup_Server_Variable_Name] = DPAG.[Server_Variable_Name],

	       [ActionGroup_Id] = AG.[ActionGroup_Id],
	       [ActionGroup_CreatedBy_User_Name] = AG.[CreatedBy_User_Name],
	       [ActionGroup_CreatedOn_Date] = AG.[CreatedOn_Date],
	       [ActionGroup_ModifiedBy_User_Name] = AG.[ModifiedBy_User_Name],
	       [ActionGroup_ModifiedOn_Date] = AG.[ModifiedOn_Date],
	       [ActionGroup_ActionGroup_Name] = AG.[ActionGroup_Name],
	       [ActionGroup_Predicate_Configuration] = AG.[Predicate_Configuration],
	       [ActionGroup_ActionGroup_Description] = AG.[ActionGroup_Description],
	       [ActionGroup_Server_Id] = AG.[Server_Id],
	       [ActionGroup_Server_Variable_Name] = AG.[Server_Variable_Name],

	       [Referenced_Application_Id] = R_A.[Application_Id],
	       [Referenced_Application_Name] = R_A.[Application_Name],

	       [DeploymentPlanActionGroup_Count] = (SELECT COUNT(*)
						                         FROM [DeploymentPlanActionGroups] _DPAG
							                    WHERE _DPAG.[ActionGroup_Id] = DPAG.[ActionGroup_Id])

	  FROM	[DeploymentPlanActionGroups] DPAG
            INNER JOIN [ActionGroups] AG
                    ON DPAG.[ActionGroup_Id] = AG.[ActionGroup_Id]
             LEFT JOIN [Deployables] R_D
                    ON R_D.[Deployable_Name] = DPAG.[Deployable_Name]
                   AND R_D.[Application_Id] = DPAG.[Deployable_Application_Id]
             LEFT JOIN [Applications] R_A
                    ON R_A.[Application_Id] = R_D.[Application_Id]
             LEFT JOIN [Servers] S
                    ON S.[Server_Id] = COALESCE(DPAG.[Server_Id], AG.[Server_Id])
			 LEFT JOIN [DeploymentPlanActionGroups] S_DPAG
			        ON DPAG.[OnSuccess_DeploymentPlanActionGroup_Id] = S_DPAG.[DeploymentPlanActionGroup_Id]
			 LEFT JOIN [ActionGroups] S_AG
			        ON S_DPAG.[ActionGroup_Id] = S_AG.[ActionGroup_Id]
			 LEFT JOIN [DeploymentPlanActionGroups] F_DPAG
			        ON DPAG.[OnFailure_DeploymentPlanActionGroup_Id] = F_DPAG.[DeploymentPlanActionGroup_Id]
			 LEFT JOIN [ActionGroups] F_AG
			        ON F_DPAG.[ActionGroup_Id] = F_AG.[ActionGroup_Id]
GO



GO
IF OBJECT_ID('[DeploymentPlans_ActionUsage]') IS NOT NULL DROP VIEW [DeploymentPlans_ActionUsage]
GO

CREATE VIEW [DeploymentPlans_ActionUsage] AS

SELECT DPAG.[DeploymentPlanActionGroup_Id],
       [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
       DPAG.[DeploymentPlanActionGroup_Sequence],
       DPAG.[ActionGroup_Id],
       DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
       AGA.[Action_Sequence],
       [Uses_DeploymentPlan_Id] = DP_Uses.[DeploymentPlan_Id],
       [Uses_DeploymentPlan_Name] = DP_Uses.[DeploymentPlan_Name]
  FROM [DeploymentPlanActionGroups] DPAG
       INNER JOIN [DeploymentPlans] DP
               ON DP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
       INNER JOIN [ActionGroups] AG
               ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
       INNER JOIN [ActionGroupActions] AGA
               ON DPAG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
       INNER JOIN [DeploymentPlans] DP_Uses
               ON DP_Uses.[DeploymentPlan_Id] = AGA.[Action_Configuration].value('/*[1]/Properties[1]/@DeploymentPlanId', 'INT')
 WHERE CAST(AGA.[Action_Configuration].query('local-name(/*[1])') AS VARCHAR(200)) = 'Inedo.BuildMaster.Extensibility.Actions.BuildMaster.ExecuteDeploymentPlanAction'
   AND AGA.[Action_Configuration].value('/*[1]/@Assembly', 'VARCHAR(200)') = 'BuildMasterExtensions'

GO



GO

IF OBJECT_ID('[EnvironmentServers_Extended]') IS NOT NULL DROP VIEW [EnvironmentServers_Extended]
GO

CREATE VIEW [EnvironmentServers_Extended] AS

	SELECT  ES.[Environment_Id],
            E.[Environment_Name],
            [Environment_Active_Indicator] = E.[Active_Indicator],
            [Server_Restricted_Indicator] = S.[Restricted_Indicator],
			S.[Server_Id],
			S.[Server_Name],
			S.[Agent_Configuration],
			S.[ServerType_Code],
			[Server_Active_Indicator] = S.[Active_Indicator]
      FROM	[Servers] S
 LEFT JOIN	[EnvironmentServers] ES
        ON	S.[Server_Id] = ES.[Server_Id]
 LEFT JOIN	[Environments] E
        ON	ES.[Environment_Id] = E.[Environment_Id]

GO



GO
IF OBJECT_ID('[Environments_Extended]') IS NOT NULL
	DROP VIEW [Environments_Extended]
GO

CREATE VIEW [Environments_Extended]
AS

	SELECT 
        E.[Environment_Id]
       ,E.[Environment_Name]
       ,E.[CreatedBy_User_Name]
       ,E.[CreatedOn_Date]
       ,E.[ModifiedBy_User_Name]
       ,E.[ModifiedOn_Date]
       ,E.[Active_Indicator]
       ,E.[Display_Sequence]
       ,E.[Parent_Environment_Id]
       ,[Parent_Environment_Name] = E_PARENT.[Environment_Name]
       ,[Parent_Display_Sequence] = E_PARENT.[Display_Sequence]
       ,[Servers_Count] = (SELECT COUNT(*) 
                            FROM [EnvironmentServers] ES 
                           WHERE ES.[Environment_Id] = E.[Environment_Id])

	  FROM [Environments] E
 
 LEFT JOIN [Environments] E_PARENT
        ON E_PARENT.[Environment_Id] = E.[Parent_Environment_Id]

GO



GO
IF OBJECT_ID('[EventListeners_Extended]') IS NOT NULL DROP VIEW [EventListeners_Extended]
GO


GO
IF OBJECT_ID('[EventListeners_ForEvent]') IS NOT NULL DROP VIEW [EventListeners_ForEvent]
GO

CREATE VIEW [EventListeners_ForEvent] AS

SELECT N.*

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]
      ,EO.[Application_Id]

  FROM [EventOccurences] EO
       INNER JOIN [EventListenerEvents] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [EventListeners] N
               ON N.[EventListener_Id] = NEC.[EventListener_Id]
 WHERE N.[Active_Indicator] = 'Y'





GO
IF OBJECT_ID('[EventNotifiers]') IS NOT NULL DROP VIEW [EventNotifiers]
GO

CREATE VIEW [EventNotifiers] AS

SELECT N.[Notifier_Id]
      ,N.[Notifier_Name]
      ,N.[Notifier_Configuration]
      ,N.[Owner_User_Name]

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]
      ,EO.[Application_Id]

  FROM [EventOccurences] EO
       INNER JOIN [NotifierEventCodes] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [Notifiers] N
               ON N.[Notifier_Id] = NEC.[Notifier_Id]
 WHERE N.[Active_Indicator] = 'Y'


GO
IF OBJECT_ID('[EventOccurences_Extended]') IS NOT NULL DROP VIEW [EventOccurences_Extended]
GO

CREATE VIEW [EventOccurences_Extended]
AS

	SELECT EO.*
	      ,ET.[Event_Description]
	      ,A.[Application_Name]
		  ,A.[ApplicationGroup_Id]
	  FROM [EventOccurences] EO
	       INNER JOIN [EventTypes] ET	              
	               ON ET.[Event_Code] = EO.[Event_Code]
	        LEFT JOIN [Applications] A
	               ON EO.[Application_Id] = A.[Application_Id]



GO
IF OBJECT_ID('[Executions_Extended]') IS NOT NULL DROP VIEW [Executions_Extended]
GO

CREATE VIEW [Executions_Extended] AS

	SELECT X.[Execution_Id]
		  ,X.[Start_Date]
		  ,X.[End_Date]
		  ,X.[ExecutionRunState_Code]
		  ,X.[ExecutionStatus_Code]
          ,X.[ExecutionMode_Code]
		  ,X.[CreatedBy_User_Name]
		  ,X.[CreatedOn_Date]
		  ,X.[CanceledBy_User_Name]
		  ,X.[CanceledOn_Date]
          ,R.[Application_Id]
          ,A.[Application_Name]
		  ,E.[Environment_Id]
		  ,E.[Environment_Name]
		  ,X.[DependentUpon_Execution_Id]

	  FROM [Executions] X
            LEFT JOIN [BuildExecutions] BX
		          ON X.[Execution_Id] = BX.[Execution_Id]
            LEFT JOIN [BuildPromotions] P
                  ON P.[Promotion_Id] = BX.[Promotion_Id]
            LEFT JOIN [Builds] B
                  ON B.[Build_Id] = P.[Build_Id]
            LEFT JOIN [Releases] R
                  ON R.[Release_Id] = B.[Release_Id]
            LEFT JOIN [Applications] A
                  ON R.[Application_Id] = A.[Application_Id]
			LEFT JOIN [DatabaseChangeScriptExecutions] DCSX
			       ON DCSX.[Execution_Id] = X.[Execution_Id]
			LEFT JOIN [DatabaseConnections] DC
			       ON DC.[DatabaseConnection_Id] = DCSX.[DatabaseConnection_Id]
			LEFT JOIN [Environments] E
			       ON E.[Environment_Id] = BX.[Environment_Id]
				   OR E.[Environment_Id] = DC.[Environment_Id]
                    
                     
GO



GO
IF OBJECT_ID('[IssueSources_Extended]') IS NOT NULL DROP VIEW [IssueSources_Extended]
GO

CREATE VIEW [IssueSources_Extended] AS
    SELECT S.*
          ,A.[Application_Name]
          ,AG.[ApplicationGroup_Name]
          ,[Execution_Start_Date] = E.[Start_Date]
          ,[Execution_End_Date] = E.[End_Date]
          ,E.[ExecutionRunState_Code]
          ,E.[ExecutionStatus_Code]
                                          
          FROM [IssueSources] S
               LEFT JOIN [Applications] A
                      ON A.[Application_Id] = S.[Application_Id]
               LEFT JOIN [ApplicationGroups] AG
                     ON AG.[ApplicationGroup_Id] = S.[ApplicationGroup_Id]
               LEFT JOIN [Executions] E
                     ON E.[Execution_Id] = S.[Last_Execution_Id]
GO


GO
IF OBJECT_ID('[Issues_Extended]') IS NOT NULL DROP VIEW [Issues_Extended]
GO

CREATE VIEW [Issues_Extended] AS
	SELECT I.[Application_Id],
	       A.[Application_Name],
		   A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
		   R.[Release_Id],
		   R.[Release_Number],
		   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
           R.[Release_Sequence],
		   I.[Issue_Id],
           I.[Type_Text],
		   I.[Title_Text],
		   I.[Description_Text],
		   I.[Closed_Indicator],
		   I.[ClosedOn_Date],
           I.[OpenedOn_Original_Build_Number],
           [OpenedOn_Original_Build_Sequence] = B_OPENED.[Build_Sequence],
           I.[ClosedOn_Original_Build_Number],
           [ClosedOn_Original_Build_Sequence]= B_CLOSED.[Build_Sequence],
		   I.[CreatedBy_User_Name],
		   I.[CreatedOn_Date],
		   I.[IssueTracker_IssueId_Text],
		   I.[IssueTracker_IssueStatus_Text],
		   I.[IssueTracker_LastSync_Date],
           I.[IssueTracker_IssueUrl_Text],
           [BuildMasterIssue_Indicator] = CASE WHEN I.[IssueTracker_IssueId_Text] IS NULL THEN 'Y' ELSE 'N' END
                                          
	  FROM [Issues] I
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = I.[Application_Id]
		    LEFT JOIN [ApplicationGroups] AG
			       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
		    LEFT JOIN [Releases] R
		           ON R.[Release_Id] = I.[Release_Id]
            LEFT JOIN [Builds] B_OPENED
                   ON B_OPENED.[Release_Id] = I.[Release_Id]
                  AND B_OPENED.[Build_Number] = I.[OpenedOn_Original_Build_Number]
            LEFT JOIN [Builds] B_CLOSED
                   ON B_CLOSED.[Release_Id] = I.[Release_Id]
                  AND B_CLOSED.[Build_Number] = I.[ClosedOn_Original_Build_Number]
GO


GO
IF OBJECT_ID('[Issues_Types]') IS NOT NULL DROP VIEW [Issues_Types]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [Issues_Types] WITH SCHEMABINDING
AS

	SELECT [Type_Text],
	       [Type_Count] = COUNT_BIG(*)
	  FROM dbo.[Issues]
	 GROUP BY [Type_Text]

	--SELECT DISTINCT([Type_Text])
	--  FROM [Issues]

GO
	CREATE UNIQUE CLUSTERED INDEX [IX__Issues_Types]
		ON [Issues_Types] ([Type_Text])

GO


GO
IF OBJECT_ID('[LogMessages_Categories]') IS NOT NULL
	DROP VIEW [LogMessages_Categories]
GO

CREATE VIEW [LogMessages_Categories]
AS

	SELECT DISTINCT([Category_Name])
	  FROM [LogMessages]

GO


GO
IF OBJECT_ID('LogMessages_Extended') IS NOT NULL DROP VIEW LogMessages_Extended
GO

CREATE VIEW LogMessages_Extended AS
SELECT L.[LogMessage_Id]
      ,L.[Logged_Date]
      ,L.[Message_Text]
      ,L.[Category_Name]
      ,L.[Message_Level]
      ,L.[Details_Bytes]
      ,L.[Server_Id]
      ,S.[Server_Name]
      ,L.[Application_Id]
      ,A.[Application_Name]
      ,L.[Schedule_Id]
      ,BS.[Schedule_Name]
      ,L.[Requirement_Id]
      ,BPR.[Requirement_Description]
      ,[Requirement_Workflow_Id] = BPR.[Workflow_Id]
      ,L.[Provider_Id]
      ,P.[Provider_Name]
      ,L.[EventListener_Id]
      ,L.[ShowNotification_Indicator]
  FROM [LogMessages] L
       LEFT JOIN [Servers] S
              ON L.[Server_Id] = S.[Server_Id]       
       LEFT JOIN [Applications] A
              ON L.[Application_Id] = A.[Application_Id]
       LEFT JOIN [BuildSchedules] BS
              ON L.[Schedule_Id] = BS.[Schedule_Id]
       LEFT JOIN [BuildPromotionRequirements] BPR
              ON L.[Requirement_Id] = BPR.[Requirement_Id]
       LEFT JOIN [Providers] P
              ON L.[Provider_Id] = P.[Provider_Id]
       LEFT JOIN [EventListeners] EL
              ON L.[EventListener_Id] = EL.[EventListener_Id]

GO





GO
IF OBJECT_ID('[ManualActionsInProgress_Extended]') IS NOT NULL DROP VIEW [ManualActionsInProgress_Extended]
GO

CREATE VIEW [ManualActionsInProgress_Extended] AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
		   R.[Release_Id],
 	       R.[Release_Number],
 	       [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
		   B.[Build_Id],
 	       B.[Build_Number],
 	       BE.[Execution_Id],
	       [ActionComplete_Indicator] = CASE WHEN MAIP.[CompletedBy_User_Name] IS NULL THEN 'N' ELSE 'Y' END,
		   MAIP.[AssignedTo_Principal_Name],
		   MAIP.[CompletedBy_User_Name],
		   MAIP.[Task_Name],
		   MAIP.[Task_Description],
	       MAIP.[ActionStarted_Date],
		   MAIP.[ManualAction_Sequence],
		   [ExecutionStart_Date] = X.[Start_Date]
	  FROM [ManualActionsInProgress] MAIP
	       INNER JOIN [BuildExecutions] BE
	               ON BE.[Execution_Id] = MAIP.[Execution_Id]
		   INNER JOIN [Executions] X
		           ON X.[Execution_Id] = BE.[Execution_Id]
		   INNER JOIN [BuildPromotions] BP
		           ON BP.[Promotion_Id] = BE.[Promotion_Id]
		   INNER JOIN [Builds] B
		           ON B.[Build_Id] = BP.[Build_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = R.[Application_Id]



GO
IF OBJECT_ID('[ManualExecutions_Extended]') IS NOT NULL DROP VIEW [ManualExecutions_Extended]
GO

CREATE VIEW [ManualExecutions_Extended]
AS
SELECT ME.[Execution_Id],
	   ME.[ExecutionType_Name],
	   ME.[Execution_Configuration],
	   X.[Start_Date],
	   X.[End_Date],
	   X.[CreatedBy_User_Name],
	   X.[CreatedOn_Date],
	   X.[ExecutionStatus_Code],
	   X.[ExecutionRunState_Code]

  FROM [ManualExecutions] ME
       INNER JOIN [Executions] X
	           ON X.[Execution_Id] = ME.[Execution_Id]



GO
IF OBJECT_ID('[Pipelines_UsedByApplications]') IS NOT NULL DROP VIEW [Pipelines_UsedByApplications]
GO

CREATE VIEW [Pipelines_UsedByApplications] AS

        SELECT DISTINCT R.[Application_Id], R.[Pipeline_Id]
          FROM [Releases] R
               INNER JOIN [Applications] A
                       ON R.[Application_Id] = A.[Application_Id]
                      AND A.[Active_Indicator] = 'Y'
         WHERE R.[Pipeline_Id] IS NOT NULL


GO


GO
IF OBJECT_ID('[Plans_Extended]') IS NOT NULL DROP VIEW [Plans_Extended]
GO

CREATE VIEW [Plans_Extended] AS

SELECT P.[Plan_Id],
       P.[Plan_Name],
       P.[Application_Id],
       A.[Application_Name],
       P.[PlanType_Code],
       P.[Plan_Bytes],
       P.[ModifiedOn_Date],
       P.[ModifiedBy_User_Name],
       P.[Environment_Id],
       E.[Environment_Name]

  FROM [Plans] P

	    LEFT JOIN [Environments] E
               ON P.[Environment_Id] = E.[Environment_Id]
        LEFT JOIN [Applications] A
               ON P.[Application_Id] = A.[Application_Id]

GO


GO
IF OBJECT_ID('[Plans_MostCommonActions]') IS NOT NULL DROP VIEW [Plans_MostCommonActions]
GO

CREATE VIEW [Plans_MostCommonActions] AS

SELECT [Action_Type],
       [Usage_Count] = COUNT([Action_Type])
  FROM (SELECT [Action_Type] = CAST([Action_Configuration].query('local-name(/*[1])') AS VARCHAR(200)) + ',' + [Action_Configuration].value('/*[1]/@Assembly', 'VARCHAR(100)')
          FROM [ActionGroupActions]) AT
 GROUP BY AT.[Action_Type]

GO



GO
IF OBJECT_ID('[Privileges_Extended]') IS NOT NULL
	DROP VIEW [Privileges_Extended]
GO

CREATE VIEW [Privileges_Extended]
AS

	SELECT P.[Privilege_Id]
		  ,P.[UserDirectory_Id]
		  ,P.[Principal_Name]
		  ,P.[PrincipalType_Code]
		  ,P.[PrivilegeType_Code]

		  ,R.[Role_Id]
		  ,R.[Role_Name]

		  ,A.[Application_Id]
		  ,A.[Application_Name]

		  ,E.[Environment_Id]
		  ,E.[Environment_Name]

		  ,AG.[ApplicationGroup_Id]
		  ,AG.[ApplicationGroup_Name]

	  FROM [Privileges] P
		   INNER JOIN [Roles] R
				   ON P.[Role_Id] = R.[Role_Id]

		   LEFT JOIN [Applications] A
				  ON P.[Application_Id] = A.[Application_Id]

		   LEFT JOIN [Environments] E
				  ON P.[Environment_Id] = E.[Environment_Id]
				  
		   LEFT JOIN [ApplicationGroups] AG
				  ON P.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

GO



GO
IF OBJECT_ID('[ReleaseDeployables_Extended]') IS NOT NULL DROP VIEW [ReleaseDeployables_Extended]
GO

CREATE VIEW [ReleaseDeployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])
      ,R.[Release_Id]
      ,R.[Release_Number]
	  ,R.[Pipeline_Id]
      ,RD.[InclusionType_Code]
	  ,RD.[Referenced_Release_Id]
      ,[Referenced_Release_Number] = RREF.[Release_Number]
      ,[Referenced_Application_Id] = RREF.[Application_Id]
      ,[Referenced_Release_Name] = COALESCE(RREF.[Release_Name], RREF.[Release_Number])

      ,[Referenced_Application_Name] = AREF.[Application_Name]

	  ,[Referenced_Pipeline_Id] = RREF.[Pipeline_Id]

FROM [ReleaseDeployables] RD
     INNER JOIN [Deployables] D
             ON D.[Deployable_Id] = RD.[Deployable_Id]
	 INNER JOIN [Releases] R
	         ON R.[Release_Id] = RD.[Release_Id]
     INNER JOIN [Applications] A
             ON A.[Application_Id] = R.[Application_Id]
     LEFT JOIN [Releases] RREF
			 ON RD.[Referenced_Release_Id] = RREF.[Release_Id]
     LEFT JOIN [Applications] AREF
			 ON AREF.[Application_Id] = RREF.[Application_Id]
GO





GO
IF OBJECT_ID('[ReleaseNotes_Extended]') IS NOT NULL DROP VIEW [ReleaseNotes_Extended]
GO

CREATE VIEW [ReleaseNotes_Extended]
AS

SELECT A.[Application_Id],
       A.[Application_Name],
	   RN.[Release_Id],
       R.[Release_Number],
	   R.[Release_Name],
	   RN.[Promotion_Id],
	   B.[Build_Id],
	   B.[Build_Number],
	   RN.[ReleaseNote_Sequence],
	   RN.[Notes_Text],
	   RN.[CreatedOn_Date],
	   RN.[CreatedBy_User_Name],
	   RN.[ModifiedOn_Date],
	   RN.[ModifiedBy_User_Name]
  FROM [ReleaseNotes] RN
       INNER JOIN [Releases] R
	           ON R.[Release_Id] = RN.[Release_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = R.[Application_Id]
	    LEFT JOIN [BuildPromotions] BP
		       ON BP.[Promotion_Id] = RN.[Promotion_Id]
	    LEFT JOIN [Builds] B
		       ON B.[Build_Id] = BP.[Build_Id]


GO
IF OBJECT_ID('[Releases_Extended]') IS NOT NULL DROP VIEW [Releases_Extended]
GO

CREATE VIEW [Releases_Extended] AS

SELECT R.[Release_Id]
      ,R.[Application_Id]
	  ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]
      ,R.[Notes_Text]
	  ,R.[CreatedBy_User_Name]
	  ,R.[CreatedOn_Date]
	  ,R.[ModifiedBy_User_Name]
	  ,R.[ModifiedOn_Date]
	  ,R.[CancelledBy_User_Name]
	  ,R.[CancelledReason_Text]
	  ,R.[CancelledOn_Date]
      ,R.[Target_Date]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
      ,R.[Release_Sequence]
      ,R.[Pipeline_Id]
	  ,P.[Pipeline_Name]
	  ,P.[Pipeline_Color]
	  ,A.[Application_Name]
	  ,R.[Furthest_Build_Number]
	  ,B_Furthest.[Furthest_PipelineStage_Name]
      ,[Furthest_Build_Id] = B_Furthest.[Build_Id]
	  ,[Furthest_Build_CreatedOn_Date] = B_Furthest.[CreatedOn_Date]
      ,[Latest_Build_Id] = B_Latest.[Build_Id]
	  ,[Latest_Build_Number] = B_Latest.[Build_Number]
	  ,[Latest_Build_CreatedOn_Date] = B_Latest.[CreatedOn_Date]
	  ,[Latest_Promotion_Id] = BP_Latest.[Promotion_Id]
	  ,[Latest_Promoted_Date] = BP_Latest.[Promoted_Date]
	  ,[Latest_PromotionStatus_Name] = BP_Latest.[PromotionStatus_Name]
	  ,[Latest_Promotion_PipelineStage_Name] = BP_Latest.[PipelineStage_Name]
      ,[Latest_Execution_Id] = E_Latest.[Execution_Id]
      ,[Latest_Execution_Start_Date] = E_Latest.[Start_Date]
      ,[Latest_Execution_RunState_Code] = E_Latest.[ExecutionRunState_Code]
      ,[Latest_Execution_Status_Code] = E_Latest.[ExecutionStatus_Code]
      ,[Latest_Execution_Environment_Id] = BE_Latest.[Environment_Id]
      ,R.[ReleaseTemplate_Name]

  FROM [Releases] R
	   INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	    LEFT JOIN [Pipelines] P
	           ON P.[Pipeline_Id] = R.[Pipeline_Id]
	    LEFT JOIN [Builds] B_Furthest
		       ON B_Furthest.[Release_Id] = R.[Release_Id]
		      AND B_Furthest.[Build_Number] = R.[Furthest_Build_Number]
		LEFT JOIN (SELECT *, [Row] = ROW_NUMBER() OVER (PARTITION BY [Release_Id] ORDER BY [Build_Id] DESC)
		             FROM [Builds]) B_Latest
			   ON B_Latest.[Release_Id] = R.[Release_Id]
			  AND B_Latest.[Row] = 1
		LEFT JOIN (SELECT _BP.*,
		                  _B.[Release_Id],
						  [Row] = ROW_NUMBER() OVER (PARTITION BY _B.[Release_Id] ORDER BY _BP.[Promotion_Id] DESC)
		             FROM [BuildPromotions] _BP
					      INNER JOIN [Builds] _B
						          ON _B.[Build_Id] = _BP.[Build_Id]) BP_Latest
			   ON BP_Latest.[Release_Id] = R.[Release_Id]
			  AND BP_Latest.[Row] = 1
        OUTER APPLY(
            SELECT TOP 1 BE.* FROM [BuildExecutions] BE
             INNER JOIN [BuildPromotions] BP ON BP.[Promotion_Id] = BE.[Promotion_Id]
             INNER JOIN [Builds] B ON B.[Build_Id] = BP.[Build_Id]
             INNER JOIN [Executions] E ON E.[Execution_Id] = BE.[Execution_Id]
                  WHERE B.[Release_Id] = R.[Release_Id]
               ORDER BY E.[Execution_Id] DESC
        ) BE_Latest
        
        LEFT JOIN [Executions] E_Latest 
               ON E_Latest.[Execution_Id] = BE_Latest.[Execution_Id]

GO


GO
IF OBJECT_ID('Releases_RequiredPromotions') IS NOT NULL DROP VIEW [Releases_RequiredPromotions]
GO



GO
IF OBJECT_ID('[Releases_Slim]') IS NOT NULL DROP VIEW [Releases_Slim]
GO

CREATE VIEW [Releases_Slim] AS

SELECT R.[Application_Id]
      ,R.[Release_Number]
      ,R.[Workflow_Id]
      ,R.[ReleaseStatus_Name]
      ,R.[Notes_Text]
      ,R.[Target_Date]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
      ,R.[CreatedBy_User_Name]
      ,R.[CreatedOn_Date]
      ,R.[ModifiedBy_User_Name]
      ,R.[ModifiedOn_Date]
      ,R.[CancelledBy_User_Name]
      ,R.[CancelledReason_Text]
      ,R.[CancelledOn_Date]
      ,R.[Release_Sequence]
      ,R.[Original_Workflow_Name]
      ,R.[Pipeline_Id]
      ,R.[Release_Id]
      ,R.[Furthest_Build_Number]
      ,R.[ReleaseTemplate_Name]

	  ,A.[Application_Name]

	  ,P.[Pipeline_Name]
	  ,P.[Pipeline_Color]

  FROM [Releases] R
	   INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	    LEFT JOIN [Pipelines] P
               ON R.[Pipeline_Id] = P.[Pipeline_Id]

GO







GO
IF OBJECT_ID('[RetentionPolicies_Extended]') IS NOT NULL DROP VIEW [RetentionPolicies_Extended]
GO

CREATE VIEW [RetentionPolicies_Extended] AS

SELECT RP.[RetentionPolicy_Id],
       RP.[RetentionPolicy_Name],
	   RP.[PurgeType_Code],
	   RP.[CreatedOn_Date],
	   RP.[CreatedBy_User_Name],
	   RP.[ModifiedOn_Date],
	   RP.[ModifiedBy_User_Name],
	   
	   RP.[PurgeIn_Application_Id],
	   [PurgeIn_Application_Name] = A.[Application_Name],
	   RP.[PurgeIn_ApplicationGroup_Id],
	   [PurgeIn_ApplicationGroup_Name] = AG.[ApplicationGroup_Name],
	   RP.[PurgeIn_Workflow_Id],
	   [PurgeIn_Workflow_Name] = W.[Workflow_Name],
       RP.[PurgeIn_Pipeline_Id],
	   RP.[PurgeIn_Environment_Id],
	   [PurgeIn_Environment_Name] = E.[Environment_Name],
	   RP.[PurgeIn_BuildStep_Indicator],
	   RP.[PurgeIn_OnlyDeployedReleases_Indicator],
	   RP.[PurgeIn_OnlyRejectedBuilds_Indicator],
       RP.[PurgeOrphanedArtifacts_Indicator],

	   RP.[Retention_Count],
	   RP.[Retention_Size],
	   RP.[Retention_Days_Count],

	   RP.[PurgeWindow_Start_Minutes],
       RP.[PurgeWindow_Duration_Minutes],
	   RP.[PurgeNow_Indicator],

       RP.[Active_Indicator],

	   RPPH.[RetentionPoliciesPurgeHistoryEntry_Id],
	   [LastStart_Date] = RPPH.[Start_Date]

  FROM [RetentionPolicies] RP
       LEFT JOIN [Applications] A
	          ON A.[Application_Id] = RP.[PurgeIn_Application_Id]
	   LEFT JOIN [ApplicationGroups] AG
	          ON AG.[ApplicationGroup_Id] = RP.[PurgeIn_ApplicationGroup_Id]
	   LEFT JOIN [Workflows] W
	          ON W.[Workflow_Id] = RP.[PurgeIn_Workflow_Id]
	   LEFT JOIN [Environments] E
	          ON E.[Environment_Id] = RP.[PurgeIn_Environment_Id]
	   LEFT JOIN (SELECT [RetentionPoliciesPurgeHistoryEntry_Id],
						 [Original_RetentionPolicy_Id],
						 [Start_Date],
	                     [Rank] = ROW_NUMBER() OVER (PARTITION BY [Original_RetentionPolicy_Id] ORDER BY [Start_Date] DESC)
	                FROM [RetentionPoliciesPurgeHistory]) RPPH
			  ON RPPH.[Original_RetentionPolicy_Id] = RP.[RetentionPolicy_Id]
			 AND RPPH.[Rank] = 1
GO


GO
IF OBJECT_ID('[dbo].[RoleTasks_Extended]') IS NOT NULL
	DROP VIEW [dbo].[RoleTasks_Extended]
GO

CREATE VIEW [dbo].[RoleTasks_Extended]
AS

	SELECT RT.[Role_Id]

		  ,T.[Task_Id]
		  ,T.[Task_Name]
          ,T.[Application_Scopeable_Indicator]
          ,T.[Environment_Scopeable_Indicator]

	  FROM [dbo].[RoleTasks] RT
		   INNER JOIN [dbo].[Tasks] T
				   ON RT.[Task_Id] = T.[Task_Id]

GO



GO
IF OBJECT_ID('[Scripts_Extended]') IS NOT NULL DROP VIEW [Scripts_Extended]
GO

CREATE VIEW [Scripts_Extended] AS

SELECT S.[Script_Id],
       S.[Script_Name],
       S.[Description_Text],
       S.[Active_Indicator],
       S.[AutoSync_Indicator],
       S.[ScriptType_Code],
       S.[CreatedBy_User_Name],
       S.[CreatedOn_Date],
       S.[ModifiedBy_User_Name],
       S.[ModifiedOn_Date],
       [LatestVersion_Number] = SV.[Version_Number],
       [LatestVersion_Bytes] = SV.[Script_Bytes]

  FROM [Scripts] S
        LEFT JOIN (SELECT _SV.[Script_Id],
                          [Version_Number] = MAX(_SV.[Version_Number])
                     FROM [ScriptVersions] _SV
                    GROUP BY _SV.[Script_Id]) _Latest_SV
               ON S.[Script_Id] = _Latest_SV.[Script_Id]
        LEFT JOIN [ScriptVersions] SV
               ON _Latest_SV.[Script_Id] = SV.[Script_Id]
              AND _Latest_SV.[Version_Number] = SV.[Version_Number]
GO



GO
IF OBJECT_ID('[ScriptTypes_Usage]') IS NOT NULL DROP VIEW [ScriptTypes_Usage]
GO

CREATE VIEW [ScriptTypes_Usage] AS

	SELECT S.[ScriptType_Code],
	       [Usage_Count] = COUNT(S.[Script_Id])
	  FROM [Scripts] S
	 WHERE S.[Active_Indicator] = 'Y'
	 GROUP BY S.[ScriptType_Code]

GO



GO
IF OBJECT_ID('[ServerRolesEnvironments_Merged]') IS NOT NULL
	DROP VIEW [ServerRolesEnvironments_Merged]
GO

CREATE VIEW [ServerRolesEnvironments_Merged]
AS

	SELECT DISTINCT

          SR.[ServerRole_Id]
         ,SR.[ServerRole_Name]
         ,E.[Environment_Id]
         ,E.[Environment_Name]
          

	  FROM [ServerRoles] SR
     
INNER JOIN [ServerRoleServers] SRS
        ON SRS.[ServerRole_Id] = SR.[ServerRole_Id]
INNER JOIN [EnvironmentServers] ES
        ON ES.[Server_Id] = SRS.[Server_Id]
INNER JOIN [Environments] E
        ON E.[Environment_Id] = ES.[Environment_Id]

        

GO



GO
IF OBJECT_ID('[ServerRoles_Extended]') IS NOT NULL DROP VIEW [ServerRoles_Extended]
GO

CREATE VIEW [ServerRoles_Extended]
AS

    SELECT
           SR.[ServerRole_Id]
          ,SR.[ServerRole_Name]
          ,[Servers_Count] = (SELECT COUNT(*)
                                FROM [ServerRoleServers] SRS 
                               WHERE SRS.[ServerRole_Id] = SR.[ServerRole_Id])

          ,[Environments_Count] = (SELECT COUNT(DISTINCT [Environment_Id]) 
                                     FROM [ServerRoleServers] SRS 
                                INNER JOIN [EnvironmentServers] ES
                                        ON ES.[Server_Id] = SRS.[Server_Id]
                                    WHERE SR.[ServerRole_Id] = SRS.[ServerRole_Id])

	  FROM [ServerRoles] SR

GO



GO
IF OBJECT_ID('[VariableDefinitions_Extended]') IS NOT NULL DROP VIEW [VariableDefinitions_Extended]
GO

CREATE VIEW [VariableDefinitions_Extended] AS

SELECT VDEF.[VariableDefinition_Id]
      ,VDEF.[Variable_Name]
      ,VDEF.[Environment_Id]
      ,VDEF.[ServerRole_Id]
      ,VDEF.[Server_Id]
      ,VDEF.[ApplicationGroup_Id]
      ,VDEF.[Application_Id]
      ,VDEF.[Deployable_Id]
	  ,VDEF.[Release_Id]
      ,R.[Release_Number]
	  ,VDEF.[Build_Id]
      ,B.[Build_Number]
      ,VDEF.[Value_Text]
      ,VDEF.[Execution_Id]
	  ,VDEF.[Scope_Code]

      ,VDEC.[VariableDeclaration_Id]
      ,VDEC.[Scoped_Application_Id]
      ,VDEC.[Required_Indicator]
      ,VDEC.[Variable_Configuration]
      ,VDEC.[DefaultValue_Text]
      ,VDEC.[Scoped_Environment_Id]

      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,A.[Application_Name]
      ,D.[Deployable_Name]
      ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
       LEFT JOIN [VariableDeclarations] VDEC
              ON VDEF.[Variable_Name] = VDEC.[Variable_Name]
             AND (VDEC.[Scope_Code] NOT IN ('R','B','P','X') OR
                  VDEF.[Application_Id] = VDEC.[Scoped_Application_Id])
             AND (VDEC.[Scope_Code] NOT IN ('P','X') OR
                  VDEF.[Environment_Id] = VDEC.[Scoped_Environment_Id])
             AND VDEC.[Scope_Code] = VDEF.[Scope_Code]
       LEFT JOIN [Applications] A
                 ON VDEF.[Application_Id] = A.[Application_Id]
	   LEFT JOIN [Releases] R
	             ON VDEF.[Release_Id] = R.[Release_Id]
	   LEFT JOIN [BuildPromotions] BP
	             ON VDEF.[Promotion_Id] = BP.[Promotion_Id]
	   LEFT JOIN [Builds] B
	             ON VDEF.[Build_Id] = BP.[Build_Id]
       LEFT JOIN [ApplicationGroups] AG
                 ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
                 ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
                 ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Servers] S
                 ON VDEF.[Server_Id] = S.[Server_Id]
GO


GO
IF OBJECT_ID('[VariableDefinitions_System]') IS NOT NULL DROP VIEW [VariableDefinitions_System]
GO

CREATE VIEW [VariableDefinitions_System] AS

SELECT VDEF.*
      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,[Application_Name] = COALESCE(A.[Application_Name],AD.[Application_Name])
      ,D.[Deployable_Name]
  FROM [VariableDefinitions] VDEF
       LEFT JOIN [Applications] A
              ON VDEF.[Application_Id] = A.[Application_Id]
       LEFT JOIN [ApplicationGroups] AG
              ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
              ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
              ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Applications] AD
              ON D.[Application_Id] = AD.[Application_Id]
       LEFT JOIN [Servers] S
              ON VDEF.[Server_Id] = S.[Server_Id]
  
  WHERE [Scope_Code] IN ('S','E','V','G','A','D')

GO


GO
IF OBJECT_ID('[Variable_Values]') IS NOT NULL DROP VIEW [Variable_Values]
GO

CREATE VIEW [Variable_Values] AS

SELECT DISTINCT
	   VDEF.[Variable_Name]
      ,VDEF.[Value_Text]
	  ,VDEF.[Scope_Code]
	  ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
GO


GO
IF OBJECT_ID('[WorkflowSteps_Extended]') IS NOT NULL DROP VIEW [WorkflowSteps_Extended]
GO



GO
IF OBJECT_ID('[Workflows_Extended]') IS NOT NULL DROP VIEW [Workflows_Extended]
GO



GO
IF OBJECT_ID('[__BuildMaster_StoredProcInfo]') IS NOT NULL
	DROP TABLE [__BuildMaster_StoredProcInfo]
GO

IF OBJECT_ID('[__StoredProcInfo]') IS NOT NULL
	DROP TABLE [__StoredProcInfo]
GO

CREATE TABLE [__StoredProcInfo]
(
    [StoredProc_Name] SYSNAME,
    CONSTRAINT [PK__StoredProcInfo]
        PRIMARY KEY ([StoredProc_Name]),
    [Internal_Indicator] YNINDICATOR,
    [ReturnType_Name] VARCHAR(100) NULL,
    CONSTRAINT [CK__StoredProcInfo__ReturnType_Name_Domain]
        CHECK ([ReturnType_Name] IS NULL OR [ReturnType_Name] IN ('void', 'DataRow', 'DataTable', 'DataSet')),
    [DataTableNames_Csv] VARCHAR(255) NULL,
    [OutputPropertyNames_Csv] VARCHAR(255) NULL,
    [Description_Text] VARCHAR(MAX),
    [Remarks_Text] VARCHAR(MAX)
)
GO

IF OBJECT_ID('[__BuildMaster_AddStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_AddStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_AddStoredProcInfo]
(
	@StoredProc_Name SYSNAME,
	@Internal_Indicator YNINDICATOR,
	@ReturnType_Name VARCHAR(100),
	@DataTableNames_Csv VARCHAR(255),
	@OutputPropertyNames_Csv VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Remarks_Text VARCHAR(MAX) = NULL
)
AS
BEGIN
	
	IF EXISTS(SELECT * FROM [__StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name)
		DELETE [__StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name

    INSERT INTO [__StoredProcInfo]
    (
         [StoredProc_Name]
        ,[Internal_Indicator]
        ,[ReturnType_Name]
        ,[DataTableNames_Csv]
        ,[OutputPropertyNames_Csv]
        ,[Description_Text]
        ,[Remarks_Text]
    )
    VALUES
    (
         @StoredProc_Name
	    ,@Internal_Indicator
	    ,@ReturnType_Name
	    ,@DataTableNames_Csv
	    ,@OutputPropertyNames_Csv
	    ,@Description_Text
	    ,@Remarks_Text
    )

END
GO

GRANT EXECUTE ON [__BuildMaster_AddStoredProcInfo] TO [BuildMasterUser_Role]
GO

IF OBJECT_ID('[__BuildMaster_GetStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetStoredProcInfo]
(
	@StoredProc_Name SYSNAME = NULL
)
AS
BEGIN
    
    SET NOCOUNT ON
    
    IF @StoredProc_Name IS NULL
	    SELECT * FROM [__StoredProcInfo]
	ElSE
		SELECT * FROM [__StoredProcInfo] WHERE [StoredProc_Name] = @StoredProc_Name

END
GO

GRANT EXECUTE ON [__BuildMaster_GetStoredProcInfo] TO [BuildMasterUser_Role]
GO


GO
IF OBJECT_ID('[__BuildMaster_GetEventReference]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetEventReference]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetEventReference]
AS
BEGIN
	
	SELECT [Event_Code], [Event_Description], [StoredProc_Name] = ROUTINE_NAME
	  FROM INFORMATION_SCHEMA.ROUTINES 
		   INNER JOIN [EventTypes]
				   ON ROUTINE_DEFINITION LIKE '%' + [Event_Code] + '%' 
				  AND ROUTINE_TYPE='PROCEDURE'
	 ORDER BY 1

END
GO

GRANT EXECUTE ON [__BuildMaster_GetEventReference] TO [BuildMasterUser_Role]
GO


GO
IF OBJECT_ID('[__BuildMaster_GetSecurityTasks]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetSecurityTasks]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetSecurityTasks]
AS
BEGIN
	
	SELECT * FROM [Tasks]

END
GO

GRANT EXECUTE ON [__BuildMaster_GetSecurityTasks] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_FillVersionGaps',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Ensures that there are no gaps in configuration file instance versions.'
GO

IF OBJECT_ID('[ConfigurationFiles_FillVersionGaps]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_FillVersionGaps]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_FillVersionGaps]
(
	@ConfigurationFile_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @InstanceGaps TABLE
	(
		[Instance_Name] VARCHAR(100) NOT NULL,
		[Version_Number] INT NOT NULL,
		[Count] INT NOT NULL
	)

	INSERT INTO @InstanceGaps
	(
		[Instance_Name],
		[Version_Number],
		[Count]
	)
	SELECT CFV.[Instance_Name],
		   [Version_Number] = MAX([Version_Number]),
		   [Count] = MAX(CFV_Max.[Max_Version_Number]) - MAX([Version_Number])
	  FROM [ConfigurationFileVersions] CFV
	       INNER JOIN (SELECT [ConfigurationFile_Id],
	                          [Max_Version_Number] = MAX([Version_Number])
	                     FROM [ConfigurationFileVersions]
	                    GROUP BY [ConfigurationFile_Id]) CFV_Max
			       ON CFV_Max.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
	 WHERE CFV.[ConfigurationFile_Id] = @ConfigurationFile_Id
	 GROUP BY CFV.[ConfigurationFile_Id], CFV.[Instance_Name]
	HAVING MAX(CFV.[Version_Number]) < MAX(CFV_Max.[Max_Version_Number])
	
	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CFV.[ConfigurationFile_Id],
	       CFV.[Instance_Name],
		   CFV.[Version_Number] + N.[Number],
		   NULL,
		   NULL,
		   NULL,
		   NULL,
		   NULL
	  FROM @InstanceGaps IG
	       INNER JOIN [Numbers] N
		           ON N.[Number] >= 1 AND N.[Number] <= IG.[Count]
		   INNER JOIN [ConfigurationFileVersions] CFV
		           ON CFV.[ConfigurationFile_Id] = @ConfigurationFile_Id
				  AND CFV.[Instance_Name] = IG.[Instance_Name]
				  AND CFV.[Version_Number] = IG.[Version_Number]

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_FillVersionGaps] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_RaiseEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Events_RaiseEvent]') IS NOT NULL
	DROP PROCEDURE [Events_RaiseEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_RaiseEvent]
(
	@Event_Code VARCHAR(8),
	@Application_Id INT = NULL,
	@detail1 SQL_VARIANT = NULL,
	@detail2 SQL_VARIANT = NULL,
	@detail3 SQL_VARIANT = NULL,
	@detail4 SQL_VARIANT = NULL,
	@detail5 SQL_VARIANT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @User_Name USERNAME
		SET @User_Name = COALESCE(dbo.Session_GetUserName(), 'SYSTEM')

	INSERT INTO [EventOccurences]
	 ([Event_Code],[User_Name],[Occurence_Date],[Application_Id])
	 VALUES
	 (@Event_Code, @User_Name, GETUTCDATE(), @Application_Id)

	DECLARE @EventOccurence_Id INT
		SET @EventOccurence_Id = SCOPE_IDENTITY()

	INSERT INTO [EventOccurenceDetails]
	 ([EventOccurence_Id],[Event_Code],[Detail_Name],[Detail_Value])
	SELECT @EventOccurence_Id
		  ,[Event_Code]
		  ,[Detail_Name]
		  ,CASE [Detail_Sequence]
				WHEN 1 THEN @detail1
				WHEN 2 THEN @detail2
				WHEN 3 THEN @detail3
				WHEN 4 THEN @detail4
				WHEN 5 THEN @detail5
				ELSE NULL END
	  FROM [EventTypeDetails]
	 WHERE [Event_Code] = @Event_Code

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Events_RaiseEvent] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'HandleError',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[HandleError]') IS NOT NULL DROP PROCEDURE [HandleError]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [HandleError] AS BEGIN
   -- Return if there is no error information to retrieve.
    IF ERROR_NUMBER() IS NULL
        RETURN;

    DECLARE 
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200);

    -- Assign variables to error-handling functions that 
    -- capture information for RAISERROR.
    SELECT 
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = REPLACE(ISNULL(ERROR_PROCEDURE(), '-'),'`','''');

    -- Building the message string that will contain original
    -- error information.
    SELECT @ErrorMessage = N'%d`%d`%d`%s`%d`'+ ERROR_MESSAGE();

    -- Raise an error: msg_str parameter of RAISERROR will contain
    -- the original error information.
    RAISERROR 
        (
        @ErrorMessage, 
        @ErrorSeverity, 
        42,               
        @ErrorNumber,    -- parameter: original error number.
        @ErrorSeverity,  -- parameter: original error severity.
        @ErrorState,     -- parameter: original error state.
        @ErrorProcedure, -- parameter: original error procedure name.
        @ErrorLine       -- parameter: original error line number.
        );
END
GO




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_InvalidateApprovals',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes received approvals for a pending promotion.'
GO

IF OBJECT_ID('[Promotions_InvalidateApprovals]') IS NOT NULL DROP PROCEDURE [Promotions_InvalidateApprovals]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_InvalidateApprovals]
(
	@Pipeline_Id INT,
	@PipelineStage_Name VARCHAR(50)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE BAR
      FROM [BuildApprovalsReceived] BAR
     INNER JOIN [Builds] B ON B.[Build_Id] = BAR.[Build_Id]
     INNER JOIN [Releases] R ON R.[Release_Id] = B.[Release_Id]
     LEFT JOIN [BuildPromotions] P 
            ON P.[Build_Id] = BAR.[Build_Id] 
           AND P.[PipelineStage_Name] = BAR.[PipelineStage_Name]
    WHERE P.[Promotion_Id] IS NULL
      AND R.[Pipeline_Id] = @Pipeline_Id
      AND BAR.[PipelineStage_Name] = @PipelineStage_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_InvalidateApprovals] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_PurgeArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an artifact from the specified build.'
GO


IF OBJECT_ID('[AdminEdit_PurgeArtifact]') IS NOT NULL DROP PROCEDURE [AdminEdit_PurgeArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_PurgeArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Artifact_Name VARCHAR(50)
)
AS
BEGIN

    DELETE BA
	  FROM [BuildArtifacts] BA
	       INNER JOIN [Builds] B
		           ON B.[Build_Id] = BA.[Build_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
     WHERE R.[Application_Id] = @Application_Id
       AND R.[Release_Number] = @Release_Number
       AND B.[Build_Number] = @Build_Number
       AND BA.[Artifact_Name] = @Artifact_Name

END
GO

GRANT EXECUTE ON [AdminEdit_PurgeArtifact] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_PurgeBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire build and all associated data from the system.'
GO

IF OBJECT_ID('[AdminEdit_PurgeBuild]') IS NOT NULL DROP PROCEDURE [AdminEdit_PurgeBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_PurgeBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
) AS
BEGIN

EXEC [Builds_PurgeBuildData] @Application_Id, @Release_Number, @Build_Number

END
GO

GRANT EXECUTE ON [AdminEdit_PurgeBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateBuildStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a build.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateBuildStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateBuildStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateBuildStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
    @BuildStatus_Name VARCHAR(8)
)
AS
BEGIN

    SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    IF EXISTS(SELECT *
	            FROM [Builds_Extended] B
               WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
                 AND [Build_Number] = @Build_Number
                 AND [BuildStatus_Name] = @BuildStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END

    IF (@BuildStatus_Name = 'Rejected')
    BEGIN

        UPDATE B
           SET B.[BuildStatus_Name] = @BuildStatus_Name,
               B.[Rejection_Notes] = 'Rejected via Administrative Edit',
               B.[RejectedBy_User_Name] = dbo.Session_GetUserName(),
               B.[Rejected_Date] = GETUTCDATE(),
               B.[ModifiedBy_User_Name] = dbo.Session_GetUserName(),
               B.[ModifiedOn_Date] = GETUTCDATE()
		  FROM [Builds] B
		       INNER JOIN [Releases] R
			           ON R.[Release_Id] = B.[Release_Id]
         WHERE R.[Application_Id] = @Application_Id
           AND R.[Release_Number] = @Release_Number
           AND B.[Build_Number] = @Build_Number

    END
    ELSE
    BEGIN

        UPDATE B
           SET B.[BuildStatus_Name] = @BuildStatus_Name,
               B.[Rejection_Notes] = NULL,
               B.[RejectedBy_User_Name] = NULL,
               B.[Rejected_Date] = NULL,
               B.[ModifiedBy_User_Name] = [dbo].Session_GetUserName(),
               B.[ModifiedOn_Date] = GETUTCDATE()
		  FROM [Builds] B
		       INNER JOIN [Releases] R
			           ON R.[Release_Id] = B.[Release_Id]
         WHERE R.[Application_Id] = @Application_Id
           AND R.[Release_Number] = @Release_Number
           AND B.[Build_Number] = @Build_Number

    END


    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateBuildStatus] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateExecutionStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of an execution.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateExecutionStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateExecutionStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateExecutionStatus]
(
	@Execution_Id INT,
    @ExecutionStatus_Code CHAR(1)
)
AS
BEGIN

    SET NOCOUNT ON


BEGIN TRY
	BEGIN TRANSACTION

        UPDATE [Executions]
           SET [ExecutionStatus_Code] = @ExecutionStatus_Code
         WHERE [Execution_Id] = @Execution_Id
    
    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateExecutionStatus] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdatePromotionStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a promotion.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdatePromotionStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdatePromotionStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdatePromotionStatus]
(
	@Promotion_Id INT,
    @PromotionStatus_Name VARCHAR(20)
)
AS
BEGIN

    SET NOCOUNT ON


BEGIN TRY
	BEGIN TRANSACTION

    IF EXISTS(SELECT * FROM [BuildPromotions]
               WHERE [Promotion_Id] = @Promotion_Id
                 AND [PromotionStatus_Name] = @PromotionStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END


    UPDATE [BuildPromotions]
       SET [PromotionStatus_Name] = @PromotionStatus_Name,
           [StatusOverriddenBy_User_Name] = [dbo].Session_GetUserName(),
           [StatusOverridden_Date] = GETUTCDATE(),
           [StatusOverriddenReason_Text] = 'Overridden via Administrative Edit'
     WHERE [Promotion_Id] = @Promotion_Id

    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdatePromotionStatus] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateReleaseStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a release.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateReleaseStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateReleaseStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateReleaseStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @ReleaseStatus_Name VARCHAR(9)
)
AS
BEGIN

    SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    -- don't do anything if the status isn't changing
    IF EXISTS(SELECT * FROM [Releases]
               WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
                 AND [ReleaseStatus_Name] = @ReleaseStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END
    
    IF (@ReleaseStatus_Name = 'Cancelled')
    BEGIN

        UPDATE [Releases]
           SET [ReleaseStatus_Name] = @ReleaseStatus_Name,
               [CancelledBy_User_Name] = [dbo].Session_GetUserName(),
               [CancelledOn_Date] = GETUTCDATE(),
               [CancelledReason_Text] = 'Canceled via Administrative Edit',
               [ModifiedOn_Date] = GETUTCDATE(),
               [ModifiedBy_User_Name] = dbo.Session_GetUserName()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number

    END
    ELSE
    BEGIN

        UPDATE [Releases]
           SET [ReleaseStatus_Name] = @ReleaseStatus_Name,
               [CancelledBy_User_Name] = NULL,
               [CancelledOn_Date] = NULL,
               [CancelledReason_Text] = NULL,
               [ModifiedOn_Date] = GETUTCDATE(),
               [ModifiedBy_User_Name] = [dbo].Session_GetUserName()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number

    END

    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateReleaseStatus] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_CreateOrUpdateApiKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApiKey_Id',
    /* Description_Text        */ 'Creates or updates the specified API key.'
GO

IF OBJECT_ID('[ApiKeys_CreateOrUpdateApiKey]') IS NOT NULL DROP PROCEDURE [ApiKeys_CreateOrUpdateApiKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_CreateOrUpdateApiKey]
(
    @ApiKey_Text VARCHAR(50),
    @ApiKey_Description NVARCHAR(MAX),
    @ApiKey_Configuration XML,
    @ApiKey_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

    IF NOT EXISTS(SELECT * FROM [ApiKeys] WHERE [ApiKey_Id] = @ApiKey_Id)
    BEGIN
        
        INSERT INTO [ApiKeys] 
        (
            [ApiKey_Text]
           ,[ApiKey_Description] 
           ,[ApiKey_Configuration]
        )
        VALUES 
        (
            @ApiKey_Text
           ,NULLIF(@ApiKey_Description, '')
           ,@ApiKey_Configuration
        )

        SET @ApiKey_Id = SCOPE_IDENTITY()

    END
    ELSE
    BEGIN

	    UPDATE [ApiKeys]
           SET [ApiKey_Text] = @ApiKey_Text,
               [ApiKey_Description] = NULLIF(@ApiKey_Description, ''),
               [ApiKey_Configuration] = @ApiKey_Configuration 
         WHERE [ApiKey_Id] = @ApiKey_Id

     END

END
GO

GRANT EXECUTE ON [ApiKeys_CreateOrUpdateApiKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_DeleteApiKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified API key.'
GO

IF OBJECT_ID('[ApiKeys_DeleteApiKey]') IS NOT NULL DROP PROCEDURE [ApiKeys_DeleteApiKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_DeleteApiKey]
(
    @ApiKey_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	DELETE [ApiKeys]
     WHERE [ApiKey_Id] = @ApiKey_Id

END
GO

GRANT EXECUTE ON [ApiKeys_DeleteApiKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_GetAccessLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApiKeyAccessLogs',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specific API access log.'
GO

IF OBJECT_ID('[ApiKeys_GetAccessLog]') IS NOT NULL DROP PROCEDURE [ApiKeys_GetAccessLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_GetAccessLog]
(
    @ApiKeyAccessLog_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ApiKeyAccessLogs]
     WHERE [ApiKeyAccessLog_Id] = @ApiKeyAccessLog_Id

END
GO

GRANT EXECUTE ON [ApiKeys_GetAccessLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_GetAccessLogs',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ApiKeyAccessLogs',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all API access logs.'
GO

IF OBJECT_ID('[ApiKeys_GetAccessLogs]') IS NOT NULL DROP PROCEDURE [ApiKeys_GetAccessLogs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_GetAccessLogs]
(
    @ApiKey_Id INT,
    @Log_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

    IF @Log_Count IS NOT NULL
        SET ROWCOUNT @Log_Count

    DECLARE @ApiKey_Text VARCHAR(50)
    SELECT @ApiKey_Text = [ApiKey_Text] 
      FROM [ApiKeys] 
     WHERE [ApiKey_Id] = @ApiKey_Id

	SELECT * FROM [ApiKeyAccessLogs]
     WHERE @ApiKey_Text IS NULL OR [ApiKey_Text] = @ApiKey_Text
     ORDER BY [ApiKeyAccessLog_Id] DESC

END
GO

GRANT EXECUTE ON [ApiKeys_GetAccessLogs] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_GetApiKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApiKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified API key by ID.'
GO

IF OBJECT_ID('[ApiKeys_GetApiKey]') IS NOT NULL DROP PROCEDURE [ApiKeys_GetApiKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_GetApiKey]
(
    @ApiKey_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ApiKeys]
     WHERE [ApiKey_Id] = @ApiKey_Id

END
GO

GRANT EXECUTE ON [ApiKeys_GetApiKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_GetApiKeyByName',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApiKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified API key by the API key.'
GO

IF OBJECT_ID('[ApiKeys_GetApiKeyByName]') IS NOT NULL DROP PROCEDURE [ApiKeys_GetApiKeyByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_GetApiKeyByName]
(
    @ApiKey_Text VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ApiKeys]
     WHERE [ApiKey_Text] = @ApiKey_Text

END
GO

GRANT EXECUTE ON [ApiKeys_GetApiKeyByName] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_GetApiKeys',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ApiKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all API keys in the system.'
GO

IF OBJECT_ID('[ApiKeys_GetApiKeys]') IS NOT NULL DROP PROCEDURE [ApiKeys_GetApiKeys]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_GetApiKeys]
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ApiKeys]

END
GO

GRANT EXECUTE ON [ApiKeys_GetApiKeys] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_LogRequest',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs a request made with an API key.'
GO

IF OBJECT_ID('[ApiKeys_LogRequest]') IS NOT NULL DROP PROCEDURE [ApiKeys_LogRequest]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_LogRequest]
(
    @ApiKey_Text VARCHAR(50),
    @Request_Date DATETIME,
    @Request_Url VARCHAR(MAX),
    @Request_Method VARCHAR(5),
    @Request_Ip_Address VARCHAR(20),
    @Response_StatusCode INT,
    @Request_Body VARBINARY(MAX),
    @Response_Body VARBINARY(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Min_ApiKeyAccessLog_Id INT
    SELECT @Min_ApiKeyAccessLog_Id = MIN([ApiKeyAccessLog_Id])
     FROM (SELECT TOP 1000 [ApiKeyAccessLog_Id] FROM [ApiKeyAccessLogs] ORDER BY [ApiKeyAccessLog_Id] DESC) [Logs]

    DELETE [ApiKeyAccessLogs]
     WHERE [ApiKeyAccessLog_Id] < @Min_ApiKeyAccessLog_Id

	INSERT INTO [ApiKeyAccessLogs]
    (
        [ApiKey_Text],
        [Request_Date],
        [Request_Url],
        [Request_Method],
        [Request_Ip_Address],
        [Request_Body],
        [Response_StatusCode],
        [Response_Body]
    )
    VALUES
    (
        @ApiKey_Text,
        @Request_Date,
        @Request_Url,
        @Request_Method,
        @Request_Ip_Address,
        @Request_Body,
        @Response_StatusCode,
        @Response_Body
    )

END
GO

GRANT EXECUTE ON [ApiKeys_LogRequest] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_CreateOrUpdateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Creates or updates a new application group'
GO

IF OBJECT_ID('[ApplicationGroups_CreateOrUpdateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
(
	@ApplicationGroup_Id INT = NULL OUT,
	@ApplicationGroup_Name VARCHAR(50),
	@ApplicationGroup_Description VARCHAR(MAX),
    @Parent_ApplicationGroup_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @ApplicationGroup_Id IS NULL
	BEGIN
		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[ApplicationGroup_Description],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
            [Parent_ApplicationGroup_Id]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			@ApplicationGroup_Description,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
            @Parent_ApplicationGroup_Id
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [ApplicationGroups]
		   SET [ApplicationGroup_Name] = @ApplicationGroup_Name,
			   [ApplicationGroup_Description] = @ApplicationGroup_Description,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
               [Parent_ApplicationGroup_Id] = @Parent_ApplicationGroup_Id
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_CreateOrUpdateApplicationGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_DeleteApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an application group'
GO

IF OBJECT_ID('[ApplicationGroups_DeleteApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
        UPDATE [Applications]
           SET [ApplicationGroup_Id] = NULL
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

         UPDATE [ApplicationGroups]
           SET [Parent_ApplicationGroup_Id] = NULL
         WHERE [Parent_ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [EventListenerApplicationGroups]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [Privileges]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [VariableDefinitions]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [RetentionPolicies]
         WHERE [PurgeIn_ApplicationGroup_Id] = @ApplicationGroup_Id

		DELETE [ApplicationGroups]
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_DeleteApplicationGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application group by ID.'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [ApplicationGroups]
	 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the application groups'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroups]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroups]
AS
BEGIN

SET NOCOUNT ON
	SELECT * FROM [ApplicationGroups] ORDER BY [ApplicationGroup_Name]

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroups] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetOrCreateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Gets or creates a new application group using the name'
GO

IF OBJECT_ID('[ApplicationGroups_GetOrCreateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
(
	@ApplicationGroup_Name VARCHAR(50),
    @ApplicationGroup_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Existing_ApplicationGroup_Id INT
    DECLARE @Existing_ApplicationGroup_Name VARCHAR(50)

    SELECT @Existing_ApplicationGroup_Name = [ApplicationGroup_Name],
           @Existing_ApplicationGroup_Id = [ApplicationGroup_Id]
      FROM [ApplicationGroups]
     WHERE [ApplicationGroup_Name] = @ApplicationGroup_Name
    
    IF (@ApplicationGroup_Name IS NULL OR LTRIM(RTRIM(@ApplicationGroup_Name)) = '')
    BEGIN
        
        SET @ApplicationGroup_Id = NULL

    END
	ELSE IF (@Existing_ApplicationGroup_Id IS NULL)
	BEGIN

		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()

	END
    ELSE
    BEGIN

        SET @ApplicationGroup_Id = @Existing_ApplicationGroup_Id

    END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetOrCreateApplicationGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_AddImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds an imported deployable to an application'
GO

IF OBJECT_ID('[Applications_AddImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_AddImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_AddImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF NOT EXISTS(
			SELECT * FROM DeployablesImported
			WHERE [Deployable_Id] = @Deployable_Id
			  AND [Application_Id] = @Application_Id
		)
		BEGIN
			INSERT INTO [DeployablesImported]
			(
				[Deployable_Id],
				[Application_Id]
			)
			VALUES
			(
				@Deployable_Id,
				@Application_Id
			)
		END

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_AddImportedDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Application_Id',
    /* Description_Text        */ 'Creates a new application.'
GO

IF OBJECT_ID('[Applications_CreateApplication]') IS NOT NULL DROP PROCEDURE [Applications_CreateApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateApplication]
(
	@Application_Name VARCHAR(50),
	@ReleaseNumber_Scheme_Name VARCHAR(20) = NULL,
	@IssueTracking_Provider_Id INT = NULL,
	@BuildNumber_Scheme_Name VARCHAR(15) = NULL,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Application_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

	/* Set defaults */
	SET @ReleaseNumber_Scheme_Name = ISNULL(@ReleaseNumber_Scheme_Name, 'MajorMinor')
	SET @BuildNumber_Scheme_Name = ISNULL(@BuildNumber_Scheme_Name, 'Sequential')
	SET @AllowMultipleActiveBuilds_Indicator = ISNULL(@AllowMultipleActiveBuilds_Indicator, 'N')

	INSERT INTO [Applications]
	(
		[Application_Name], 
		[ReleaseNumber_Scheme_Name],
		[BuildNumber_Scheme_Name],
		[IssueTracking_Provider_Id],
		[AllowMultipleActiveBuilds_Indicator],
		[ApplicationGroup_Id],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[VariableSupport_Code],
        [IssueTracking_AllowBuildMasterIssues_Indicator]
	)
	VALUES
	(
		@Application_Name,
		@ReleaseNumber_Scheme_Name,
		@BuildNumber_Scheme_Name,
		@IssueTracking_Provider_Id,
		@AllowMultipleActiveBuilds_Indicator,
		@ApplicationGroup_Id,
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		'N',
        'Y'
	)
	SET @Application_Id = SCOPE_IDENTITY()
	
	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPNEW',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateApplication] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateOrUpdateDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Deployable_Id',
    /* Description_Text        */ 'Creates a deployable for a specified application, or updates an existing one'
GO


IF OBJECT_ID('[Applications_CreateOrUpdateDeployable]') IS NOT NULL DROP PROCEDURE [Applications_CreateOrUpdateDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateOrUpdateDeployable]
(
	@Deployable_Id INT OUT,
	@Application_Id INT,
	@Deployable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Deployable_Id IS NOT NULL
	BEGIN

		DECLARE @Old_Deployable_Name VARCHAR(50)
		SET @Old_Deployable_Name = (SELECT [Deployable_Name]
		                              FROM [Deployables]
									 WHERE [Deployable_Id] = @Deployable_Id)

		UPDATE [Deployables]
		   SET [Deployable_Name] = @Deployable_Name,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Deployable_Id] = @Deployable_Id

		IF @@ROWCOUNT=0 RAISERROR('Deployable %i does not exist.',16,1,@Deployable_Id)

		IF @Old_Deployable_Name <> @Deployable_Name
		BEGIN

			-- Update deployable names in deployment plans
			UPDATE DPAG
			   SET DPAG.[Deployable_Name] = @Deployable_Name
			  FROM [DeploymentPlanActionGroups] DPAG
			       LEFT JOIN [ApplicationDeploymentPlans] ADP
				          ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
			 WHERE (DPAG.[Deployable_Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)
			    OR (DPAG.[Deployable_Application_Id] IS NULL AND ADP.[Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)

		END

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	END
	ELSE
	BEGIN
		INSERT INTO [Deployables]
		(
			[Application_Id],
			[Deployable_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
        (
			@Application_Id,
			@Deployable_Name,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		SET @Deployable_Id = SCOPE_IDENTITY()

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPNWD',@Application_Id,@Deployable_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateOrUpdateDeployable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_DeactivateApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deactivates an application, removing it from menu options but preserves data.'
GO

IF OBJECT_ID('[Applications_DeactivateApplication]') IS NOT NULL
	DROP PROCEDURE [Applications_DeactivateApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_DeactivateApplication]
(
	@Application_Id INT
) 
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	UPDATE [Applications]
	   SET [Active_Indicator] = 'N'
	 WHERE [Application_Id] = @Application_Id

     IF @@ROWCOUNT = 0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

    UPDATE [BuildSchedules]
       SET [Active_Indicator] = 'N'
	 WHERE [Application_Id] = @Application_Id

    UPDATE [RetentionPolicies]
       SET [Active_Indicator] = 'N'
	 WHERE [PurgeIn_Application_Id] = @Application_Id

     DECLARE @ActiveReleases TABLE 
     (
        [Release_Number] RELEASENUMBER
     )

     INSERT @ActiveReleases ([Release_Number])
     SELECT [Release_Number] 
       FROM [Releases]
      WHERE [Application_Id] = @Application_Id
        AND [ReleaseStatus_Name] = 'Active'

      DECLARE @Current_ReleaseNumber RELEASENUMBER

      WHILE EXISTS(SELECT * FROM @ActiveReleases)
      BEGIN

        SELECT TOP 1 @Current_ReleaseNumber = [Release_Number] FROM @ActiveReleases

        EXEC [Releases_CancelRelease]
                @Application_Id,
	            @Release_Number = @Current_ReleaseNumber,
	            @CancelledReason_Text = 'Application was deactivated'
        
        DELETE @ActiveReleases WHERE [Release_Number] = @Current_ReleaseNumber

      END


	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_DeactivateApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_DeleteImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an existing imported deployable relation, but does not delete the deployable from the host application'
GO

IF OBJECT_ID('[Applications_DeleteImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_DeleteImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_DeleteImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployablesImported]
		WHERE [Deployable_Id] = @Deployable_Id
		  AND [Application_Id] = @Application_Id

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_DeleteImportedDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_EditApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Edits an existing application.'
GO


IF OBJECT_ID('[Applications_EditApplication]') IS NOT NULL DROP PROCEDURE [Applications_EditApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_EditApplication]
(
	@Application_Id INT,
	@Application_Name VARCHAR(50),
	@Application_Description VARCHAR(MAX),
	@ApplicationGroup_Id INT,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR = NULL,
	@VariableSupport_Code CHAR(1) = NULL,
    @ReleaseNumber_Scheme_Name VARCHAR(20) = NULL,
	@BuildNumber_Scheme_Name VARCHAR(15) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Application_Name] = @Application_Name
		  ,[Application_Description] = @Application_Description
		  ,[ApplicationGroup_Id] = @ApplicationGroup_Id
		  ,[AllowMultipleActiveBuilds_Indicator] = COALESCE(@AllowMultipleActiveBuilds_Indicator, [AllowMultipleActiveBuilds_Indicator])
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
		  ,[VariableSupport_Code] = COALESCE(@VariableSupport_Code, [VariableSupport_Code])
          ,[ReleaseNumber_Scheme_Name] = COALESCE(@ReleaseNumber_Scheme_Name, [ReleaseNumber_Scheme_Name])
		  ,[BuildNumber_Scheme_Name] = COALESCE(@BuildNumber_Scheme_Name, [BuildNumber_Scheme_Name])
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_EditApplication] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_FindDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Searches for deployables in the system.'
GO

IF OBJECT_ID('[Applications_FindDeployables]') IS NOT NULL DROP PROCEDURE [Applications_FindDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_FindDeployables]
(
	@SearchTerm_Text VARCHAR(50),
	@Max_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	SET @SearchTerm_Text = NULLIF(@SearchTerm_Text, '')
	IF @SearchTerm_Text IS NOT NULL
	BEGIN
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '_', '\_')
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '%', '\%')
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '[', '\[')
		SET @SearchTerm_Text = @SearchTerm_Text + '%'
	END

	IF @Max_Count IS NOT NULL
		SET ROWCOUNT @Max_Count

	SELECT *
	  FROM [Deployables_Extended]
     WHERE @SearchTerm_Text IS NULL
	    OR [Deployable_Name] LIKE @SearchTerm_Text ESCAPE '\'
		OR [Application_Name] LIKE @SearchTerm_Text ESCAPE '\'

END
GO

GRANT EXECUTE ON [Applications_FindDeployables] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an application along with the associated deployables'
GO


IF OBJECT_ID('[Applications_GetApplication]') IS NOT NULL DROP PROCEDURE [Applications_GetApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplication]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Applications_Extended]
     WHERE [Application_Id] = @Application_Id

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Application_Id] = @Application_Id
	ORDER BY [Deployable_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplication] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified number of applications in the system, or all if null is passed in'
GO


IF OBJECT_ID('[Applications_GetApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplications]
(
    @Application_Count INT,
    @IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Count IS NOT NULL
        SET ROWCOUNT @Application_Count

	SELECT * FROM [Applications_Extended]
    WHERE @IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y'
	ORDER BY [ApplicationGroup_Name], [Application_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplications] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDependentApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all applications with imported deployables from a specified application'
GO

IF OBJECT_ID('[Applications_GetDependentApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetDependentApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDependentApplications]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

        SELECT * 
          FROM [Applications_Extended]
         WHERE [Application_Id] IN (SELECT [Application_Id]     
                                      FROM [DeployablesImported_Extended] 
                                     WHERE [ImportedFrom_Application_Id] = @Application_Id)

END
GO

GRANT EXECUTE ON [Applications_GetDependentApplications] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified deployable'
GO


IF OBJECT_ID('[Applications_GetDeployable]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployable]
(
	@Deployable_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Deployable_Id] = @Deployable_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployableDependencies',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of dependencies for the deployables in a given application or a specific deployable if one is specified'
GO

IF OBJECT_ID('[Applications_GetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployableDependencies]
(
	@Application_Id INT,
	@Deployable_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM [DeployableDependencies]
		 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM [Deployables] 
		                            WHERE [Application_Id] = @Application_Id)
		   AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetDeployableDependencies] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the deployables for a specific application or for all applications.'
GO

IF OBJECT_ID('[Applications_GetDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE @Application_Id IS NULL
	    OR [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployables] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetImportedDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployablesImported_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set a imported deployables for a specified application'
GO

IF OBJECT_ID('[Applications_GetImportedDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetImportedDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetImportedDeployables]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM DeployablesImported_Extended
		WHERE @Application_Id IS NULL 
		   OR [Application_Id] = @Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetImportedDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetLatestEnvironmentExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_LatestEnvironmentExecutions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a view of all the latest executions grouped by application and environment, optionally filtered by application or application group.'
GO


IF OBJECT_ID('[Applications_GetLatestEnvironmentExecutions]') IS NOT NULL DROP PROCEDURE [Applications_GetLatestEnvironmentExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetLatestEnvironmentExecutions]
(
	@Application_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL
)
AS
BEGIN

	SELECT *
	  FROM [Applications_LatestEnvironmentExecutions] ALEE

	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
       AND (@ApplicationGroup_Id IS NULL OR EXISTS(SELECT * 
                                                     FROM [ApplicationGroups] AG
                                              CROSS APPLY dbo.ApplicationGroups_GetAncestorGroups(AG.[ApplicationGroup_Id]) AGA
                                                    WHERE AGA.[ApplicationGroup_Id] = @ApplicationGroup_Id
                                                      AND AG.[ApplicationGroup_Id] = ALEE.[ApplicationGroup_Id])
            )
	 ORDER BY   
	          CASE WHEN ALEE.[ApplicationGroup_Id] IS NULL THEN 1 ELSE 0 END
	         ,[ApplicationGroup_Name]
	         ,[Application_Name]

END
GO

GRANT EXECUTE ON [Applications_GetLatestEnvironmentExecutions] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetNavigationData',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Builds_Extended,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the information required to display the application navigation bar.'
GO


IF OBJECT_ID('[Applications_GetNavigationData]') IS NOT NULL DROP PROCEDURE [Applications_GetNavigationData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetNavigationData]
(
    @Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Applications_Extended]
	 WHERE [Application_Id] = @Application_Id

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [BuildStatus_Name] = 'Active'
	 ORDER BY [CreatedOn_Date] DESC

	SELECT *
	  FROM [ApplicationDeploymentPlans]
	 WHERE [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetNavigationData] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetSpecifiedApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets only the active applications in the specified CSV string.'
GO


IF OBJECT_ID('[Applications_GetSpecifiedApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetSpecifiedApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetSpecifiedApplications]
(
    @ApplicationIds_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Applications_Slim] A
    INNER JOIN dbo.CsvToTable(@ApplicationIds_Csv, ',') CSV 
            ON CAST(CSV.[Value_Text] AS INT) = A.[Application_Id]
     WHERE A.[Active_Indicator] = 'Y'
	ORDER BY [Application_Name]

END
GO

GRANT EXECUTE ON [Applications_GetSpecifiedApplications] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Applications_GetWorkflowSteps]') IS NOT NULL DROP PROCEDURE [Applications_GetWorkflowSteps]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeDeployableData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a deployable and all its associations from the system.'
GO

IF OBJECT_ID('[Applications_PurgeDeployableData]') IS NOT NULL DROP PROCEDURE [Applications_PurgeDeployableData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeDeployableData]
(
	@Deployable_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ConfigurationFiles TABLE ([ConfigurationFile_Id] INT NOT NULL)
    INSERT INTO @ConfigurationFiles ([ConfigurationFile_Id])
    SELECT [ConfigurationFile_Id]
      FROM [ConfigurationFiles]
     WHERE [Deployable_Id] = @Deployable_Id

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
    INSERT INTO @DatabaseChangeScripts ([Script_Id])
    SELECT [Script_Id]
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

    DELETE [DeployableDependencies]
     WHERE [Deployable_Id] = @Deployable_Id
	    OR [DependsOn_Deployable_Id] = @Deployable_Id

	DELETE [DeployablesImported]
	 WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [BuildArtifacts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [ConfigurationFileDeployments]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

    DELETE
      FROM [ReleaseConfigurationFiles]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

	DELETE
      FROM [ConfigurationFileVersions]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFileInstances]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFiles]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [ReleaseDeployables]
     WHERE [Deployable_Id] = @Deployable_Id 

    DELETE
      FROM [VariableDefinitions]
     WHERE [Deployable_Id] = @Deployable_Id 


	DECLARE @Deployable_Name SQL_VARIANT, @Application_Id INT
	 SELECT @Deployable_Name = [Deployable_Name] 
	       ,@Application_Id = [Application_Id] 
       FROM [Deployables] 
      WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [Deployables]
     WHERE [Deployable_Id] = @Deployable_Id 

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPDLD',@Application_Id,@Deployable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeDeployableData] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_ReactivateApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Reactivates an application that has been deactivated.'
GO

IF OBJECT_ID('[Applications_ReactivateApplication]') IS NOT NULL
	DROP PROCEDURE [Applications_ReactivateApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_ReactivateApplication]
(
	@Application_Id INT
) 
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
	   SET [Active_Indicator] = 'Y'
	 WHERE [Application_Id] = @Application_Id

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_ReactivateApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SearchApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Searches for applications by application name and group name that contain the search term.'
GO


IF OBJECT_ID('[Applications_SearchApplications]') IS NOT NULL DROP PROCEDURE [Applications_SearchApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SearchApplications]
(
    @SearchTerm_Text NVARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
      FROM [Applications_Slim]
     WHERE [Active_Indicator] = 'Y'
       AND (NULLIF(@SearchTerm_Text, '') IS NULL
        OR CHARINDEX(@SearchTerm_Text, [Application_Name]) > 0
        OR CHARINDEX(@SearchTerm_Text, [ApplicationGroup_Name]) > 0)
	ORDER BY 
        CASE WHEN [ApplicationGroup_Id] IS NULL THEN 0 ELSE 1 END, 
        [ApplicationGroup_Name], 
        [Application_Name]

END
GO

GRANT EXECUTE ON [Applications_SearchApplications] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dashboard text for an application'
GO


IF OBJECT_ID('[Applications_SetDashboard]') IS NOT NULL DROP PROCEDURE [Applications_SetDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDashboard]
(
	@Application_Id INT,
	@Dashboard_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Dashboard_Text] = @Dashboard_Text
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDashboard] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dependency relationships between the specified deployable and its dependents'
GO

IF OBJECT_ID('[Applications_SetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_SetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDeployableDependencies]
(
	@Deployable_Id INT,
	@DependsOn_Deployables_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployableDependencies]
		 WHERE [Deployable_Id] = @Deployable_Id

		INSERT INTO [DeployableDependencies]
			(
				[Deployable_Id],
				[DependsOn_Deployable_Id]
			)
		SELECT
				@Deployable_Id,
				CAST(CSV.[Value_Text] AS INT)
		  FROM [dbo].[CsvToTable](@DependsOn_Deployables_Csv, ',') CSV

		DECLARE @Application_Id INT 
		    SET @Application_Id = (SELECT [Application_Id] 
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDeployableDependencies] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetIssueTracker',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the issue tracking provider of a specified application.'
GO

IF OBJECT_ID('[Applications_SetIssueTracker]') IS NOT NULL
	DROP PROCEDURE [Applications_SetIssueTracker]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetIssueTracker]
(
	@Application_Id INT,
	@IssueTracking_Provider_Id INT,
	@IssueTracking_CategoryIdList_Text VARCHAR(MAX) = NULL,
	@IssueTracking_Filter_Configuration XML = NULL,
    @IssueTracking_AllowBuildMasterIssues_Indicator YNINDICATOR = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
	   SET [IssueTracking_Provider_Id] = @IssueTracking_Provider_Id
		  ,[IssueTracking_CategoryIdList_Text] = @IssueTracking_CategoryIdList_Text
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
		  ,[IssueTracking_Filter_Configuration] = @IssueTracking_Filter_Configuration
          ,[IssueTracking_AllowBuildMasterIssues_Indicator] = COALESCE(@IssueTracking_AllowBuildMasterIssues_Indicator, [IssueTracking_AllowBuildMasterIssues_Indicator])
	 WHERE [Application_Id] = @Application_Id

	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetIssueTracker] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_ValidateHealth',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Applications_Health',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets data relevant to the Application Health page.'
GO


IF OBJECT_ID('[Applications_ValidateHealth]') IS NOT NULL DROP PROCEDURE [Applications_ValidateHealth]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_ValidateHealth]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * FROM [Applications_Health]
    WHERE [Application_Id] = @Application_Id
	
END
GO

GRANT EXECUTE ON [Applications_ValidateHealth] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_CreateOrReplaceArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a reference to an artifact in the artifact library.'
GO


IF OBJECT_ID('[Artifacts_CreateOrReplaceArtifact]') IS NOT NULL DROP PROCEDURE [Artifacts_CreateOrReplaceArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_CreateOrReplaceArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Deployable_Id INT,
	@Artifact_Name VARCHAR(50),
	@Execution_Id INT,
	@Artifact_Path VARCHAR(250)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	SELECT @Build_Id = [Build_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE [BuildArtifacts]
	 WHERE [Build_Id] = @Build_Id
       AND (
             (@Deployable_Id IS NULL AND [Deployable_Id] IS NULL)
             OR
             ([Deployable_Id] = @Deployable_Id)
           )
	   AND [Artifact_Name] = @Artifact_Name

	INSERT INTO [BuildArtifacts]
	(
		[Build_Id],
		[Deployable_Id],
		[Artifact_Name],
		[Execution_Id],
		[Created_Date],
		[Artifact_Path]
	)
	VALUES
	(
		@Build_Id,
		@Deployable_Id,
		@Artifact_Name,
		@Execution_Id,
		GETUTCDATE(),
		@Artifact_Path
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Artifacts_CreateOrReplaceArtifact] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an artifact based on the specified criteria.'
GO


IF OBJECT_ID('[Artifacts_GetArtifact]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Artifact_Name VARCHAR(50)
)
AS
BEGIN

	SELECT *
	  FROM [BuildArtifacts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND [Artifact_Name] = @Artifact_Name

END
GO

GRANT EXECUTE ON [Artifacts_GetArtifact] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifacts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts based on the specified criteria.'
GO


IF OBJECT_ID('[Artifacts_GetArtifacts]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifacts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifacts]
(
	@Application_Id INT = NULL,
	@Release_Number RELEASENUMBER = NULL,
	@Build_Number BUILDNUMBER = NULL,
	@Deployable_Id INT = NULL,
    @Artifact_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

    IF (@Artifact_Count IS NOT NULL)
        SET ROWCOUNT @Artifact_Count

	SELECT *
	  FROM [BuildArtifacts_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@Release_Number IS NULL OR [Release_Number] = @Release_Number)
	   AND (@Build_Number IS NULL OR [Build_Number] = @Build_Number)
	   AND (@Deployable_Id IS NULL OR [Deployable_Id] IS NULL OR [Deployable_Id] = @Deployable_Id)
	 ORDER BY [Created_Date] DESC

END
GO

GRANT EXECUTE ON [Artifacts_GetArtifacts] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifactsByBuildStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts that are associated with a specified build status, optionally filtered by application and release.'
GO


IF OBJECT_ID('[Artifacts_GetArtifactsByBuildStatus]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@BuildStatus_Name VARCHAR(8)
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Id IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE IF @Release_Number IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
END
GO

GRANT EXECUTE ON [Artifacts_GetArtifactsByBuildStatus] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildExecutions_GetPipelineStageNames',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_PipelineStages',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all stage names based on the target of an execution.'
GO

IF OBJECT_ID('[BuildExecutions_GetPipelineStageNames]') IS NOT NULL DROP PROCEDURE [BuildExecutions_GetPipelineStageNames]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildExecutions_GetPipelineStageNames]
AS
BEGIN

SET NOCOUNT ON

    SELECT * FROM [BuildExecutions_PipelineStages]
     ORDER BY [PipelineStage_Name]

END
GO

GRANT EXECUTE ON [BuildExecutions_GetPipelineStageNames] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetAlerts',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Alerts_Xml',
    /* Description_Text        */ 'Gets the alerts.'
GO

IF OBJECT_ID('[BuildMaster_GetAlerts]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetAlerts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetAlerts]
(
	@Alerts_Xml XML = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

SET @Alerts_Xml =
    (SELECT
           [@IgnoreServiceWarning] = COALESCE((SELECT [Value_Text]
                                        FROM [Configuration]
										WITH (READUNCOMMITTED)
                                       WHERE [Scope_Name] = 'Web'
                                         AND [Key_Name] = 'IgnoreServiceWarning'), 'False'),

           [@ErrorsInLog] = (CASE WHEN EXISTS(SELECT [LogMessage_Id]
                                                FROM [LogMessages]
												WITH (READUNCOMMITTED)
                                              WHERE [ShowNotification_Indicator] = 'Y')
                                         THEN 'True'
                                         ELSE 'False' END),

           [@ServersInErrorStatus] = (SELECT COUNT(*)
                                        FROM [Servers]
                                        WITH (READUNCOMMITTED)
                                       WHERE [ServerStatus_Code] = 'ERROR'
									     AND [ServerType_Code] = 'S'
                                         AND [Active_Indicator] = 'Y'),

           [@TotalServers] = (SELECT COUNT(*) 
                                        FROM [Servers] 
                                        WITH (READUNCOMMITTED)
                                       WHERE [ServerType_Code] = 'S'
                                         AND [Active_Indicator] = 'Y'),

           [@BmUpdateReminder] = (SELECT [Value_Text]
                                    FROM [Configuration]
									WITH (READUNCOMMITTED)
                                   WHERE [Scope_Name] = 'CoreEx'
                                     AND [Key_Name] = 'BuildMasterUpdateCheck_ShowReminder'),

           [@ExtUpdateReminder] = (SELECT [Value_Text]
                                     FROM [Configuration]
									 WITH (READUNCOMMITTED)
                                    WHERE [Scope_Name] = 'CoreEx'
                                      AND [Key_Name] = 'ExtensionUpdateCheck_ShowReminder'),
       
           [ManualActionsInProgress] = (SELECT [@AssignedTo_Principal_Name] = [AssignedTo_Principal_Name]
                                FROM [ManualActionsInProgress]
							    WITH (READUNCOMMITTED)
                               WHERE [CompletedBy_User_Name] IS NULL
                                 FOR XML PATH('ManualAction'), TYPE),

		   [ExecutionsInProgress] = (SELECT [@ExecutionId] = X.[Execution_Id],
		                                    [@ApplicationId] = R.[Application_Id],
                                            [@EnvironmnetId] = BE.[Environment_Id]
		                               FROM [BuildExecutions] BE
									   WITH (READUNCOMMITTED)
									        INNER JOIN [Executions] X WITH (READUNCOMMITTED)
											       ON X.[Execution_Id] = BE.[Execution_Id]
											LEFT JOIN [BuildPromotions] BP WITH (READUNCOMMITTED)
											        ON BP.[Promotion_Id] = BE.[Promotion_Id]
											LEFT JOIN [Builds] B WITH (READUNCOMMITTED)
											        ON B.[Build_Id] = BP.[Build_Id]
											LEFT JOIN [Releases] R WITH (READUNCOMMITTED)
											        ON R.[Release_Id] = B.[Release_Id]
									  WHERE X.[ExecutionRunState_Code] = 'X'
									    FOR XML PATH('Execution'), TYPE)

   FOR XML PATH('Alerts'))

END
GO

GRANT EXECUTE ON [BuildMaster_GetAlerts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetCurrentDateTime',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Current_Date',
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[BuildMaster_GetCurrentDateTime]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetCurrentDateTime]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetCurrentDateTime]
(
	@Current_Date DATETIME OUT
)
AS
BEGIN

	SET NOCOUNT ON
    
    SET @Current_Date = GETUTCDATE()

END
GO

GRANT EXECUTE ON [BuildMaster_GetCurrentDateTime] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetUsageInfo',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildMaster_UsageInfo',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets miscellaneous usage data from the system.'
GO


IF OBJECT_ID('[BuildMaster_GetUsageInfo]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetUsageInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetUsageInfo]
AS
BEGIN

	SET NOCOUNT ON

    SELECT * FROM [BuildMaster_UsageInfo]

END
GO

GRANT EXECUTE ON [BuildMaster_GetUsageInfo] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_AddOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a build output report to a specified build execution'
GO

IF OBJECT_ID('[BuildOutputs_AddOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_AddOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_AddOutput]
(
    @Execution_Id INT,
	@Output_Name VARCHAR(50),
	@Output_Bytes VARBINARY(MAX),
    @OutputType_Code CHAR(1)
)
AS
BEGIN

	SET NOCOUNT ON

	BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Output_Sequence INT
	SET @Output_Sequence = COALESCE((SELECT MAX([Output_Sequence])
	                                   FROM [BuildOutputs]
							          WHERE [Execution_Id] = @Execution_Id), 0) + 1

	INSERT INTO [BuildOutputs]
	(
		[Execution_Id],
		[Output_Sequence],
		[Output_Name],
	    [Output_Bytes],
		[OutputType_Code]
	)
	VALUES
	(
		@Execution_Id,
		@Output_Sequence,
		@Output_Name,
		@Output_Bytes,
		@OutputType_Code
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildOutputs_AddOutput] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output report for a specified build and sequence number'
GO


IF OBJECT_ID('[BuildOutputs_GetOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutput]
(
	@Execution_Id INT,
	@Output_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Output_Sequence] = @Output_Sequence

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutput] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutputs',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output reports for a specified build'
GO


IF OBJECT_ID('[BuildOutputs_GetOutputs]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutputs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutputs]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Application_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER
	
	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number]
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	 

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
  ORDER BY [Output_Sequence]

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutputs] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateRecurring',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a recurring build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateRecurring]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@ReleaseFilter_Pipeline_Id INT,
    @BuildImporter_Configuration XML,
    @Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[ReleaseFilter_ReleaseType_Code]
            ,[ReleaseFilter_Pipeline_Id]
            ,[BuildImporter_Configuration])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'Recurring'
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@ReleaseFilter_ReleaseType_Code
			,@ReleaseFilter_Pipeline_Id
            ,@BuildImporter_Configuration)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'Recurring' RAISERROR('Existing BuildSchedule #%i is not a Recurring schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[ReleaseFilter_Pipeline_Id] = @ReleaseFilter_Pipeline_Id
              ,[BuildImporter_Configuration] = @BuildImporter_Configuration
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateRecurring] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a scheduled build based on the specified parameters, or updates an existing scheduled build if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSchedule]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
    @Active_Indicator YNINDICATOR,
    @Schedule_Type VARCHAR(15),
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@Url_AllowedIP_CSV varchar(MAX),
	@Url_Username_Text varchar(50),
	@Url_Password_Text varchar(50),
	@SourceControl_Path_Name varchar(250),
	@SourceControl_Provider_Id int,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@ReleaseFilter_Pipeline_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[ReleaseFilter_Pipeline_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,@Schedule_Type
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@ReleaseFilter_Pipeline_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[Schedule_Type] = @Schedule_Type
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[ReleaseFilter_Pipeline_Id] = @ReleaseFilter_Pipeline_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id)

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSchedule] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSCMTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a source control triggered automated build based on the specified parameters, or updates an existing one if one with a given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSCMTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@SourceControl_Path_Name VARCHAR(250),
	@SourceControl_Provider_Id INT,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@ReleaseFilter_Pipeline_Id INT,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[ReleaseFilter_Pipeline_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'SCM Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@ReleaseFilter_Pipeline_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'SCM Triggered' RAISERROR('Existing BuildSchedule #%i is not an SCM Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[ReleaseFilter_Pipeline_Id] = @ReleaseFilter_Pipeline_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSCMTriggered] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateURLTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a URL-triggered build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateURLTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@Url_AllowedIP_CSV VARCHAR(MAX),
	@Url_Username_Text VARCHAR(50),
	@Url_Password_Text VARCHAR(50),
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@ReleaseFilter_Pipeline_Id INT,
    @BuildImporter_Configuration XML,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[ReleaseFilter_ReleaseType_Code]
			,[ReleaseFilter_Pipeline_Id]
            ,[BuildImporter_Configuration])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'URL Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@ReleaseFilter_ReleaseType_Code
			,@ReleaseFilter_Pipeline_Id
            ,@BuildImporter_Configuration)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'URL Triggered' RAISERROR('Existing BuildSchedule #%i is not a URL Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[ReleaseFilter_Pipeline_Id] = @ReleaseFilter_Pipeline_Id
              ,[BuildImporter_Configuration] = @BuildImporter_Configuration
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateURLTriggered] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_DeleteBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_DeleteBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Schedule_Name SQL_VARIANT
	 SELECT @Application_Id = [Application_Id], @Schedule_Name = [Schedule_Name]
	   FROM [BuildSchedules]
	  WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildSchedules]
	 WHERE [Schedule_Id] = @Schedule_Id

	IF @@ROWCOUNT <> 0 EXEC [Events_RaiseEvent] 'BSHDEL', @Application_Id, @Schedule_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_DeleteBuildSchedule] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended,BuildScheduleVariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_GetBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE [Schedule_Id] = @Schedule_Id

    SELECT *
      FROM [BuildScheduleVariableDefinitions_Extended]
     WHERE [Schedule_Id] = @Schedule_Id

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_GetBuildSchedule] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedules',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of builds schedules for the specified application and active indicator, or all if null is supplied'
GO

IF OBJECT_ID('[BuildSchedules_GetBuildSchedules]') IS NOT NULL
	DROP PROCEDURE [BuildSchedules_GetBuildSchedules]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_GetBuildSchedules]
(
	@Application_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE ( @Application_Id IS NULL OR [Application_Id] = @Application_Id )
	   AND [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])

END
GO

GRANT EXECUTE ON [BuildSchedules_GetBuildSchedules] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_SetLastExecutionDate',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the last execution date for a scheduled build.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_SetLastExecutionDate]') IS NOT NULL DROP PROCEDURE [BuildSchedules_SetLastExecutionDate]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_SetLastExecutionDate]
(
	@Schedule_Id INT,
	@LastExecution_Date DATETIME
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		UPDATE [BuildSchedules]
		   SET [LastExecution_Date] = @LastExecution_Date
		 WHERE [Schedule_Id] = @Schedule_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_SetLastExecutionDate] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_SetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates a variable definition to use with a scheduled build',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_SetVariableDefinition]') IS NOT NULL DROP PROCEDURE [BuildSchedules_SetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_SetVariableDefinition]
(
	@Schedule_Id INT,
	@VariableDeclaration_Id INT,
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @Value_Text IS NULL OR @Value_Text = ''
	BEGIN

		DELETE [BuildScheduleVariableDefinitions]
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id

	END
	ELSE IF EXISTS(SELECT *
		            FROM [BuildScheduleVariableDefinitions]
				   WHERE [Schedule_Id] = @Schedule_Id
				     AND [VariableDeclaration_Id] = @VariableDeclaration_Id)
	BEGIN
		
		UPDATE [BuildScheduleVariableDefinitions]
		   SET [Value_Text] = @Value_Text
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id
          
	END
	ELSE
	BEGIN

		INSERT INTO [BuildScheduleVariableDefinitions]
		([Schedule_Id], [VariableDeclaration_Id], [Value_Text])
		VALUES
		(@Schedule_Id, @VariableDeclaration_Id, @Value_Text)

	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_SetVariableDefinition] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Builds_AddApproval]') IS NOT NULL DROP PROCEDURE [Builds_AddApproval]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CompletePromotion',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Completes a build promotion.'
GO


IF OBJECT_ID('[Builds_CompletePromotion]') IS NOT NULL DROP PROCEDURE [Builds_CompletePromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CompletePromotion]
(
	@Promotion_Id INT,
	@PromotionStatus_Name VARCHAR(20),
	@StatusOverriddenReason_Text VARCHAR(100)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildPromotions]
	   SET [PromotionStatus_Name] = @PromotionStatus_Name,
	       [StatusOverridden_Date] = CASE WHEN @StatusOverriddenReason_Text IS NOT NULL THEN GETUTCDATE() ELSE NULL END,
		   [StatusOverriddenBy_User_Name] = CASE WHEN @StatusOverriddenReason_Text IS NOT NULL THEN dbo.Session_GetUserName() ELSE NULL END,
		   [StatusOverriddenReason_Text] = @StatusOverriddenReason_Text
	 WHERE [Promotion_Id] = @Promotion_Id

     DECLARE @Application_Id INT
     DECLARE @Release_Number RELEASENUMBER
     DECLARE @Build_Number BUILDNUMBER
     DECLARE @PipelineStage_Name VARCHAR(50)

     SELECT @Application_Id = R.[Application_Id],
            @Release_Number = R.[Release_Number],
            @Build_Number = B.[Build_Number],
            @PipelineStage_Name = P.[PipelineStage_Name]
       FROM [BuildPromotions] P
      INNER JOIN [Builds] B ON P.[Build_Id] = B.[Build_Id]
      INNER JOIN [Releases] R ON B.[Release_Id] = R.[Release_Id]
      WHERE P.[Promotion_Id] = @Promotion_Id

	EXEC [Events_RaiseEvent] 'BLDCPP', @Application_Id, @Release_Number, @Build_Number, @PipelineStage_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Builds_CompletePromotion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CreateBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new build of an application.'
GO

IF OBJECT_ID('[Builds_CreateBuild]') IS NOT NULL DROP PROCEDURE [Builds_CreateBuild]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CreateBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@BuildImporter_Configuration XML = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	DECLARE @Build_Sequence INT
	SET @Build_Sequence = COALESCE((SELECT MAX([Build_Sequence])
	                                  FROM [Builds]
							         WHERE [Release_Id] = @Release_Id), 0) + 1

	INSERT INTO [Builds]
	(
		[Release_Id],
		[Build_Number],
		[BuildStatus_Name],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[BuildImporter_Configuration],
		[Build_Sequence]
	)
	VALUES
	(
		@Release_Id,
		@Build_Number,
		'Active',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		@BuildImporter_Configuration,
		@Build_Sequence
	)

	DECLARE @Build_Id INT
	SET @Build_Id = SCOPE_IDENTITY()

    UPDATE [Issues]
       SET [OpenedOn_Original_Build_Number] = @Build_Number
     WHERE [Release_Id] = @Release_Id
       AND [Closed_Indicator] = 'N'
       AND [OpenedOn_Original_Build_Number] IS NULL

    UPDATE [Issues]
       SET [ClosedOn_Original_Build_Number] = @Build_Number
     WHERE [Release_Id] = @Release_Id
       AND [Closed_Indicator] = 'Y'
       AND [ClosedOn_Original_Build_Number] IS NULL

	EXEC [Events_RaiseEvent] 'BLDNEW', @Application_Id, @Release_Number, @Build_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_CreateBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_DeployBuild',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ '',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks a build and its associated release as deployed based on the settings in its workflow.'
GO

IF OBJECT_ID('[Builds_DeployBuild]') IS NOT NULL DROP PROCEDURE [Builds_DeployBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_DeployBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

    BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @Release_Id INT
		DECLARE @Build_Id INT
		SELECT @Release_Id = B.[Release_Id],
		       @Build_Id = B.[Build_Id]
		  FROM [Builds] B
		       INNER JOIN [Releases] R
			           ON R.[Release_Id] = B.[Release_Id]
		 WHERE R.[Application_Id] = @Application_Id
		   AND R.[Release_Number] = @Release_Number
		   AND B.[Build_Number] = @Build_Number
				

        IF (SELECT [ReleaseStatus_Name] FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number) <> 'Active'
			RAISERROR('Can only deploy an active release.', 16, 1)

		IF (SELECT [BuildStatus_Name] FROM [Builds_Extended] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number) <> 'Active'
			RAISERROR('Can only deploy an active build.', 16, 1)

	    UPDATE [Releases]
		   SET [ReleaseStatus_Name] = 'Deployed'
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

        DECLARE @Release_Sequence BIGINT

        SELECT @Release_Sequence = [Release_Sequence]
          FROM [Releases]
         WHERE [Application_Id] = @Application_Id 
           AND [Release_Number] = @Release_Number

		UPDATE [Builds]
		   SET [BuildStatus_Name] = 'Deployed'
		 WHERE [Build_Id] = @Build_Id

		UPDATE B
		   SET B.[BuildStatus_Name] = 'Rejected',
			   B.[RejectedBy_User_Name] = dbo.Session_GetUserName(),
			   B.[Rejected_Date] = GETUTCDATE()
		  FROM [Builds] B
		       INNER JOIN [Releases] R
			           ON R.[Release_Id] = B.[Release_Id]
		 WHERE R.[Application_Id] = @Application_Id
		   AND R.[Release_Number] = @Release_Number
		   AND B.[BuildStatus_Name] = 'Active'

		INSERT INTO [ReleaseConfigurationFiles]
		(
			[Release_Id],
			[ConfigurationFile_Id],
			[Version_Number]
		)
		SELECT @Release_Id,
			   CF.[ConfigurationFile_Id],
			   CFV_Latest.[Version_Number]
		  FROM [ConfigurationFiles] CF
		       INNER JOIN [Deployables] D
			           ON D.[Deployable_Id] = CF.[Deployable_Id]
		       INNER JOIN (SELECT [ConfigurationFile_Id],
			                      [Version_Number] = MAX([Version_Number])
			                 FROM [ConfigurationFileVersions]
							GROUP BY [ConfigurationFile_Id]) CFV_Latest
					   ON CFV_Latest.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]
		 WHERE D.[Application_Id] = @Application_Id
		   AND CF.[ConfigurationFile_Id] NOT IN (SELECT [ConfigurationFile_Id]
		                                           FROM [ReleaseConfigurationFiles] RCF
												  WHERE RCF.[Release_Id] = @Release_Id)
		

	    EXEC [Events_RaiseEvent] 'RELDEP', @Application_Id, @Release_Number

	COMMIT
    END TRY BEGIN CATCH
	    IF XACT_STATE()<>0 ROLLBACK
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [Builds_DeployBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build.'
GO


IF OBJECT_ID('[Builds_GetBuild]') IS NOT NULL DROP PROCEDURE [Builds_GetBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %s does not exist.', 16, 1, @Application_Id, @Release_Number, @Build_Number)

END
GO

GRANT EXECUTE ON [Builds_GetBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuildById',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build by ID.'
GO


IF OBJECT_ID('[Builds_GetBuildById]') IS NOT NULL DROP PROCEDURE [Builds_GetBuildById]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuildById]
(
	@Build_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Build_Id] = @Build_Id


END
GO

GRANT EXECUTE ON [Builds_GetBuildById] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuilds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all builds that match the specified criteria.'
GO


IF OBJECT_ID('[Builds_GetBuilds]') IS NOT NULL DROP PROCEDURE [Builds_GetBuilds]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuilds]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@BuildStatus_Name VARCHAR(8) = NULL,
	@Build_Count INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	IF (@Build_Count IS NOT NULL AND @Build_Count > 0)
	BEGIN
		SET ROWCOUNT @Build_Count
	END

	SELECT *
	  FROM [Builds_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@Release_Number IS NULL OR [Release_Number] = @Release_Number)
	   AND (@BuildStatus_Name IS NULL OR [BuildStatus_Name] = @BuildStatus_Name)
	  ORDER BY [Release_Sequence] DESC, [Build_Sequence] DESC, [CreatedOn_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetBuilds] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution'
GO

IF OBJECT_ID('[Builds_GetExecution]') IS NOT NULL DROP PROCEDURE [Builds_GetExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecution]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id


END
GO

GRANT EXECUTE ON [Builds_GetExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionExtended',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended,BuildExecution_DeploymentPlans,BuildExecution_DeploymentPlanActionGroups,BuildExecution_ActionGroupActions,BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution.'
GO

IF OBJECT_ID('[Builds_GetExecutionExtended]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionExtended]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionExtended]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id

	SELECT *
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] = @Execution_Id

	SELECT BEDPAG.*
	  FROM [BuildExecution_DeploymentPlanActionGroups] BEDPAG
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id
	 
	SELECT BEAGA.*
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id

	SELECT BEAGALE.*
	  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		   INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id
	   AND BEAGA.[Original_Action_Sequence] = 0
	   AND BEAGA.[Original_DeploymentPlanActionGroup_Sequence] = 0
	   AND BEDP.[Original_DeploymentPlan_Id] = 0
	 ORDER BY BEAGALE.[LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionExtended] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the build executions for the specified build'
GO


IF OBJECT_ID('[Builds_GetExecutions]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@PipelineStage_Name VARCHAR(50) = NULL,
	@Execution_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF NULLIF(@Execution_Count, 0) IS NOT NULL
		SET ROWCOUNT @Execution_Count

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@Release_Number IS NULL OR @Release_Number = [Release_Number])
	   AND (@Build_Number IS NULL OR @Build_Number = [Build_Number])
	   AND (@PipelineStage_Name IS NULL OR @PipelineStage_Name = [PipelineStage_Name])
	 ORDER BY [ExecutionStart_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetExecutions] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionsByServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets recent build executions for the specified server.'
GO


IF OBJECT_ID('[Builds_GetExecutionsByServer]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionsByServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionsByServer]
(
	@Server_Id INT,
	@Execution_Count INT = 50
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Execution_Count = COALESCE(@Execution_Count, 50)

	DECLARE @Server_Name VARCHAR(50)
	SET @Server_Name = (SELECT [Server_Name]
	                      FROM [Servers]
	                     WHERE [Server_Id] = @Server_Id)

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] IN
	 (
		SELECT DISTINCT TOP(@Execution_Count) BEDP.[Execution_Id]
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEAGA.[Original_Server_Name] = @Server_Name
		 ORDER BY BEDP.[Execution_Id] DESC
	 )

END
GO

GRANT EXECUTE ON [Builds_GetExecutionsByServer] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionsInProgress',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all executions in the executing state optionally filtered by application.'
GO


IF OBJECT_ID('[Builds_GetExecutionsInProgress]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionsInProgress]
(
	@Application_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND [ExecutionStatus_Name] = 'Executing'
	 ORDER BY [ExecutionStart_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetExecutionsInProgress] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a build promotion.'
GO


IF OBJECT_ID('[Builds_GetPromotion]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotion]
(
	@Promotion_Id INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Promotion_Id] = @Promotion_Id

END
GO

GRANT EXECUTE ON [Builds_GetPromotion] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the promotions that have occurred.'
GO


IF OBJECT_ID('[Builds_GetPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER = NULL
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND (@Build_Number IS NULL OR ([Build_Number] = @Build_Number))

END
GO

GRANT EXECUTE ON [Builds_GetPromotions] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Builds_GetRequiredPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetRequiredPromotions]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_InitiateBuildExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Execution_Id',
    /* Description_Text        */ 'Initiates a new build execution.'
GO

IF OBJECT_ID('[Builds_InitiateBuildExecution]') IS NOT NULL DROP PROCEDURE [Builds_InitiateBuildExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_InitiateBuildExecution]
(
	@Promotion_Id INT,
	@Plan_Name VARCHAR(255),
	@Start_Date DATETIME,
	@Environment_Id INT,
	@DependentUpon_Execution_Id INT = NULL,
    @PipelineStageTarget_Index INT = NULL,
	@Execution_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Start_Date = COALESCE(@Start_Date, GETUTCDATE())

	DECLARE @Application_Id INT
	SELECT @Application_Id = R.[Application_Id]
	  FROM [BuildPromotions] BP
	       INNER JOIN [Builds] B
		           ON B.[Build_Id] = BP.[Promotion_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]

	INSERT INTO [Executions]
	(
		[Start_Date],
		[ExecutionStatus_Code],
		[ExecutionRunState_Code],
		[ExecutionMode_Code],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[DependentUpon_Execution_Id]
	)
	VALUES
	(
		@Start_Date,
		'S',
		'P',
		'B',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		@DependentUpon_Execution_Id
	)

	SET @Execution_Id = SCOPE_IDENTITY()

	INSERT INTO [BuildExecutions]
	(
		[Execution_Id],
		[Promotion_Id],
		[Plan_Name],
		[Environment_Id],
        [PipelineStageTarget_Index]
	)
	VALUES
	(
		@Execution_Id,
		@Promotion_Id,
		@Plan_Name,
		@Environment_Id,
        @PipelineStageTarget_Index
	)

	EXEC [Events_RaiseEvent] 'EXEINI', @Application_Id, @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Builds_InitiateBuildExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PromoteBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Promotion_Id',
    /* Description_Text        */ 'Promotes a build to to a pipeline stage.'
GO

IF OBJECT_ID('[Builds_PromoteBuild]') IS NOT NULL DROP PROCEDURE [Builds_PromoteBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PromoteBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Comments_Text VARCHAR(255),
	@ForcePromotion_Indicator YNINDICATOR,
	@PromoteTo_PipelineStage_Name VARCHAR(50),
	@SetFurthestStage_Indicator YNINDICATOR = NULL,
	@Promotion_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	SELECT @Build_Id = [Build_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	INSERT INTO [BuildPromotions]
	(
		[Build_Id],
		[PipelineStage_Name],
		[PromotedBy_Name],
		[Comments_Text],
		[PromotionStatus_Name],
		[Forced_Indicator],
		[Promoted_Date]
	)
	VALUES
	(
		@Build_Id,
		@PromoteTo_PipelineStage_Name,
		dbo.Session_GetUserName(),
		NULLIF(@Comments_Text, ''),
		'AwaitingExecution',
		@ForcePromotion_Indicator,
		GETUTCDATE()
	)

	SET @Promotion_Id = SCOPE_IDENTITY()

	DELETE [BuildPromotionsScheduled]
	 WHERE [Build_Id] = @Build_Id
	   AND [PipelineStage_Name] = @PromoteTo_PipelineStage_Name

	IF @SetFurthestStage_Indicator = 'Y'
	BEGIN

		UPDATE [Builds]
		   SET [Furthest_PipelineStage_Name] = @PromoteTo_PipelineStage_Name
		 WHERE [Build_Id] = @Build_Id

	END

	EXEC [Events_RaiseEvent] 'BLDPMT', @Application_Id, @Release_Number, @Build_Number, @PromoteTo_PipelineStage_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
		   
END
GO

GRANT EXECUTE ON [Builds_PromoteBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PurgeBuildData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire build and all associated data from the system.'
GO

IF OBJECT_ID('[Builds_PurgeBuildData]') IS NOT NULL DROP PROCEDURE [Builds_PurgeBuildData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PurgeBuildData]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	DECLARE @Release_Id INT
	DECLARE @Application_Name VARCHAR(50)
	SELECT @Build_Id = B.[Build_Id],
	       @Release_Id = B.[Release_Id],
	       @Application_Name = A.[Application_Name]
	  FROM [Builds] B
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
		   INNER JOIN [Applications] A
		           ON A.[Application_Id] = R.[Application_Id]
	 WHERE A.[Application_Id] = @Application_Id
           AND R.[Release_Number] = @Release_Number
           AND B.[Build_Number] = @Build_Number
          

	DECLARE @Promotions TABLE ([Promotion_Id] INT NOT NULL)
	INSERT INTO @Promotions
	SELECT [Promotion_Id]
	  FROM [BuildPromotions]
	 WHERE [Build_Id] = @Build_Id

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)

	DELETE [ReleaseNotes]
	 WHERE [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)

	DELETE [VariableDefinitions]
	 WHERE [Build_Id] = @Build_Id
	    OR [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)
		OR [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildArtifacts]
	 WHERE [Build_Id] = @Build_Id

    DELETE [BuildPromotionsScheduled]
	 WHERE [Build_Id] = @Build_Id

	DECLARE @BuildExecution_DeploymentPlans TABLE ([BuildExecution_DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_DeploymentPlans ([BuildExecution_DeploymentPlan_Id])
	SELECT [BuildExecution_DeploymentPlan_Id]
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DECLARE @BuildExecution_ActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_ActionGroupActions ([BuildExecution_ActionGroupAction_Id])
	SELECT [BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE [BuildTestResults]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildOutputs]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

    DELETE [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE [ManualActionsInProgress]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE [BuildExecution_DeploymentPlanActionGroups]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	UPDATE [Executions]
	   SET [DependentUpon_Execution_Id] = NULL
	 WHERE [DependentUpon_Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildExecution_DeploymentPlans]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [ScopedExecutionLogEntries]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [ScopedExecutionLogs]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildExecutions]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildPromotions]
	 WHERE [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)

	DELETE [Builds]
	 WHERE [Build_Id] = @Build_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END


GO

GRANT EXECUTE ON [Builds_PurgeBuildData] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PurgeExecutionLogs',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'LogMessagesDeleted_Count',
    /* Description_Text        */ 'Purges execution logs in a specific build before a specified time.'
GO

IF OBJECT_ID('[Builds_PurgeExecutionLogs]') IS NOT NULL DROP PROCEDURE [Builds_PurgeExecutionLogs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PurgeExecutionLogs]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@PipelineStage_Name VARCHAR(50),
	@Before_Date DATETIME,
	@LogMessagesDeleted_Count BIGINT = NULL OUT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ToDelete TABLE ([Execution_Id] INT NOT NULL)

	INSERT INTO @ToDelete
	SELECT BE.[Execution_Id]
	  FROM [BuildExecutions] BE
		   INNER JOIN [BuildPromotions] BP
		           ON BP.[Promotion_Id] = BE.[Promotion_Id]
		   INNER JOIN [Builds] B
			       ON B.[Build_Id] = BP.[Build_Id]
		   INNER JOIN [Releases] R
			       ON R.[Release_Id] = B.[Release_Id]
		   INNER JOIN [Executions] E
		           ON E.[Execution_Id] = BE.[Execution_Id]
	 WHERE R.[Application_Id] = @Application_Id
	   AND R.[Release_Number] = @Release_Number
	   AND B.[Build_Number] = @Build_Number
	   AND (@Environment_Id IS NULL OR BE.[Environment_Id] = @Environment_Id)
	   AND (@PipelineStage_Name IS NULL OR BP.[PipelineStage_Name] = @PipelineStage_Name)
	   AND (@Before_Date IS NULL OR E.[End_Date] < @Before_Date)

	UPDATE [Executions]
	   SET [DependentUpon_Execution_Id] = NULL
	 WHERE [DependentUpon_Execution_Id] IN (SELECT [Execution_Id] FROM @ToDelete)

	DELETE BEAGALE
	  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] IN (SELECT [Execution_Id] FROM @ToDelete)

	SET @LogMessagesDeleted_Count = ROWCOUNT_BIG()

	DELETE
	  FROM [ScopedExecutionLogs]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @ToDelete)

	SET @LogMessagesDeleted_Count = @LogMessagesDeleted_Count + ROWCOUNT_BIG()

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_PurgeExecutionLogs] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_RejectBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Rejects the specified build.'
GO


IF OBJECT_ID('[Builds_RejectBuild]') IS NOT NULL DROP PROCEDURE [Builds_RejectBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_RejectBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Rejection_Notes NVARCHAR(255) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	SELECT @Build_Id = B.[Build_Id]
	  FROM [Builds] B
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
	 WHERE R.[Application_Id] = @Application_Id
	   AND R.[Release_Number] = @Release_Number
	   AND B.[Build_Number] = @Build_Number

	UPDATE [Builds]
	   SET [BuildStatus_Name] = 'Rejected'
		  ,[Rejection_Notes] = @Rejection_Notes
		  ,[Rejected_Date] = GETUTCDATE()
		  ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Build_Id] = @Build_Id
	   AND [BuildStatus_Name] <> 'Deployed'

	EXEC [Events_RaiseEvent] 'BLDRJC', @Application_Id, @Release_Number, @Build_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_RejectBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_SetBuildNumber',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the build number of a build.'
GO

IF OBJECT_ID('[Builds_SetBuildNumber]') IS NOT NULL DROP PROCEDURE [Builds_SetBuildNumber]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_SetBuildNumber]
(
	@Build_Id INT,
    @Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Existing_Build_Number BUILDNUMBER
    DECLARE @Release_Id INT
    SELECT @Existing_Build_Number = [Build_Number],
           @Release_Id = [Release_Id]
      FROM [Builds] 
     WHERE [Build_Id] = @Build_Id

     IF @Existing_Build_Number <> @Build_Number
     BEGIN

        UPDATE [Builds]
           SET [Build_Number] = @Build_Number
         WHERE [Build_Id] = @Build_Id

        UPDATE [Issues]
           SET [OpenedOn_Original_Build_Number] = @Build_Number
         WHERE [Release_Id] = @Release_Id
           AND [OpenedOn_Original_Build_Number] = @Existing_Build_Number

        UPDATE [Issues]
           SET [ClosedOn_Original_Build_Number] = @Build_Number
         WHERE [Release_Id] = @Release_Id
           AND [ClosedOn_Original_Build_Number] = @Existing_Build_Number

       END       	

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_SetBuildNumber] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Builds_ValidateForPromotion]') IS NOT NULL DROP PROCEDURE [Builds_ValidateForPromotion]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetAllGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_AllGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the test result groups in the system.'
GO


IF OBJECT_ID('[BuildTestResults_GetAllGroups]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetAllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetAllGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_AllGroups]
	 ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetAllGroups] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for a specified build execution.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResults]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResults] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResultsForBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for the specified build.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResultsForBuild]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
(
	@Application_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Max_Execution_Id INT

    SELECT TOP 1 @Max_Execution_Id = [Execution_Id]
      FROM [BuildTestResults_TestGroupResults] TGR
	       INNER JOIN [Builds] B
		           ON B.[Build_Id] = TGR.Build_Id
		   INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
	 WHERE R.[Application_Id] = @Application_Id
       AND R.[Release_Number] = @Release_Number
       AND B.[Build_Number] = @Build_Number
     ORDER BY [Execution_Id] DESC

    SELECT *
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Max_Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResultsForBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestHistory',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets this history of a test by its name, and optionally filtered by test group and application.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestHistory]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestHistory]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestHistory]
(
	@Test_Name VARCHAR(255),
	@Group_Name VARCHAR(50),
	@Application_Id INT,
	@TestStatus_Code CHAR(1),
	@Row_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
		SET ROWCOUNT @Row_Count

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE (@Group_Name IS NULL OR [Group_Name] = @Group_Name)
	   AND (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@TestStatus_Code IS NULL OR [TestStatus_Code] = @TestStatus_Code)
	   AND (@Test_Name = [Test_Name])
	 ORDER BY [Application_Name], [Group_Name], [Execution_Id] DESC

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestHistory] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the results of a specific unit test'
GO


IF OBJECT_ID('[BuildTestResults_GetTestLog]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestLog]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Group_Name] = @Group_Name
	   AND [Test_Sequence] = @Test_Sequence

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestLog] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build test results based on the specified criteria.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestResults]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50) = NULL
)
AS
BEGIN

	SET NOCOUNT ON


	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
       AND (@Group_Name IS NULL OR [Group_Name] = @Group_Name)
	ORDER BY [Group_Name], [Test_Sequence]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestResults] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_RecordTestResult',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs the result of an individual unit test.'
GO

IF OBJECT_ID('[BuildTestResults_RecordTestResult]') IS NOT NULL DROP PROCEDURE [BuildTestResults_RecordTestResult]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_RecordTestResult]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Name VARCHAR(255),
	@TestStatus_Code CHAR(1),
	@TestResult_Text VARCHAR(MAX),
	@TestStarted_Date DATETIME,
	@TestEnded_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	INSERT INTO [BuildTestResults]
			   ([Execution_Id]
			   ,[Test_Sequence]
			   ,[Group_Name]
			   ,[Test_Name]
			   ,[TestStatus_Code]
			   ,[TestResult_Text]
			   ,[TestStarted_Date]
			   ,[TestEnded_Date])
		 SELECT
				@Execution_Id,
				ISNULL(MAX([Test_Sequence]), 0) + 1,
				@Group_Name,
				@Test_Name,
				@TestStatus_Code,
				@TestResult_Text,
				@TestStarted_Date,
				@TestEnded_Date
		   FROM [BuildTestResults]
           WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_RecordTestResult] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControlPerformances_GetReleaseSummary',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_ReleaseSummary',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a summary of the change control performances for a specified change control.'
GO

IF OBJECT_ID('[ChangeControlPerformances_GetReleaseSummary]') IS NOT NULL DROP PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

    SELECT RS.*
	  FROM [ChangeControlPerformances_ReleaseSummary] RS
	       INNER JOIN [Environments] E
		           ON E.[Environment_Id] = RS.[Environment_Id]
     WHERE RS.[ChangeControl_Id] = @ChangeControl_Id
	 ORDER BY E.[Display_Sequence]

END
GO

GRANT EXECUTE ON [ChangeControlPerformances_GetReleaseSummary] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_CreateOrUpdateChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ChangeControl_Id',
    /* Description_Text        */ 'Creates a new change control, or updates an existing one.'
GO

IF OBJECT_ID('[ChangeControls_CreateOrUpdateChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
(
	@ChangeControl_Id INT = NULL OUT,
	@ChangeControl_Name VARCHAR(100),
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Group_Name VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@PerformanceType_Code CHAR(1),
	@Active_Indicator YNINDICATOR,
	@Environments_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

    IF @ChangeControl_Id IS NULL
    BEGIN
    
        INSERT INTO [ChangeControls]
        (
			[Release_Id]
	       ,[ChangeControl_Name]
	       ,[Group_Name]
	       ,[Description_Text]
	       ,[Principal_Name]
	       ,[PrincipalType_Code]
	       ,[PerformanceType_Code]
	       ,[CreatedBy_User_Name]
	       ,[CreatedOn_Date]
	       ,[ModifiedBy_User_Name]
	       ,[ModifiedOn_Date]
	       ,[Active_Indicator]
        )
        VALUES
        (
	         @Release_Id
	        ,@ChangeControl_Name
	        ,@Group_Name
	        ,@Description_Text
	        ,@Principal_Name
	        ,@PrincipalType_Code
	        ,@PerformanceType_Code
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,@Active_Indicator
        )
        
        SET @ChangeControl_Id = SCOPE_IDENTITY()
        
        INSERT INTO [ChangeControlEnvironments]
		(
			[ChangeControl_Id],
			[Environment_Id]
		)
		SELECT @ChangeControl_Id,
			   CAST(CSV.[Value_Text] AS INT)
		  FROM dbo.CsvToTable(@Environments_Csv, ',') CSV
        
        EXEC [Events_RaiseEvent] 'CCTNEW', @Application_Id, @ChangeControl_Id
    
    END
    ELSE
    BEGIN
    
        UPDATE [ChangeControls]
           SET  [ChangeControl_Name] = @ChangeControl_Name
	           ,[Group_Name] = @Group_Name
	           ,[Description_Text] = @Description_Text
	           ,[Principal_Name] = @Principal_Name
	           ,[PrincipalType_Code] = @PrincipalType_Code
	           ,[PerformanceType_Code] = @PerformanceType_Code
	           ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	           ,[ModifiedOn_Date] = GETUTCDATE()
	           ,[Active_Indicator] = @Active_Indicator
	     WHERE [ChangeControl_Id] = @ChangeControl_Id
	     
	     DELETE [ChangeControlEnvironments]
		 WHERE [ChangeControl_Id] = @ChangeControl_Id
        
         INSERT INTO [ChangeControlEnvironments] ([ChangeControl_Id], [Environment_Id])
	     SELECT
			    @ChangeControl_Id,
			    CAST(CSV.[Value_Text] AS INT)
	      FROM [dbo].[CsvToTable](@Environments_Csv, ',') CSV
	     
	     EXEC [Events_RaiseEvent] 'CCTMOD', @Application_Id, @ChangeControl_Id
    
    END

COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_CreateOrUpdateChangeControl] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_DeleteChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified change control'
GO

IF OBJECT_ID('[ChangeControls_DeleteChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_DeleteChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_DeleteChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Application_Id INT
    
    SELECT @Application_Id = R.[Application_Id]
      FROM [ChangeControls] CC
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = CC.[Release_Id]
     WHERE CC.[ChangeControl_Id] = @ChangeControl_Id

    DELETE
      FROM [ChangeControlEnvironments]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE
      FROM [ChangeControlPerformances]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE 
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    EXEC [Events_RaiseEvent] 'CCTDEL', @Application_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_DeleteChangeControl] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls_Extended,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified change control.'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

    SELECT *
	  FROM [ChangeControls_Extended]
     WHERE [ChangeControl_Id] = @ChangeControl_Id

    SELECT *
	  FROM [ChangeControlEnvironments_Extended]
     WHERE [ChangeControl_Id] = @ChangeControl_Id

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControl] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControlPerformances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change control performances for a specified change control or all change controls if none is specified.'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControlPerformances]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControlPerformances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControlPerformances]
(
	@Application_Id INT,
	@ChangeControl_Id INT,
	@Release_Number RELEASENUMBER,
	@IncludeInactive_Indicator YNINDICATOR,
	@Environment_Id INT,
	@Row_Count INT
)
AS
BEGIN

SET NOCOUNT ON

    IF @Row_Count IS NOT NULL
        SET ROWCOUNT @Row_Count
    
    SELECT *
      FROM [ChangeControlPerformances_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND [ChangeControl_Id] = COALESCE(@ChangeControl_Id, [ChangeControl_Id])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
       AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControlPerformances] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControls',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls_Extended,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change controls based on the specified criteria.'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControls]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControls]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControls]
(
    @Application_Id INT,
    @Release_Number RELEASENUMBER,
    @IncludeInactive_Indicator YNINDICATOR,
    @ChangeControl_Count INT
)
AS
BEGIN

SET NOCOUNT ON

    IF @ChangeControl_Count IS NOT NULL
        SET ROWCOUNT @ChangeControl_Count

    SELECT *
	  FROM [ChangeControls_Extended]
     WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')

    SELECT CCE.*
	  FROM [ChangeControlEnvironments_Extended] CCE
	       INNER JOIN [ChangeControls] CC
		           ON CC.[ChangeControl_Id] = CCE.ChangeControl_Id
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = CC.[Release_Id]
     WHERE R.[Application_Id] = COALESCE(@Application_Id, R.[Application_Id])
       AND R.[Release_Number] = COALESCE(@Release_Number, R.[Release_Number])
       AND (CC.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControls] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_PerformChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a change control performance for a specified environment.'
GO

IF OBJECT_ID('[ChangeControls_PerformChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_PerformChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_PerformChangeControl]
(
	@ChangeControl_Id INT,
	@Environment_Id INT,
	@Notes_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON
    
BEGIN TRY
	BEGIN TRANSACTION
    
    DECLARE @Application_Id INT
    DECLARE @ChangeControlPerformance_Id INT
    
    SELECT @Application_Id = R.[Application_Id]
    FROM [ChangeControls] CC
		       INNER JOIN [Releases] R
		           ON R.[Release_Id] = CC.[Release_Id]
    WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    INSERT INTO [ChangeControlPerformances]
    (
        [ChangeControl_Id]
       ,[Environment_Id]
       ,[PerformedBy_User_Name]
       ,[Performed_Date]
       ,[Notes_Text]
    )
    VALUES
    (
        @ChangeControl_Id,
        @Environment_Id,
        dbo.Session_GetUserName(),
        GETUTCDATE(),
        @Notes_Text
    )
    
    SET @ChangeControlPerformance_Id = SCOPE_IDENTITY()
    
    EXEC [Events_RaiseEvent] 'CCTPRF', @Application_Id, @ChangeControl_Id, @ChangeControlPerformance_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_PerformChangeControl] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFile_Id',
    /* Description_Text        */ 'Creates a configuration file in the system.'
GO

IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
(
	@Application_Id INT,
	@Deployable_Id INT,
	@FilePath_Text VARCHAR(500),
	@ConfigurationFile_Name NVARCHAR(100),
	@Description_Text NVARCHAR(MAX),
	@ConfigurationFile_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [ConfigurationFiles] 
	(
		[Application_Id],
		[Deployable_Id],
		[FilePath_Text],
		[Active_Indicator],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[ConfigurationFile_Name],
		[Description_Text]
	)
	VALUES 
	(
		@Application_Id,
		@Deployable_Id, 
		@FilePath_Text, 
		'Y',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		NULLIF(@ConfigurationFile_Name, ''),
		NULLIF(@Description_Text, '')
	)

	SET @ConfigurationFile_Id = SCOPE_IDENTITY()
	
	EXEC [Events_RaiseEvent] 'CFGNEW', @Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFile] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new instance of a configuration file.',
	/* Remarks_Text            */ 'If an instance with the specified name already exists but has been deactivated, the old instance will be reactivated and a blank version will be inserted as the latest.'
GO


IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Environment_Id INT,
	@Template_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100),
	@TransformType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT *
                    FROM [ConfigurationFileInstances]
				   WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
				     AND [Instance_Name] = @Instance_Name)
	BEGIN

		INSERT INTO [ConfigurationFileInstances]
        (
			[ConfigurationFile_Id],
			[Instance_Name],
			[Environment_Id],
			[Active_Indicator],
			[Template_Indicator],
			[Template_Instance_Name],
			[TransformType_Code]
		)
	    VALUES
	    (
			@ConfigurationFile_Id,
			@Instance_Name,
			@Environment_Id,
			'Y',
			@Template_Indicator,
			@Template_Instance_Name,
			@TransformType_Code
		)

	END
	ELSE IF (SELECT [Active_Indicator]
	           FROM [ConfigurationFileInstances]
			  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
			    AND [Instance_Name] = @Instance_Name) = 'N'
	BEGIN

		UPDATE [ConfigurationFileInstances]
		   SET [Active_Indicator] = 'Y',
		       [Environment_Id] = @Environment_Id,
			   [Template_Indicator] = @Template_Indicator,
			   [Template_Instance_Name] = @Template_Instance_Name,
			   [TransformType_Code] = @TransformType_Code
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Instance_Name] = @Instance_Name

	END
	ELSE
		RAISERROR('Configuration file instance already exists.', 16, 1)

	-- If there are any versions at all
	IF EXISTS(SELECT *
	            FROM [ConfigurationFileVersions]
			   WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)
	BEGIN

		-- Find latest version with actual data
		DECLARE @Latest_NonNull_Version INT
		SET @Latest_NonNull_Version = (SELECT MAX([Version_Number])
		                                 FROM [ConfigurationFileVersions]
										WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
										  AND [Instance_Name] = @Instance_Name
										  AND [Null_Indicator] = 'N')

		-- If there isn't a latest version
		IF @Latest_NonNull_Version IS NULL
		-- or there is, but it is not empty
		OR (SELECT DATALENGTH([File_Bytes])
		      FROM [ConfigurationFileVersions]
			 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
			   AND [Instance_Name] = @Instance_Name
			   AND [Version_Number] = @Latest_NonNull_Version) <> 0
		BEGIN

			DECLARE @Rubbish INT
			SET @Rubbish = COALESCE((SELECT MAX([Version_Number])
				                       FROM [ConfigurationFileVersions]
				                      WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
				                        AND [Instance_Name] = @Instance_Name), 0) + 1

			-- Insert a blank version as the latest for this instance
			INSERT INTO [ConfigurationFileVersions]
			(
				[ConfigurationFile_Id],
				[Instance_Name],
				[Version_Number],
				[VersionNotes_Text],
				[File_Date],
				[File_Bytes],
				[CreatedBy_User_Name],
				[CreatedOn_Date]
			)
			VALUES
			(
				@ConfigurationFile_Id,
				@Instance_Name,
				@Rubbish,
				NULL,
				GETUTCDATE(),
				0x,
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)

			-- Make sure there are no gaps
			EXEC [ConfigurationFiles_FillVersionGaps] @ConfigurationFile_Id

		END

	END


	DECLARE @Application_Id INT
	 SELECT @Application_Id = [Application_Id]
	   FROM [ConfigurationFiles]
	  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGNWI', @Application_Id, @ConfigurationFile_Id, @Instance_Name
		
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileInstance] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileVersions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new configuration file versions for each instance specified in the ConfigurationFiles_Xml for the given configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileVersions]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
(
	@ConfigurationFile_Id INT,
	@ConfigurationFiles_Xml XML,
	@ReleaseNumbers_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- Do nothing if there are no instances specified
	IF COALESCE(@ConfigurationFiles_Xml.exist('/ConfigFiles/Version/@Instance_Name'), 0) = 0
		RETURN

	EXEC [ConfigurationFiles_FillVersionGaps] @ConfigurationFile_Id


	/*******************************************************************
	 * Now versions should be consistent, so figure out next version
	 ******************************************************************/
	DECLARE @Version_Number INT
	    SET @Version_Number = COALESCE((SELECT MAX([Version_Number])
		                                  FROM [ConfigurationFileVersions]
								         WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id), 0) + 1

	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ConfigFiles>
			<Version
				Instance_Name="Default"
				VersionNotes_Text="Notes..."
				File_Bytes="::base64::"
			/>
		</ConfigFiles>
	*/

	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CFI.[ConfigurationFile_Id],
	       CFI.[Instance_Name],
		   @Version_Number,
		   CFV_Input.[VersionNotes_Text],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END,
		   CFV_Input.[File_Bytes],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN dbo.Session_GetUserName() ELSE NULL END,
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END
	  FROM [ConfigurationFileInstances] CFI
	       LEFT JOIN (SELECT [Instance_Name] = CFVX.[Version].[value]('@Instance_Name', 'VARCHAR(100)'),
		                     [VersionNotes_Text] = CFVX.[Version].[value]('@VersionNotes_Text', 'VARCHAR(500)'),
							 [File_Bytes] = COALESCE(CFVX.[Version].[value]('@File_Bytes', 'VARBINARY(MAX)'), CAST('' AS VARBINARY(MAX)))
		                FROM @ConfigurationFiles_Xml.[nodes]('/ConfigFiles/Version') AS CFVX(Version)) CFV_Input
				  ON CFV_Input.[Instance_Name] = CFI.[Instance_Name]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id

	 DECLARE @Application_Id INT
	 SET @Application_Id = (SELECT [Application_Id]
	                          FROM [ConfigurationFiles] CF
							 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	 DECLARE @ReleaseNumbers TABLE ([Release_Number] RELEASENUMBER NOT NULL)
	 INSERT INTO @ReleaseNumbers
	 SELECT [Value_Text] FROM dbo.CsvToTable(@ReleaseNumbers_Csv, ',')

	 UPDATE RCF
	    SET RCF.[Version_Number] = @Version_Number
	   FROM [ReleaseConfigurationFiles] RCF
	        INNER JOIN [Releases] R
			        ON R.[Release_Id] = RCF.[Release_Id]
	  WHERE R.[Application_Id] = @Application_Id
	    AND R.[Release_Number] IN (SELECT [Release_Number] FROM @ReleaseNumbers)
		AND RCF.[ConfigurationFile_Id] = @ConfigurationFile_Id

	 INSERT INTO [ReleaseConfigurationFiles]
	 (
		[Release_Id],
		[ConfigurationFile_Id],
		[Version_Number]
	 )
	 SELECT R.[Release_Id],
			@ConfigurationFile_Id,
			@Version_Number
	   FROM @ReleaseNumbers RN
	        INNER JOIN [Releases] R
			        ON R.[Application_Id] = @Application_Id
				   AND R.[Release_Number] = RN.[Release_Number]
	  WHERE NOT EXISTS(SELECT *
	                     FROM [ReleaseConfigurationFiles] RCF
						WHERE RCF.[Release_Id] = R.[Release_Id]
						  AND RCF.[ConfigurationFile_Id] = @ConfigurationFile_Id)

	 EXEC [Events_RaiseEvent] 'CFGNWV', @Application_Id, @ConfigurationFile_Id, @Version_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileVersions] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_DeployConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFileDeployment_Id',
    /* Description_Text        */ 'Saves the result of a deployed configuration file (this will not physically deploy the configuration file).'
GO

IF OBJECT_ID('[ConfigurationFiles_DeployConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
(
	@Execution_Id INT,
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Version_Number INT,
	@Server_Id INT,
	@DeployedTo_Path VARCHAR(500),
	@DeployedBy_User_Name USERNAME,
	@ConfigurationFileDeployment_Id INT = NULL OUT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Template_Indicator YNINDICATOR
	SET @Template_Indicator = (SELECT [Template_Indicator]
	                             FROM [ConfigurationFileInstances]
								WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
								  AND [Instance_Name] = @Instance_Name)

	IF @Template_Indicator IS NULL
		RAISERROR('Configuration file does not exist.', 16, 1)

	INSERT INTO [ConfigurationFileDeployments]
	(
		[ConfigurationFile_Id],
		[Execution_Id],
		[Instance_Name],
		[Version_Number],
		[Server_Id],
		[DeployedTo_Path],
		[DeployedBy_User_Name],
		[DeployedOn_Date]
	)
	VALUES
	(
		@ConfigurationFile_Id,
		@Execution_Id,
		@Instance_Name,
		@Version_Number,
		@Server_Id,
		@DeployedTo_Path,
		COALESCE(@DeployedBy_User_Name, dbo.Session_GetUserName()),
		GETUTCDATE()
	)

	SET @ConfigurationFileDeployment_Id = SCOPE_IDENTITY()

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [ConfigurationFiles]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	EXEC [Events_RaiseEvent] 'CFGDPL', @Application_Id, @ConfigurationFileDeployment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_DeployConfigurationFile] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended,ConfigurationFileInstances_Extended,ConfigurationFileVersions_Combined,ConfigurationFileVersions_ModifiedInstances',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a configuration file and all of its instances.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@IncludeInactiveInstances_Indicator YNINDICATOR = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	
	IF (@@ROWCOUNT = 0) BEGIN
		RAISERROR('Configuration file %i does not exist.', 16, 1, @ConfigurationFile_Id)
		RETURN
	END

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
	       LEFT JOIN [Environments] E
		           ON E.[Environment_Id] = CFI.[Environment_Id]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND (CFI.[Active_Indicator] = 'Y' OR @IncludeInactiveInstances_Indicator = 'Y')
	 ORDER BY CFI.[Template_Indicator], E.[Display_Sequence], CFI.[Instance_Name]

	SELECT *
	  FROM [ConfigurationFileVersions_Combined]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	SELECT *
	  FROM [ConfigurationFileVersions_ModifiedInstances] CFVMI
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileDeployments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileDeployments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file deployments based on the specified criteria.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileDeployments]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
(
	@Application_Id INT,
    @ConfigurationFile_Id INT = NULL,
    @Deployable_Id INT = NULL,
    @Instance_Name VARCHAR(100) = NULL,
    @Server_Id INT = NULL,
	@File_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@File_Count IS NOT NULL AND @File_Count > 0)
	BEGIN
		SET ROWCOUNT @File_Count
	END

	SELECT *
	  FROM [ConfigurationFileDeployments_Extended]
     WHERE [ConfigurationFile_Id] = COALESCE(@ConfigurationFile_Id, [ConfigurationFile_Id])
	   AND [Application_Id] = COALESCE(@Application_Id, [Application_Id])
       AND (@Deployable_Id IS NULL OR ([Deployable_Id] = @Deployable_Id))
       AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])
       AND [Server_Id] = COALESCE(@Server_Id, [Server_Id])
	  ORDER BY [DeployedOn_Date] DESC

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileDeployments] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileInstances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileInstances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file instances that match the specified criteria.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileInstances]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
		   LEFT JOIN [Environments] E
				 ON E.[Environment_Id] = CFI.[Environment_Id]
	 WHERE (@Application_Id IS NULL OR CFI.[Application_Id] = @Application_Id)
	   AND (@Deployable_Id IS NULL OR CFI.[Deployable_Id] = @Deployable_Id)
	   AND (@Active_Indicator IS NULL OR CFI.[Active_Indicator] = @Active_Indicator)
	ORDER BY E.[Display_Sequence], CFI.[Instance_Name]
END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileInstances] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFiles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended,ConfigurationFileInstances_Extended,ConfigurationFileVersions_Combined,ConfigurationFileVersions_ModifiedInstances',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all configuration files that match the specified criteria.'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFiles]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@IncludeInstances_Indicator YNINDICATOR = NULL,
	@IncludeVersions_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [Application_Id] = ISNULL(@Application_Id, [Application_Id])
	   AND (@Deployable_Id IS NULL OR [Deployable_Id] = @Deployable_Id)
	   AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	 ORDER BY [Deployable_Name], [Active_Indicator] DESC, [FilePath_Text]

	SELECT *
	  FROM [ConfigurationFileInstances_Extended]
	 WHERE @IncludeInstances_Indicator = 'Y'
	   AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND [Application_Id] = ISNULL(@Application_Id, [Application_Id])
	   AND (@Deployable_Id IS NULL OR [Deployable_Id] = @Deployable_Id)

	SELECT CFV.*
	  FROM [ConfigurationFileVersions_Combined] CFV
	       INNER JOIN [ConfigurationFiles] CF
		           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
	 WHERE @IncludeVersions_Indicator = 'Y'
	   AND (CF.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND (@Deployable_Id IS NULL OR CF.[Deployable_Id] = @Deployable_Id)
	   AND CF.[Application_Id] = ISNULL(@Application_Id, CF.[Application_Id])

	SELECT CFVMI.*
	  FROM [ConfigurationFileVersions_ModifiedInstances] CFVMI
	       INNER JOIN [ConfigurationFiles] CF
		           ON CF.[ConfigurationFile_Id] = CFVMI.[ConfigurationFile_Id]
	 WHERE @IncludeVersions_Indicator = 'Y'
	   AND (CF.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND (@Deployable_Id IS NULL OR CF.[Deployable_Id] = @Deployable_Id)
	   AND CF.[Application_Id] = ISNULL(@Application_Id, CF.[Application_Id])

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFiles] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileVersions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of configuration file versions for a specified configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersion]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100) = NULL,
	@Version_Number INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Version_Number IS NULL
	BEGIN

		SELECT @Version_Number = MAX([Version_Number])
		  FROM [ConfigurationFileVersions_Extended]
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])

	END	

	SELECT *
	  FROM [ConfigurationFileVersions_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])
	   AND [Version_Number] = @Version_Number

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileVersion] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_PurgeConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a configuration file, including all previous versions of all instances and deployment history.'
GO

IF OBJECT_ID('[ConfigurationFiles_PurgeConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_PurgeConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_PurgeConfigurationFile]
(
	@ConfigurationFile_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT
	DECLARE @ConfigurationFile_Name NVARCHAR(500)
	SELECT @Application_Id = [Application_Id],
	       @ConfigurationFile_Name = COALESCE([ConfigurationFile_Name], [FilePath_Text])
	  FROM [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ReleaseConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileInstances]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGDEL', @Application_Id, @ConfigurationFile_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_PurgeConfigurationFile] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_PurgeConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a configuration file instance, including all previous versions and deployment history.'
GO

IF OBJECT_ID('[ConfigurationFiles_PurgeConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_PurgeConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_PurgeConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@UnlinkTemplate_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT
	SELECT @Application_Id = [Application_Id]
	  FROM [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	IF @UnlinkTemplate_Indicator = 'Y'
	BEGIN
		UPDATE [ConfigurationFileInstances]
		   SET [Template_Instance_Name] = NULL
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Template_Instance_Name] = @Instance_Name
	END

	DELETE [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

	DELETE [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

    DELETE [ConfigurationFileInstances]
     WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
       AND [Instance_Name] = @Instance_Name

	EXEC [Events_RaiseEvent] 'CFGDLI', @Application_Id, @ConfigurationFile_Id, @Instance_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_PurgeConfigurationFileInstance] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates metadata for an existing configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@Active_Indicator YNINDICATOR,
	@ConfigurationFile_Name NVARCHAR(100),
	@FilePath_Text VARCHAR(500),
	@Description_Text NVARCHAR(MAX),
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [ConfigurationFiles]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	DECLARE @Other_Application_Id INT
	SET @Other_Application_Id = (SELECT [Application_Id] 
	                               FROM [Deployables]
 	                              WHERE [Deployable_Id] = @Deployable_Id)

	IF @Other_Application_Id IS NOT NULL AND @Other_Application_Id <> @Application_Id
		RAISERROR('Cannot change configuration file deployable to a deployable of a different application.', 16, 1)

	UPDATE [ConfigurationFiles]
	   SET [Active_Indicator] = @Active_Indicator,
	       [ConfigurationFile_Name] = NULLIF(@ConfigurationFile_Name, ''),
		   [FilePath_Text] = @FilePath_Text,
		   [Description_Text] = NULLIF(@Description_Text, ''),
		   [Deployable_Id] = @Deployable_Id
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGMOD', @Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFile] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates metadata for an existing configuration file instance.'
GO

IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
    @Environment_Id INT,
	@Active_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100),
	@TransformType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Template_Instance_Name IS NOT NULL
	BEGIN
		IF (SELECT [Template_Indicator] FROM [ConfigurationFileInstances] WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id AND [Instance_Name] = @Template_Instance_Name) = 'N'
			RAISERROR('Specified template instance name must actually refer to a template instance.', 16, 1)
	END

	UPDATE [ConfigurationFileInstances]
	   SET [Environment_Id] = @Environment_Id,
	       [Active_Indicator] = @Active_Indicator,
		   [Template_Instance_Name] = @Template_Instance_Name,
		   [TransformType_Code] = @TransformType_Code
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [ConfigurationFiles]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	EXEC [Events_RaiseEvent] 'CFGMDI', @Application_Id, @ConfigurationFile_Id, @Instance_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFileInstance] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Configuration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of BuildMaster configuration settings.'
GO

IF OBJECT_ID('[Configuration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [Configuration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Configuration_GetConfiguration] AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Configuration]
	 WHERE [Scope_Name] <> 'Internal'
	 ORDER BY [Scope_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [Configuration_GetConfiguration] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetInternalValue',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster internal configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetInternalValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetInternalValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetInternalValue]
(
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = 'Internal'
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetInternalValue] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = @Scope_Name
	   AND [Key_Name] = @Key_Name
	   AND [Scope_Name] <> 'Internal'

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetValue] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Scope_Name <> 'Internal'
	BEGIN

		IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
			UPDATE [Configuration]
	           SET [Value_Text] = @Value_Text
	         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

	    END ELSE BEGIN

			INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
			VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
	    END

	EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValue] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValueWithoutEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValueWithoutEvent]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
		UPDATE [Configuration]
           SET [Value_Text] = @Value_Text
         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

    END ELSE BEGIN

		INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
		VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValueWithoutEvent] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Credentials_CreateOrUpdateCredential',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Credential_Id',
    /* Description_Text        */ 'Creates a new credential or updates an existing one.'
GO

IF OBJECT_ID('[Credentials_CreateOrUpdateCredential]') IS NOT NULL DROP PROCEDURE [Credentials_CreateOrUpdateCredential]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Credentials_CreateOrUpdateCredential]
(
	@Credential_Name NVARCHAR(50),
    @Environment_Id INT,
    @CredentialType_Name VARCHAR(50),
    @Configuration_Xml XML,
    @AllowFunctionAccess_Indicator YNINDICATOR,
    @Credential_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Credential_Id IS NULL)
	BEGIN
		INSERT INTO [Credentials] 
		(
            [Environment_Id],
            [Credential_Name],
            [CredentialType_Name],
            [Configuration_Xml],
            [AllowFunctionAccess_Indicator]
		)
		VALUES
		(
			@Environment_Id,
            @Credential_Name,
            @CredentialType_Name,
            @Configuration_Xml,
            COALESCE(@AllowFunctionAccess_Indicator, 'N')
		)
		SET @Credential_Id = SCOPE_IDENTITY()

		-- EXEC [Events_RaiseEvent] 'ENVNWS', NULL, @Server_Id
	END
	ELSE
	BEGIN

		UPDATE [Credentials]
		   SET [Environment_Id] = @Environment_Id,
			   [Credential_Name] = @Credential_Name,
               [CredentialType_Name] = @CredentialType_Name,
               [Configuration_Xml] = @Configuration_Xml,
               [AllowFunctionAccess_Indicator] = COALESCE(@AllowFunctionAccess_Indicator, [AllowFunctionAccess_Indicator])
         WHERE [Credential_Id] = @Credential_Id

		--EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @Server_Id
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Credentials_CreateOrUpdateCredential] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Credentials_DeleteCredential',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Credentials_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified credential.'
GO


IF OBJECT_ID('[Credentials_DeleteCredential]') IS NOT NULL DROP PROCEDURE [Credentials_DeleteCredential]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Credentials_DeleteCredential]
(
    @Credential_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	DELETE [Credentials]
     WHERE [Credential_Id] = @Credential_Id

END
GO

GRANT EXECUTE ON [Credentials_DeleteCredential] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Credentials_GetCredential',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Credentials_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified credential.'
GO


IF OBJECT_ID('[Credentials_GetCredential]') IS NOT NULL DROP PROCEDURE [Credentials_GetCredential]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Credentials_GetCredential]
(
    @Credential_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Credentials_Extended]
     WHERE [Credential_Id] = @Credential_Id

END
GO

GRANT EXECUTE ON [Credentials_GetCredential] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Credentials_GetCredentialByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Credentials_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified credential by its qualified name.'
GO


IF OBJECT_ID('[Credentials_GetCredentialByName]') IS NOT NULL DROP PROCEDURE [Credentials_GetCredentialByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Credentials_GetCredentialByName]
(
    @CredentialType_Name NVARCHAR(50),
    @Credential_Name NVARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	IF @CredentialType_Name IS NOT NULL
    BEGIN

        SELECT * FROM [Credentials_Extended]
         WHERE [CredentialType_Name] = @CredentialType_Name
           AND [Credential_Name] = @Credential_Name

    END
    ELSE
    BEGIN
        IF (SELECT COUNT([Credential_Id])
                   FROM [Credentials]
                  WHERE [Credential_Name] = @Credential_Name) > 1
        BEGIN
            RAISERROR('The specified credential is not unique solely by its name, please specify a type name in addition to the credential name.', 16, 1)
        END
        ELSE
        BEGIN

            SELECT * 
              FROM [Credentials_Extended]
             WHERE [Credential_Name] = @Credential_Name
            
        END
    END
END
GO

GRANT EXECUTE ON [Credentials_GetCredentialByName] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Credentials_GetCredentials',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Credentials_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all credentials in the system.'
GO


IF OBJECT_ID('[Credentials_GetCredentials]') IS NOT NULL DROP PROCEDURE [Credentials_GetCredentials]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Credentials_GetCredentials]
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Credentials_Extended]

END
GO

GRANT EXECUTE ON [Credentials_GetCredentials] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetApplications',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Applications_LatestDeployedReleases',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application summary for all applications in BuildMaster or a specific group of applications.'
GO


IF OBJECT_ID('[Dashboards_GetApplications]') IS NOT NULL DROP PROCEDURE [Dashboards_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetApplications]
(
	@Application_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL
)
AS
BEGIN

	SELECT AE.*
	  FROM [Applications_Extended] AE

	 WHERE ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
       AND (@ApplicationGroup_Id IS NULL OR EXISTS(SELECT * 
                                                     FROM [ApplicationGroups] AG
                                              CROSS APPLY dbo.ApplicationGroups_GetAncestorGroups(AG.[ApplicationGroup_Id]) AGA
                                                    WHERE AGA.[ApplicationGroup_Id] = @ApplicationGroup_Id
                                                      AND AG.[ApplicationGroup_Id] = AE.[ApplicationGroup_Id])
            )
	 ORDER BY   
	          CASE WHEN AE.[ApplicationGroup_Id] IS NULL THEN 1 ELSE 0 END
	         ,[ApplicationGroup_Name]
	         ,[Application_Name]

	SELECT *
	  FROM [Applications_LatestDeployedReleases]
	 WHERE ([ApplicationGroup_Id] = @ApplicationGroup_Id OR @ApplicationGroup_Id IS NULL)
	   AND ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
	 ORDER BY [Application_Id]

END
GO

GRANT EXECUTE ON [Dashboards_GetApplications] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_CreateExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Execution_Id',
    /* Description_Text        */ 'Creates a database change script execution.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_CreateExecution]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_CreateExecution]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_CreateExecution]
(
	@DatabaseConnection_Id INT,
	@ScriptIds_Csv VARCHAR(MAX),
	@Execution_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [Executions]
	(
		[Start_Date],
		[ExecutionStatus_Code],
		[ExecutionRunState_Code],
		[ExecutionMode_Code],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	VALUES
	(
		GETUTCDATE(),
		'S',
		'P',
		'D',
		dbo.Session_GetUserName(),
		GETUTCDATE()
	)

	SET @Execution_Id = SCOPE_IDENTITY()

	INSERT INTO [DatabaseChangeScriptExecutions]
	(
		[Execution_Id],
		[DatabaseConnection_Id],
		[ScriptIds_Csv]
	)
	VALUES
	(
		@Execution_Id,
		@DatabaseConnection_Id,
		@ScriptIds_Csv
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_CreateExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_CreateOrUpdateChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates or edits a database change script.'
GO

IF OBJECT_ID('[DatabaseChangeScripts_CreateOrUpdateChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_CreateOrUpdateChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_CreateOrUpdateChangeScript]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Script_Name VARCHAR(250),
	@Script_Text NVARCHAR(MAX),
	@Active_Indicator YNINDICATOR,
	@Script_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	IF @Script_Id IS NULL
	BEGIN
		INSERT INTO [DatabaseChangeScripts]
		(
			[Release_Id],
			[Deployable_Id],
			[Script_Name],
			[Script_Text],
			[Active_Indicator],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Script_Guid]
		)
		VALUES
		(
			@Release_Id,
			@Deployable_Id,
			@Script_Name,
			@Script_Text,
			@Active_Indicator,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			NEWID()
		)

		SET @Script_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'DBSNWS', @Application_Id, @Script_Id
	END
	ELSE
	BEGIN
		UPDATE [DatabaseChangeScripts]
		   SET [Release_Id] = @Release_Id,
		       [Deployable_Id] = @Deployable_Id,
			   [Script_Name] = @Script_Name,
			   [Script_Text] = @Script_Text,
			   [Active_Indicator] = @Active_Indicator,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

		EXEC [Events_RaiseEvent] 'DBSMDS', @Application_Id, @Script_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_CreateOrUpdateChangeScript] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_DeleteChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database change script.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_DeleteChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Script_Name SQL_VARIANT
	    SET @Script_Name    = (SELECT [Script_Name] 
	                             FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)

	DELETE [DatabaseChangeScriptLegacyExecutions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [DatabaseChangeScripts]
	 WHERE [Script_Id] = @Script_Id	

	--IF @@ROWCOUNT > 0
	--EXEC [Events_RaiseEvent] 'DBSDLS', @Application_Id, @Script_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_DeleteChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database change script.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScript]
(
	@Script_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Script_Id] = @Script_Id

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database change scripts that match the specified criteria.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScripts]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@DatabaseChangeScript_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	IF ISNULL(@DatabaseChangeScript_Count, 0) > 0
		SET ROWCOUNT @DatabaseChangeScript_Count

	SELECT * 
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND (	[Active_Indicator] = 'Y'
			 OR @IncludeInactive_Indicator = 'Y' )
  ORDER BY [Numeric_Release_Number] DESC

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScripts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a database change script execution.'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetExecution]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseChangeScriptExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseConnections_CreateOrUpdateConnection',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'DatabaseConnection_Id',
    /* Description_Text        */ 'Creates a new database connection or updates an existing one.'
GO


IF OBJECT_ID('[DatabaseConnections_CreateOrUpdateConnection]') IS NOT NULL DROP PROCEDURE [DatabaseConnections_CreateOrUpdateConnection]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseConnections_CreateOrUpdateConnection]
(
	@DatabaseConnection_Name NVARCHAR(50),
	@DatabaseConnection_Description NVARCHAR(MAX),
	@DatabaseConnection_Configuration XML,
	@ExecuteOn_Server_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@DatabaseConnection_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @DatabaseConnection_Id IS NULL
	BEGIN
		
		INSERT INTO [DatabaseConnections]
		(
			[DatabaseConnection_Name],
			[DatabaseConnection_Description],
			[DatabaseConnection_Configuration],
			[ExecuteOn_Server_Id],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Application_Id],
			[Environment_Id]
		)
		VALUES
		(
			NULLIF(@DatabaseConnection_Name, ''),
			NULLIF(@DatabaseConnection_Description, ''),
			@DatabaseConnection_Configuration,
			@ExecuteOn_Server_Id,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Application_Id,
			@Environment_Id
		)

		SET @DatabaseConnection_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [DatabaseConnections]
		   SET [DatabaseConnection_Name] = NULLIF(@DatabaseConnection_Name, ''),
		       [DatabaseConnection_Description] = NULLIF(@DatabaseConnection_Description, ''),
			   [DatabaseConnection_Configuration] = @DatabaseConnection_Configuration,
			   [ExecuteOn_Server_Id] = @ExecuteOn_Server_Id,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Environment_Id] = @Environment_Id
	        WHERE [DatabaseConnection_Id] = @DatabaseConnection_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseConnections_CreateOrUpdateConnection] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseConnections_DeleteConnection',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database connection.'
GO


IF OBJECT_ID('[DatabaseConnections_DeleteConnection]') IS NOT NULL DROP PROCEDURE [DatabaseConnections_DeleteConnection]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseConnections_DeleteConnection]
(
	@DatabaseConnection_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [DatabaseConnections]
	 WHERE [DatabaseConnection_Id] = @DatabaseConnection_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseConnections_DeleteConnection] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseConnections_GetConnection',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseConnections_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a database connection.'
GO


IF OBJECT_ID('[DatabaseConnections_GetConnection]') IS NOT NULL DROP PROCEDURE [DatabaseConnections_GetConnection]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseConnections_GetConnection]
(
	@DatabaseConnection_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseConnections_Extended]
	 WHERE [DatabaseConnection_Id] = @DatabaseConnection_Id

END
GO

GRANT EXECUTE ON [DatabaseConnections_GetConnection] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseConnections_GetConnections',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseConnections_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets database connections, optionally filtered by application and environment.'
GO


IF OBJECT_ID('[DatabaseConnections_GetConnections]') IS NOT NULL DROP PROCEDURE [DatabaseConnections_GetConnections]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseConnections_GetConnections]
(
	@Application_Id INT = NULL,
	@Environment_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseConnections_Extended]
	 WHERE (@Application_Id IS NULL OR ([Application_Id] = @Application_Id))
	   AND (@Environment_Id IS NULL OR ([Environment_Id] = @Environment_Id))

END
GO

GRANT EXECUTE ON [DatabaseConnections_GetConnections] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_AddOrRemoveServer]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServer]
GO


GO
IF OBJECT_ID('[Environments_AddOrRemoveServerFromGroup]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServerFromGroup]
GO


GO
IF OBJECT_ID('[Environments_CreateOrUpdateAutomaticRequirement]') IS NOT NULL 
	DROP PROCEDURE [Environments_CreateOrUpdateAutomaticRequirement]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Environment_Id',
    /* Description_Text        */ 'Creates a new environment or updates an existing one'
GO


IF OBJECT_ID('[Environments_CreateOrUpdateEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateEnvironment]
(
	@Environment_Name VARCHAR(50) = NULL,
    @ServerIds_Csv VARCHAR(MAX) = NULL,
    @Parent_Environment_Id INT = NULL,
    @Display_Sequence INT = NULL,  
    @Environment_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Environment_Id IS NULL) BEGIN
		 SELECT @Display_Sequence = COALESCE(MAX([Display_Sequence] ),0)+1
		   FROM [Environments]
		BEGIN
			INSERT INTO [Environments] 
			(
				[Environment_Name],
				[Display_Sequence],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
                [Parent_Environment_Id]
			)
			VALUES 
			(
				@Environment_Name,
				@Display_Sequence,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
                @Parent_Environment_Id
			)
			SET @Environment_Id = SCOPE_IDENTITY()

			EXEC [Events_RaiseEvent] 'ENVNEW', NULL, @Environment_Id
		END

	END ELSE
		BEGIN
			UPDATE [Environments]
			   SET [Environment_Name] = COALESCE(@Environment_Name, [Environment_Name]),
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE(),
                   [Display_Sequence] = COALESCE(@Display_Sequence, [Display_Sequence]),
                   [Parent_Environment_Id] = @Parent_Environment_Id
			 WHERE [Environment_Id] = @Environment_Id

			EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id
		END


    IF @ServerIds_Csv IS NOT NULL
    BEGIN
     DELETE [EnvironmentServers] WHERE [Environment_Id] = @Environment_Id

        INSERT [EnvironmentServers] ([Environment_Id], [Server_Id])
        SELECT @Environment_Id, [Value_Text] 
          FROM [dbo].[CsvToTable] (@ServerIds_Csv, ',')
    END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateEnvironment] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_CreateOrUpdateManualRequirement]') IS NOT NULL 
	DROP PROCEDURE [Environments_CreateOrUpdateManualRequirement]
GO


GO
IF OBJECT_ID('[Environments_CreateOrUpdateServer]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateServer]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_DeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Soft deletes an environment'
GO

IF OBJECT_ID('[Environments_DeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_DeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_DeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF EXISTS(SELECT * FROM WorkflowSteps WHERE [Environment_Id] = @Environment_Id)
	BEGIN
		RAISERROR('Environment %i is currently associated with a workflow and cannot be removed.', 16, 1, @Environment_Id)
	END

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'N'
	 WHERE [Environment_Id] = @Environment_Id
	 
	EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_DeleteEnvironment] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_DeleteServer]') IS NOT NULL DROP PROCEDURE [Environments_DeleteServer]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments_Extended,Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an environment.'
GO


IF OBJECT_ID('[Environments_GetEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT OFF 

	SELECT *
	  FROM [Environments_Extended]
     WHERE [Environment_Id] = @Environment_Id

	SELECT *
	  FROM [Servers]
	 WHERE [Server_Id] IN (SELECT [Server_Id] FROM [EnvironmentServers]
	                        WHERE [Environment_Id] = @Environment_Id)

END
GO

GRANT EXECUTE ON [Environments_GetEnvironment] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Environments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all environments.'
GO

IF OBJECT_ID('[Environments_GetEnvironments]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironments]
(
    @IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SELECT E.*
	  FROM [Environments_Extended] E
	 WHERE @IncludeInactive_Indicator = 'Y' OR E.[Active_Indicator] = 'Y'
	 ORDER BY [Display_Sequence]

END
GO

GRANT EXECUTE ON [Environments_GetEnvironments] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironmentsAndServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments_Extended,EnvironmentServers_Extended,Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a dataset containing all environments, all environment/server relations, and servers'
GO


IF OBJECT_ID('[Environments_GetEnvironmentsAndServers]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironmentsAndServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironmentsAndServers]
(
	@IncludeInactive_Environments_Indicator YNINDICATOR,
	@IncludeInactive_Servers_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Environments_Extended] 
	 WHERE [Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y'
	 ORDER BY [Display_Sequence]

	SELECT * FROM [EnvironmentServers_Extended] 
	 WHERE [Environment_Id] IS NULL OR
	       (([Environment_Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y') AND
	       ([Server_Active_Indicator] = 'Y' OR @IncludeInactive_Servers_Indicator = 'Y'))
	 ORDER BY [Server_Name]

	SELECT *
	  FROM [Servers]
     WHERE @IncludeInactive_Servers_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name]

END
GO

GRANT EXECUTE ON [Environments_GetEnvironmentsAndServers] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_GetServer]') IS NOT NULL DROP PROCEDURE [Environments_GetServer]
GO


GO
IF OBJECT_ID('[Environments_GetServers]') IS NOT NULL DROP PROCEDURE [Environments_GetServers]
GO


GO
IF OBJECT_ID('[Environments_GetServersInGroup]') IS NOT NULL DROP PROCEDURE [Environments_GetServersInGroup]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_ResequenceEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified environment by placing it where another environment is sequenced'
GO


IF OBJECT_ID('[Environments_ResequenceEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_ResequenceEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_ResequenceEnvironment]
(
	@Environment_Id INT,
	@New_Environment_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Environment_Sequence INT
	SELECT @Environment_Sequence = [Display_Sequence]
	  FROM [Environments]
	 WHERE [Environment_Id] = @Environment_Id

	IF @Environment_Sequence IS NULL BEGIN
	   RAISERROR('Environment (ID=%i) was not found.', 16, 1, @Environment_Id)
	   RETURN
    END

	UPDATE [Environments] 
	   SET [Display_Sequence] = CASE
	       WHEN /* id */ [Environment_Id] = @Environment_Id THEN @New_Environment_Sequence

		   WHEN /* up */ @New_Environment_Sequence > @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @Environment_Sequence AND @New_Environment_Sequence THEN [Display_Sequence] - 1
				ELSE [Display_Sequence] END
		   WHEN /* dn */ @New_Environment_Sequence < @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @New_Environment_Sequence AND @Environment_Sequence THEN [Display_Sequence] + 1
				ELSE [Display_Sequence] END
		   ELSE @New_Environment_Sequence END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_ResequenceEnvironment] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_SetRestrictedServer]') IS NOT NULL DROP PROCEDURE [Environments_SetRestrictedServer]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UndeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Undeletes the specified environment'
GO


IF OBJECT_ID('[Environments_UndeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_UndeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UndeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'Y'
	 WHERE [Environment_Id] = @Environment_Id

    EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_UndeleteEnvironment] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Environments_UpdateEnvironmentServers]') IS NOT NULL DROP PROCEDURE [Environments_UpdateEnvironmentServers]
GO


GO
IF OBJECT_ID('[Environments_UpdateServerGroupServers]') IS NOT NULL DROP PROCEDURE [Environments_UpdateServerGroupServers]
GO


GO
IF OBJECT_ID('[Environments_UpdateServerStatus]') IS NOT NULL DROP PROCEDURE [Environments_UpdateServerStatus]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_CreateOrUpdateEventListener',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'EventListener_Id',
    /* Description_Text        */ 'Creates or updates an event listener.'
GO

IF OBJECT_ID('[EventListeners_CreateOrUpdateEventListeners]') IS NOT NULL DROP PROCEDURE [EventListeners_CreateOrUpdateEventListeners]
GO
IF OBJECT_ID('[EventListeners_CreateOrUpdateEventListener]') IS NOT NULL DROP PROCEDURE [EventListeners_CreateOrUpdateEventListener]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_CreateOrUpdateEventListener]
(
	@EventListener_Description NVARCHAR(MAX),
	@EventListener_Configuration XML,
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,

	@EventCodeList_Csv VARCHAR(MAX),
	@ApplicationIdList_Csv VARCHAR(MAX),
	@ApplicationGroupIdList_Csv VARCHAR(MAX),
	@PipelineStageNameList_Csv VARCHAR(MAX),

    @PipelineStageName_Delimeter VARCHAR(10) = NULL,

	@EventListener_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @EventListener_Id IS NULL BEGIN

		INSERT INTO [EventListeners]
		(
			[EventListener_Description],
			[EventListener_Configuration],
			[Owner_User_Name],
			[Active_Indicator]
		)
		VALUES
		(
			@EventListener_Description,
			@EventListener_Configuration,
			@Owner_User_Name,
			@Active_Indicator
		)
		
		SET @EventListener_Id = SCOPE_IDENTITY()

	END ELSE BEGIN

		UPDATE [EventListeners]
		   SET [EventListener_Description] = @EventListener_Description,
		       [EventListener_Configuration] = @EventListener_Configuration,
		       [Owner_User_Name] = @Owner_User_Name,
		       [Active_Indicator] = @Active_Indicator
		 WHERE [EventListener_Id] = @EventListener_Id

	END

	DELETE [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerStages] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	INSERT INTO [EventListenerApplications] ([EventListener_Id],[Application_Id])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@ApplicationIdList_Csv,',') V

	INSERT INTO [EventListenerApplicationGroups] ([EventListener_Id],[ApplicationGroup_Id])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@ApplicationGroupIdList_Csv,',') V

	INSERT INTO [EventListenerStages] ([EventListener_Id],[PipelineStage_Name])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@PipelineStageNameList_Csv,COALESCE(@PipelineStageName_Delimeter, ',')) V

	INSERT INTO [EventListenerEvents] ([EventListener_Id],[Event_Code])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@EventCodeList_Csv, ',') V


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [EventListeners_CreateOrUpdateEventListener] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_DeleteEventListener',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an event listener.'
GO

IF OBJECT_ID('[EventListeners_DeleteEventListener]') IS NOT NULL DROP PROCEDURE [EventListeners_DeleteEventListener]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_DeleteEventListener]
(
	@EventListener_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	DELETE [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	DELETE [EventListeners] WHERE [EventListener_Id] = @EventListener_Id
	

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [EventListeners_DeleteEventListener] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_GetEventListener',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventListeners,EventListenerApplications,EventListenerApplicationGroups,EventListenerStages,EventListenerEvents',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a EventListener and a list of event codes that the EventListener is subscribed to.'
GO

IF OBJECT_ID('[EventListeners_GetEventListener]') IS NOT NULL DROP PROCEDURE [EventListeners_GetEventListener]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_GetEventListener]
(
	@EventListener_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [EventListeners] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerStages] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

END
GO

GRANT EXECUTE ON [EventListeners_GetEventListener] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_GetEventListeners',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventListeners,EventListenerApplications,EventListenerApplicationGroups,EventListenerStages,EventListenerEvents',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of event listeners for the specified user, or all users if no user is specified.'
GO

IF OBJECT_ID('[EventListeners_GetEventListeners]') IS NOT NULL DROP PROCEDURE [EventListeners_GetEventListeners]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_GetEventListeners]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	DECLARE @IDS TABLE ([Id] INT)
	
	INSERT INTO @IDS
	SELECT [EventListener_Id] FROM [EventListeners] 
	 WHERE ([Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name]) OR (@Owner_User_Name IS NULL AND [Owner_User_Name] IS NULL))
	   AND (COALESCE([WorkflowStep_Workflow_Id], [WorkflowStep_Environment_Id], [BuildStep_Workflow_Id]) IS NULL)

	SELECT * FROM [EventListeners] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerApplications] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerApplicationGroups] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerStages] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerEvents] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)

END
GO

GRANT EXECUTE ON [EventListeners_GetEventListeners] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Events_GetEventListeners]') IS NOT NULL DROP PROCEDURE [Events_GetEventListeners]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurrence',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Get a specific event occurrence along with its event details.'
GO

IF OBJECT_ID('[Events_GetEventOccurence]') IS NOT NULL DROP PROCEDURE [Events_GetEventOccurence]
GO
IF OBJECT_ID('[Events_GetEventOccurrence]') IS NOT NULL DROP PROCEDURE [Events_GetEventOccurrence]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurrence]
(
	@EventOccurence_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id

	SELECT *
	  FROM [EventOccurenceDetails]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id

END
GO

GRANT EXECUTE ON [Events_GetEventOccurrence] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurrences',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the event occurrences in the system for a specific application, or all applications, limited to a specified count if provided.'
GO

IF OBJECT_ID('[Events_GetEventOccurences]') IS NOT NULL DROP PROCEDURE [Events_GetEventOccurences]
GO
IF OBJECT_ID('[Events_GetEventOccurrences]') IS NOT NULL DROP PROCEDURE [Events_GetEventOccurrences]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurrences]
(
	@Application_Id INT = NULL,
	@Event_Count INT = NULL,
	@After_EventOccurence_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Event_Count IS NOT NULL
		SET ROWCOUNT @Event_Count

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@After_EventOccurence_Id IS NULL OR [EventOccurence_Id] > @After_EventOccurence_Id)
	 ORDER BY [EventOccurence_Id] DESC

END
GO

GRANT EXECUTE ON [Events_GetEventOccurrences] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Events_GetNotifiers]') IS NOT NULL DROP PROCEDURE [Events_GetNotifiers]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_AddLogEntry',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a log entry.'
GO

IF OBJECT_ID('[Executions_AddLogEntry]') IS NOT NULL DROP PROCEDURE [Executions_AddLogEntry]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_AddLogEntry]
(
	@Execution_Id INT,
	@Scope_Sequence INT,
	@LogEntry_Level INT,
	@LogEntry_Text VARBINARY(MAX),
	@LogEntry_Date DATETIME
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @LogEntry_Sequence INT
	SET @LogEntry_Sequence = COALESCE((SELECT MAX([LogEntry_Sequence])
	                                     FROM [ScopedExecutionLogEntries]
									    WHERE [Execution_Id] = @Execution_Id), 0) + 1

	INSERT INTO [ScopedExecutionLogEntries]
	(
		[Execution_Id],
		[Scope_Sequence],
		[LogEntry_Sequence],
		[LogEntry_Level],
		[LogEntry_Text],
		[LogEntry_Date]
	)
	VALUES
	(
		@Execution_Id,
		@Scope_Sequence,
		@LogEntry_Sequence,
		@LogEntry_Level,
		@LogEntry_Text,
		@LogEntry_Date
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Executions_AddLogEntry] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_BeginExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the state of an execution from Pending to Executing.'
GO


IF OBJECT_ID('[Executions_BeginExecution]') IS NOT NULL DROP PROCEDURE [Executions_BeginExecution]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_BeginExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Executions]
	   SET [ExecutionRunState_Code] = 'X',
	       [Start_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Executions_BeginExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CancelExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[dbo].[Executions_CancelExecution]') IS NOT NULL
	DROP PROCEDURE [dbo].[Executions_CancelExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CancelExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ExecutionStatus_Name VARCHAR(10)
	SET @ExecutionStatus_Name =
		(SELECT [ExecutionRunState_Code]
		   FROM [Executions]
		  WHERE [Execution_Id] = @Execution_Id)

	IF @ExecutionStatus_Name NOT IN ('P', 'X')
	BEGIN
		RAISERROR('This execution is not currently running; only executions in progress may be canceled.', 16, 1)
		RETURN
	END

	/* Pending executions immediately get failed; ones that are currently
	   executing will be canceled and picked up by the Plan Executer */
	UPDATE [Executions]
	   SET [CanceledBy_User_Name] = dbo.Session_GetUserName()
		  ,[CanceledOn_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CancelExecution] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteLogScope',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Completes a log scope.'
GO

IF OBJECT_ID('[Executions_CompleteLogScope]') IS NOT NULL DROP PROCEDURE [Executions_CompleteLogScope]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteLogScope]
(
	@Execution_Id INT,
	@Scope_Sequence INT,
	@End_Date DATETIME
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @HighestLogEntry_Level INT
	SET @HighestLogEntry_Level = (SELECT MAX([LogEntry_Level])
		                            FROM [ScopedExecutionLogEntries]
								   WHERE [Execution_Id] = @Execution_Id
								     AND [Scope_Sequence] = @Scope_Sequence)

	UPDATE [ScopedExecutionLogs]
	   SET [End_Date] = @End_Date,
	       [HighestLogEntry_Level] = @HighestLogEntry_Level
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Scope_Sequence] = @Scope_Sequence

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Executions_CompleteLogScope] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CreateLogScope',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Scope_Sequence',
    /* Description_Text        */ 'Creates a log scope.'
GO

IF OBJECT_ID('[Executions_CreateLogScope]') IS NOT NULL DROP PROCEDURE [Executions_CreateLogScope]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CreateLogScope]
(
	@Execution_Id INT,
	@Scope_Name VARBINARY(MAX),
	@Parent_Scope_Sequence INT,
	@Start_Date DATETIME,
	@Scope_Sequence INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Scope_Sequence = COALESCE((SELECT MAX([Scope_Sequence])
	                                  FROM [ScopedExecutionLogs]
									 WHERE [Execution_Id] = @Execution_Id), 0) + 1

	INSERT INTO [ScopedExecutionLogs]
	(
		[Execution_Id],
		[Scope_Sequence],
		[Scope_Name],
		[Parent_Scope_Sequence],
		[Start_Date]
	)
	VALUES
	(
		@Execution_Id,
		@Scope_Sequence,
		COALESCE(@Scope_Name, 0x),
		@Parent_Scope_Sequence,
		@Start_Date
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Executions_CreateLogScope] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CreateManualExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Execution_Id',
    /* Description_Text        */ 'Creates a manual execution.'
GO

IF OBJECT_ID('[Executions_CreateManualExecution]') IS NOT NULL DROP PROCEDURE [Executions_CreateManualExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CreateManualExecution]
(
	@ExecutionType_Name VARCHAR(50),
	@Execution_Configuration XML,
	@Execution_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [Executions]
	(
		[Start_Date],
		[ExecutionStatus_Code],
		[ExecutionRunState_Code],
		[ExecutionMode_Code],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	VALUES
	(
		GETUTCDATE(),
		'S',
		'P',
		'M',
		dbo.Session_GetUserName(),
		GETUTCDATE()
	)

	SET @Execution_Id = SCOPE_IDENTITY()

	INSERT INTO [ManualExecutions]
	(
		[Execution_Id],
		[ExecutionType_Name],
		[Execution_Configuration]
	)
	VALUES
	(
		@Execution_Id,
		@ExecutionType_Name,
		@Execution_Configuration
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Executions_CreateManualExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CreateOrUpdateManualAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ManualAction_Sequence',
    /* Description_Text        */ 'Creates a new manual action or marks an existing one as complete.'
GO

IF OBJECT_ID('[Executions_CreateOrUpdateManualAction]') IS NOT NULL DROP PROCEDURE [Executions_CreateOrUpdateManualAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CreateOrUpdateManualAction]
(
	@Execution_Id INT,
	@CompletedBy_User_Name USERNAME = NULL,
	@AssignedTo_Principal_Name USERNAME = NULL,
	@Task_Name VARCHAR(50),
	@Task_Description TEXT = NULL,
	@ManualAction_Sequence INT = NULL OUT
)
AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @ManualAction_Sequence IS NULL
	BEGIN

		SET @ManualAction_Sequence = COALESCE((SELECT MAX([ManualAction_Sequence])
		                                         FROM [ManualActionsInProgress]
												WHERE [Execution_Id] = @Execution_Id), 0) + 1

		INSERT INTO [ManualActionsInProgress]
		(
			[Execution_Id],
            [ManualAction_Sequence],
			[Task_Name],
			[Task_Description],
			[AssignedTo_Principal_Name],
			[ActionStarted_Date]
		)
		VALUES
		(
			@Execution_Id,
            @ManualAction_Sequence,
			@Task_Name,
			@Task_Description,
			@AssignedTo_Principal_Name,
			GETUTCDATE()
		)

	END

	IF @CompletedBy_User_Name IS NOT NULL
	BEGIN

		UPDATE [ManualActionsInProgress]
		   SET [CompletedBy_User_Name] = @CompletedBy_User_Name
		 WHERE [Execution_Id] = @Execution_Id
		   AND [ManualAction_Sequence] = @ManualAction_Sequence

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CreateOrUpdateManualAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_FailExecutingPlans',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_FailExecutingPlans]') IS NOT NULL DROP PROCEDURE [Executions_FailExecutingPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_FailExecutingPlans]
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	EXEC [Events_RaiseEvent] 'EXEFAL', NULL

	/* Fail all promotions */
	UPDATE [BuildPromotions]
	   SET [PromotionStatus_Name] = 'Failed'
	  FROM [BuildPromotions] BP
	       INNER JOIN [BuildExecutions] BE
	               ON BE.[Promotion_Id] = BP.[Promotion_Id]
		   INNER JOIN [Executions] E
		           ON E.[Execution_Id] = BE.[Execution_Id]
	 WHERE E.[ExecutionRunState_Code] = 'X'

	/* Fail all actions */
	UPDATE [BuildExecution_ActionGroupActions]
	   SET [ExecutionStatus_Name] = 'Failed',
	       [ExecutionEnded_Date] = GETUTCDATE()
	 WHERE [ExecutionStatus_Name] IN ('Executing', 'Canceled')

	UPDATE [Executions]
	   SET [ExecutionRunState_Code] = 'C',
	       [ExecutionStatus_Code] = 'E',
		   [End_Date] = GETUTCDATE()
	 WHERE [ExecutionRunState_Code] = 'X'

	/* Kill all manual actions */
	DELETE [ManualActionsInProgress]

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH		 


END
GO

GRANT EXECUTE ON [Executions_FailExecutingPlans] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetEstimatedDuration',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DeploymentPlanRuns',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the estimated length of time for an execution based on its run context hash.'
GO

IF OBJECT_ID('[Executions_GetEstimatedDuration]') IS NOT NULL DROP PROCEDURE [Executions_GetEstimatedDuration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetEstimatedDuration]
(
	@DeploymentPlan_Id INT,
    @RunContextHash_Bytes BINARY(16)
)
AS
BEGIN

SET NOCOUNT ON

        SELECT TOP 1 * 
          FROM [DeploymentPlanRuns]
         WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id
         ORDER BY 
                CASE WHEN [RunContextHash_Bytes] = @RunContextHash_Bytes THEN 0 ELSE 1 END,
                [LastRun_Date] DESC

END
GO

GRANT EXECUTE ON [Executions_GetEstimatedDuration] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Executions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an execution.'
GO

IF OBJECT_ID('[Executions_GetExecution]') IS NOT NULL DROP PROCEDURE [Executions_GetExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

    SELECT * 
      FROM [Executions_Extended]
     WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [Executions_GetExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionLogScopes',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScopedExecutionLogs',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all log scopes for the specified execution.'
GO

IF OBJECT_ID('[Executions_GetExecutionLogScopes]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionLogScopes]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionLogScopes]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

	SELECT *
	  FROM [ScopedExecutionLogs]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [Executions_GetExecutionLogScopes] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Executions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all executions in the system.'
GO

IF OBJECT_ID('[Executions_GetExecutions]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutions]
(
    @Execution_Count INT = NULL,
    @ExecutionRunState_Code CHAR(1) = NULL,
    @ExecutionStatus_Code CHAR(1) = NULL
)
AS
BEGIN

SET NOCOUNT ON

    IF @Execution_Count IS NOT NULL
        SET ROWCOUNT @Execution_Count

    SELECT * 
      FROM [Executions_Extended]
     WHERE (@ExecutionRunState_Code IS NULL OR [ExecutionRunState_Code] = @ExecutionRunState_Code)
       AND (@ExecutionStatus_Code IS NULL OR [ExecutionStatus_Code] = @ExecutionStatus_Code)
     ORDER BY [Execution_Id] DESC

END
GO

GRANT EXECUTE ON [Executions_GetExecutions] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionsPendingCancellation',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Executions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns executions which are awaiting cancellation.'
GO


IF OBJECT_ID('[Executions_GetExecutionsPendingCancellation]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionsPendingCancellation]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionsPendingCancellation]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Executions_Extended]
	 WHERE [ExecutionRunState_Code] = 'X'
	   AND [CanceledOn_Date] IS NOT NULL

END
GO

GRANT EXECUTE ON [Executions_GetExecutionsPendingCancellation] TO [BuildMasterUser_Role]

GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionsToExecute',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Executions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns executions which are ready to be started by the executer.'
GO

IF OBJECT_ID('[Executions_GetExecutionsToExecute]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionsToExecute]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionsToExecute]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Executions_Extended]
	 WHERE [ExecutionRunState_Code] = 'P'
	   AND ([Start_Date] < GETUTCDATE()
        OR [CanceledOn_Date] IS NOT NULL)

END
GO

GRANT EXECUTE ON [Executions_GetExecutionsToExecute] TO [BuildMasterUser_Role]

GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetLogEntries',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScopedExecutionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the latest log entries for a specified execution and log scope.'
GO

IF OBJECT_ID('[Executions_GetLogEntries]') IS NOT NULL DROP PROCEDURE [Executions_GetLogEntries]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetLogEntries]
(
	@Execution_Id INT,
    @Scope_Sequence INT,
    @Minimum_Scope_HighestLogEntry_Level INT,
    @IncludeDebug_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ScopedExecutionLogEntries] SELE
     INNER JOIN [ScopedExecutionLogs] SEL
             ON SEL.[Execution_Id] = SELE.[Execution_Id]
            AND SEL.[Scope_Sequence] = SELE.[Scope_Sequence]
	 WHERE SELE.[Execution_Id] = @Execution_Id
	   AND (@Scope_Sequence IS NULL OR SELE.[Scope_Sequence] = @Scope_Sequence)
       AND (@Minimum_Scope_HighestLogEntry_Level IS NULL OR SEL.[HighestLogEntry_Level] >= @Minimum_Scope_HighestLogEntry_Level)
	   AND (@IncludeDebug_Indicator = 'Y' OR SELE.[LogEntry_Level] > 0)
	 ORDER BY [LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Executions_GetLogEntries] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualActionsInProgress',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ManualActionsInProgress_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns manual actions which are currently in progress.'
GO


IF OBJECT_ID('[Executions_GetManualActionsInProgress]') IS NOT NULL DROP PROCEDURE [Executions_GetManualActionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualActionsInProgress]
(
	@Application_Id INT = NULL,
	@AssignedTo_Principal_Name VARCHAR(255) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ManualActionsInProgress_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@AssignedTo_Principal_Name IS NULL OR [AssignedTo_Principal_Name] = @AssignedTo_Principal_Name)
	   AND [ActionComplete_Indicator] = 'N'
	
END
GO

GRANT EXECUTE ON [Executions_GetManualActionsInProgress] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ManualExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a manual execution.'
GO

IF OBJECT_ID('[Executions_GetManualExecution]') IS NOT NULL DROP PROCEDURE [Executions_GetManualExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ManualExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [Executions_GetManualExecution] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetPartialExecutionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScopedExecutionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the latest log entries for a specified execution.'
GO

IF OBJECT_ID('[Executions_GetPartialExecutionLog]') IS NOT NULL DROP PROCEDURE [Executions_GetPartialExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetPartialExecutionLog]
(
	@Execution_Id INT,
    @Minimum_LogEntry_Sequence INT,
    @IncludeDebug_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

	SELECT *
	  FROM [ScopedExecutionLogEntries]
	 WHERE [Execution_Id] = @Execution_Id
	   AND ([LogEntry_Sequence] > @Minimum_LogEntry_Sequence)
	   AND (@IncludeDebug_Indicator = 'Y' OR [LogEntry_Level] > 0)
	 ORDER BY [Scope_Sequence], [LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Executions_GetPartialExecutionLog] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_IsManualActionComplete',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'CompletedBy_User_Name',
    /* Description_Text        */ 'Returns the user which has completed the manual action, or null if the action is not completed.'
GO

IF OBJECT_ID('[Executions_IsManualActionComplete]') IS NOT NULL DROP PROCEDURE [Executions_IsManualActionComplete]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_IsManualActionComplete]
(
	@Execution_Id INT,
	@ManualAction_Sequence INT,
	@CompletedBy_User_Name USERNAME = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

	SET @CompletedBy_User_Name = (SELECT [CompletedBy_User_Name]
	                                FROM [ManualActionsInProgress]
								   WHERE [Execution_Id] = @Execution_Id
								     AND [ManualAction_Sequence] = @ManualAction_Sequence)

END
GO

GRANT EXECUTE ON [Executions_IsManualActionComplete] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_SetRunContextHash',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DeploymentPlanRuns',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the estimated length of time for an execution based on its run context hash.'
GO

IF OBJECT_ID('[Executions_SetRunContextHash]') IS NOT NULL DROP PROCEDURE [Executions_SetRunContextHash]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_SetRunContextHash]
(
	@Execution_Id INT,
    @RunContextHash_Bytes BINARY(16)
)
AS
BEGIN

SET NOCOUNT ON

        UPDATE [BuildExecutions]
           SET [RunContextHash_Bytes] = @RunContextHash_Bytes
         WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [Executions_SetRunContextHash] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_DeleteConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an extension configuration'
GO

IF OBJECT_ID('[ExtensionConfiguration_DeleteConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_DeleteConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_DeleteConfiguration] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @QualifiedType_Name VARCHAR(500)
		    SET @QualifiedType_Name = (SELECT [QualifiedType_Name]
		                             FROM [ExtensionConfigurations]
		                            WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id)
		IF @@ROWCOUNT = 0 RETURN

        UPDATE [ActionGroupActions]
           SET [ExtensionConfiguration_Id] = NULL
         WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		DELETE [ExtensionConfigurations]
		 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_DeleteConfiguration] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified name'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfiguration] 
(
	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [QualifiedType_Name] = @QualifiedType_Name
	   AND ([Profile_Name] = @Profile_Name OR [Default_Indicator] = 'Y')


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfiguration] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurationById',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified ID'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurationById]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurationById]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurationById] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [ExtensionConfiguration_Id]  = @ExtensionConfiguration_Id 

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurationById] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all extension configurers in the system, or a specific one if specified'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurations]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurations] 
(
	@QualifiedType_Name VARCHAR(500) = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE @QualifiedType_Name IS NULL OR [QualifiedType_Name] = @QualifiedType_Name
  ORDER BY [Default_Indicator] DESC, [Profile_Name] ASC


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurations] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_SaveConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ExtensionConfiguration_Id',
    /* Description_Text        */ 'Creates an extension configurer in the system, or updates an existing one'
GO

IF OBJECT_ID('[ExtensionConfiguration_SaveConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_SaveConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_SaveConfiguration] 
(
	@ExtensionConfiguration_Id INT OUT,

	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50),

	@Extension_Configuration VARCHAR(MAX),
	@Default_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF (@ExtensionConfiguration_Id IS NULL)
		BEGIN

			INSERT INTO [ExtensionConfigurations]
					   ([QualifiedType_Name]
					   ,[Profile_Name]
					   ,[Extension_Configuration]
					   ,[CreatedBy_User_Name]
					   ,[CreatedOn_Date]
					   ,[ModifiedBy_User_Name]
					   ,[ModifiedOn_Date]
					   ,[Default_Indicator])
				 VALUES
					   (@QualifiedType_Name
					   ,@Profile_Name
					   ,@Extension_Configuration
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,@Default_Indicator)
			SET @ExtensionConfiguration_Id = SCOPE_IDENTITY()

		END ELSE BEGIN

			UPDATE [ExtensionConfigurations]
			   SET [QualifiedType_Name] = @QualifiedType_Name
			      ,[Profile_Name] = @Profile_Name
			      ,[Extension_Configuration] = @Extension_Configuration
			      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			      ,[ModifiedOn_Date] = GETUTCDATE()
			      ,[Default_Indicator] = @Default_Indicator
			 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		END

        IF (@Default_Indicator = 'Y')
        BEGIN 
                
        UPDATE [ExtensionConfigurations]
            SET [Default_Indicator] = 'N'
            WHERE [QualifiedType_Name] = @QualifiedType_Name
            AND [ExtensionConfiguration_Id] <> @ExtensionConfiguration_Id

        END

		/* Raise Extension Configured event */
		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_SaveConfiguration] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Infrastructure_GetInfrastructure',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Servers,Environments_Extended,ServerRoles_Extended,EnvironmentServers_Extended,ServerRoleServers,Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the infrastructure; this includes servers, environments, and roles.'
GO

IF OBJECT_ID('[Infrastructure_GetInfrastructure]') IS NOT NULL DROP PROCEDURE [Infrastructure_GetInfrastructure]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Infrastructure_GetInfrastructure]
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
     WHERE [ServerType_Code] = 'S'
       AND CAST([Agent_Configuration] AS VARCHAR(MAX)) NOT LIKE '<Inedo.BuildMaster.Extensibility.Agents.Soap.SoapAgent%'
       AND CAST([Agent_Configuration] AS VARCHAR(MAX)) NOT LIKE '<Inedo.BuildMaster.Extensibility.Agents.LegacyTcp.LegacyTcpAgent%'

	SELECT *
	  FROM [Environments_Extended]
     WHERE [Active_Indicator] = 'Y'

	SELECT *
	  FROM [ServerRoles_Extended]

	SELECT *
	  FROM [EnvironmentServers_Extended]

	SELECT *
	  FROM [ServerRoleServers]

	SELECT *
	  FROM [Variables]
	 WHERE ([Server_Id] IS NOT NULL AND COALESCE([Environment_Id], [ServerRole_Id], [ApplicationGroup_Id], [Application_Id], [Deployable_Id]) IS NULL)
	    OR ([Environment_Id] IS NOT NULL AND COALESCE([Server_Id], [ServerRole_Id], [ApplicationGroup_Id], [Application_Id], [Deployable_Id]) IS NULL)
		OR ([ServerRole_Id] IS NOT NULL AND COALESCE([Environment_Id], [Server_Id], [ApplicationGroup_Id], [Application_Id], [Deployable_Id]) IS NULL)

END
GO

GRANT EXECUTE ON [Infrastructure_GetInfrastructure] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'IssueSources_CreateOrUpdateIssueSource',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'IssueSource_Id',
    /* Description_Text        */ 'Creates or updates the specified issue source.'
GO

IF OBJECT_ID('[IssueSources_CreateOrUpdateIssueSource]') IS NOT NULL DROP PROCEDURE [IssueSources_CreateOrUpdateIssueSource]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [IssueSources_CreateOrUpdateIssueSource]
(
    @IssueSource_Name VARCHAR(255),
    @Application_Id INT,
    @ApplicationGroup_Id INT,
    @IssueSource_Configuration XML,
    @IssueSource_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

    IF NOT EXISTS(SELECT * FROM [IssueSources] WHERE [IssueSource_Id] = @IssueSource_Id)
    BEGIN
        
        INSERT INTO [IssueSources] 
        (
            [IssueSource_Name]
           ,[Application_Id]
           ,[ApplicationGroup_Id]
           ,[IssueSource_Configuration]
        )
        VALUES 
        (
            @IssueSource_Name
           ,@Application_Id
           ,@ApplicationGroup_Id 
           ,@IssueSource_Configuration
        )

        SET @IssueSource_Id = SCOPE_IDENTITY()

    END
    ELSE
    BEGIN

	    UPDATE [IssueSources]
           SET [IssueSource_Name] = @IssueSource_Name
              ,[Application_Id] = @Application_Id
              ,[ApplicationGroup_Id] = @ApplicationGroup_Id
              ,[IssueSource_Configuration] = @IssueSource_Configuration
         WHERE [IssueSource_Id] = @IssueSource_Id

     END

END
GO

GRANT EXECUTE ON [IssueSources_CreateOrUpdateIssueSource] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'IssueSources_DeleteIssueSource',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an issue source.'
GO


IF OBJECT_ID('[IssueSources_DeleteIssueSource]') IS NOT NULL DROP PROCEDURE [IssueSources_DeleteIssueSource]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [IssueSources_DeleteIssueSource]
(
	@IssueSource_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	DELETE [IssueSources]
	 WHERE [IssueSource_Id] = @IssueSource_Id

END
GO

GRANT EXECUTE ON [IssueSources_DeleteIssueSource] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'IssueSources_GetIssueSource',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'IssueSources_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an issue source.'
GO


IF OBJECT_ID('[IssueSources_GetIssueSource]') IS NOT NULL DROP PROCEDURE [IssueSources_GetIssueSource]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [IssueSources_GetIssueSource]
(
	@IssueSource_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [IssueSources_Extended]
	 WHERE [IssueSource_Id] = @IssueSource_Id

END
GO

GRANT EXECUTE ON [IssueSources_GetIssueSource] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'IssueSources_GetIssueSources',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'IssueSources_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the issue sources, optionally filtered to a release scope.'
GO


IF OBJECT_ID('[IssueSources_GetIssueSources]') IS NOT NULL DROP PROCEDURE [IssueSources_GetIssueSources]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [IssueSources_GetIssueSources]
(
    @Release_Id INT = NULL
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT * 
      FROM [IssueSources_Extended]
     WHERE @Release_Id IS NULL
        OR ([Application_Id] IS NULL AND [ApplicationGroup_Id] IS NULL)
        OR ([Application_Id] IS NOT NULL 
            AND [Application_Id] = (SELECT [Application_Id] FROM [Releases] WHERE [Release_Id] = @Release_Id))
        OR ([ApplicationGroup_Id] IS NOT NULL 
            AND [ApplicationGroup_Id] IN (SELECT G.[ApplicationGroup_Id]
                                            FROM [Releases] R 
                                                 INNER JOIN [Applications] A
                                                         ON R.[Application_Id] = A.[Application_Id]
                                                 CROSS APPLY dbo.[ApplicationGroups_GetAncestorGroups](A.[ApplicationGroup_Id]) G
                                          WHERE [Release_Id] = @Release_Id))
                                          

END
GO

GRANT EXECUTE ON [IssueSources_GetIssueSources] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'IssueSources_UpdateLastPull',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'IssueSource_Id',
    /* Description_Text        */ 'Updates the last pull and execution for the specified issue source.'
GO

IF OBJECT_ID('[IssueSources_UpdateLastPull]') IS NOT NULL DROP PROCEDURE [IssueSources_UpdateLastPull]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [IssueSources_UpdateLastPull]
(
    @IssueSource_Id INT,
    @Last_Pull_Date DATETIME,
    @Last_Execution_Id INT
)
AS
BEGIN

        UPDATE [IssueSources]
           SET [Last_Pull_Date] = COALESCE(@Last_Pull_Date,[Last_Pull_Date])
              ,[Last_Execution_Id] = COALESCE(@Last_Execution_Id,[Last_Execution_Id])
         WHERE [IssueSource_Id] = @IssueSource_Id

END
GO

GRANT EXECUTE ON [IssueSources_UpdateLastPull] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_CloseIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Closes a BuildMaster issue.'
GO

IF OBJECT_ID('[Issues_CloseIssue]') IS NOT NULL DROP PROCEDURE [Issues_CloseIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_CloseIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Issues]
	   SET [Closed_Indicator] = 'Y',
	       [ClosedOn_Date] = GETUTCDATE(),
		   [ClosedOn_Original_Build_Number] = @Build_Number
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id
       AND [IssueTracker_IssueId_Text] IS NULL

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_CloseIssue] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_CreateIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Issue_Id',
    /* Description_Text        */ 'Creates an issue for a release of an application.'
GO

IF OBJECT_ID('[Issues_CreateIssue]') IS NOT NULL DROP PROCEDURE [Issues_CreateIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_CreateIssue]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Type_Text NVARCHAR(50),
	@Title_Text NVARCHAR(200),
	@Description_Text NVARCHAR(MAX),
    @Closed_Indicator YNINDICATOR, 
    @Build_Number BUILDNUMBER = NULL,
	@CreatedBy_User_Name USERNAME = NULL,
	@CreatedOn_Date DATETIME = NULL,
	@IssueTracker_IssueId_Text NVARCHAR(100) = NULL,
	@IssueTracker_IssueStatus_Text NVARCHAR(100) = NULL,
	@IssueTracker_LastSync_Date DATETIME = NULL,
    @IssueTracker_IssueUrl_Text NVARCHAR(255) = NULL,
	@Issue_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Issue_Id = COALESCE((SELECT MAX([Issue_Id])
	                            FROM [Issues]
							   WHERE [Application_Id] = @Application_Id), 0) + 1

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	INSERT INTO [Issues]
	(
		[Issue_Id],
		[Application_Id],
		[Release_Id],
		[IssueTracker_IssueId_Text],
		[IssueTracker_IssueStatus_Text],
        [IssueTracker_IssueUrl_Text],
		[IssueTracker_LastSync_Date],
        [Type_Text],
		[Title_Text],
		[Description_Text],
		[Closed_Indicator],
        [ClosedOn_Date],
        [OpenedOn_Original_Build_Number],
        [ClosedOn_Original_Build_Number],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	VALUES
	(
		@Issue_Id,
		@Application_Id,
		@Release_Id,
		@IssueTracker_IssueId_Text,
		@IssueTracker_IssueStatus_Text,
        @IssueTracker_IssueUrl_Text,
		@IssueTracker_LastSync_Date,
        @Type_Text,
		@Title_Text,
		NULLIF(@Description_Text, ''),
		@Closed_Indicator,
        CASE WHEN @Closed_Indicator = 'Y' THEN GETUTCDATE() ELSE NULL END,
        @Build_Number,
        CASE WHEN @Closed_Indicator = 'Y' THEN @Build_Number ELSE NULL END,
		COALESCE(@CreatedBy_User_Name, dbo.Session_GetUserName()),
		COALESCE(@CreatedOn_Date, GETUTCDATE())
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_CreateIssue] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_DeleteIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an issue.'
GO

IF OBJECT_ID('[Issues_DeleteIssue]') IS NOT NULL DROP PROCEDURE [Issues_DeleteIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_DeleteIssue]
(
	@Application_Id INT,
	@Issue_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [Issues]
		 WHERE [Application_Id] = @Application_Id
		   AND [Issue_Id] = @Issue_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_DeleteIssue] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_GetIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Issues_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified.'
GO


IF OBJECT_ID('[Issues_GetIssue]') IS NOT NULL DROP PROCEDURE [Issues_GetIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_GetIssue]
(
    @Application_Id INT,
	@Issue_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Issues_Extended]
	 WHERE [Application_Id] = @Application_Id
       AND [Issue_Id] = @Issue_Id

END
GO

GRANT EXECUTE ON [Issues_GetIssue] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_GetIssues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Issues_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the issues for an application, optionally filtered by release or build.'
GO


IF OBJECT_ID('[Issues_GetIssues]') IS NOT NULL DROP PROCEDURE [Issues_GetIssues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_GetIssues]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
    @Build_Number BUILDNUMBER = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT IE.*
	  FROM [Issues_Extended] IE
 LEFT JOIN [Builds] B
        ON B.[Release_Id] = IE.[Release_Id]
       AND B.[Build_Number] = @Build_Number
	 WHERE (@Application_Id IS NULL OR IE.[Application_Id] = @Application_Id)
	   AND (@Release_Number IS NULL OR IE.[Release_Number] = @Release_Number)
       AND (@Build_Number IS NULL OR (IE.[OpenedOn_Original_Build_Sequence] IS NULL AND IE.[ClosedOn_Original_Build_Sequence] IS NOT NULL) OR B.[Build_Sequence] >= IE.[OpenedOn_Original_Build_Sequence])
       AND (@Build_Number IS NULL OR IE.[ClosedOn_Original_Build_Sequence] IS NULL OR B.[Build_Sequence] = IE.[ClosedOn_Original_Build_Sequence])
  ORDER BY [Application_Name], [Release_Sequence] DESC, [Closed_Indicator], [Issue_Id] DESC

END
GO

GRANT EXECUTE ON [Issues_GetIssues] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_GetIssueTypes',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Issues_Types',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of distinct issue types that have been synchronized by the system.'
GO


IF OBJECT_ID('[Issues_GetIssueTypes]') IS NOT NULL DROP PROCEDURE [Issues_GetIssueTypes]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_GetIssueTypes]
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Issues_Types]
    ORDER BY [Type_Text]

END
GO

GRANT EXECUTE ON [Issues_GetIssueTypes] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_ReopenIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Re-opens a BuildMaster issue.'
GO

IF OBJECT_ID('[Issues_ReopenIssue]') IS NOT NULL DROP PROCEDURE [Issues_ReopenIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_ReopenIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Issues]
	   SET [Closed_Indicator] = 'N',
	       [ClosedOn_Date] = NULL,
		   [ClosedOn_Original_Build_Number] = NULL,
           [OpenedOn_Original_Build_Number] = @Build_Number
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id
       AND [IssueTracker_IssueId_Text] IS NULL

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_ReopenIssue] TO [BuildMasterUser_Role]



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_UpdateIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates one or more properties of an issue.'
GO

IF OBJECT_ID('[Issues_UpdateIssue]') IS NOT NULL DROP PROCEDURE [Issues_UpdateIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_UpdateIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Release_Number RELEASENUMBER,
    @Type_Text NVARCHAR(50) = NULL,
	@Title_Text NVARCHAR(200) = NULL,
	@Description_Text NVARCHAR(MAX) = NULL,
    @Closed_Indicator YNINDICATOR = NULL,
	@IssueTracker_IssueStatus_Text NVARCHAR(100) = NULL,
    @IssueTracker_IssueUrl_Text NVARCHAR(255) = NULL,
	@IssueTracker_LastSync_Date DATETIME = NULL,
    @OpenedOn_Original_Build_Number BUILDNUMBER = NULL,
    @ClosedOn_Original_Build_Number BUILDNUMBER = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	UPDATE [Issues]
	   SET [Release_Id] = @Release_Id,
           [Type_Text] = NULLIF(COALESCE(@Type_Text, [Type_Text]), ''),
	       [Title_Text] = COALESCE(@Title_Text, [Title_Text]),
		   [Description_Text] = COALESCE(@Description_Text, [Description_Text]),
           [Closed_Indicator] = COALESCE(@Closed_Indicator, [Closed_Indicator]),
		   [IssueTracker_IssueStatus_Text] = COALESCE(@IssueTracker_IssueStatus_Text, [IssueTracker_IssueStatus_Text]),
           [IssueTracker_IssueUrl_Text] = COALESCE(@IssueTracker_IssueUrl_Text, [IssueTracker_IssueUrl_Text]),
		   [IssueTracker_LastSync_Date] = COALESCE(@IssueTracker_LastSync_Date, [IssueTracker_LastSync_Date]),
           [OpenedOn_Original_Build_Number] = NULLIF(COALESCE(@OpenedOn_Original_Build_Number, [OpenedOn_Original_Build_Number]), ''),
           [ClosedOn_Original_Build_Number] = NULLIF(COALESCE(@ClosedOn_Original_Build_Number, [ClosedOn_Original_Build_Number]), '')
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_UpdateIssue] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_AppendActionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Writes a log message to an action log.'
GO

IF OBJECT_ID('[LegacyExecutions_AppendActionLog]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_AppendActionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_AppendActionLog]
(
	@BuildExecution_PlanAction_Id INT,
	@LogEntry_Level INT,
	@LogEntry_Text VARCHAR(MAX)
)
AS
BEGIN

	INSERT INTO [BuildExecution_ActionGroupActionLogEntries]
	(
		[BuildExecution_ActionGroupAction_Id],
		[LogEntry_Sequence],
		[LogEntry_Level],
		[LogEntry_Text]
	)
	SELECT @BuildExecution_PlanAction_Id,
		   COALESCE(MAX([LogEntry_Sequence]),0) + 1,
		   @LogEntry_Level,
		   @LogEntry_Text
	  FROM [BuildExecution_ActionGroupActionLogEntries] 
	  WITH (TABLOCKX) -- added table lock to resolve issue BM-897
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_PlanAction_Id

END
GO

GRANT EXECUTE ON [LegacyExecutions_AppendActionLog] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_BeginAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'BuildExecution_ActionGroupAction_Id',
    /* Description_Text        */ 'Initiates execution of a specific action.'
GO


IF OBJECT_ID('[LegacyExecutions_BeginAction]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_BeginAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_BeginAction]
(
	@BuildExecution_DeploymentPlan_Id INT,
	@DeploymentPlanActionGroup_Sequence INT,
	@Action_Sequence INT,
	@Server_Name VARCHAR(50),
	@Attempt_Sequence INT,
	@VariableValues_Xml XML,
	@ExecutionAction_Description VARCHAR(500),
	@BuildExecution_ActionGroupAction_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [BuildExecution_ActionGroupActions]
	(
		[ExecutionStarted_Date],
		[ExecutionStatus_Name],
		[ExecutionAction_Description],
		[Original_Server_Name],
		[Original_Action_Sequence],
		[Attempt_Sequence],
		[WarningLogEntry_Indicator],
		[BuildExecution_DeploymentPlan_Id],
		[Original_DeploymentPlanActionGroup_Sequence]
	)
	VALUES
	(
		GETUTCDATE(),
		'Executing',
		@ExecutionAction_Description,
		@Server_Name,
		@Action_Sequence,
		@Attempt_Sequence,
		'N',
		@BuildExecution_DeploymentPlan_Id,
		@DeploymentPlanActionGroup_Sequence
	)

	SET @BuildExecution_ActionGroupAction_Id = SCOPE_IDENTITY()

	INSERT INTO [BuildExecution_ActionGroupActionVariableValues]
	(
		[BuildExecution_ActionGroupAction_Id],
		[Variable_Name],
		[Value_Text]
	)
	SELECT [BuildExecution_PlanAction_Id] = @BuildExecution_ActionGroupAction_Id,
	       [Variable_Name] = V.value('@N', 'VARCHAR(50)'),
	       V.value('@V', 'VARCHAR(MAX)')
	  FROM @VariableValues_Xml.nodes('/Vars/Var') AS _V(V)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LegacyExecutions_BeginAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_BeginDeploymentPlan',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'BuildExecution_DeploymentPlan_Id',
    /* Description_Text        */ 'Initiates execution of a specific deployment plan.'
GO


IF OBJECT_ID('[LegacyExecutions_BeginDeploymentPlan]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_BeginDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_BeginDeploymentPlan]
(
	@Execution_Id INT,
	@DeploymentPlan_Id INT,
	@InitiatedBy_BuildExecution_ActionGroupAction_Id INT = NULL,
	@BuildExecution_DeploymentPlan_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @DeploymentPlan_Name VARCHAR(50)
	SELECT @DeploymentPlan_Name = [DeploymentPlan_Name]
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	INSERT INTO [BuildExecution_DeploymentPlans]
	(
		[Execution_Id],
		[Original_DeploymentPlan_Id],
		[Original_DeploymentPlan_Name]
	)
	VALUES
	(
		@Execution_Id,
		@DeploymentPlan_Id,
		@DeploymentPlan_Name
	)

	SET @BuildExecution_DeploymentPlan_Id = SCOPE_IDENTITY()
	
	IF @InitiatedBy_BuildExecution_ActionGroupAction_Id IS NOT NULL
	BEGIN

		UPDATE [BuildExecution_ActionGroupActions]
		   SET [Nested_BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		 WHERE [BuildExecution_ActionGroupAction_Id] = @InitiatedBy_BuildExecution_ActionGroupAction_Id

	END
	ELSE IF @DeploymentPlan_Id > 0
	BEGIN

		UPDATE [BuildExecutions]
		   SET [Initial_BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		 WHERE [Execution_Id] = @Execution_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LegacyExecutions_BeginDeploymentPlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_BeginDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Initiates execution of a specific deployment plan action group.'
GO


IF OBJECT_ID('[LegacyExecutions_BeginDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_BeginDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_BeginDeploymentPlanActionGroup]
(
	@BuildExecution_DeploymentPlan_Id INT,
	@DeploymentPlanActionGroup_Id INT,
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
		IF @DeploymentPlanActionGroup_Id IS NOT NULL
		BEGIN

			INSERT INTO [BuildExecution_DeploymentPlanActionGroups]
			(
				[BuildExecution_DeploymentPlan_Id],
				[Original_DeploymentPlanActionGroup_Sequence],
				[Original_Deployable_Name],
				[Original_Deployable_Application_Name],
				[Original_DeploymentPlanActionGroup_Id],
				[Original_DeploymentPlanActionGroup_Name]
			)
			SELECT @BuildExecution_DeploymentPlan_Id,
			       DPAG.[DeploymentPlanActionGroup_Sequence],
			       D.[Deployable_Name],
				   D_A.[Application_Name],
			       @DeploymentPlanActionGroup_Id,
			       COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name])
			  FROM [DeploymentPlanActionGroups] DPAG
			       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
			               ON BEDP.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
			       INNER JOIN [BuildExecutions] BE
				           ON BE.[Execution_Id] = BEDP.[Execution_Id]
				   INNER JOIN [BuildPromotions] BP
				           ON BP.[Promotion_Id] = BE.[Promotion_Id]
				   INNER JOIN [Builds] B
				           ON B.[Build_Id] = BP.[Build_Id]
				   INNER JOIN [Releases] R
				           ON R.[Release_Id] = B.[Release_Id]
					LEFT JOIN [Deployables] D
			               ON D.[Deployable_Id] = @Deployable_Id
			        LEFT JOIN [Applications] D_A
			               ON D_A.[Application_Id] = D.[Application_Id]
				          AND D_A.[Application_Id] <> R.[Application_Id]
				   INNER JOIN [ActionGroups] AG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
			 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

		 END
		 ELSE
		 BEGIN

			INSERT INTO [BuildExecution_DeploymentPlanActionGroups]
			(
				[BuildExecution_DeploymentPlan_Id],
				[Original_DeploymentPlanActionGroup_Sequence],
				[Original_DeploymentPlanActionGroup_Id]
			)
			VALUES
			(
				@BuildExecution_DeploymentPlan_Id,
				0,
				0
			)

		 END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LegacyExecutions_BeginDeploymentPlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_CompleteAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks an action as completed.'
GO


IF OBJECT_ID('[LegacyExecutions_CompleteAction]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_CompleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_CompleteAction]
(
	@BuildExecution_ActionGroupAction_Id INT,
	@ExecutionStatus_Name VARCHAR(10)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildExecution_ActionGroupActions]
	   SET [ExecutionEnded_Date] = GETUTCDATE(),
	       [ExecutionStatus_Name] = @ExecutionStatus_Name,
	       [WarningLogEntry_Indicator] = CASE WHEN @ExecutionStatus_Name = 'Succeeded'
	                                           AND EXISTS(SELECT *
	                                                        FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	                                                       WHERE BEAGALE.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	                                                         AND BEAGALE.[LogEntry_Level] >= 20
	                                                         AND BEAGALE.[LogEntry_Level] < 30)
	                                          THEN 'Y'
	                                          ELSE 'N'
	                                     END
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LegacyExecutions_CompleteAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_GetExecutionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended,BuildExecution_DeploymentPlanActionGroups,BuildExecution_ActionGroupActions,BuildExecution_ActionGroupActionLogEntries,BuildExecution_ActionGroupActionVariableValues',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build execution, and plan or action if specified.'
GO

IF OBJECT_ID('[LegacyExecutions_GetExecutionLog]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_GetExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_GetExecutionLog]
(
	@Execution_Id INT = NULL,
    @BuildExecution_DeploymentPlan_Id INT = NULL,
    @Original_DeploymentPlanActionGroup_Sequence INT = NULL,
	@BuildExecution_ActionGroupAction_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Execution_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT *
		  FROM [BuildExecutions_Extended]
		 WHERE [Execution_Id] = @Execution_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT BEDPAG.*
		  FROM [BuildExecution_DeploymentPlanActionGroups] BEDPAG
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActions
		SELECT BEAGA.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
			           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT BEAGAVV.*
		  FROM [BuildExecution_ActionGroupActionVariableValues] BEAGAVV
			   INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGAVV.[BuildExecution_ActionGroupAction_Id]
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

	END
	ELSE IF @BuildExecution_DeploymentPlan_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT BEX.*
		  FROM [BuildExecution_DeploymentPlans] BEDP
		       INNER JOIN [BuildExecutions_Extended] BEX
		               ON BEX.[Execution_Id] = BEDP.[Execution_Id]
		 WHERE BEDP.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT *
		  FROM [BuildExecution_DeploymentPlanActionGroups]
		 WHERE [BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActions
		SELECT *
		  FROM [BuildExecution_ActionGroupActions]
		 WHERE [BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
	       AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
			           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		 WHERE BEAGA.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT BEAGAVV.*
		  FROM [BuildExecution_ActionGroupActionVariableValues] BEAGAVV
			   INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGAVV.[BuildExecution_ActionGroupAction_Id]
		 WHERE BEAGA.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

	END
	ELSE IF @BuildExecution_ActionGroupAction_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT BEX.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		       INNER JOIN [BuildExecutions_Extended] BEX
		               ON BEX.[Execution_Id] = BEDP.[Execution_Id]
		 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT BEDPAG.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlanActionGroups] BEDPAG
		               ON BEDPAG.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		              AND BEDPAG.[Original_DeploymentPlanActionGroup_Sequence] = BEAGA.[Original_DeploymentPlanActionGroup_Sequence]
		 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActions
		SELECT *
		  FROM [BuildExecution_ActionGroupActions]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT *
		  FROM [BuildExecution_ActionGroupActionLogEntries]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT *
		  FROM [BuildExecution_ActionGroupActionVariableValues]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	END

END
GO

GRANT EXECUTE ON [LegacyExecutions_GetExecutionLog] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_GetExecutionLogForActions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the log entries for the actions specified.'
GO

IF OBJECT_ID('[LegacyExecutions_GetExecutionLogForActions]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_GetExecutionLogForActions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_GetExecutionLogForActions]
(
	@BuildExecution_ActionGroupActions_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT BEAGALE.*
	  FROM dbo.CsvToTable(@BuildExecution_ActionGroupActions_Csv, ',') T
	       INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	               ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = T.[Value_Text]

END
GO

GRANT EXECUTE ON [LegacyExecutions_GetExecutionLogForActions] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_GetPartialExecutionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the latest log entries for a specified build execution.'
GO

IF OBJECT_ID('[LegacyExecutions_GetPartialExecutionLog]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_GetPartialExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_GetPartialExecutionLog]
(
	@Execution_Id INT,
    @After_BuildExecution_ActionGroupAction_Id INT,
	@After_LogEntry_Sequence INT,
    @IncludeDebug_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
			   INNER JOIN [BuildExecutions] BE
			           ON BE.[Execution_Id] = BEDP.[Execution_Id]
			   INNER JOIN [BuildPromotions] BP
			           ON BP.[Promotion_Id] = BE.[Promotion_Id]
			   INNER JOIN [Builds] B
			           ON B.[Build_Id] = BP.[Build_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id
		   AND (@IncludeDebug_Indicator = 'Y' OR BEAGALE.[LogEntry_Level] > 0)
		   AND (BEAGA.[Original_Action_Sequence] > 0)
		   AND (@After_BuildExecution_ActionGroupAction_Id IS NULL
		        OR (BEAGALE.[BuildExecution_ActionGroupAction_Id] > @After_BuildExecution_ActionGroupAction_Id)
		        OR (BEAGALE.[BuildExecution_ActionGroupAction_Id] = @After_BuildExecution_ActionGroupAction_Id
		            AND (@After_LogEntry_Sequence IS NULL OR BEAGALE.[LogEntry_Sequence] > @After_LogEntry_Sequence)))
		 ORDER BY BEAGALE.[BuildExecution_ActionGroupAction_Id]

END
GO

GRANT EXECUTE ON [LegacyExecutions_GetPartialExecutionLog] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_GetVariableValues',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionVariableValues',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets execution time variable values.'
GO

IF OBJECT_ID('[LegacyExecutions_GetVariableValues]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_GetVariableValues]
(
	@BuildExecution_ActionGroupAction_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	 ORDER BY [Variable_Name]

END
GO

GRANT EXECUTE ON [LegacyExecutions_GetVariableValues] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LegacyExecutions_SetVariableValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates, updates, or deletes an execution time variable value.'
GO

IF OBJECT_ID('[LegacyExecutions_SetVariableValue]') IS NOT NULL DROP PROCEDURE [LegacyExecutions_SetVariableValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LegacyExecutions_SetVariableValue]
(
	@BuildExecution_ActionGroupAction_Id INT,
	@Variable_Name VARCHAR(50),
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON
	
BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @ExecutionStatus_Name VARCHAR(10)
	
	SELECT @ExecutionStatus_Name = BE.[ExecutionStatus_Name]
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
	       INNER JOIN [BuildExecutions_Extended] BE
	               ON BE.[Execution_Id] = BEDP.[Execution_Id]
	 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	IF @ExecutionStatus_Name <> 'Pending' AND @ExecutionStatus_Name <> 'Executing'
		RAISERROR('Variables cannot be set on a %s execution.', 16, 1, @ExecutionStatus_Name)

	IF EXISTS(SELECT *
	            FROM [BuildExecution_ActionGroupActionVariableValues]
	           WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	             AND [Variable_Name] = @Variable_Name)
	BEGIN
		IF @Value_Text IS NULL
		BEGIN
			DELETE [BuildExecution_ActionGroupActionVariableValues]
             WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
               AND [Variable_Name] = @Variable_Name
		END
		ELSE
		BEGIN
			UPDATE [BuildExecution_ActionGroupActionVariableValues]
			   SET [Value_Text] = @Value_Text
	         WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	           AND [Variable_Name] = @Variable_Name
	    END
	END
	ELSE IF @Value_Text IS NOT NULL
	BEGIN
		INSERT INTO [BuildExecution_ActionGroupActionVariableValues]
		(
			[BuildExecution_ActionGroupAction_Id],
			[Variable_Name],
			[Value_Text]
		)
		VALUES
		(
			@BuildExecution_ActionGroupAction_Id,
			@Variable_Name,
			@Value_Text
		)
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LegacyExecutions_SetVariableValue] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_ActivateLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the activation code for a license key.'
GO

IF OBJECT_ID('[LicenseKeys_ActivateLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_ActivateLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_ActivateLicenseKey]
(
	@LicenseKey_Text CHAR(36),
	@ActivationCode_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

IF @ActivationCode_Text IS NULL
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = NULL,
	       [ActivatedBy_User_Name] = NULL,
	       [ActivatedOn_Date] = NULL
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END
ELSE
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = @ActivationCode_Text,
	       [ActivatedBy_User_Name] = dbo.Session_GetUserName(),
	       [ActivatedOn_Date] = GETUTCDATE()
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END

END
GO

GRANT EXECUTE ON [LicenseKeys_ActivateLicenseKey] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_AddLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a new license key.'
GO

IF OBJECT_ID('[LicenseKeys_AddLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_AddLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_AddLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

INSERT INTO [LicenseKeys]
(
	[LicenseKey_Text],
	[AddedBy_User_Name],
	[AddedOn_Date]
)
VALUES
(
	@LicenseKey_Text,
	dbo.Session_GetUserName(),
	GETUTCDATE()
)

END
GO

GRANT EXECUTE ON [LicenseKeys_AddLicenseKey] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_DeleteLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a license key.'
GO

IF OBJECT_ID('[LicenseKeys_DeleteLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_DeleteLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_DeleteLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

DELETE [LicenseKeys]
 WHERE [LicenseKey_Text] = @LicenseKey_Text

END
GO

GRANT EXECUTE ON [LicenseKeys_DeleteLicenseKey] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_GetLicenseKeys',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LicenseKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the license keys.'
GO

IF OBJECT_ID('[LicenseKeys_GetLicenseKeys]') IS NOT NULL DROP PROCEDURE [LicenseKeys_GetLicenseKeys]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_GetLicenseKeys]
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [LicenseKeys]
	 ORDER BY [AddedOn_Date]

END
GO

GRANT EXECUTE ON [LicenseKeys_GetLicenseKeys] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_AddMessage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[LogMessages_AddMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_AddMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_AddMessage]
(
	@Message_Text VARCHAR(MAX),
    @Message_Level INT = NULL,
    @Category_Name VARCHAR(30) = NULL,
    @Details_Bytes VARBINARY(MAX) = NULL,
    @Application_Id INT = NULL,
    @Schedule_Id INT = NULL,
    @Requirement_Id INT = NULL,
    @Provider_Id INT = NULL,
    @EventListener_Id INT = NULL,
    @Server_Id INT = NULL,
    @ShowNotification_Indicator YNINDICATOR = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

     INSERT INTO [LogMessages]
     (
            [Logged_Date]
           ,[Message_Text]
           ,[Message_Level]
           ,[Category_Name]
           ,[Details_Bytes]
           ,[Application_Id]
           ,[Schedule_Id]
           ,[Requirement_Id]
           ,[Provider_Id]
           ,[EventListener_Id]
           ,[Server_Id]
           ,[ShowNotification_Indicator]
     )
     VALUES
     (
            GETUTCDATE()
           ,@Message_Text
           ,COALESCE(@Message_Level, 30)
           ,@Category_Name
           ,@Details_Bytes
           ,@Application_Id
           ,@Schedule_Id
           ,@Requirement_Id
           ,@Provider_Id
           ,@EventListener_Id
           ,@Server_Id
           ,COALESCE(@ShowNotification_Indicator, 'N')
     )

    DECLARE @Row_Count INT
	SET @Row_Count = (SELECT COUNT(*) FROM [LogMessages])

	IF @Row_Count > 1000
	BEGIN

		DECLARE @DeleteBelow_Id INT
		SET @DeleteBelow_Id = (SELECT MIN(A.[LogMessage_Id])
		                         FROM (SELECT TOP 1000 [LogMessage_Id]
								        FROM [LogMessages]
								       ORDER BY [LogMessage_Id] DESC) A)

		DELETE [LogMessages]
		 WHERE [LogMessage_Id] < @DeleteBelow_Id

	END

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_AddMessage] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_DeleteMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes one or all message from the error log'
GO

IF OBJECT_ID('[LogMessages_DeleteMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_DeleteMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_DeleteMessages]
(
	@LogMessage_Id INT = NULL
) AS
BEGIN

    SET NOCOUNT ON

    BEGIN TRY

	    DELETE [LogMessages]
	     WHERE @LogMessage_Id IS NULL
            OR [LogMessage_Id] = @LogMessage_Id

    END TRY BEGIN CATCH
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_DeleteMessages] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetCategories',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Categories',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the distinct category names in the log messages table.'
GO

IF OBJECT_ID('[LogMessages_GetCategories]') IS NOT NULL DROP PROCEDURE [LogMessages_GetCategories]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetCategories]
AS
BEGIN

    SET NOCOUNT ON

	SELECT *
	  FROM [LogMessages_Categories]

END
GO

GRANT EXECUTE ON [LogMessages_GetCategories] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specified message from the error log'
GO

IF OBJECT_ID('[LogMessages_GetMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessage]
(
	@LogMessage_Id INT
) AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [LogMessages_Extended] WHERE @LogMessage_Id = [LogMessage_Id]
	
END
GO

GRANT EXECUTE ON [LogMessages_GetMessage] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the messages from the error log that match the specified criteria'
GO

IF OBJECT_ID('[LogMessages_GetMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessages]
(
	@Row_Count INT,
	@Message_Level INT = NULL,
    @Category_Name VARCHAR(30) = NULL,
    @Application_Id INT = NULL,
    @Schedule_Id INT = NULL,
    @Requirement_Id INT = NULL,
    @Provider_Id INT = NULL,
    @EventListener_Id INT = NULL,
    @Server_Id INT = NULL
) AS
BEGIN

    SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
	    SET ROWCOUNT @Row_Count

    SELECT * FROM [LogMessages_Extended]
     WHERE (@Message_Level IS NULL OR [Message_Level] >= @Message_Level)
       AND (@Category_Name IS NULL OR [Category_Name] = @Category_Name)
       AND (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
       AND (@Schedule_Id IS NULL OR [Schedule_Id] = @Schedule_Id)
       AND (@Requirement_Id IS NULL OR [Requirement_Id] = @Requirement_Id)
       AND (@Provider_Id IS NULL OR [Provider_Id] = @Provider_Id)
       AND (@EventListener_Id IS NULL OR [EventListener_Id] = @EventListener_Id)
       AND (@Server_Id IS NULL OR [Server_Id] = @Server_Id)
     ORDER BY [Logged_Date] DESC

END
GO

GRANT EXECUTE ON [LogMessages_GetMessages] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_CreateOrUpdateNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new notifier, or updates an existing one'
GO

IF OBJECT_ID('[Notifiers_CreateOrUpdateNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_CreateOrUpdateNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_CreateOrUpdateNotifier]
(
	@Notifier_Id INT = NULL,
	@Notifier_Name VARCHAR(50),
	@Notifier_Description VARCHAR(MAX),
	@Notifier_Configuration VARCHAR(MAX),
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,
	@EventCodeList_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Notifier_Id IS NULL BEGIN
		INSERT INTO [Notifiers]
				   ([Notifier_Name]
				   ,[Notifier_Description]
				   ,[Notifier_Configuration]
				   ,[Owner_User_Name]
				   ,[Active_Indicator])
			 VALUES
				   (@Notifier_Name
				   ,@Notifier_Description
				   ,@Notifier_Configuration
				   ,@Owner_User_Name
				   ,@Active_Indicator)
		SET @Notifier_Id = SCOPE_IDENTITY()
	END ELSE BEGIN
		UPDATE [Notifiers] SET
		   [Notifier_Name]=@Notifier_Name
		   ,[Notifier_Description] = @Notifier_Description
		   ,[Notifier_Configuration] = @Notifier_Configuration
		   ,[Active_Indicator] = @Active_Indicator
		WHERE [Notifier_Id] = @Notifier_Id
		IF @@ROWCOUNT=0 RAISERROR('Notifier #%i does not exist.',16,1,@Notifier_Id)

		DELETE [NotifierEventCodes] 
		 WHERE [Notifier_Id] = @Notifier_Id
	END

	INSERT INTO [NotifierEventCodes] ([Notifier_Id],[Event_Code])
	SELECT @Notifier_Id, V.[Value_Text]
	  FROM dbo.CsvToTable(@EventCodeList_Csv,',') V

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_CreateOrUpdateNotifier] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_DeleteNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a notifier from the system'
GO

IF OBJECT_ID('[Notifiers_DeleteNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_DeleteNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_DeleteNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NotifierEventCodes] WHERE [Notifier_Id] = @Notifier_Id

	DELETE [Notifiers] WHERE [Notifier_Id] = @Notifier_Id

 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_DeleteNotifier] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Notifiers,NotifierEventCodes',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a notifier and a list of event codes that the notifier is subscribed to'
GO

IF OBJECT_ID('[Notifiers_GetNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Notifier_Id] = @Notifier_Id
	IF @@ROWCOUNT=0 RAISERROR('Notifier %i does not exist.', 16,1,@Notifier_Id)

	SELECT * FROM [NotifierEventCodes]
	 WHERE [Notifier_Id] = @Notifier_Id

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifier] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Notifiers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of notifiers for the specified user, or all users if no user is specified'
GO

IF OBJECT_ID('[Notifiers_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifiers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifiers]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name])
	   OR ([Owner_User_Name] IS NULL AND @Owner_User_Name IS NULL)

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifiers] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_CreatePipeline',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Pipeline_Id',
    /* Description_Text        */ 'Creates a new pipeline.'
GO

IF OBJECT_ID('[Pipelines_CreatePipeline]') IS NOT NULL DROP PROCEDURE [Pipelines_CreatePipeline]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_CreatePipeline]
(
	@Pipeline_Name VARCHAR(255),
    @Pipeline_Configuration XML,
    @Active_Indicator YNINDICATOR,
    @Application_Id INT,
	@Pipeline_Color VARCHAR(20),
	@Pipeline_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [Pipelines]
	(
        [Pipeline_Name],
        [Application_Id],
        [Pipeline_Configuration],
        [Active_Indicator],
        [ModifiedOn_Date],
        [ModifiedBy_User_Name],
		[Pipeline_Color]
	)
	VALUES
	(
        @Pipeline_Name,
        @Application_Id,
        @Pipeline_Configuration,
        @Active_Indicator,
        GETUTCDATE(),
        dbo.Session_GetUserName(),
		@Pipeline_Color
	)

	SET @Pipeline_Id = SCOPE_IDENTITY()

	--EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @Pipeline_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Pipelines_CreatePipeline] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_DeletePipeline',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a pipeline.'
GO


IF OBJECT_ID('[Pipelines_DeletePipeline]') IS NOT NULL DROP PROCEDURE [Pipelines_DeletePipeline]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_DeletePipeline]
(
	@Pipeline_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 	DELETE [Pipelines] WHERE [Pipeline_Id] = @Pipeline_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Pipelines_DeletePipeline] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_GetPipeline',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Pipelines',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the specified pipeline.'
GO


IF OBJECT_ID('[Pipelines_GetPipeline]') IS NOT NULL DROP PROCEDURE [Pipelines_GetPipeline]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_GetPipeline]
(
	@Pipeline_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Pipelines]
     WHERE [Pipeline_Id] = @Pipeline_Id

END
GO

GRANT EXECUTE ON [Pipelines_GetPipeline] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_GetPipelines',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Pipelines',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the pipelines in the system, optionally filtered by application.'
GO


IF OBJECT_ID('[Pipelines_GetPipelines]') IS NOT NULL DROP PROCEDURE [Pipelines_GetPipelines]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_GetPipelines]
(
	@Application_Id INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Pipelines]
     WHERE @Application_Id IS NULL OR [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Pipelines_GetPipelines] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_GetPipelinesUsedByApplications',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Pipelines_UsedByApplications',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the pipelines used by applications.'
GO


IF OBJECT_ID('[Pipelines_GetPipelinesUsedByApplications]') IS NOT NULL DROP PROCEDURE [Pipelines_GetPipelinesUsedByApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_GetPipelinesUsedByApplications]
AS
BEGIN
	SET NOCOUNT ON

        SELECT * FROM [Pipelines_UsedByApplications]
END
GO

GRANT EXECUTE ON [Pipelines_GetPipelinesUsedByApplications] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Pipelines_UpdatePipeline',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates a pipeline.'
GO

IF OBJECT_ID('[Pipelines_UpdatePipeline]') IS NOT NULL DROP PROCEDURE [Pipelines_UpdatePipeline]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Pipelines_UpdatePipeline]
(
	@Pipeline_Id INT,
    @Pipeline_Name VARCHAR(255) = NULL,
    @Pipeline_Configuration XML = NULL,
    @Active_Indicator YNINDICATOR = NULL,
	@Pipeline_Color VARCHAR(20) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	UPDATE [Pipelines]
       SET [Pipeline_Name] = COALESCE(@Pipeline_Name, [Pipeline_Name]),
           [Pipeline_Configuration] = COALESCE(@Pipeline_Configuration, [Pipeline_Configuration]),
           [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator]),
           [ModifiedOn_Date] = GETUTCDATE(),
           [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
		   [Pipeline_Color] = COALESCE(@Pipeline_Color, [Pipeline_Color])
     WHERE [Pipeline_Id] = @Pipeline_Id

	--EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @Pipeline_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Pipelines_UpdatePipeline] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves an action into a different action group.'
GO


IF OBJECT_ID('[Plans_ChangeActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_DeploymentPlanActionGroup_Id INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlan_Id INT
	 SELECT @ActionGroup_Id = [ActionGroup_Id],
	        @DeploymentPlan_Id = [DeploymentPlan_Id]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	DECLARE @New_ActionGroup_Id INT
	 SELECT @New_ActionGroup_Id = [ActionGroup_Id]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @New_DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	IF (@ActionGroup_Id = @New_ActionGroup_Id)
	    RAISERROR('Actions cannot be moved to the same plan; use Plans_ResequenceAction instead.', 16, 1)

   DECLARE @Last_Action_Sequence INT
	SELECT @Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 1)
	  FROM [ActionGroupActions]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

   DECLARE @Target_Last_Action_Sequence INT
	SELECT @Target_Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 0) + 1
      FROM [ActionGroupActions]
     WHERE [ActionGroup_Id] = @New_ActionGroup_Id


	-- move action to end of list to appease trigger
	EXEC [Plans_ResequenceAction] @DeploymentPlanActionGroup_Id, @Action_Sequence, @Last_Action_Sequence

    -- move the action to the end of its new action group 
     UPDATE [ActionGroupActions]
        SET [ActionGroup_Id] = @New_ActionGroup_Id,
            [Action_Sequence] = @Target_Last_Action_Sequence
      WHERE [ActionGroup_Id] = @ActionGroup_Id
	    AND [Action_Sequence] = @Last_Action_Sequence

	-- move the action from the end of its action group to its new sequence
	EXEC [Plans_ResequenceAction] @New_DeploymentPlanActionGroup_Id, @Target_Last_Action_Sequence, @New_Action_Sequence

	-- disable reexecution for anything that used the containing plan of either group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                                 INNER JOIN [DeploymentPlanActionGroups] DPAG
	                                         ON BEDP.[Original_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
	                           WHERE DPAG.[DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	                              OR DPAG.[DeploymentPlanActionGroup_Id] = @New_DeploymentPlanActionGroup_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroupDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a plan action group to the final position of a different deployment plan.'
GO


IF OBJECT_ID('[Plans_ChangeActionGroupDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroupDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroupDeploymentPlan]
(
	@DeploymentPlanActionGroup_Id INT,
	@New_DeploymentPlan_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

    DECLARE @DeploymentPlanActionGroup_Sequence INT
    DECLARE @ActionGroup_Id INT
    DECLARE @DeploymentPlan_Id INT
	
	SELECT @ActionGroup_Id = [ActionGroup_Id],
	       @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence],
	       @DeploymentPlan_Id = [DeploymentPlan_Id]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

    DECLARE @Last_DeploymentPlanActionGroup_Sequence INT
    SELECT @Last_DeploymentPlanActionGroup_Sequence = MAX([DeploymentPlanActionGroup_Sequence])
      FROM [DeploymentPlanActionGroups]
     WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

    -- move action group to the end of the sequence in the current environment to appease trigger
    EXEC [Plans_ResequenceActionGroup] @DeploymentPlanActionGroup_Id, @Last_DeploymentPlanActionGroup_Sequence

    -- move action group to the end of the sequence of action groups in the new environment
    UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = (SELECT COALESCE(MAX([DeploymentPlanActionGroup_Sequence]), 0) 
	                                                 FROM [DeploymentPlanActionGroups] 
	                                                WHERE [DeploymentPlan_Id] = @New_DeploymentPlan_Id) + 1,
	       [DeploymentPlan_Id] = @New_DeploymentPlan_Id
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

 	-- disable reexecution for anything that used either plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id
	                              OR BEDP.[Original_DeploymentPlan_Id] = @New_DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id
	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @New_DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroupDeploymentPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CloneDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'NewDeploymentPlanIdsCsv',
    /* Description_Text        */ 'Clones one or more deployment plans'
GO

IF OBJECT_ID('[Plans_CloneDeploymentPlans]') IS NOT NULL
	DROP PROCEDURE [Plans_CloneDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CloneDeploymentPlans]
(
	@DeploymentPlanCloneData_Xml XML,
	@NewDeploymentPlanIdsCsv VARCHAR(MAX) = NULL OUT
) AS
BEGIN
	/*<DeploymentPlanCloneData LinkAllActionGroups_Indicator="Y">
        <DeploymentPlan>
          <New_DeploymentPlan_Name>Standard</New_DeploymentPlan_Name>
          <DeploymentPlan_Id>3</DeploymentPlan_Id>
          <Sequence>1000</Sequence>
        </DeploymentPlan>
	  </DeploymentPlanCloneData>*/

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

		DECLARE @LinkAllActionGroups_Indicator YNINDICATOR
		    SET @LinkAllActionGroups_Indicator =
		       (SELECT DPCD.[value]('@LinkAllActionGroups_Indicator', 'CHAR(1)')
	              FROM @DeploymentPlanCloneData_Xml.[nodes]('/DeploymentPlanCloneData') AS DeploymentPlanCloneData(DPCD))

		DECLARE @Sequenced_DeploymentPlans TABLE ([Old_DeploymentPlan_Id] INT, [Seq] INT)

        INSERT @Sequenced_DeploymentPlans ([Old_DeploymentPlan_Id], [Seq])
        SELECT DPCD.[value]('DeploymentPlan_Id[1]', 'INT') [Old_DeploymentPlan_Id],
               DPCD.[value]('Sequence[1]', 'INT') [Seq]
          FROM @DeploymentPlanCloneData_Xml.[nodes]('/DeploymentPlanCloneData/DeploymentPlan') AS DeploymentPlanCloneData(DPCD)
        

        DECLARE @DeploymentPlans TABLE ([Old_DeploymentPlan_Id] INT, [New_DeploymentPlan_Id] INT, [AllowLocalChanges_Indicator] CHAR(1), [New_DeploymentPlan_Name] VARCHAR(50), [Seq] INT)

        INSERT INTO @DeploymentPlans ([Old_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator])
		SELECT DISTINCT [Old_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator]
		  FROM (SELECT DPCD.[value]('New_DeploymentPlan_Name[1]', 'VARCHAR(20)') [New_DeploymentPlan_Name],
					   DPCD.[value]('DeploymentPlan_Id[1]', 'INT') [Old_DeploymentPlan_Id]
				  FROM @DeploymentPlanCloneData_Xml.[nodes]('/DeploymentPlanCloneData/DeploymentPlan') AS DeploymentPlanCloneData(DPCD)) DPCD
			   INNER JOIN [DeploymentPlans] DP ON DP.[DeploymentPlan_Id] = DPCD.[Old_DeploymentPlan_Id]

		DECLARE @ActionGroups TABLE ([Old_ActionGroup_Id] INT, [New_ActionGroup_Id] INT, [Old_DeploymentPlan_Id] INT, [Old_DeploymentPlanActionGroup_Sequence] INT)

		IF @LinkAllActionGroups_Indicator = 'Y' BEGIN
			
			INSERT INTO @ActionGroups ([Old_ActionGroup_Id], [New_ActionGroup_Id], [Old_DeploymentPlan_Id], [Old_DeploymentPlanActionGroup_Sequence])
			SELECT  AG.[ActionGroup_Id], 
					AG.[ActionGroup_Id], 
					DPAG.[DeploymentPlan_Id],
                    DPAG.[DeploymentPlanActionGroup_Sequence]
			  FROM [ActionGroups] AG
				   INNER JOIN [DeploymentPlanActionGroups] DPAG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
				   INNER JOIN @DeploymentPlans DP
						   ON DP.[Old_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]

		END ELSE BEGIN 

			INSERT INTO @ActionGroups ([Old_ActionGroup_Id], [Old_DeploymentPlan_Id], [Old_DeploymentPlanActionGroup_Sequence])
		    SELECT  AG.[ActionGroup_Id],
                    DPAG.[DeploymentPlan_Id],
                    DPAG.[DeploymentPlanActionGroup_Sequence]
			  FROM [ActionGroups] AG
				   INNER JOIN [DeploymentPlanActionGroups] DPAG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
				   INNER JOIN @DeploymentPlans DP
						   ON DP.[Old_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
		
			DECLARE @ActionGroupsOldId INT
			DECLARE @ActionGroupsInsertCur CURSOR
			   SET @ActionGroupsInsertCur = CURSOR FOR 
				SELECT [Old_ActionGroup_Id] FROM @ActionGroups
					FOR UPDATE OF [New_ActionGroup_Id]
			OPEN @ActionGroupsInsertCur
			FETCH NEXT FROM @ActionGroupsInsertCur INTO @ActionGroupsOldId
			WHILE @@FETCH_STATUS = 0 
			BEGIN

				INSERT INTO [ActionGroups] 
				(
					[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],
					[ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description],
					[Server_Id],[Server_Variable_Name]
				)
				SELECT	
					dbo.Session_GetUserName(), GETUTCDATE(), [ModifiedBy_User_Name],[ModifiedOn_Date],
					[ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description],
					[Server_Id],[Server_Variable_Name]   
				FROM [ActionGroups] 
				WHERE [ActionGroup_Id] = @ActionGroupsOldId

				UPDATE @ActionGroups SET [New_ActionGroup_Id] = SCOPE_IDENTITY()
				 WHERE CURRENT OF @ActionGroupsInsertCur
			
				FETCH NEXT FROM @ActionGroupsInsertCur INTO @ActionGroupsOldId
			END
			CLOSE @ActionGroupsInsertCur


			--==[ActionGroupActions]==--
			INSERT INTO [ActionGroupActions] (
				   [ActionGroup_Id]
				  ,[Action_Sequence]
				  ,[Server_Id]
				  ,[Long_Description]
				  ,[ResumeNextOnFailure_Indicator]
				  ,[Action_Configuration]
				  ,[Short_Description]
				  ,[Active_Indicator]
				  ,[Retry_Count]
				  ,[LogFailureAsWarning_Indicator]
				  ,[Server_Variable_Name]
				  ,[Target_Server_Id]
				  ,[Target_Server_Variable_Name]
				  ,[ExtensionConfiguration_Id]
			) SELECT 
				   [New_ActionGroup_Id]
				  ,[Action_Sequence]
				  ,[Server_Id]
				  ,[Long_Description]
				  ,[ResumeNextOnFailure_Indicator]
				  ,[Action_Configuration]
				  ,[Short_Description]
				  ,[Active_Indicator]
				  ,[Retry_Count]
				  ,[LogFailureAsWarning_Indicator]
				  ,[Server_Variable_Name]
				  ,[Target_Server_Id]
				  ,[Target_Server_Variable_Name]
				  ,[ExtensionConfiguration_Id]
			  FROM [ActionGroupActions] JOIN @ActionGroups ON [ActionGroup_Id] = [Old_ActionGroup_Id]
		END

		--==[DeploymentPlans]==--
		DECLARE @DeploymentPlansNewId INT, @DeploymentPlansNewName VARCHAR(50), @AllowLocalChanges_Indicator CHAR(1)
		DECLARE @DeploymentPlansInsertCur CURSOR
			SET @DeploymentPlansInsertCur = CURSOR FOR 
			SELECT [New_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator] FROM @DeploymentPlans
				FOR UPDATE OF [New_DeploymentPlan_Id]
		OPEN @DeploymentPlansInsertCur
		FETCH NEXT FROM @DeploymentPlansInsertCur INTO @DeploymentPlansNewId, @DeploymentPlansNewName, @AllowLocalChanges_Indicator
		WHILE @@FETCH_STATUS = 0 
		BEGIN

			INSERT INTO [DeploymentPlans] ([DeploymentPlan_Name], [AllowLocalChanges_Indicator]) VALUES (@DeploymentPlansNewName, @AllowLocalChanges_Indicator)
			
			UPDATE @DeploymentPlans SET [New_DeploymentPlan_Id] = SCOPE_IDENTITY()
				WHERE CURRENT OF @DeploymentPlansInsertCur

			FETCH NEXT FROM @DeploymentPlansInsertCur INTO @DeploymentPlansNewId, @DeploymentPlansNewName, @AllowLocalChanges_Indicator

		END
		CLOSE @DeploymentPlansInsertCur

		--==[DeploymentPlanActionGroups]==--
		INSERT INTO [DeploymentPlanActionGroups] (
			  /*[DeploymentPlanActionGroup_Id]*/
			  [DeploymentPlan_Id]
			  ,[DeploymentPlanActionGroup_Sequence]
			  ,[ActionGroup_Id]
			  ,[Deployable_Name]
			  ,[Deployable_Application_Id]
			  ,[Active_Indicator]
			  ,[ActionGroup_Name]
			  ,[ActionGroup_Description]
			  ,[Predicate_Configuration]
			  ,[Server_Id]
			  ,[Server_Variable_Name]
			  ,[Iterate_Servers_Indicator]
			  ,[Iterate_Deployables_Indicator]
			  ,[Parallel_Indicator]
			  ,[OnSuccess_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_Continue_Indicator]
			  ,[ModifiedBy_User_Name]
			  ,[ModifiedOn_Date]
		) SELECT 
			  [New_DeploymentPlan_Id]
			  ,[DeploymentPlanActionGroup_Sequence]
			  ,[New_ActionGroup_Id]
			  ,[Deployable_Name]
			  ,[Deployable_Application_Id]
			  ,[Active_Indicator]
			  ,[ActionGroup_Name]
			  ,[ActionGroup_Description]
			  ,[Predicate_Configuration]
			  ,[Server_Id]
			  ,[Server_Variable_Name]
			  ,[Iterate_Servers_Indicator]
			  ,[Iterate_Deployables_Indicator]
			  ,[Parallel_Indicator]
			  ,[OnSuccess_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_Continue_Indicator]
			  ,[ModifiedBy_User_Name]
			  ,[ModifiedOn_Date]
		  FROM [DeploymentPlanActionGroups] DPAG
         INNER JOIN @ActionGroups AG
                 ON DPAG.[DeploymentPlan_Id] = AG.[Old_DeploymentPlan_Id] 
                AND DPAG.[ActionGroup_Id] = AG.[Old_ActionGroup_Id] 
                AND DPAG.[DeploymentPlanActionGroup_Sequence] = AG.[Old_DeploymentPlanActionGroup_Sequence]
         INNER JOIN @DeploymentPlans DP 
                 ON [DeploymentPlan_Id] = DP.[Old_DeploymentPlan_Id] 
                AND [DeploymentPlanActionGroup_Sequence] = AG.[Old_DeploymentPlanActionGroup_Sequence]

		SELECT @NewDeploymentPlanIdsCsv = ISNULL(@NewDeploymentPlanIdsCsv,'') + CAST([New_DeploymentPlan_Id] AS VARCHAR(MAX)) + ','
		  FROM @DeploymentPlans DP
        INNER JOIN @Sequenced_DeploymentPlans SEQ ON SEQ.Old_DeploymentPlan_Id = DP.Old_DeploymentPlan_Id
         ORDER BY SEQ.[Seq]
		SET @NewDeploymentPlanIdsCsv = SUBSTRING(@NewDeploymentPlanIdsCsv, 1, LEN(@NewDeploymentPlanIdsCsv)-1)

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CloneDeploymentPlans] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'DeploymentPlanActionGroup_Id__ActionGroup_Id',
    /* Description_Text        */ 'Creates a new action group in the specified deployment plan.'
GO


IF OBJECT_ID('[Plans_CreateDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_CreateDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateDeploymentPlanActionGroup]
(
	@DeploymentPlan_Id INT = NULL,
	@Deployable_Name VARCHAR(50) = NULL,
	@Deployable_Application_Id INT = NULL,
	@DeploymentPlanActionGroup_Sequence INT = NULL,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,

	@Active_Indicator YNINDICATOR = NULL,
	@ActionGroup_Name VARCHAR(50) = NULL,
	@ActionGroup_Description VARCHAR(MAX) = NULL,
	@Predicate_Configuration XML = NULL,

	@ActionGroup_Id INT = NULL,
	@DeploymentPlanActionGroup_Id__ActionGroup_Id VARCHAR(50) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @DeploymentPlanActionGroup_Id INT

	--If sequence is not specified, create at the end	
	IF @DeploymentPlanActionGroup_Sequence IS NULL
	BEGIN
		SET @DeploymentPlanActionGroup_Sequence = COALESCE(
			(SELECT MAX([DeploymentPlanActionGroup_Sequence])
			   FROM [DeploymentPlanActionGroups]
			  WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id), 0) + 1
	END

	IF @ActionGroup_Id IS NULL
	BEGIN
		INSERT INTO [ActionGroups]
		(
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[ActionGroup_Name],
			[Predicate_Configuration],
			[ActionGroup_Description],
			[Server_Id],
			[Server_Variable_Name]
		)
		VALUES
		(
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@ActionGroup_Name,
			@Predicate_Configuration,
			@ActionGroup_Description,
			@Server_Id,
			NULLIF(@Server_Variable_Name, '')
		)

		SET @ActionGroup_Id = SCOPE_IDENTITY()

		INSERT INTO [DeploymentPlanActionGroups]
		(
			[DeploymentPlan_Id],
			[DeploymentPlanActionGroup_Sequence],
			[ActionGroup_Id],
			[Deployable_Name],
			[Deployable_Application_Id],
			[Active_Indicator],
			[ActionGroup_Name],
			[ActionGroup_Description],
			[Predicate_Configuration],
			[Server_Id],
			[Server_Variable_Name],
			[Iterate_Servers_Indicator],
			[Iterate_Deployables_Indicator],
			[Parallel_Indicator],
			[OnFailure_Continue_Indicator],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@DeploymentPlan_Id,
			@DeploymentPlanActionGroup_Sequence,
			@ActionGroup_Id,
			@Deployable_Name,
			@Deployable_Application_Id,
			COALESCE(@Active_Indicator, 'Y'),
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			'N',
			'N',
			'N',
			'N',
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @DeploymentPlanActionGroup_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		INSERT INTO [DeploymentPlanActionGroups]
		(
			[DeploymentPlan_Id],
			[DeploymentPlanActionGroup_Sequence],
			[ActionGroup_Id],
			[Deployable_Name],
			[Deployable_Application_Id],
			[Active_Indicator],
			[ActionGroup_Name],
			[ActionGroup_Description],
			[Predicate_Configuration],
			[Server_Id],
			[Server_Variable_Name],
			[Iterate_Servers_Indicator],
			[Iterate_Deployables_Indicator],
			[Parallel_Indicator],
			[OnFailure_Continue_Indicator],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@DeploymentPlan_Id,
			@DeploymentPlanActionGroup_Sequence,
			@ActionGroup_Id,
			@Deployable_Name,
			@Deployable_Application_Id,
			@Active_Indicator,
			NULLIF(@ActionGroup_Name, ''),
			NULLIF(@ActionGroup_Description, ''),
			@Predicate_Configuration,
			@Server_Id,
			NULLIF(@Server_Variable_Name, ''),
			'N',
			'N',
			'N',
			'N',
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @DeploymentPlanActionGroup_Id = SCOPE_IDENTITY()
	END

	SET @DeploymentPlanActionGroup_Id__ActionGroup_Id = CAST(COALESCE(@DeploymentPlanActionGroup_Id, 0) AS VARCHAR(50)) + ':' + CAST(COALESCE(@ActionGroup_Id, 0) AS VARCHAR(50))
	
 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateDeploymentPlanActionGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateOrUpdateAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Action_Sequence',
    /* Description_Text        */ 'Creates an action in the specified action group or updates an existing one.'
GO


IF OBJECT_ID('[Plans_CreateOrUpdateAction]') IS NOT NULL DROP PROCEDURE [Plans_CreateOrUpdateAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateOrUpdateAction]
(
	@ActionGroup_Id INT = NULL,
	@Action_Sequence INT = NULL OUT,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,
	@Long_Description VARCHAR(500) = NULL,
	@ResumeNextOnFailure_Indicator YNINDICATOR = NULL,
	@Action_Configuration XML = NULL,
	@Short_Description VARCHAR(100) = NULL,
	@Active_Indicator YNINDICATOR = NULL,
	@Retry_Count INT = NULL,
	@LogFailureAsWarning_Indicator YNINDICATOR = NULL,
	@Target_Server_Id INT = NULL,
	@Target_Server_Variable_Name VARCHAR(50) = NULL,
	@ExtensionConfiguration_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @ActionGroup_Id IS NULL
	BEGIN
		RAISERROR('ActionGroup_Id must be specified.', 16, 1)
	END

	IF (@Action_Sequence IS NULL)
	BEGIN
		SET @Action_Sequence =
			(SELECT COALESCE(MAX([Action_Sequence]), 0) + 1
		       FROM [ActionGroupActions] WHERE [ActionGroup_Id] = @ActionGroup_Id)

		INSERT INTO [ActionGroupActions]
		(
			[ActionGroup_Id],
			[Action_Sequence],
			[Server_Id],
			[Server_Variable_Name],
			[Long_Description],
			[ResumeNextOnFailure_Indicator],
			[Action_Configuration],
			[Short_Description],
			[Active_Indicator],
			[Retry_Count],
			[LogFailureAsWarning_Indicator],
			[ExtensionConfiguration_Id],
			[Target_Server_Id],
			[Target_Server_Variable_Name]
		)
		VALUES
		(
			@ActionGroup_Id,
			@Action_Sequence,
			@Server_Id,
			@Server_Variable_Name,
			@Long_Description,
			COALESCE(@ResumeNextOnFailure_Indicator, 'N'),
			@Action_Configuration,
			@Short_Description,
			COALESCE(@Active_Indicator, 'Y'),
			COALESCE(@Retry_Count, 0),
			COALESCE(@LogFailureAsWarning_Indicator, 'N'),
			@ExtensionConfiguration_Id,
			@Target_Server_Id,
			@Target_Server_Variable_Name
		)

		-- disable reexecution for anything that used the plan containing the action group
		UPDATE [BuildExecutions]
		   SET [ReexecutionState_Xml] = NULL
		 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
			                        FROM [BuildExecution_DeploymentPlans] BEDP
			                             INNER JOIN [DeploymentPlanActionGroups] DPAG
			                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
			                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	END
	ELSE
	BEGIN
		DECLARE @Previous_Active_Indicator YNINDICATOR
		DECLARE @Previous_ResumeNextOnFailure_Indicator YNINDICATOR
		
		SELECT @Previous_Active_Indicator = [Active_Indicator]
		  FROM [ActionGroupActions]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id
		   AND [Action_Sequence] = @Action_Sequence

		UPDATE [ActionGroupActions]
		   SET [Server_Id] = @Server_Id
		      ,[Server_Variable_Name] = @Server_Variable_Name
			  ,[Long_Description] = COALESCE(@Long_Description, [Long_Description])
		      ,[Action_Configuration] = COALESCE(@Action_Configuration, [Action_Configuration])
		      ,[ResumeNextOnFailure_Indicator] = COALESCE(@ResumeNextOnFailure_Indicator, [ResumeNextOnFailure_Indicator])
			  ,[Short_Description] = COALESCE(@Short_Description, [Short_Description])
			  ,[Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])
			  ,[Retry_Count] = COALESCE(@Retry_Count, [Retry_Count])
			  ,[LogFailureAsWarning_Indicator] = COALESCE(@LogFailureAsWarning_Indicator, [LogFailureAsWarning_Indicator])
			  ,[Target_Server_Id] = @Target_Server_Id
			  ,[Target_Server_Variable_Name] = @Target_Server_Variable_Name
			  ,[ExtensionConfiguration_Id] = @ExtensionConfiguration_Id
		 WHERE [ActionGroup_Id] = @ActionGroup_Id
		   AND [Action_Sequence] = @Action_Sequence
	END

	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateOrUpdateAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateOrUpdateDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'DeploymentPlan_Id',
    /* Description_Text        */ 'Creates a new deployment plan or changes the name of an existing deployment plan.'
GO


IF OBJECT_ID('[Plans_CreateOrUpdateDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_CreateOrUpdateDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateOrUpdateDeploymentPlan]
(
	@DeploymentPlan_Name NVARCHAR(50),
	@AllowLocalChanges_Indicator YNINDICATOR = NULL,
    @Application_Id INT = NULL,
	@DeploymentPlan_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @DeploymentPlan_Id IS NULL
	BEGIN

		INSERT INTO [DeploymentPlans]
		(
			[DeploymentPlan_Name],
			[AllowLocalChanges_Indicator],
            [Application_Id]
		)
		VALUES
		(
			@DeploymentPlan_Name,
			COALESCE(@AllowLocalChanges_Indicator, 'N'),
            @Application_Id
		)

		SET @DeploymentPlan_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [DeploymentPlans]
		   SET [DeploymentPlan_Name] = @DeploymentPlan_Name,
		       [AllowLocalChanges_Indicator] = COALESCE(@AllowLocalChanges_Indicator, 'N'),
               [Application_Id] = COALESCE(@Application_Id, [Application_Id])
		 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	END

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateOrUpdateDeploymentPlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreatePlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Plan_Id',
    /* Description_Text        */ 'Creates a new OtterScript deployment plan.'
GO


IF OBJECT_ID('[Plans_CreatePlan]') IS NOT NULL DROP PROCEDURE [Plans_CreatePlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreatePlan]
(
	@Plan_Name VARCHAR(255),
    @Application_Id INT,
    @Environment_Id INT,
    @PlanType_Code CHAR(1),
    @Plan_Bytes VARBINARY(MAX) = NULL,
	@Plan_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [Plans]
	(
        [Plan_Name],
        [Application_Id],
        [Environment_Id],
        [PlanType_Code],
        [Plan_Bytes],
        [ModifiedOn_Date],
        [ModifiedBy_User_Name]
	)
	VALUES
	(
        @Plan_Name,
        @Application_Id,
        @Environment_Id,
        @PlanType_Code,
        COALESCE(@Plan_Bytes, 0x),
        GETUTCDATE(),
        dbo.Session_GetUserName()
	)

	SET @Plan_Id = SCOPE_IDENTITY()

	--EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @Plan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreatePlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an action from an action group.'
GO


IF OBJECT_ID('[Plans_DeleteAction]') IS NOT NULL DROP PROCEDURE [Plans_DeleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteAction]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	DECLARE @ActionGroup_Id INT
	    SET @ActionGroup_Id = (SELECT [ActionGroup_Id]
	                             FROM [DeploymentPlanActionGroups]
	                            WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

	IF NOT EXISTS(SELECT * FROM [ActionGroupActions] WHERE [ActionGroup_Id] = @ActionGroup_Id AND [Action_Sequence] = @Action_Sequence)
	BEGIN
		/* Silently Return */
		RETURN;
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- move action to the end of the sequence before deletion to appease trigger
    DECLARE @New_Action_Sequence INT
    SELECT @New_Action_Sequence = MAX([Action_Sequence])
      FROM [ActionGroupActions]
     WHERE [ActionGroup_Id] = @ActionGroup_Id

	UPDATE [ActionGroupActions]
	   SET [Action_Sequence] = CASE
	          WHEN [ActionGroupActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] - 1
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] + 1
	          ELSE [ActionGroupActions].[Action_Sequence]
	          END
	  FROM [ActionGroupActions],
	       [ActionGroupActions] CUR,
	       [ActionGroupActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [ActionGroupActions].[ActionGroup_Id] = @ActionGroup_Id
	   AND CUR.[ActionGroup_Id] = @ActionGroup_Id
	   AND NEW.[ActionGroup_Id] = @ActionGroup_Id

	-- delete the action now that it's at the end
	DELETE [ActionGroupActions]
	WHERE [ActionGroup_Id] = @ActionGroup_Id
	  AND [Action_Sequence] = @New_Action_Sequence
	  
	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Plans_DeleteAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a deployment plan.'
GO


IF OBJECT_ID('[Plans_DeleteDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_DeleteDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteDeploymentPlan]
(
	@DeploymentPlan_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 	DECLARE @TmpDeploymentPlanActionGroups TABLE ([DeploymentPlanActionGroup_Id] INT)
	INSERT @TmpDeploymentPlanActionGroups
	SELECT [DeploymentPlanActionGroup_Id] 
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	 WHILE ((SELECT COUNT(*) FROM @TmpDeploymentPlanActionGroups) > 0 )
	 BEGIN
		DECLARE @DeploymentPlanActionGroup_Id INT
		SET @DeploymentPlanActionGroup_Id = (SELECT TOP 1 [DeploymentPlanActionGroup_Id] FROM @TmpDeploymentPlanActionGroups)

		EXEC [Plans_DeleteDeploymentPlanActionGroup] @DeploymentPlanActionGroup_Id

		DELETE @TmpDeploymentPlanActionGroups WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	 END

    UPDATE [Workflows] 
       SET [Build_DeploymentPlan_Id] = NULL 
     WHERE [Build_DeploymentPlan_Id] = @DeploymentPlan_Id

    UPDATE [WorkflowSteps] 
       SET [DeploymentPlan_Id] = NULL
     WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	DELETE [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeleteDeploymentPlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified action group.'
GO

IF OBJECT_ID('[Plans_DeleteDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_DeleteDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlanActionGroup_Sequence INT
	DECLARE @DeploymentPlan_Id INT

	 SELECT @DeploymentPlan_Id = [DeploymentPlan_Id],
	        @ActionGroup_Id = [ActionGroup_Id],
	        @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	DECLARE @Inst_Count INT
	SELECT @Inst_Count = COUNT(*)
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	-- move plan action group to the end of the sequence before deletion to appease triggers
	DECLARE @New_DeploymentPlanActionGroup_Sequence INT
	 SELECT @New_DeploymentPlanActionGroup_Sequence = MAX([DeploymentPlanActionGroup_Sequence])
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = CASE
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] = CUR.[DeploymentPlanActionGroup_Sequence] 
	               THEN NEW.[DeploymentPlanActionGroup_Sequence]
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN CUR.[DeploymentPlanActionGroup_Sequence] AND NEW.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] - 1
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN NEW.[DeploymentPlanActionGroup_Sequence] AND CUR.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] + 1
	          ELSE [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence]
	          END
	  FROM [DeploymentPlanActionGroups],
	       [DeploymentPlanActionGroups] CUR,
	       [DeploymentPlanActionGroups] NEW
	 WHERE CUR.[DeploymentPlanActionGroup_Sequence] = @DeploymentPlanActionGroup_Sequence
	   AND NEW.[DeploymentPlanActionGroup_Sequence] = @New_DeploymentPlanActionGroup_Sequence
	   AND @DeploymentPlanActionGroup_Sequence <> @New_DeploymentPlanActionGroup_Sequence
	   AND [DeploymentPlanActionGroups].[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND CUR.[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND NEW.[DeploymentPlan_Id] = @DeploymentPlan_Id

	-- is this the last PlanActionGroup for the ActionGroup?
	IF @Inst_Count = 1
	BEGIN

		DELETE [ActionGroupActions]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id

		-- delete the PlanActionGroup
		DELETE [DeploymentPlanActionGroups]
		 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

		-- delete the ActionGroup
		DELETE [ActionGroups]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id

	END
	ELSE
	BEGIN

		-- if there is only one other instance left, unshare it
		IF @Inst_Count = 2
		BEGIN
			DECLARE @Other_DeploymentPlanActionGroup_Id INT
			SELECT @Other_DeploymentPlanActionGroup_Id = [DeploymentPlanActionGroup_Id]
			  FROM [DeploymentPlanActionGroups]
			 WHERE [DeploymentPlanActionGroup_Id] <> @DeploymentPlanActionGroup_Id
			   AND [ActionGroup_Id] = @ActionGroup_Id

			UPDATE [ActionGroups]
			   SET [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
			       [Predicate_Configuration] = COALESCE(DPAG.[Predicate_Configuration], AG.[Predicate_Configuration]),
			       [ActionGroup_Description] = COALESCE(DPAG.[ActionGroup_Description], AG.[ActionGroup_Description]),
			       [Server_Id] = COALESCE(DPAG.[Server_Id], AG.[Server_Id]),
			       [Server_Variable_Name] = COALESCE(DPAG.[Server_Variable_Name], AG.[Server_Variable_Name])
			  FROM [ActionGroups] AG
			       INNER JOIN [DeploymentPlanActionGroups] DPAG
			               ON DPAG.[DeploymentPlanActionGroup_Id] = @Other_DeploymentPlanActionGroup_Id
			 WHERE AG.[ActionGroup_Id] = @ActionGroup_Id

			 UPDATE [DeploymentPlanActionGroups]
			    SET [ActionGroup_Name] = NULL,
			        [ActionGroup_Description] = NULL,
			        [Predicate_Configuration] = NULL,
                    [Server_Id] = NULL,
                    [Server_Variable_Name] = NULL
			  WHERE [DeploymentPlanActionGroup_Id] = @Other_DeploymentPlanActionGroup_Id
		END

		-- delete the PlanActionGroup
		DELETE [DeploymentPlanActionGroups]
		 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	END

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeleteDeploymentPlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeletePlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an OtterScript plan.'
GO


IF OBJECT_ID('[Plans_DeletePlan]') IS NOT NULL DROP PROCEDURE [Plans_DeletePlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeletePlan]
(
	@Plan_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 	DELETE [Plans] WHERE [Plan_Id] = @Plan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeletePlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ActionGroupActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified action.'
GO


IF OBJECT_ID('[Plans_GetAction]') IS NOT NULL DROP PROCEDURE [Plans_GetAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetAction]
(
	@ActionGroup_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	   AND [Action_Sequence] = @Action_Sequence
	IF @@ROWCOUNT=0 RAISERROR('Action %i (action group #%i) does not exist.',16,1,@Action_Sequence,@ActionGroup_Id)

END
GO

GRANT EXECUTE ON [Plans_GetAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ActionGroups,ActionGroupActions_Extended,ActionGroupUsage_Slim',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroup]
(
	@ActionGroup_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [ActionGroups]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Action group %i does not exist.', 16, 1, @ActionGroup_Id)

	SELECT *
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [ActionGroupUsage_Slim]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

END
GO

GRANT EXECUTE ON [Plans_GetActionGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ActionGroups',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets all action groups.'
GO

IF OBJECT_ID('[Plans_GetActionGroups]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroups]
AS
BEGIN
	
    SET NOCOUNT ON

    SELECT * FROM [ActionGroups]

END
GO

GRANT EXECUTE ON [Plans_GetActionGroups] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlans,ApplicationDeploymentPlans,DeploymentPlanActionGroups_Extended,ActionGroupActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns action groups and actions for a deployment plan.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlan]
(
	@DeploymentPlan_Id INT,
	@IncludeActionUsage_Indicator YNINDICATOR = 'N'
)
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT *
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	SELECT *
	  FROM [ApplicationDeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	SELECT *
	  FROM [DeploymentPlanActionGroups_Extended]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id
	 ORDER BY [DeploymentPlanActionGroup_Sequence]

	SELECT AGA.*
	  FROM [ActionGroupActions_Extended] AGA
	 WHERE AGA.[ActionGroup_Id] IN (SELECT DPAG.[ActionGroup_Id]
	                                  FROM [DeploymentPlanActionGroups] DPAG
									 WHERE DPAG.[DeploymentPlan_Id] = @DeploymentPlan_Id)
	 ORDER BY AGA.[ActionGroup_Id], AGA.[Action_Sequence]

	IF @IncludeActionUsage_Indicator = 'Y'
	BEGIN
		SELECT *
		  FROM [DeploymentPlans_ActionUsage]
		 WHERE [Uses_DeploymentPlan_Id] = @DeploymentPlan_Id
	END
	ELSE
	BEGIN
		SELECT *
		  FROM [DeploymentPlans_ActionUsage]
		 WHERE 0=1
	END

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlanActionGroups_Extended,ActionGroupActions_Extended,ActionGroupUsage_Slim,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [DeploymentPlanActionGroups_Extended]
     WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	
	DECLARE @ActionGroup_Id INT
	SET @ActionGroup_Id = (SELECT [ActionGroup_Id]
	                  FROM [DeploymentPlanActionGroups]
	                 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

	SELECT *
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [ActionGroupUsage_Slim]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	SELECT ADP.*
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlanActionGroups_Extended] DPAG
	               ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
	 WHERE DPAG.[DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlanActionGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlans,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the deployment plans in the system.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlans]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlans]
(
	@IncludeAnonymous_Indicator YNINDICATOR
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [DeploymentPlans]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR [DeploymentPlan_Name] IS NOT NULL
	 ORDER BY [DeploymentPlan_Name]

	SELECT ADP.*
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlans] DP
	               ON DP.[DeploymentPlan_Id] = ADP.[DeploymentPlan_Id]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR DP.[DeploymentPlan_Name] IS NOT NULL

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlans] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetGlobalDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns named deployment plans and optionally plans orphaned.'
GO


IF OBJECT_ID('[Plans_GetGlobalDeploymentPlans]') IS NOT NULL DROP PROCEDURE [Plans_GetGlobalDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetGlobalDeploymentPlans]  AS
BEGIN
	SET NOCOUNT ON
	
	SELECT * INTO #DeploymentPlans
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Name] IS NOT NULL
	    OR (    [DeploymentPlan_Id] NOT IN (SELECT [DeploymentPlan_Id] FROM [WorkflowSteps] WHERE [DeploymentPlan_Id] IS NOT NULL)
	        AND [DeploymentPlan_Id] NOT IN (SELECT [Build_DeploymentPlan_Id] FROM [Workflows] WHERE [Build_DeploymentPlan_Id] IS NOT NULL))

	SELECT * FROM #DeploymentPlans
         ORDER BY [DeploymentPlan_Name]
									 
END
GO

GRANT EXECUTE ON [Plans_GetGlobalDeploymentPlans] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetMostCommonActions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ 'Plans_MostCommonActions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the most commonly used actions in the system.'
GO


IF OBJECT_ID('[Plans_GetMostCommonActions]') IS NOT NULL DROP PROCEDURE [Plans_GetMostCommonActions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetMostCommonActions]
(
    @MostRecent_Count INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF (@MostRecent_Count IS NOT NULL)
        SET ROWCOUNT @MostRecent_Count

	SELECT *
      FROM [Plans_MostCommonActions]
     ORDER BY [Usage_Count] DESC

END
GO

GRANT EXECUTE ON [Plans_GetMostCommonActions] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Plans_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the specified OtterScript plan.'
GO


IF OBJECT_ID('[Plans_GetPlan]') IS NOT NULL DROP PROCEDURE [Plans_GetPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlan]
(
	@Plan_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Plans_Extended]
     WHERE [Plan_Id] = @Plan_Id

END
GO

GRANT EXECUTE ON [Plans_GetPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlanByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Plans_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the plan which has the specified name and type that is visible to the specified application.'
GO

IF OBJECT_ID('[Plans_GetPlanByName]') IS NOT NULL DROP PROCEDURE [Plans_GetPlanByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlanByName]
(
	@Application_Id INT,
	@Plan_Name VARCHAR(255),
    @PlanType_Code CHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Plans_Extended]
     WHERE ([Application_Id] = @Application_Id OR [Application_Id] IS NULL)
	   AND [Plan_Name] = @Plan_Name
       AND [PlanType_Code] = @PlanType_Code
     ORDER BY COALESCE([Application_Id], 0) DESC 

END
GO

GRANT EXECUTE ON [Plans_GetPlanByName] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Plans_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns all OtterScript plans of the specified type which are visible to the specified application.'
GO

IF OBJECT_ID('[Plans_GetPlans]') IS NOT NULL DROP PROCEDURE [Plans_GetPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlans]
(
	@Application_Id INT = NULL,
    @PlanType_Code CHAR(1) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Plans_Extended]
     WHERE ([Application_Id] IS NULL OR [Application_Id] = @Application_Id)
       AND (@PlanType_Code IS NULL OR [PlanType_Code] = @PlanType_Code)

END
GO

GRANT EXECUTE ON [Plans_GetPlans] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlansByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Plans_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all plans in the system with the specified name and type.'
GO

IF OBJECT_ID('[Plans_GetPlansByName]') IS NOT NULL DROP PROCEDURE [Plans_GetPlansByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlansByName]
(
	@Plan_Name VARCHAR(255),
    @PlanType_Code CHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Plans_Extended]
     WHERE [Plan_Name] = @Plan_Name
       AND [PlanType_Code] = @PlanType_Code

END
GO

GRANT EXECUTE ON [Plans_GetPlansByName] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of an action within a plan.'
GO


IF OBJECT_ID('[Plans_ResequenceAction]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceAction]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @ActionGroup_Id INT
	SELECT @ActionGroup_Id = [ActionGroup_Id]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	UPDATE [ActionGroupActions]
	   SET [Action_Sequence] = CASE
	          WHEN [ActionGroupActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] - 1
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] + 1
	          ELSE [ActionGroupActions].[Action_Sequence]
	          END
	  FROM [ActionGroupActions],
	       [ActionGroupActions] CUR,
	       [ActionGroupActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [ActionGroupActions].[ActionGroup_Id] = @ActionGroup_Id
	   AND CUR.[ActionGroup_Id] = @ActionGroup_Id
	   AND NEW.[ActionGroup_Id] = @ActionGroup_Id


	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

 	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceAction] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of a deployment plan action group.'
GO


IF OBJECT_ID('[Plans_ResequenceActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@New_DeploymentPlanActionGroup_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

    DECLARE @DeploymentPlanActionGroup_Sequence INT
    DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlan_Id INT

	SELECT @DeploymentPlan_Id = [DeploymentPlan_Id],
	       @ActionGroup_Id = [ActionGroup_Id],
	       @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = CASE
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] = CUR.[DeploymentPlanActionGroup_Sequence] 
	               THEN NEW.[DeploymentPlanActionGroup_Sequence]
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN CUR.[DeploymentPlanActionGroup_Sequence] AND NEW.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] - 1
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN NEW.[DeploymentPlanActionGroup_Sequence] AND CUR.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] + 1
	          ELSE [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence]
	          END
	  FROM [DeploymentPlanActionGroups],
	       [DeploymentPlanActionGroups] CUR,
	       [DeploymentPlanActionGroups] NEW
	 WHERE CUR.[DeploymentPlanActionGroup_Sequence] = @DeploymentPlanActionGroup_Sequence
	   AND NEW.[DeploymentPlanActionGroup_Sequence] = @New_DeploymentPlanActionGroup_Sequence
	   AND @DeploymentPlanActionGroup_Sequence <> @New_DeploymentPlanActionGroup_Sequence
	   AND [DeploymentPlanActionGroups].[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND CUR.[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND NEW.[DeploymentPlan_Id] = @DeploymentPlan_Id

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_SetOtterScript',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the Otter script of a plan.'
GO


IF OBJECT_ID('[Plans_SetOtterScript]') IS NOT NULL DROP PROCEDURE [Plans_SetOtterScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_SetOtterScript]
(
	@DeploymentPlan_Id INT,
	@OtterScript_Bytes VARBINARY(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [DeploymentPlans]
	   SET [OtterScript_Bytes] = @OtterScript_Bytes
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_SetOtterScript] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdateActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the specified action group.'
GO


IF OBJECT_ID('[Plans_UpdateActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdateActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdateActionGroup]
(
	@ActionGroup_Id INT,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Old_Predicate_Configuration XML
	SELECT @Old_Predicate_Configuration = [Predicate_Configuration]
	  FROM [ActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	UPDATE [ActionGroups]
	   SET [ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
		  ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
	      ,[Predicate_Configuration] = @Predicate_Configuration
	      ,[ModifiedOn_Date] = GETUTCDATE()
	      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	      ,[Server_Id] = @Server_Id
	      ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

 	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdateActionGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdateDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
     /* Description_Text        */ 'Updates the specified deployment plan action group.'
GO

IF OBJECT_ID('[Plans_UpdateDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdateDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdateDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@Active_Indicator YNINDICATOR,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50),
	@Deployable_Name VARCHAR(50),
	@Deployable_Application_Id INT,
	@Iterate_Servers_Indicator YNINDICATOR,
	@Iterate_Deployables_Indicator YNINDICATOR,
	@Parallel_Indicator YNINDICATOR,
	@OnSuccess_DeploymentPlanActionGroup_Id INT,
	@OnFailure_DeploymentPlanActionGroup_Id INT,
	@OnFailure_Continue_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @DeploymentPlan_Id INT
	SET @DeploymentPlan_Id = (SELECT [DeploymentPlan_Id]
	                            FROM [DeploymentPlanActionGroups]
	                           WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

    UPDATE [DeploymentPlanActionGroups]
       SET [Active_Indicator] = @Active_Indicator
          ,[ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
          ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
          ,[Predicate_Configuration] = @Predicate_Configuration
          ,[Server_Id] = @Server_Id
          ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
          ,[Deployable_Name] = NULLIF(@Deployable_Name, '')
          ,[Deployable_Application_Id] = @Deployable_Application_Id
          ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
          ,[ModifiedOn_Date] = GETUTCDATE()
          ,[Iterate_Servers_Indicator] = @Iterate_Servers_Indicator
          ,[Iterate_Deployables_Indicator] = @Iterate_Deployables_Indicator
          ,[Parallel_Indicator] = @Parallel_Indicator
          ,[OnSuccess_DeploymentPlanActionGroup_Id] = @OnSuccess_DeploymentPlanActionGroup_Id
          ,[OnFailure_DeploymentPlanActionGroup_Id] = @OnFailure_DeploymentPlanActionGroup_Id
          ,[OnFailure_Continue_Indicator] = @OnFailure_Continue_Indicator
     WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdateDeploymentPlanActionGroup] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdatePlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates an OtterScript deployment plan.'
GO


IF OBJECT_ID('[Plans_UpdatePlan]') IS NOT NULL DROP PROCEDURE [Plans_UpdatePlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdatePlan]
(
	@Plan_Id INT,
    @Plan_Name VARCHAR(255),
    @Plan_Bytes VARBINARY(MAX),
    @Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	UPDATE [Plans]
       SET [Plan_Name] = @Plan_Name,
           [Plan_Bytes] = @Plan_Bytes,
           [ModifiedOn_Date] = GETUTCDATE(),
           [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
           [Environment_Id] = @Environment_Id
     WHERE [Plan_Id] = @Plan_Id

	--EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @Plan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdatePlan] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_AddApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Approval_Id',
    /* Description_Text        */ 'Adds an approval for a promotion.'
GO

IF OBJECT_ID('[Promotions_AddApproval]') IS NOT NULL DROP PROCEDURE [Promotions_AddApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_AddApproval]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@PipelineStage_Name VARCHAR(50),
	@Requirement_Id INT,
	@RequirementType_Code CHAR(2),
	@User_Name USERNAME,
	@Requirement_Description NVARCHAR(1000),
	@Comments_Text NVARCHAR(1000) = NULL,
	@Approval_Id INT = NULL OUT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions_Extended]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [PipelineStage_Name] = @PipelineStage_Name)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot add approval.', 16, 1)
	END

	DECLARE @Build_Id INT
	SELECT @Build_Id = [Build_Id]
	  FROM [Builds_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	INSERT INTO [BuildApprovalsReceived]
	(
		[Build_Id],
		[PipelineStage_Name],
		[Original_Requirement_Id],
		[RequirementType_Code],
		[User_Name],
		[Comments_Text],
		[Received_Date],
		[Requirement_Description]
	)
	VALUES
	(
		@Build_Id,
		@PipelineStage_Name,
		@Requirement_Id,
		@RequirementType_Code,
		@User_Name,
		NULLIF(@Comments_Text, ''),
		GETUTCDATE(),
		@Requirement_Description
	)

	SET @Approval_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'BLDAPR', @Application_Id, @Approval_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_AddApproval] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Promotions_CreateOrUpdateRequirement]') IS NOT NULL DROP PROCEDURE [Promotions_CreateOrUpdateRequirement]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_DeleteApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an approval for a promotion.'
GO

IF OBJECT_ID('[Promotions_DeleteApproval]') IS NOT NULL DROP PROCEDURE [Promotions_DeleteApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_DeleteApproval]
(
	@Approval_Id INT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions] BP
				     INNER JOIN [BuildApprovalsReceived] BAR
					         ON BAR.[Build_Id] = BP.[Build_Id]
							AND BAR.[PipelineStage_Name] = BP.[PipelineStage_Name]
			   WHERE BAR.[Approval_Id] = @Approval_Id)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot delete approval.', 16, 1)
	END

	DELETE [BuildApprovalsReceived]
	 WHERE [Approval_Id] = @Approval_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_DeleteApproval] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Promotions_DeleteRequirement]') IS NOT NULL DROP PROCEDURE [Promotions_DeleteRequirement]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetApprovalsReceived',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildApprovalsReceived_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all requirements and approvals received for the specified promotion.'
GO

IF OBJECT_ID('[Promotions_GetApprovalsReceived]') IS NOT NULL DROP PROCEDURE [Promotions_GetApprovalsReceived]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetApprovalsReceived]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@PipelineStage_Name VARCHAR(50),
	@Approval_Id INT = NULL
) AS BEGIN

SET NOCOUNT ON

	IF @Approval_Id IS NULL
	BEGIN

		SELECT *
		  FROM [BuildApprovalsReceived_Extended]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND (@PipelineStage_Name IS NULL OR [PipelineStage_Name] = @PipelineStage_Name)

	END
	ELSE
	BEGIN

		SELECT *
		  FROM [BuildApprovalsReceived_Extended]
		 WHERE [Approval_Id] = @Approval_Id

	END

END
GO

GRANT EXECUTE ON [Promotions_GetApprovalsReceived] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Promotions_GetRequirement]') IS NOT NULL DROP PROCEDURE [Promotions_GetRequirement]
GO



GO
IF OBJECT_ID('[Promotions_GetRequirements]') IS NOT NULL DROP PROCEDURE [Promotions_GetRequirements]
GO



GO
IF OBJECT_ID('[Promotions_GetRequirementsToVerify]') IS NOT NULL DROP PROCEDURE [Promotions_GetRequirementsToVerify]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetScheduled',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotionsScheduled_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets scheduled promotions, optionally filtered by promotion and date.'
GO

IF OBJECT_ID('[Promotions_GetScheduled]') IS NOT NULL DROP PROCEDURE [Promotions_GetScheduled]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetScheduled]
(
	@Application_Id INT = NULL,
	@Release_Number RELEASENUMBER = NULL,
	@Build_Number BUILDNUMBER = NULL,
	@PipelineStage_Name VARCHAR(50) = NULL,
	@Before_Date DATETIME = NULL
) AS BEGIN

SET NOCOUNT ON

	IF @Application_Id IS NULL
	BEGIN

		SELECT BPS.*
		  FROM [BuildPromotionsScheduled_Extended] BPS
		       INNER JOIN [Builds] B
			           ON B.[Build_Id] = BPS.[Build_Id]
		       LEFT JOIN [BuildPromotions] BP
			          ON BP.[Build_Id] = BPS.[Build_Id]
					 AND BP.[PipelineStage_Name] = BPS.[PipelineStage_Name]
		 WHERE @Before_Date IS NULL OR BPS.[PromoteOn_Date] <= @Before_Date
		   AND BP.[Promotion_Id] IS NULL
		   AND B.[BuildStatus_Name] = 'Active'
		 ORDER BY BPS.[PromoteOn_Date] DESC

	END
	ELSE
	BEGIN

		SELECT BPS.*
		  FROM [BuildPromotionsScheduled_Extended] BPS
		       INNER JOIN [Builds] B
			           ON B.[Build_Id] = BPS.[Build_Id]
			   INNER JOIN [Releases] R
			           ON R.[Release_Id] = B.[Release_Id]
		       LEFT JOIN [BuildPromotions] BP
			          ON BP.[Build_Id] = BPS.[Build_Id]
					 AND BP.[PipelineStage_Name] = BPS.[PipelineStage_Name]
		 WHERE R.[Application_Id] = @Application_Id
		   AND R.[Release_Number] = COALESCE(@Release_Number, R.[Release_Number])
		   AND B.[Build_Number] = COALESCE(@Build_Number, B.[Build_Number])
		   AND BPS.[PipelineStage_Name] = COALESCE(@PipelineStage_Name, BPS.[PipelineStage_Name])
		   AND (@Before_Date IS NULL OR BPS.[PromoteOn_Date] <= @Before_Date)
		   AND BP.[Promotion_Id] IS NULL
		   AND B.[BuildStatus_Name] = 'Active'
		 ORDER BY BPS.[PromoteOn_Date] DESC

	END

END
GO

GRANT EXECUTE ON [Promotions_GetScheduled] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_SchedulePromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Schedules a promotion or removes a previously scheduled promotion.'
GO

IF OBJECT_ID('[Promotions_SchedulePromotion]') IS NOT NULL 
	DROP PROCEDURE [Promotions_SchedulePromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_SchedulePromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@PipelineStage_Name VARCHAR(50),
	@PromoteOn_Date DATETIME
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions_Extended]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [PipelineStage_Name] = @PipelineStage_Name)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot schedule promotion.', 16, 1)
	END

	DECLARE @Build_Id INT
	SELECT @Build_Id = B.[Build_Id]
	  FROM [Builds] B
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
	 WHERE R.[Application_Id] = @Application_Id
	   AND R.[Release_Number] = @Release_Number
	   AND B.[Build_Number] = @Build_Number

	IF @PromoteOn_Date IS NULL
	BEGIN

		DELETE [BuildPromotionsScheduled]
	     WHERE [Build_Id] = @Build_Id
		   AND [PipelineStage_Name] = @PipelineStage_Name

	END
	ELSE IF EXISTS(SELECT *
	            FROM [BuildPromotionsScheduled]
	           WHERE [Build_Id] = @Build_Id
		         AND [PipelineStage_Name] = @PipelineStage_Name)
	BEGIN

		UPDATE [BuildPromotionsScheduled]
		   SET [PromoteOn_Date] = @PromoteOn_Date
	     WHERE [Build_Id] = @Build_Id
		   AND [PipelineStage_Name] = @PipelineStage_Name

	END
	ELSE
	BEGIN

		INSERT INTO [BuildPromotionsScheduled]
		(
			[Build_Id],
			[PipelineStage_Name],
			[PromoteOn_Date]
		)
		VALUES
		(
			@Build_Id,
			@PipelineStage_Name,
			@PromoteOn_Date
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_SchedulePromotion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_SetAutomaticRequirementStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Records the status of an automatic promotion requirement.'
GO

IF OBJECT_ID('[Promotions_SetAutomaticRequirementStatus]') IS NOT NULL DROP PROCEDURE [Promotions_SetAutomaticRequirementStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_SetAutomaticRequirementStatus]
(
	@Requirement_Id INT,
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@PipelineStage_Name VARCHAR(50),
	@Applicable_Indicator YNINDICATOR,
	@Met_Indicator YNINDICATOR,
	@Requirement_Description NVARCHAR(1000)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	SELECT @Build_Id = [Build_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	IF EXISTS(SELECT *
	            FROM [BuildApprovalsReceived]
			   WHERE [Build_Id] = @Build_Id
		   AND [PipelineStage_Name] = @PipelineStage_Name
				 AND [Original_Requirement_Id] = @Requirement_Id
				 AND [RequirementType_Code] IN ('AW', 'AE'))
	BEGIN


		UPDATE [BuildApprovalsReceived]
		   SET [Received_Date] = GETUTCDATE(),
		       [Comments_Text] = CASE WHEN @Applicable_Indicator = 'N' THEN 'not applicable'
                                      WHEN @Met_Indicator = 'Y' THEN 'met' 
                                 ELSE 'not met' END,
			   [Requirement_Description] = @Requirement_Description
		 WHERE [Build_Id] = @Build_Id
		   AND [PipelineStage_Name] = @PipelineStage_Name
		   AND [Original_Requirement_Id] = @Requirement_Id
		   AND [RequirementType_Code] IN ('AW', 'AE')

	END
	ELSE
	BEGIN

		INSERT INTO [BuildApprovalsReceived]
		(
			[Build_Id],
			[PipelineStage_Name],
			[Original_Requirement_Id],
			[RequirementType_Code],
			[Comments_Text],
			[Received_Date],
			[Requirement_Description]
		)
		VALUES
		(
			@Build_Id,
		    @PipelineStage_Name,
			@Requirement_Id,
			'AW',
			CASE WHEN @Applicable_Indicator = 'N' THEN 'not applicable'
                 WHEN @Met_Indicator = 'Y' THEN 'met' 
             ELSE 'not met' END,
			GETUTCDATE(),
			@Requirement_Description
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_SetAutomaticRequirementStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_CreateOrUpdateProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Provider_Id',
    /* Description_Text        */ 'Creates a new provider or updates an existing one.'
GO


IF OBJECT_ID('[Providers_CreateOrUpdateProvider]') IS NOT NULL DROP PROCEDURE [Providers_CreateOrUpdateProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_CreateOrUpdateProvider]
(
	@Provider_Id INT = NULL OUT,
	@ProviderType_Code CHAR(1),
	@ExecuteOn_Server_Id INT,
	@Provider_Name VARCHAR(50),
	@Provider_Description VARCHAR(100),
	@Provider_Configuration VARCHAR(MAX),
	@Internal_Indicator YNINDICATOR	 = 'N'
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    -- BitChecker recipe checks for the existence of a provider before attempting to create one, but
    -- Providers_GetProviders doesn't return internal providers - so without a new SDK it can never know
    -- if an example provider was created already. This hack will ensure the recipe can be run more than once.
    IF (@Provider_Id IS NULL AND @Internal_Indicator = 'Y')
    BEGIN

        SELECT @Provider_Id = [Provider_Id]
          FROM [Providers]
         WHERE [Provider_Name] = @Provider_Name
           AND [ProviderType_Code] = @ProviderType_Code

            IF (@Provider_Id IS NOT NULL)
            BEGIN
                COMMIT
                RETURN
            END

    END

	IF (@Provider_Id IS NULL) 
	BEGIN

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		INSERT INTO [Providers]
            (
				[ProviderType_Code],
				[Provider_Name],
				[ExecuteOn_Server_Id],
				[Provider_Description],
				[Provider_Configuration],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
				[Internal_Indicator]
			)
		VALUES
            (
				@ProviderType_Code,
				@Provider_Name,
				@ExecuteOn_Server_Id,
				@Provider_Description,
				@Provider_Configuration,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				@Internal_Indicator
			)
		SET @Provider_Id = SCOPE_IDENTITY()

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSNEW', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDNEW', NULL, @Provider_Id
	END
	ELSE
	BEGIN
		IF EXISTS(SELECT * FROM [Providers]
		           WHERE [Provider_Id] = @Provider_Id
		             AND [Internal_Indicator] = 'Y')
		    RAISERROR('A provider marked as internal may not be edited.', 16, 1)

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [Provider_Id] <> @Provider_Id AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		UPDATE [Providers]
		   SET [ProviderType_Code] = @ProviderType_Code,
		       [Provider_Name] = @Provider_Name,
			   [ExecuteOn_Server_Id] = @ExecuteOn_Server_Id,
			   [Provider_Description] = @Provider_Description,
			   [Provider_Configuration] = @Provider_Configuration,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Provider_Id] = @Provider_Id

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSMOD', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDMOD', NULL, @Provider_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_CreateOrUpdateProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_DeleteProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified provider'
GO


IF OBJECT_ID('[Providers_DeleteProvider]') IS NOT NULL DROP PROCEDURE [Providers_DeleteProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_DeleteProvider]
(
	@Provider_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ProviderType_Code CHAR(1), @Provider_Name SQL_VARIANT
		SELECT @Provider_Name = [Provider_Name],
			   @ProviderType_Code = [ProviderType_Code]
		  FROM [Providers]
		 WHERE [Provider_Id] = @Provider_Id

	IF @ProviderType_Code = 'B' BEGIN
		RAISERROR('Database providers may not be deleted with this procedure.', 16, 1)
		END

	ELSE IF @ProviderType_Code = 'I' BEGIN
		UPDATE [Applications] 
		   SET [IssueTracking_Provider_Id] = NULL
		 WHERE [IssueTracking_Provider_Id] = @Provider_Id
		END
	
	ELSE IF @ProviderType_Code = 'S' BEGIN
		DELETE [BuildSchedules]
		 WHERE [SourceControl_Provider_Id] = @Provider_Id
		END

	ELSE IF @ProviderType_Code IS NOT NULL BEGIN
		RAISERROR('Deleting Provider type %s is not be supported.', 16, 1, @ProviderType_Code)
		END

	DELETE [Providers]
	 WHERE [Provider_Id] = @Provider_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'PVDDEL', NULL, @Provider_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_DeleteProvider] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified provider'
GO


IF OBJECT_ID('[Providers_GetProvider]') IS NOT NULL DROP PROCEDURE [Providers_GetProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProvider]
(
	@Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [Provider_Id] = @Provider_Id

END
GO

GRANT EXECUTE ON [Providers_GetProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProviders',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of providers in the system based on the specified criteria'
GO

IF OBJECT_ID('[Providers_GetProviders]') IS NOT NULL DROP PROCEDURE [Providers_GetProviders]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProviders]
(
	@ProviderType_Code CHAR(1) = NULL,
	@Application_Id INT = NULL,
	@Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [ProviderType_Code] = COALESCE(@ProviderType_Code, [ProviderType_Code])
	   AND [Internal_Indicator] = 'N'	 
	 ORDER BY [Provider_Name]
END
GO

GRANT EXECUTE ON [Providers_GetProviders] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Cancels the specified release.'
GO


IF OBJECT_ID('[Releases_CancelRelease]') IS NOT NULL DROP PROCEDURE [Releases_CancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@CancelledReason_Text VARCHAR(500)
)
AS
BEGIN

SET NOCOUNT ON

	IF EXISTS(SELECT * 
	            FROM [Releases] 
			   WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
			     AND [ReleaseStatus_Name] IN ('Deployed', 'Cancelled'))
	BEGIN
		RETURN
	END



BEGIN TRY
	BEGIN TRANSACTION


    UPDATE B
       SET B.[BuildStatus_Name] = 'Rejected'
          ,B.[RejectedBy_User_Name] = dbo.Session_GetUserName()
          ,B.[Rejected_Date] = GETUTCDATE()
          ,B.[Rejection_Notes] = 'Release was cancelled'
	  FROM [Builds] B
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = B.[Release_Id]
     WHERE R.[Application_Id] = @Application_Id
       AND R.[Release_Number] = @Release_Number
       AND B.[BuildStatus_Name] <> 'Rejected'
	 
    UPDATE [Releases]
       SET [ReleaseStatus_Name] = 'Cancelled',
		   [CancelledBy_User_Name] = dbo.Session_GetUserName(),
		   [CancelledReason_Text] = @CancelledReason_Text,
           [CancelledOn_Date] = GETUTCDATE()
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number

	EXEC [Events_RaiseEvent] 'RELCNL', @Application_Id, @Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CancelRelease] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a release or updates an existing one.'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateRelease]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Pipeline_Id INT,
	@Target_Date DATETIME,
	@Release_Name VARCHAR(50),
    @ReleaseTemplate_Name VARCHAR(50) = NULL
) AS
BEGIN

IF REPLACE(@Release_Name,' ','') = ''
    SET @Release_Name = NULLIF(REPLACE(@Release_Name,' ',''),'')

DECLARE @Pipeline_Name NVARCHAR(50)
    SET @Pipeline_Name = (SELECT [Pipeline_Name] FROM [Pipelines] WHERE [Pipeline_Id] = @Pipeline_Id)

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Release_Id INT
    SELECT @Release_Id = [Release_Id] FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	SET @Pipeline_Name = (SELECT [Pipeline_Name]
	                        FROM [Pipelines]
						   WHERE [Pipeline_Id] = @Pipeline_Id)

	IF @Release_Id IS NULL
	BEGIN

		DECLARE @Release_Sequence INT
			SET @Release_Sequence = COALESCE((SELECT MAX([Release_Sequence])
			                                    FROM [Releases]
									           WHERE [Application_Id] = @Application_Id), 0) + 1

		INSERT INTO [Releases]
		(
			[Application_Id],
			[Release_Number],
			[Pipeline_Id],
			[ReleaseStatus_Name],
			[Target_Date],
			[Release_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
            [Release_Sequence],
			[Original_Workflow_Name],
            [ReleaseTemplate_Name]
		)
		VALUES
		(
			@Application_Id,
			@Release_Number,
			@Pipeline_Id,
			'Active',
			@Target_Date,
			NULLIF(LTRIM(RTRIM(@Release_Name)), ''),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Release_Sequence,
			@Pipeline_Name,
            @ReleaseTemplate_Name
		)

        SET @Release_Id = SCOPE_IDENTITY()

		INSERT INTO [ReleaseConfigurationFiles]
		(
			[Release_Id],
			[ConfigurationFile_Id],
			[Version_Number]
		)
		SELECT @Release_Id,
			   CFV.[ConfigurationFile_Id],
			   MAX(CFV.[Version_Number])
		  FROM [ConfigurationFileVersions] CFV
		       INNER JOIN [ConfigurationFiles] CF
			           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			   INNER JOIN [Deployables] D
			           ON D.[Deployable_Id] = CF.[Deployable_Id]
		 WHERE D.[Application_Id] = @Application_Id
		 GROUP BY CFV.[ConfigurationFile_Id]

		EXEC [Events_RaiseEvent] 'RELNEW', @Application_Id, @Release_Number

	END
	ELSE
	BEGIN

		UPDATE [Releases]
		   SET [Release_Name] = NULLIF(LTRIM(RTRIM(@Release_Name)), ''),
			   [Pipeline_Id] = COALESCE(@Pipeline_Id, [Pipeline_Id]),
			   [Target_Date] = @Target_Date,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Original_Workflow_Name] = COALESCE(@Pipeline_Name, [Original_Workflow_Name]),
               [ReleaseTemplate_Name] = @ReleaseTemplate_Name
		 WHERE [Release_Id] = @Release_Id

		EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateRelease] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateReleaseDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the furthest build number for the release.'
GO


IF OBJECT_ID('[Releases_CreateOrUpdateReleaseDeployable]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateReleaseDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateReleaseDeployable]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Referenced_Application_Id INT,
	@Referenced_Release_Number RELEASENUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	DECLARE @Referenced_Release_Id INT
	SELECT @Referenced_Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = COALESCE(@Referenced_Application_Id, @Application_Id)
	   AND [Release_Number] = @Referenced_Release_Number

	IF EXISTS(SELECT * FROM [ReleaseDeployables] WHERE [Release_Id] = @Release_Id AND [Deployable_Id] = @Deployable_Id)
	BEGIN

		UPDATE [ReleaseDeployables]
		   SET [InclusionType_Code] = CASE WHEN @Referenced_Release_Id IS NULL THEN 'I' ELSE 'R' END,
		       [Referenced_Release_Id] = @Referenced_Release_Id
		 WHERE [Release_Id] = @Release_Id
		   AND [Deployable_Id] = @Deployable_Id

	END
	ELSE
	BEGIN

		INSERT INTO [ReleaseDeployables]
		(
			[Release_Id],
			[Deployable_Id],
			[InclusionType_Code],
			[Referenced_Release_Id]
		)
		VALUES
		(
			@Release_Id,
			@Deployable_Id,
			CASE WHEN @Referenced_Release_Id IS NULL THEN 'I' ELSE 'R' END,
			@Referenced_Release_Id
		)

	END

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateReleaseDeployable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseNote_Sequence',
    /* Description_Text        */ 'Creates a new release note, or updates an existing one.'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateReleaseNote]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT = NULL OUT,
	@Build_Number BUILDNUMBER = NULL,
	@Promotion_Id INT = NULL,
	@Notes_Text VARCHAR(MAX)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	DECLARE @Release_Id INT

	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	IF @ReleaseNote_Sequence IS NOT NULL

		UPDATE [ReleaseNotes]
		   SET [Notes_Text] = @Notes_Text
		      ,[Promotion_Id] = @Promotion_Id
		      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
		      ,[ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Release_Id] = @Release_Id
		   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
	
	ELSE BEGIN

		SET @ReleaseNote_Sequence = COALESCE(
		            (SELECT MAX([ReleaseNote_Sequence]) + 1
		               FROM [ReleaseNotes]
		              WHERE [Release_Id] = @Release_Id), 1)

		INSERT INTO [ReleaseNotes]
		(
			[Release_Id],
			[ReleaseNote_Sequence],
			[Promotion_Id],
			[Notes_Text],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Release_Id,
			@ReleaseNote_Sequence,
			@Promotion_Id,
			@Notes_Text,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH
		 

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateReleaseNote] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_DeleteReleaseDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Removes a deployable from the release.'
GO


IF OBJECT_ID('[Releases_DeleteReleaseDeployable]') IS NOT NULL DROP PROCEDURE [Releases_DeleteReleaseDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_DeleteReleaseDeployable]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	DELETE [ReleaseDeployables]
	 WHERE [Release_Id] = @Release_Id
	   AND [Deployable_Id] = @Deployable_Id

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_DeleteReleaseDeployable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_DeleteReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a release note from the system.'
GO

IF OBJECT_ID('[Releases_DeleteReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_DeleteReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_DeleteReleaseNote]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	DELETE RN
	  FROM [ReleaseNotes] RN
	       INNER JOIN [Releases] R
		           ON R.[Release_Id] = RN.[Release_Id]
	 WHERE R.[Application_Id] = @Application_Id
	   AND R.[Release_Number] = @Release_Number
	   AND RN.[ReleaseNote_Sequence] = @ReleaseNote_Sequence

END
GO

GRANT EXECUTE ON [Releases_DeleteReleaseNote] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetBuildExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of build executions for the specified release'
GO

IF OBJECT_ID('[Releases_GetBuildExecutions]') IS NOT NULL
	DROP PROCEDURE [Releases_GetBuildExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetBuildExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

END
GO

GRANT EXECUTE ON [Releases_GetBuildExecutions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetDependentReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all releases with referenced deployables from a specified release.'
GO

IF OBJECT_ID('[Releases_GetDependentReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetDependentReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetDependentReleases]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 
	SELECT RE.*
	  FROM [Releases_Extended] RE
	 WHERE EXISTS 
           (
	         SELECT *
			   FROM [ReleaseDeployables] RD
			        INNER JOIN [Releases] R
					        ON R.[Release_Id] = RD.[Release_Id]
					LEFT JOIN [Releases] R_REF
					       ON R_REF.[Release_Id] = RD.[Referenced_Release_Id]
	          WHERE RE.[Application_Id] = R.[Application_Id]
	            AND RE.[Release_Number] = R.[Release_Number]
	            AND R_REF.[Application_Id] = @Application_Id
	            AND R_REF.[Release_Number] = @Release_Number
	            
	       )
	   AND NOT ([Application_Id] = @Application_Id AND 
	            [Release_Number] = @Release_Number)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_GetDependentReleases] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Releases_GetNextReleaseNumber]') IS NOT NULL DROP PROCEDURE [Releases_GetNextReleaseNumber]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Releases_Extended,ReleaseDeployables_Extended,ReleaseConfigurationFiles',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified release, the deployables associated with the release, and the configuration file versions associated with the release.'
GO

IF OBJECT_ID('[Releases_GetRelease]') IS NOT NULL DROP PROCEDURE [Releases_GetRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Release_Id INT
	SELECT @Release_Id = [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	SELECT * 
	  FROM [Releases_Extended]
     WHERE [Release_Id] = @Release_Id

	SELECT *
	  FROM [ReleaseDeployables_Extended]
     WHERE [Release_Id] = @Release_Id
	ORDER BY [Deployable_Name]

	SELECT *
	  FROM [ReleaseConfigurationFiles]
     WHERE [Release_Id] = @Release_Id

END
GO

GRANT EXECUTE ON [Releases_GetRelease] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseById',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified release by ID.'
GO

IF OBJECT_ID('[Releases_GetReleaseById]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseById]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseById]
(
	@Release_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	SELECT * 
	  FROM [Releases_Extended]
     WHERE [Release_Id] = @Release_Id

END
GO

GRANT EXECUTE ON [Releases_GetReleaseById] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseDeployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of deployables associated with any release for an application.'
GO


IF OBJECT_ID('[Releases_GetReleaseDeployables]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ReleaseDeployables_Extended]
	 WHERE @Application_Id IS NULL OR [Application_Id] = @Application_Id
END
GO

GRANT EXECUTE ON [Releases_GetReleaseDeployables] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a release note from the system.'
GO

IF OBJECT_ID('[Releases_GetReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNote]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	SELECT *
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
END
GO

GRANT EXECUTE ON [Releases_GetReleaseNote] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNotes',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the release notes that match the specified criteria.'
GO

IF OBJECT_ID('[Releases_GetReleaseNotes]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNotes]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNotes]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Promotion_Id INT = NULL
) AS BEGIN

	SELECT *
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND (@Promotion_Id IS NULL OR [Promotion_Id] = @Promotion_Id)

END
GO

GRANT EXECUTE ON [Releases_GetReleaseNotes] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of releases that match the specified criteria.'
GO


IF OBJECT_ID('[Releases_GetReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleases]
(
	@Application_Id INT,
	@ReleaseStatus_Name VARCHAR(9) = NULL,
	@Release_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Release_Count IS NOT NULL
		SET ROWCOUNT @Release_Count

	SELECT * 
	  FROM [Releases_Extended]
     WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@ReleaseStatus_Name IS NULL OR [ReleaseStatus_Name] = @ReleaseStatus_Name)
      ORDER BY CASE WHEN @Application_Id IS NULL THEN [Latest_Promoted_Date] ELSE [Release_Sequence] END DESC
               

END
GO

GRANT EXECUTE ON [Releases_GetReleases] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseSlim',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Releases_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified release.'
GO


IF OBJECT_ID('[Releases_GetReleaseSlim]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseSlim]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseSlim]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Releases_Slim]
     WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

END
GO

GRANT EXECUTE ON [Releases_GetReleaseSlim] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleasesSlim',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of releases that match the specified criteria with minimal join data.'
GO


IF OBJECT_ID('[Releases_GetReleasesSlim]') IS NOT NULL DROP PROCEDURE [Releases_GetReleasesSlim]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleasesSlim]
(
	@Application_Id INT,
	@ReleaseStatus_Name VARCHAR(9) = NULL,
	@Release_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Release_Count IS NOT NULL
		SET ROWCOUNT @Release_Count

	SELECT * 
	  FROM [Releases_Slim]
     WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@ReleaseStatus_Name IS NULL OR [ReleaseStatus_Name] = @ReleaseStatus_Name)
      ORDER BY [Release_Sequence] DESC
               

END
GO

GRANT EXECUTE ON [Releases_GetReleasesSlim] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('Releases_GetRequiredPromotions') IS NOT NULL DROP PROCEDURE Releases_GetRequiredPromotions
GO



GO
IF OBJECT_ID('[Releases_GetSortableReleaseNumber]') IS NOT NULL DROP PROCEDURE [Releases_GetSortableReleaseNumber]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_PurgeReleaseData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire release and all associated data from the system.'
GO

IF OBJECT_ID('[Releases_PurgeReleaseData]') IS NOT NULL DROP PROCEDURE [Releases_PurgeReleaseData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_PurgeReleaseData]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Release_Id INT
	DECLARE @Release_Sequence BIGINT
	SELECT @Release_Id = [Release_Id],
	       @Release_Sequence = [Release_Sequence]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	IF @Application_Name IS NULL
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)
	
	IF @Release_Id IS NULL
		RAISERROR('Release %s of application %i does not exist.', 16, 1, @Release_Number, @Application_Id)

	DECLARE @Builds TABLE ([Build_Id] INT NOT NULL)
	INSERT INTO @Builds ([Build_Id])
	SELECT [Build_Id]
	  FROM [Builds]
	 WHERE [Release_Id] = @Release_Id

	DECLARE @Promotions TABLE ([Promotion_Id] INT NOT NULL)
	INSERT INTO @Promotions
	SELECT [Promotion_Id]
	  FROM [BuildPromotions]
	 WHERE [Build_Id] IN (SELECT [Build_Id] FROM @Builds)

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)

	DECLARE @ChangeControls TABLE ([ChangeControl_Id] INT NOT NULL)
	INSERT INTO @ChangeControls ([ChangeControl_Id])
	SELECT [ChangeControl_Id]
	  FROM [ChangeControls]
	 WHERE [Release_Id] = @Release_Id

	DELETE
	  FROM [VariableDefinitions]
	 WHERE [Release_Id] = @Release_Id
	    OR [Build_Id] IN (SELECT [Build_Id] FROM @Builds)
		OR [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)
		OR [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [ReleaseDeployables]
	 WHERE [Release_Id] = @Release_Id
	    OR [Referenced_Release_Id] = @Release_Id

	DECLARE @BuildExecution_DeploymentPlans TABLE ([BuildExecution_DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_DeploymentPlans ([BuildExecution_DeploymentPlan_Id])
	SELECT [BuildExecution_DeploymentPlan_Id]
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DECLARE @BuildExecution_ActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_ActionGroupActions ([BuildExecution_ActionGroupAction_Id])
	SELECT [BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

    DELETE
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN  (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [BuildExecution_DeploymentPlanActionGroups]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	/* Have to delete change script executions before Build Executions... */
	DELETE [DatabaseChangeScriptLegacyExecutions]
	 WHERE [Script_Id] IN (SELECT [Script_Id] FROM [DatabaseChangeScripts] WHERE [Release_Id] = @Release_Id)
	    OR [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)


	DELETE
	  FROM [Executions]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE
	  FROM [BuildPromotions]
	 WHERE [Promotion_Id] IN (SELECT [Promotion_Id] FROM @Promotions)

    DELETE
      FROM [BuildArtifacts]
     WHERE [Build_Id] IN (SELECT [Build_Id] FROM @Builds)

	DELETE
	  FROM [Builds]
	 WHERE [Build_Id] IN (SELECT [Build_Id] FROM @Builds)


	 /* 2d) Change Controls */
	DELETE
	  FROM [ChangeControlEnvironments]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE
	  FROM [ChangeControlPerformances]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE
	  FROM [ChangeControls]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)


	/* 3) Release */
	DELETE [Releases]
	 WHERE [Release_Id] = @Release_Id

    UPDATE [Releases]
       SET [Release_Sequence] = [Release_Sequence] - 1
     WHERE [Application_Id] = @Application_Id
       AND [Release_Sequence] > @Release_Sequence

	 /* Raise Event */
	EXEC [Events_RaiseEvent] 'RELDEL', @Application_Id, @Application_Name, @Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_PurgeReleaseData] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ReleaseExists',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseExists_Indicator,NewerDeployedReleaseExists_Indicator',
    /* Description_Text        */ 'Determines whether a release already exists, and also whether a newer release of an application exists'
GO

IF OBJECT_ID('[Releases_ReleaseExists]') IS NOT NULL DROP PROCEDURE [Releases_ReleaseExists]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ReleaseExists]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseExists_Indicator YNINDICATOR = NULL OUT,
	@NewerDeployedReleaseExists_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Release_Id INT

	SET @ReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases]
						  WHERE [Release_Number] = @Release_Number
							AND [Application_Id] = @Application_Id)
			 THEN 'Y'
			 ELSE 'N' END

    DECLARE @Release_Sequence BIGINT

    SELECT @Release_Sequence = [Release_Sequence]
     FROM [Releases]
    WHERE [Application_Id] = @Application_Id 
      AND [Release_Number] = @Release_Number

	SET @NewerDeployedReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases]
						  WHERE [Release_Sequence] > @Release_Sequence
							AND [Application_Id] = @Application_Id
							AND [ReleaseStatus_Name] = 'Deployed')
			 THEN 'Y'
			 ELSE 'N' END

END
GO

GRANT EXECUTE ON [Releases_ReleaseExists] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ResequenceRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified release by placing it where another release is sequenced.'
GO


IF OBJECT_ID('[Releases_ResequenceRelease]') IS NOT NULL DROP PROCEDURE [Releases_ResequenceRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ResequenceRelease]
(
	@Application_Id INT,
    @Release_Number RELEASENUMBER,
	@Release_Sequence BIGINT
)
AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Old_Release_Sequence BIGINT
	SELECT @Old_Release_Sequence = [Release_Sequence]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	
	UPDATE [Releases]
	   SET [Release_Sequence] = CASE
	          WHEN [Releases].[Release_Sequence] = RTAR.[Release_Sequence] 
	               THEN RPOS.[Release_Sequence]
	          WHEN [Releases].[Release_Sequence] BETWEEN RTAR.[Release_Sequence] AND RPOS.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] - 1
	          WHEN [Releases].[Release_Sequence] BETWEEN RPOS.[Release_Sequence] AND RTAR.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] + 1
	          ELSE [Releases].[Release_Sequence]
	          END
	  FROM [Releases],
	       [Releases] RTAR,
	       [Releases] RPOS
	 WHERE [Releases].[Application_Id] = @Application_Id
       AND RTAR.[Application_Id] = @Application_Id
       AND RTAR.[Release_Sequence] = @Old_Release_Sequence
	   AND RPOS.[Application_Id] = @Application_Id
       AND RPOS.[Release_Sequence] = @Release_Sequence
	   AND @Old_Release_Sequence <> @Release_Sequence


END
GO

GRANT EXECUTE ON [Releases_ResequenceRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_SetConfigurationFileVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the configuration file version for the specified release.'
GO


IF OBJECT_ID('[Releases_SetConfigurationFileVersion]') IS NOT NULL DROP PROCEDURE [Releases_SetConfigurationFileVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_SetConfigurationFileVersion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ConfigurationFile_Id INT,
    @Version_Number INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Release_Id INT

    SELECT @Release_Id = [Release_Id] 
      FROM [Releases_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number

    DELETE [ReleaseConfigurationFiles]
     WHERE [Release_Id] = @Release_Id
       AND [ConfigurationFile_Id] = @ConfigurationFile_Id

    IF (@Version_Number IS NOT NULL) 
    BEGIN

        INSERT INTO [ReleaseConfigurationFiles]
        (
            [Release_Id],
            [ConfigurationFile_Id],
            [Version_Number]
        )
        VALUES
        (
            @Release_Id,
	        @ConfigurationFile_Id,
            @Version_Number
        )

    END

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_SetConfigurationFileVersion] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_SetFurthestBuild',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the furthest build number for the release.'
GO


IF OBJECT_ID('[Releases_SetFurthestBuild]') IS NOT NULL DROP PROCEDURE [Releases_SetFurthestBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_SetFurthestBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Releases]
	   SET [Furthest_Build_Number] = @Build_Number
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_SetFurthestBuild] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_UnCancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Restores a previously canceled release.'
GO

IF OBJECT_ID('[Releases_UnCancelRelease]') IS NOT NULL DROP PROCEDURE [Releases_UnCancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_UnCancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Release_Id INT

    SELECT @Release_Id = [Release_Id] 
      FROM [Releases_Extended]
     WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseStatus_Name] = 'Cancelled'

	IF @Release_Id IS NULL
	BEGIN
		RAISERROR('Release ''%s'' does not exist or its status is not ''Cancelled''', 16, 1, @Release_Number)
		RETURN
	END


BEGIN TRY
	BEGIN TRANSACTION

		UPDATE [Releases]
		   SET [ReleaseStatus_Name] = 'Active',
		       [Original_Workflow_Name] = COALESCE(P.[Pipeline_Name], '{unknown}')
		  FROM [Releases] R 
			   LEFT JOIN [Pipelines] P
					  ON R.[Pipeline_Id] = P.[Pipeline_Id]
		 WHERE [Release_Id] = @Release_Id

	EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_UnCancelRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ReleaseTemplates_CreateOrUpdateTemplate',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseTemplate_Id',
    /* Description_Text        */ 'Creates a release template or updates an existing one.'
GO

IF OBJECT_ID('[ReleaseTemplates_CreateOrUpdateTemplate]') IS NOT NULL DROP PROCEDURE [ReleaseTemplates_CreateOrUpdateTemplate]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ReleaseTemplates_CreateOrUpdateTemplate]
(
	@Application_Id INT,
	@ReleaseTemplate_Name VARCHAR(50),
    @ReleaseTemplate_Configuration XML,
	@ReleaseTemplate_Id INT = NULL OUT
) AS
BEGIN

BEGIN TRY
	BEGIN TRANSACTION

    IF @ReleaseTemplate_Id IS NOT NULL
    BEGIN

        UPDATE [ReleaseTemplates]
           SET [ReleaseTemplate_Name] = @ReleaseTemplate_Name,
               [ReleaseTemplate_Configuration] = @ReleaseTemplate_Configuration
         WHERE [ReleaseTemplate_Id] = @ReleaseTemplate_Id

    END
    ELSE
    BEGIN

        INSERT INTO [ReleaseTemplates]
        (
            [Application_Id],
            [ReleaseTemplate_Name],
            [ReleaseTemplate_Configuration]
        )
        VALUES
        (
            @Application_Id,
            @ReleaseTemplate_Name,
            @ReleaseTemplate_Configuration
        )
        
        SET @ReleaseTemplate_Id = SCOPE_IDENTITY()

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ReleaseTemplates_CreateOrUpdateTemplate] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ReleaseTemplates_DeleteTemplate',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified release template.'
GO

IF OBJECT_ID('[ReleaseTemplates_DeleteTemplate]') IS NOT NULL DROP PROCEDURE [ReleaseTemplates_DeleteTemplate]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ReleaseTemplates_DeleteTemplate]
(
	@ReleaseTemplate_Id INT
) 
AS
BEGIN

    DELETE [ReleaseTemplates]
     WHERE [ReleaseTemplate_Id] = @ReleaseTemplate_Id


END
GO

GRANT EXECUTE ON [ReleaseTemplates_DeleteTemplate] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ReleaseTemplates_GetTemplate',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ReleaseTemplates',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified release template.'
GO

IF OBJECT_ID('[ReleaseTemplates_GetTemplate]') IS NOT NULL DROP PROCEDURE [ReleaseTemplates_GetTemplate]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ReleaseTemplates_GetTemplate]
(
	@ReleaseTemplate_Id INT
) 
AS
BEGIN

    SELECT * 
      FROM [ReleaseTemplates]
     WHERE [ReleaseTemplate_Id] = @ReleaseTemplate_Id


END
GO

GRANT EXECUTE ON [ReleaseTemplates_GetTemplate] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ReleaseTemplates_GetTemplateByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ReleaseTemplates',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified release template by application and name.'
GO

IF OBJECT_ID('[ReleaseTemplates_GetTemplateByName]') IS NOT NULL DROP PROCEDURE [ReleaseTemplates_GetTemplateByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ReleaseTemplates_GetTemplateByName]
(
    @Application_Id INT,
	@ReleaseTemplate_Name VARCHAR(50)
) 
AS
BEGIN

    SELECT * 
      FROM [ReleaseTemplates]
     WHERE [Application_Id] = @Application_Id
       AND [ReleaseTemplate_Name] = @ReleaseTemplate_Name


END
GO

GRANT EXECUTE ON [ReleaseTemplates_GetTemplateByName] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ReleaseTemplates_GetTemplates',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseTemplates',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the release templates for the specified application.'
GO

IF OBJECT_ID('[ReleaseTemplates_GetTemplates]') IS NOT NULL DROP PROCEDURE [ReleaseTemplates_GetTemplates]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ReleaseTemplates_GetTemplates]
(
	@Application_Id INT
) 
AS
BEGIN

    SELECT * 
      FROM [ReleaseTemplates]
     WHERE [Application_Id] = @Application_Id
     ORDER BY [ReleaseTemplate_Name]


END
GO

GRANT EXECUTE ON [ReleaseTemplates_GetTemplates] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_BeginPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'RetentionPoliciesPurgeHistoryEntry_Id',
    /* Description_Text        */ 'Marks the start of a new purge.'
GO


IF OBJECT_ID('[RetentionPolicies_BeginPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_BeginPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_BeginPurge]
(
	@RetentionPolicy_Id INT,
	@Original_RetentionPolicy_Name NVARCHAR(50),
	@RetentionPoliciesPurgeHistoryEntry_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	--Make sure this always gets reset
	UPDATE [RetentionPolicies]
	   SET [PurgeNow_Indicator] = 'N'
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	DECLARE @History_Count INT
	SET @History_Count = (SELECT COUNT(*) FROM [RetentionPoliciesPurgeHistory] WITH (TABLOCKX))

	IF @History_Count > 1000
	BEGIN

		DECLARE @RowsToDelete TABLE ([RetentionPolicyPurgeHistoryEntry_Id] INT NOT NULL)

		INSERT INTO @RowsToDelete
		SELECT [RetentionPoliciesPurgeHistoryEntry_Id]
		  FROM (SELECT *,
		               [Rank] = ROW_NUMBER() OVER(ORDER BY [Start_Date] DESC)
				  FROM [RetentionPoliciesPurgeHistory]) _TMP
		 WHERE _TMP.[Rank] > 1000

		DELETE [RetentionPoliciesPurgeHistory]
		 WHERE [RetentionPoliciesPurgeHistoryEntry_Id] IN (SELECT [RetentionPolicyPurgeHistoryEntry_Id] FROM @RowsToDelete)

	END

	INSERT INTO [RetentionPoliciesPurgeHistory]
	(
		[Original_RetentionPolicy_Id],
		[Original_RetentionPolicy_Name],
		[Start_Date],
		[End_Date],
		[LogText_Bytes]
	)
	VALUES
	(
		@RetentionPolicy_Id,
		@Original_RetentionPolicy_Name,
		GETUTCDATE(),
		NULL,
		NULL
	)

	SET @RetentionPoliciesPurgeHistoryEntry_Id = SCOPE_IDENTITY()

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_BeginPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_CreateOrUpdateRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'RetentionPolicy_Id',
    /* Description_Text        */ 'Creates or updates a retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_CreateOrUpdateRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_CreateOrUpdateRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_CreateOrUpdateRetentionPolicy]
(
	@RetentionPolicy_Name NVARCHAR(50),
	@PurgeType_Code CHAR(1),
	@PurgeIn_Application_Id INT,
	@PurgeIn_ApplicationGroup_Id INT,
	@PurgeIn_OnlyDeployedReleases_Indicator YNINDICATOR,
	@PurgeIn_OnlyRejectedBuilds_Indicator YNINDICATOR,
    @PurgeOrphanedArtifacts_Indicator YNINDICATOR,
	@PurgeIn_Pipeline_Id INT,
	@PurgeIn_Environment_Id INT,
	@PurgeIn_BuildStep_Indicator YNINDICATOR,
	@Retention_Count INT,
	@Retention_Size BIGINT,
    @Retention_Days_Count INT,
	@PurgeWindow_Start_Minutes INT,
    @PurgeWindow_Duration_Minutes INT,
    @Active_Indicator YNINDICATOR,
	@RetentionPolicy_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [RetentionPolicies]
			    WITH (UPDLOCK)
			   WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id)
	BEGIN

		UPDATE [RetentionPolicies]
		   SET [RetentionPolicy_Name] = NULLIF(RTRIM(LTRIM(@RetentionPolicy_Name)), ''),
		       [PurgeType_Code] = @PurgeType_Code,
			   [PurgeIn_Application_Id] = @PurgeIn_Application_Id,
			   [PurgeIn_ApplicationGroup_Id] = @PurgeIn_ApplicationGroup_Id,
			   [PurgeIn_OnlyDeployedReleases_Indicator] = @PurgeIn_OnlyDeployedReleases_Indicator,
			   [PurgeIn_OnlyRejectedBuilds_Indicator] = @PurgeIn_OnlyRejectedBuilds_Indicator,
               [PurgeOrphanedArtifacts_Indicator] = @PurgeOrphanedArtifacts_Indicator,
			   [PurgeIn_Pipeline_Id] = @PurgeIn_Pipeline_Id,
			   [PurgeIn_Environment_Id] = @PurgeIn_Environment_Id,
			   [PurgeIn_BuildStep_Indicator] = @PurgeIn_BuildStep_Indicator,
			   [Retention_Count] = @Retention_Count,
			   [Retention_Size] = @Retention_Size,
               [Retention_Days_Count] = @Retention_Days_Count,
			   [PurgeWindow_Start_Minutes] = @PurgeWindow_Start_Minutes,
               [PurgeWindow_Duration_Minutes] = @PurgeWindow_Duration_Minutes,
               [Active_Indicator] = @Active_Indicator,
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName()
		 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	END
	ELSE
	BEGIN

		INSERT INTO [RetentionPolicies]
		(
			[RetentionPolicy_Name],
			[PurgeType_Code],
			[PurgeIn_Application_Id],
			[PurgeIn_ApplicationGroup_Id],
			[PurgeIn_OnlyDeployedReleases_Indicator],
			[PurgeIn_OnlyRejectedBuilds_Indicator],
            [PurgeOrphanedArtifacts_Indicator],
			[PurgeIn_Pipeline_Id],
			[PurgeIn_Environment_Id],
			[PurgeIn_BuildStep_Indicator],
			[Retention_Count],
			[Retention_Size],
            [Retention_Days_Count],
			[PurgeWindow_Start_Minutes],
            [PurgeWindow_Duration_Minutes],
			[PurgeNow_Indicator],
            [Active_Indicator],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@RetentionPolicy_Name,
			@PurgeType_Code,
			@PurgeIn_Application_Id,
			@PurgeIn_ApplicationGroup_Id,
			@PurgeIn_OnlyDeployedReleases_Indicator,
			@PurgeIn_OnlyRejectedBuilds_Indicator,
            @PurgeOrphanedArtifacts_Indicator,
			@PurgeIn_Pipeline_Id,
			@PurgeIn_Environment_Id,
			@PurgeIn_BuildStep_Indicator,
			@Retention_Count,
			@Retention_Size,
            @Retention_Days_Count,
			@PurgeWindow_Start_Minutes,
            @PurgeWindow_Duration_Minutes,
			'N',
            @Active_Indicator,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @RetentionPolicy_Id = SCOPE_IDENTITY()

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_CreateOrUpdateRetentionPolicy] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_DeleteRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_DeleteRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_DeleteRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_DeleteRetentionPolicy]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [RetentionPolicies]
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_DeleteRetentionPolicy] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_EndPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks the end of a purge.'
GO


IF OBJECT_ID('[RetentionPolicies_EndPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_EndPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_EndPurge]
(
	@RetentionPoliciesPurgeHistoryEntry_Id INT,
	@LogText_Bytes VARBINARY(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    IF (@LogText_Bytes <> 0x)
    BEGIN

	    UPDATE [RetentionPoliciesPurgeHistory]
	       SET [End_Date] = GETUTCDATE(),
	           [LogText_Bytes] = @LogText_Bytes
	     WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

    END
    ELSE
    BEGIN

        DELETE [RetentionPoliciesPurgeHistory]
         WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_EndPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetArtifactsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the artifacts for the specified retention policy that can be purged.'
GO

IF OBJECT_ID('[RetentionPolicies_GetArtifactsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetArtifactsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetArtifactsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @ArtifactsMatchingFilter_Count INT
    DECLARE @ArtifactsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedBuildsOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Pipeline_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedBuildsOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Pipeline_Id = [PurgeIn_Pipeline_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @Retention_Count IS NOT NULL
    BEGIN

        SELECT @ArtifactsMatchingFilter_Count = COUNT(*) 
                                      FROM [BuildArtifacts_Slim] BA
	                            INNER JOIN [Applications] A
		                                ON A.[Application_Id] = BA.[Application_Id]
		                        INNER JOIN [Releases] R
		                                ON R.[Release_Id] = BA.[Release_Id]
	                        WHERE (@Application_Id IS NULL OR BA.[Application_Id] = @Application_Id)
	                          AND (@ApplicationGroup_Id IS NULL OR A.[ApplicationGroup_Id] = @ApplicationGroup_Id)
                              AND (@Pipeline_Id IS NULL OR R.[Pipeline_Id] = @Pipeline_Id)
	                          AND (@RejectedBuildsOnly_Indicator = 'N' OR BA.[BuildStatus_Name] = 'Rejected')
	                          AND (@DeployedReleasesOnly_Indicator = 'N' OR R.[ReleaseStatus_Name] = 'Deployed')

        SET @ArtifactsToDelete_Count = @ArtifactsMatchingFilter_Count - @Retention_Count
        
        IF (@ArtifactsToDelete_Count < 1)
            RETURN

        SET ROWCOUNT @ArtifactsToDelete_Count

    END

	SELECT *
	    FROM [BuildArtifacts_Slim] BA
	        INNER JOIN [Applications] A
		            ON A.[Application_Id] = BA.[Application_Id]
		    INNER JOIN [Releases] R
		             ON R.[Release_Id] = BA.[Release_Id]
	    WHERE (@Application_Id IS NULL OR BA.[Application_Id] = @Application_Id)
	    AND (@ApplicationGroup_Id IS NULL OR A.[ApplicationGroup_Id] = @ApplicationGroup_Id)
        AND (@Pipeline_Id IS NULL OR R.[Pipeline_Id] = @Pipeline_Id)
	    AND (@RejectedBuildsOnly_Indicator = 'N' OR BA.[BuildStatus_Name] = 'Rejected')
	    AND (@DeployedReleasesOnly_Indicator = 'N' OR R.[ReleaseStatus_Name] = 'Deployed')
	    AND (@AlwaysRetainAfter_Date IS NULL OR BA.[Created_Date] < @AlwaysRetainAfter_Date)
        ORDER BY [Created_Date]

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetArtifactsToPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildCount',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Build_Count',
    /* Description_Text        */ 'Gets the total number of builds in the specified scope.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildCount]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildCount]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildCount]
(
	@Application_Id INT = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Build_Count INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Application_Id IS NOT NULL
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds_Extended]
		 WHERE [Application_Id] = @Application_Id

	END
	ELSE IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds_Extended] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id

	END
	ELSE
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildCount] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the builds for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @BuildsMatchingFilter_Count INT
    DECLARE @BuildsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Pipeline_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Pipeline_Id = [PurgeIn_Workflow_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] B
		           INNER JOIN [Applications] A
			               ON A.[Application_Id] = B.[Application_Id]
		                WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                          AND (@Pipeline_Id IS NULL OR B.[Pipeline_Id] = @Pipeline_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT B.*
		  FROM [Builds_Extended] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Pipeline_Id IS NULL OR B.[Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR B.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended]
		                WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                          AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT *
		  FROM [Builds_Extended]
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildsToPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildsWithLogsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets only the builds that have logs remaining for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildsWithLogsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildsWithLogsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildsWithLogsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @BuildsMatchingFilter_Count INT
    DECLARE @BuildsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Pipeline_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Pipeline_Id = [PurgeIn_Pipeline_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] B
		           INNER JOIN [Applications] A
			               ON A.[Application_Id] = B.[Application_Id]
		                WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                          AND (@Pipeline_Id IS NULL OR B.[Pipeline_Id] = @Pipeline_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
                          AND EXISTS(
                                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                                  FROM [BuildExecutions_Extended] BEX
                            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
                            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
                            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                                 WHERE B.[Application_Id] = BEX.[Application_Id]
                                   AND B.[Release_Number] = BEX.[Release_Number]
                                   AND B.[Build_Number] = BEX.[Build_Number]
                          )

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT B.*
		  FROM [Builds_Extended] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Pipeline_Id IS NULL OR B.[Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR B.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
           AND EXISTS(
                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                    FROM [BuildExecutions_Extended] BEX
            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                    WHERE B.[Application_Id] = BEX.[Application_Id]
                    AND B.[Release_Number] = BEX.[Release_Number]
                    AND B.[Build_Number] = BEX.[Build_Number]
            )
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] BE
		                WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                          AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
                          AND EXISTS(
                                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                                  FROM [BuildExecutions_Extended] BEX
                            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
                            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
                            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                                 WHERE BE.[Application_Id] = BEX.[Application_Id]
                                   AND BE.[Release_Number] = BEX.[Release_Number]
                                   AND BE.[Build_Number] = BEX.[Build_Number]
                          ) 

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT *
		  FROM [Builds_Extended] B
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
           AND EXISTS(
                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                    FROM [BuildExecutions_Extended] BEX
            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                    WHERE B.[Application_Id] = BEX.[Application_Id]
                    AND B.[Release_Number] = BEX.[Release_Number]
                    AND B.[Build_Number] = BEX.[Build_Number]
            )
		 ORDER BY [CreatedOn_Date]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildsWithLogsToPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetPurgeHistory',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'RetentionPoliciesPurgeHistory',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the purge history for one or all retention policies.'
GO


IF OBJECT_ID('[RetentionPolicies_GetPurgeHistory]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetPurgeHistory]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetPurgeHistory]
(
	@RetentionPolicy_Id INT = NULL,
	@Max_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Max_Count IS NOT NULL
		SET ROWCOUNT @Max_Count

	SELECT *
	  FROM [RetentionPoliciesPurgeHistory]
	 WHERE @RetentionPolicy_Id IS NULL OR [Original_RetentionPolicy_Id] = @RetentionPolicy_Id
	 ORDER BY [Start_Date] DESC

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetPurgeHistory] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetPurgeLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'RetentionPoliciesPurgeHistory',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified purge history log.'
GO


IF OBJECT_ID('[RetentionPolicies_GetPurgeLog]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetPurgeLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetPurgeLog]
(
	@RetentionPoliciesPurgeHistoryEntry_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPoliciesPurgeHistory]
	 WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetPurgeLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetReleaseCount',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Release_Count',
    /* Description_Text        */ 'Gets the total number of releases in the specified scope.'
GO


IF OBJECT_ID('[RetentionPolicies_GetReleaseCount]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetReleaseCount]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetReleaseCount]
(
	@Application_Id INT = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Release_Count INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Application_Id IS NOT NULL
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases_Slim]
		 WHERE [Application_Id] = @Application_Id

	END
	ELSE IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases_Slim] R
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = R.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id

	END
	ELSE
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetReleaseCount] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetReleasesToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the releases for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetReleasesToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetReleasesToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetReleasesToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @ReleasesMatchingFilter_Count INT
    DECLARE @ReleasesToDelete_Count INT
	DECLARE @DeployedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Pipeline_Id INT
    
    SELECT @DeployedOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Pipeline_Id = [PurgeIn_Pipeline_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @ReleasesMatchingFilter_Count = COUNT(*) 
                          FROM [Releases_Extended] R
		            INNER JOIN [Applications] A
			                ON A.[Application_Id] = R.[Application_Id]
		                 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                           AND (@Pipeline_Id IS NULL OR R.[Pipeline_Id] = @Pipeline_Id)
		                   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')

            SET @ReleasesToDelete_Count = @ReleasesMatchingFilter_Count - @Retention_Count
        
            IF (@ReleasesToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @ReleasesToDelete_Count

        END

		SELECT R.*
		  FROM [Releases_Extended] R
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = R.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Pipeline_Id IS NULL OR R.[Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
           AND (@AlwaysRetainAfter_Date IS NULL OR R.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @ReleasesMatchingFilter_Count = COUNT(*) 
                          FROM [Releases_Extended]
		                 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                           AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		                   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')

            SET @ReleasesToDelete_Count = @ReleasesMatchingFilter_Count - @Retention_Count
        
            IF (@ReleasesToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @ReleasesToDelete_Count

        END

		SELECT *
		  FROM [Releases_Extended]
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Pipeline_Id IS NULL OR [Pipeline_Id] = @Pipeline_Id)
		   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
           AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END 

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetReleasesToPurge] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetRetentionPolicies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'RetentionPolicies_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the retention policies.'
GO


IF OBJECT_ID('[RetentionPolicies_GetRetentionPolicies]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetRetentionPolicies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetRetentionPolicies]
(
    @Include_Inactive YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPolicies_Extended]
     WHERE @Include_Inactive = 'Y' OR [Active_Indicator] = 'Y'

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetRetentionPolicies] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'RetentionPolicies_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_GetRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetRetentionPolicy]
(
    @RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPolicies_Extended]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetRetentionPolicy] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_PurgeNow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks a retention policy as ready to run immediately.'
GO


IF OBJECT_ID('[RetentionPolicies_PurgeNow]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_PurgeNow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_PurgeNow]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [RetentionPolicies]
	   SET [PurgeNow_Indicator] = 'Y'
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_PurgeNow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ScriptAssets_CreateOrUpdateScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ScriptAsset_Id',
    /* Description_Text        */ 'Creates a new script or updates an existing one.'
GO

IF OBJECT_ID('[ScriptAssets_CreateOrUpdateScript]') IS NOT NULL DROP PROCEDURE [ScriptAssets_CreateOrUpdateScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ScriptAssets_CreateOrUpdateScript]
(
	@ScriptAsset_Name VARCHAR(200),
	@Application_Id INT,
	@Script_Text VARBINARY(MAX),
	@ScriptAsset_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @ScriptAsset_Id IS NULL
	BEGIN

		INSERT INTO [ScriptAssets]
		(
			[ScriptAsset_Name],
			[Application_Id],
			[Script_Text],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ScriptAsset_Name,
			@Application_Id,
			@Script_Text,
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @ScriptAsset_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [ScriptAssets]
		   SET [ScriptAsset_Name] = @ScriptAsset_Name,
		       [Application_Id] = @Application_Id,
			   [Script_Text] = @Script_Text,
		       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
		       [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [ScriptAsset_Id] = @ScriptAsset_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ScriptAssets_CreateOrUpdateScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ScriptAssets_DeleteScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a script.'
GO

IF OBJECT_ID('[ScriptAssets_DeleteScript]') IS NOT NULL DROP PROCEDURE [ScriptAssets_DeleteScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ScriptAssets_DeleteScript]
(
	@ScriptAsset_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptAssets]
	 WHERE [ScriptAsset_Id] = @ScriptAsset_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ScriptAssets_DeleteScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ScriptAssets_GetScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ScriptAssets',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the script with the specified ID.'
GO

IF OBJECT_ID('[ScriptAssets_GetScript]') IS NOT NULL DROP PROCEDURE [ScriptAssets_GetScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ScriptAssets_GetScript]
(
	@ScriptAsset_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ScriptAssets]
	 WHERE [ScriptAsset_Id] = @ScriptAsset_Id

END
GO

GRANT EXECUTE ON [ScriptAssets_GetScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ScriptAssets_GetScriptByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ScriptAssets',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the script with the specified name, searching from the specified application.'
GO

IF OBJECT_ID('[ScriptAssets_GetScriptByName]') IS NOT NULL DROP PROCEDURE [ScriptAssets_GetScriptByName]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ScriptAssets_GetScriptByName]
(
	@ScriptAsset_Name VARCHAR(200),
	@Application_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT TOP 1 *
	  FROM [ScriptAssets]
	 WHERE ([Application_Id] = @Application_Id OR [Application_Id] IS NULL)
	   AND [ScriptAsset_Name] = @ScriptAsset_Name
	 ORDER BY CASE WHEN [Application_Id] IS NULL THEN 1 ELSE 0 END

END
GO

GRANT EXECUTE ON [ScriptAssets_GetScriptByName] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ScriptAssets_GetScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScriptAssets',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the scripts which are visible to the specified application.'
GO

IF OBJECT_ID('[ScriptAssets_GetScripts]') IS NOT NULL DROP PROCEDURE [ScriptAssets_GetScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ScriptAssets_GetScripts]
(
	@Application_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ScriptAssets]
	 WHERE [Application_Id] IS NULL OR [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [ScriptAssets_GetScripts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script parameter or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateParameter]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50),
	@ParameterType_Code CHAR(1),
	@DefaultValue_Text NVARCHAR(MAX),
	@Description_Text NVARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ScriptParameters]
	           WHERE [Script_Id] = @Script_Id
	             AND [Parameter_Name] = @Parameter_Name)
	BEGIN

		UPDATE [ScriptParameters]
		   SET [ParameterType_Code] = @ParameterType_Code,
		       [DefaultValue_Text] = @DefaultValue_Text,
		       [Description_Text] = @Description_Text
		 WHERE [Script_Id] = @Script_Id
		   AND [Parameter_Name] = @Parameter_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ScriptParameters]
		(
			[Script_Id],
			[Parameter_Name],
			[ParameterType_Code],
			[DefaultValue_Text],
			[Description_Text]
		)
		VALUES
		(
			@Script_Id,
			@Parameter_Name,
			@ParameterType_Code,
			@DefaultValue_Text,
			@Description_Text
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateParameter] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates a new script or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateScript]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateScript]
(
	@Script_Id INT OUT,
	@Script_Name NVARCHAR(50),
	@Description_Text NVARCHAR(MAX),
	@ScriptType_Code VARCHAR(20),
	@Active_Indicator YNINDICATOR,
	@AutoSync_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Script_Id IS NULL
	BEGIN

		INSERT INTO [Scripts]
		(
			[Script_Name],
			[Description_Text],
			[Active_Indicator],
			[AutoSync_Indicator],
			[ScriptType_Code],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Script_Name,
			@Description_Text,
			@Active_Indicator,
			@AutoSync_Indicator,
			@ScriptType_Code,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @Script_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [Scripts]
		   SET [Script_Name] = @Script_Name,
		       [Description_Text] = @Description_Text,
		       [ScriptType_Code] = @ScriptType_Code,
		       [Active_Indicator] = @Active_Indicator,
		       [AutoSync_Indicator] = @AutoSync_Indicator,
		       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
		       [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script variable or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateVariable]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateVariable]
(
	@Script_Id INT,
	@ScriptVariable_Name NVARCHAR(50),
	@Value_Text NVARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ScriptVariables]
	           WHERE [Script_Id] = @Script_Id
	             AND [ScriptVariable_Name] = @ScriptVariable_Name)
	BEGIN

		UPDATE [ScriptVariables]
		   SET [Value_Text] = @Value_Text
		 WHERE [Script_Id] = @Script_Id
		   AND [ScriptVariable_Name] = @ScriptVariable_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ScriptVariables]
		(
			[Script_Id],
			[ScriptVariable_Name],
			[Value_Text]
		)
		VALUES
		(
			@Script_Id,
			@ScriptVariable_Name,
			@Value_Text
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateVariable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Version_Number',
    /* Description_Text        */ 'Creates a new version of a script.'
GO


IF OBJECT_ID('[Scripts_CreateVersion]') IS NOT NULL DROP PROCEDURE [Scripts_CreateVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateVersion]
(
	@Version_Number INT OUT,
	@Script_Id INT,
	@Script_Bytes VARBINARY(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Version_Number = (SELECT MAX([Version_Number])
	                         FROM [ScriptVersions]
	                        WHERE [Script_Id] = @Script_Id)

	DECLARE @Latest_Bytes VARBINARY(MAX)
	SET @Latest_Bytes = (SELECT [Script_Bytes]
	                       FROM [ScriptVersions]
	                      WHERE [Script_Id] = @Script_Id
	                        AND [Version_Number] = @Version_Number)

	--Do not insert a copy of the most recent version
	IF @Latest_Bytes IS NULL OR @Latest_Bytes <> @Script_Bytes
	BEGIN

		SET @Version_Number = COALESCE(@Version_Number, 0) + 1

		INSERT INTO [ScriptVersions]
		(
			[Script_Id],
			[Version_Number],
			[Script_Bytes],
			[CreatedBy_User_Name],
			[CreatedOn_Date]
		)
		VALUES
		(
			@Script_Id,
			@Version_Number,
			@Script_Bytes,
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		UPDATE [Scripts]
		   SET [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateVersion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a script parameter.'
GO


IF OBJECT_ID('[Scripts_DeleteParameter]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	   AND [Parameter_Name] = @Parameter_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteParameter] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Permanently deletes a script.'
GO


IF OBJECT_ID('[Scripts_DeleteScript]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptVariables]
	 WHERE [Script_Id] = @Script_Id

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id

	DELETE [ScriptVersions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [Scripts]
	 WHERE [Script_Id] = @Script_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a script variable.'
GO


IF OBJECT_ID('[Scripts_DeleteVariable]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteVariable]
(
	@Script_Id INT,
	@ScriptVariable_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptVariables]
	 WHERE [Script_Id] = @Script_Id
	   AND [ScriptVariable_Name] = @ScriptVariable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteVariable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Scripts_Extended,ScriptParameters,ScriptVariables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a script and all of its associated information.'
GO


IF OBJECT_ID('[Scripts_GetScript]') IS NOT NULL DROP PROCEDURE [Scripts_GetScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE [Script_Id] = @Script_Id

	SELECT *
	  FROM [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	 ORDER BY [Parameter_Name]

	SELECT *
	  FROM [ScriptVariables]
	 WHERE [Script_Id] = @Script_Id
	 ORDER BY [ScriptVariable_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScript] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Scripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the scripts in the system, optionally filtered by type code.'
GO


IF OBJECT_ID('[Scripts_GetScripts]') IS NOT NULL DROP PROCEDURE [Scripts_GetScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScripts]
(
	@ScriptType_Code VARCHAR(20),
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE (@IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y')
	   AND (@ScriptType_Code IS NULL OR [ScriptType_Code] = @ScriptType_Code)
	 ORDER BY [Script_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScripts] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScriptTypeUsage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScriptTypes_Usage',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets usage data for the script types in the database.'
GO


IF OBJECT_ID('[Scripts_GetScriptTypeUsage]') IS NOT NULL DROP PROCEDURE [Scripts_GetScriptTypeUsage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScriptTypeUsage]
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ScriptTypes_Usage]

END
GO

GRANT EXECUTE ON [Scripts_GetScriptTypeUsage] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns the specified user as a named user with the specified user class type'
GO

IF OBJECT_ID('[Security_AddNamedUser]') IS NOT NULL DROP PROCEDURE [Security_AddNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddNamedUser]
(
	@User_Name USERNAME
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [NamedUsers] ([User_Name])
	VALUES (@User_Name)

	EXEC [Events_RaiseEvent] 'SECNWU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddNamedUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddPrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Privilege_Id',
    /* Description_Text        */ 'Grants or denies the specified principal the ability to perform a certain role'
GO

IF OBJECT_ID('[Security_AddPrivilege]') IS NOT NULL
	DROP PROCEDURE [Security_AddPrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddPrivilege]
(
	@Privilege_Id INT = NULL OUT,
	@UserDirectory_Id INT,
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@Role_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@PrivilegeType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Catch duplicate data */
	IF EXISTS(SELECT *
				FROM [Privileges]
			   WHERE [Principal_Name] = @Principal_Name
				 AND [UserDirectory_Id] = @UserDirectory_Id
				 AND [PrincipalType_Code] = @PrincipalType_Code
				 AND [Role_Id] = @Role_Id
				 AND (	[Application_Id] = @Application_Id
					 OR COALESCE([Application_Id], @Application_Id) IS NULL )
				 AND (	[Environment_Id] = @Environment_Id
					 OR COALESCE([Environment_Id], @Environment_Id) IS NULL )
				 AND (  [ApplicationGroup_Id] = @ApplicationGroup_Id
				     OR COALESCE([ApplicationGroup_Id], @ApplicationGroup_Id) IS NULL )
				 AND [PrivilegeType_Code] = @PrivilegeType_Code )
	BEGIN
	    ROLLBACK
		RETURN
    END

	INSERT INTO [Privileges]
		([UserDirectory_Id]
		,[Principal_Name]
		,[PrincipalType_Code]
		,[Role_Id]
		,[ApplicationGroup_Id]
		,[Application_Id]
		,[Environment_Id]
		,[PrivilegeType_Code])
	VALUES
		(@UserDirectory_Id
		,@Principal_Name
		,@PrincipalType_Code
		,@Role_Id
		,@ApplicationGroup_Id
		,@Application_Id
		,@Environment_Id
		,@PrivilegeType_Code)
	SET @Privilege_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'SECNWP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddPrivilege] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_CreateOrUpdateRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Creates or updates a role.'
GO

IF OBJECT_ID('[Security_CreateOrUpdateRole]') IS NOT NULL DROP PROCEDURE [Security_CreateOrUpdateRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_CreateOrUpdateRole]
(
	@Role_Id INT = NULL OUT,
	@Role_Name NVARCHAR(50),
    @Role_Description VARCHAR(MAX),
	@Tasks_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Application_Scopeable_Indicator YNINDICATOR
            SET @Application_Scopeable_Indicator = CASE
                WHEN EXISTS (SELECT * FROM [Tasks] 
                              WHERE [Application_Scopeable_Indicator] = 'N'
                               AND [Task_Id] IN (SELECT [Value_Text] FROM [CsvToTable](@Tasks_Csv, ',')))
                     THEN 'N' ELSE 'Y' END

     DECLARE @Environment_Scopeable_Indicator YNINDICATOR
            SET @Environment_Scopeable_Indicator = CASE
                WHEN EXISTS (SELECT * FROM [Tasks] 
                              WHERE [Environment_Scopeable_Indicator] = 'N'
                               AND [Task_Id] IN (SELECT [Value_Text] FROM [CsvToTable](@Tasks_Csv, ',')))
                     THEN 'N' ELSE 'Y' END


	IF @Role_Id IS NULL
	BEGIN

		INSERT INTO [Roles]
		(
			[Role_Name],
            [Role_Description],
            [Application_Scopeable_Indicator],
            [Environment_Scopeable_Indicator]
		)
		VALUES
		(
			@Role_Name,
            @Role_Description,
            @Application_Scopeable_Indicator,
            @Environment_Scopeable_Indicator
		)

		SET @Role_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'SECNWR', NULL, @Role_Id

	END
	ELSE
	BEGIN

		UPDATE [Roles]
		   SET [Role_Name] = @Role_Name,
               [Role_Description] = @Role_Description,
               [Application_Scopeable_Indicator] = @Application_Scopeable_Indicator,
               [Environment_Scopeable_Indicator] = @Environment_Scopeable_Indicator
		 WHERE [Role_Id] = @Role_Id

		DELETE [RoleTasks]
		 WHERE [Role_Id] = @Role_Id

		EXEC [Events_RaiseEvent] 'SECMDR', NULL, @Role_Id
	END

	INSERT INTO [RoleTasks]
	(
		[Role_Id],
		[Task_Id]
	)
	SELECT @Role_Id,
	       [Value_Text]
	  FROM dbo.CsvToTable(@Tasks_Csv, ',')

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_CreateOrUpdateRole] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_DeleteRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified role from the system'
GO

IF OBJECT_ID('[dbo].[Security_DeleteRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_DeleteRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_DeleteRole]
(
	@Role_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Role_Name SQL_VARIANT
	    SET @Role_Name = (SELECT CAST([Role_Name] AS VARCHAR(50)) FROM [Roles] WHERE [Role_Id] = @Role_Id)

	DELETE [Privileges]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [RoleTasks]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [Roles]
	 WHERE [Role_Id] = @Role_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLR', NULL, @Role_Name
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [dbo].[Security_DeleteRole] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified named user'
GO

IF OBJECT_ID('[Security_GetNamedUser]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUser]
(
    @User_Name USERNAME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]
	WHERE [User_Name] = @User_Name


END
GO

GRANT EXECUTE ON [Security_GetNamedUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of the named users in the system'
GO

IF OBJECT_ID('[Security_GetNamedUsers]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUsers]  AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]


END
GO

GRANT EXECUTE ON [Security_GetNamedUsers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetOrAddAnonymousRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Gets an anonymous role containing the specified tasks, creating one if necessary.'
GO

IF OBJECT_ID('[Security_GetOrAddAnonymousRole]') IS NOT NULL
	DROP PROCEDURE [Security_GetOrAddAnonymousRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetOrAddAnonymousRole]
(
	@Tasks_Csv VARCHAR(MAX),
	@Role_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Tasks TABLE([Task_Id] INT NOT NULL)

	INSERT INTO @Tasks
	SELECT DISTINCT CAST([Value_Text] AS INT)
	  FROM dbo.CsvToTable(@Tasks_Csv, ',')

	DECLARE @Task_Count INT
	SET @Task_Count = (SELECT COUNT(*) FROM @Tasks)

	SET @Role_Id = (SELECT TOP 1 RT.[Role_Id]
	                  FROM @Tasks T
	                       INNER JOIN [RoleTasks] RT
		                           ON RT.[Task_Id] = T.[Task_Id]
		                   INNER JOIN [Roles] R
		                           ON R.[Role_Id] = RT.[Role_Id]
	                 WHERE R.[Role_Name] IS NULL
	                 GROUP BY RT.[Role_Id]
	                HAVING COUNT(*) = @Task_Count)

	IF @Role_Id IS NULL
	BEGIN

		INSERT INTO [Roles] ([Role_Name]) VALUES (NULL)
		SET @Role_Id = SCOPE_IDENTITY()

		INSERT INTO [RoleTasks]
		(
			[Role_Id],
			[Task_Id]
		)
		SELECT [Role_Id] = @Role_Id,
		       [Task_Id]
		  FROM @Tasks

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_GetOrAddAnonymousRole] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of privileges assigned in the system'
GO

IF OBJECT_ID('[dbo].[Security_GetPrivileges]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetPrivileges]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetPrivileges]
(
	@UserDirectory_Id INT,
	@PrincipalType_Code CHAR(1) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [PrincipalType_Code] = COALESCE(@PrincipalType_Code, [PrincipalType_Code])
	   AND [UserDirectory_Id] = @UserDirectory_Id
END
GO

GRANT EXECUTE ON [dbo].[Security_GetPrivileges] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified role, along with all the tasks that comprise the role.'
GO

IF OBJECT_ID('[Security_GetRole]') IS NOT NULL
	DROP PROCEDURE [Security_GetRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetRole]
(
	@Role_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE [Role_Id] = @Role_Id
	IF @@ROWCOUNT=0 RAISERROR('Role #%i does not exist.',16,1,@Role_Id)

	SELECT *
	  FROM [RoleTasks_Extended]
	 WHERE [Role_Id] = @Role_Id

END
GO

GRANT EXECUTE ON [Security_GetRole] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the roles in the system, along with all the tasks comprising each role.'
GO

IF OBJECT_ID('[dbo].[Security_GetRoles]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetRoles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetRoles]
(
	@IncludeAnonymous_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR [Role_Name] IS NOT NULL
	 ORDER BY [Role_Name]

	SELECT RT.*
	  FROM [RoleTasks_Extended] RT
	       INNER JOIN [Roles] R
		           ON R.[Role_Id] = RT.[Role_Id]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR R.[Role_Name] IS NOT NULL
	 ORDER BY RT.[Task_Name]

END
GO

GRANT EXECUTE ON [dbo].[Security_GetRoles] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetTasks',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Tasks',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the tasks in the system.'
GO

IF OBJECT_ID('[Security_GetTasks]') IS NOT NULL
	DROP PROCEDURE [Security_GetTasks]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetTasks]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Tasks]

END
GO

GRANT EXECUTE ON [Security_GetTasks] TO [BuildMasterUser_Role]
GO



GO
IF OBJECT_ID('[Security_GetUserClassCounts]') IS NOT NULL DROP PROCEDURE [Security_GetUserClassCounts]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemoveNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Removes the specified user as a named user'
GO

IF OBJECT_ID('[Security_RemoveNamedUser]') IS NOT NULL DROP PROCEDURE [Security_RemoveNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_RemoveNamedUser]
(
	@User_Name USERNAME
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'SECDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_RemoveNamedUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemovePrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a privilege from the system.'
GO

IF OBJECT_ID('[Security_RemovePrivilege]') IS NOT NULL
	DROP PROCEDURE ..[Security_RemovePrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_RemovePrivilege]
(
	@Privilege_Id INT
)
AS
BEGIN
SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [Privileges]
	 WHERE [Privilege_Id] = @Privilege_Id

	-- Clean up orphaned anonymous roles
	DELETE RT
	  FROM [RoleTasks] RT
	       INNER JOIN [Roles] R
		           ON R.[Role_Id] = RT.[Role_Id]
	 WHERE R.[Role_Name] IS NULL
	   AND RT.[Role_Id] NOT IN (SELECT [Role_Id] FROM [Privileges])

	DELETE R
	  FROM [Roles] R
	 WHERE R.[Role_Name] IS NULL
	   AND R.[Role_Id] NOT IN (SELECT [Role_Id] FROM [Privileges])

	EXEC [Events_RaiseEvent] 'SECDLP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Security_RemovePrivilege] TO [BuildMasterUser_Role]
GO



GO
IF OBJECT_ID('[Security_UpdateUserClassType]') IS NOT NULL DROP PROCEDURE [Security_UpdateUserClassType]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_UserIsNamed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UserIsNamed_Indicator',
    /* Description_Text        */ 'Determines whether or not a user is a named user in the system'
GO

IF OBJECT_ID('[Security_UserIsNamed]') IS NOT NULL DROP PROCEDURE [Security_UserIsNamed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_UserIsNamed]
(
	@User_Name USERNAME,
	@UserIsNamed_Indicator YNINDICATOR = NULL OUT
) AS
BEGIN

	SET NOCOUNT ON

	SET @UserIsNamed_Indicator = 'N'

	IF EXISTS ( SELECT *
				  FROM [NamedUsers]
				 WHERE [User_Name] = @User_Name )
		SET @UserIsNamed_Indicator = 'Y'

END
GO

GRANT EXECUTE ON [Security_UserIsNamed] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_CreateOrUpdateServerRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ServerRole_Id',
    /* Description_Text        */ 'Creates or updates a server role.'
GO


IF OBJECT_ID('[ServerRoles_CreateOrUpdateServerRole]') IS NOT NULL DROP PROCEDURE [ServerRoles_CreateOrUpdateServerRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_CreateOrUpdateServerRole]
(
	@ServerRole_Name NVARCHAR(50),
    @ServerIds_Csv VARCHAR(MAX) = NULL,
    @ServerRole_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@ServerRole_Id IS NULL)
	BEGIN

		INSERT INTO [ServerRoles] ([ServerRole_Name])
		VALUES (@ServerRole_Name)

		SET @ServerRole_Id = SCOPE_IDENTITY()

	END ELSE BEGIN

		UPDATE [ServerRoles]
		   SET [ServerRole_Name] = @ServerRole_Name
		 WHERE [ServerRole_Id] = @ServerRole_Id

	END

    IF @ServerIds_Csv IS NOT NULL
    BEGIN

        DELETE [ServerRoleServers] 
         WHERE [ServerRole_Id] = @ServerRole_Id

        INSERT [ServerRoleServers] ([Server_Id], [ServerRole_Id])
        SELECT CAST([Value_Text] AS INT), @ServerRole_Id 
          FROM dbo.CsvToTable(@ServerIds_Csv, ',')

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ServerRoles_CreateOrUpdateServerRole] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_DeleteServerRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified server and its configurations.'
GO


IF OBJECT_ID('[ServerRoles_DeleteServerRole]') IS NOT NULL DROP PROCEDURE [ServerRoles_DeleteServerRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_DeleteServerRole]
(
    @ServerRole_Id INT
)   
AS
BEGIN

	SET NOCOUNT ON

	DELETE [ServerRoles] WHERE [ServerRole_Id] = @ServerRole_Id


END
GO

GRANT EXECUTE ON [ServerRoles_DeleteServerRole] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_GetServerRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ServerRoles_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified server role.'
GO


IF OBJECT_ID('[ServerRoles_GetServerRole]') IS NOT NULL DROP PROCEDURE [ServerRoles_GetServerRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_GetServerRole]
(
    @ServerRole_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ServerRoles_Extended]
     WHERE [ServerRole_Id] = @ServerRole_Id

    

END
GO

GRANT EXECUTE ON [ServerRoles_GetServerRole] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_GetServerRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ServerRoles_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers roles in the system.'
GO


IF OBJECT_ID('[ServerRoles_GetServerRoles]') IS NOT NULL DROP PROCEDURE [ServerRoles_GetServerRoles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_GetServerRoles]
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ServerRoles_Extended] SR
          ORDER BY [ServerRole_Name]

END
GO

GRANT EXECUTE ON [ServerRoles_GetServerRoles] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_GetServerRolesEnvironments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ServerRolesEnvironments_Merged',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the server role and environment relations that match the specified criteria.'
GO


IF OBJECT_ID('[ServerRoles_GetServerRolesEnvironments]') IS NOT NULL DROP PROCEDURE [ServerRoles_GetServerRolesEnvironments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_GetServerRolesEnvironments]
(
    @Environment_Id INT = NULL,
    @ServerRole_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ServerRolesEnvironments_Merged]
     WHERE (@Environment_Id IS NULL OR [Environment_Id] = @Environment_Id)
       AND (@ServerRole_Id IS NULL OR [ServerRole_Id] = @ServerRole_Id)

END
GO

GRANT EXECUTE ON [ServerRoles_GetServerRolesEnvironments] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_GetServerRoleServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ServerRoleServers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the server role servers for a specific role, or all the roles in the system.'
GO


IF OBJECT_ID('[ServerRoles_GetServerRoleServers]') IS NOT NULL DROP PROCEDURE [ServerRoles_GetServerRoleServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_GetServerRoleServers]
(
    @ServerRole_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ServerRoleServers]
     WHERE @ServerRole_Id IS NULL OR [ServerRole_Id] = @ServerRole_Id

END
GO

GRANT EXECUTE ON [ServerRoles_GetServerRoleServers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_SearchServerRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ServerRoles_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Searches servers roles in the system.'
GO


IF OBJECT_ID('[ServerRoles_SearchServerRoles]') IS NOT NULL DROP PROCEDURE [ServerRoles_SearchServerRoles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_SearchServerRoles]
(
    @Environment_Id INT = NULL
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT * 
      FROM [ServerRoles_Extended] SR
     WHERE (@Environment_Id IS NULL OR EXISTS(SELECT * 
                                                FROM [ServerRolesEnvironments_Merged] SRE
                                               WHERE [Environment_Id] = @Environment_Id
                                                 AND SR.[ServerRole_Id] = SRE.ServerRole_Id))
    ORDER BY SR.[ServerRole_Name]

    

END
GO

GRANT EXECUTE ON [ServerRoles_SearchServerRoles] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_CreateOrUpdateServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Server_Id',
    /* Description_Text        */ 'Creates a new server, or updates an existing one.'
GO


IF OBJECT_ID('[Servers_CreateOrUpdateServer]') IS NOT NULL DROP PROCEDURE [Servers_CreateOrUpdateServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_CreateOrUpdateServer]
(
	@Server_Name VARCHAR(50),
	@Agent_Configuration VARCHAR(MAX) = NULL,
	@Active_Indicator YNINDICATOR = NULL,
    @ServerType_Code CHAR(1) = NULL,
    @Restricted_Indicator YNINDICATOR = NULL,
	@EnvironmentIds_Csv VARCHAR(MAX) = NULL,
	@ServerIdsInGroup_Csv VARCHAR(MAX) = NULL,
    @ServerRoleIds_Csv VARCHAR(MAX) = NULL,
    @LogServerErrors_Indicator YNINDICATOR = NULL,
    @Server_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Server_Id IS NULL)
		BEGIN
			INSERT INTO [Servers] 
				(
					[Server_Name],
					[CreatedBy_User_Name],
					[CreatedOn_Date],
					[ModifiedBy_User_Name],
					[ModifiedOn_Date],
					[ServerType_Code],
					[Agent_Configuration],
					[ServerStatus_Code],
					[Active_Indicator],
                    [LogServerErrors_Indicator],
                    [Restricted_Indicator]
				)
			VALUES
				(
					@Server_Name,
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					COALESCE(@ServerType_Code, 'S'),
					@Agent_Configuration,
					'UNKWN',
					@Active_Indicator,
                    COALESCE(@LogServerErrors_Indicator, 'Y'),
                    COALESCE(@Restricted_Indicator, 'N')
				)
			SET @Server_Id = SCOPE_IDENTITY()

			EXEC [Events_RaiseEvent] 'ENVNWS', NULL, @Server_Id
		END

	ELSE
		BEGIN
			UPDATE [Servers]
			   SET [Server_Name] = @Server_Name,
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE(),
				   [ServerType_Code] = COALESCE(@ServerType_Code, [ServerType_Code]),
				   [Agent_Configuration] = COALESCE(@Agent_Configuration, [Agent_Configuration]),
				   [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator]),
                   [LogServerErrors_Indicator] = COALESCE(@LogServerErrors_Indicator, [LogServerErrors_Indicator]),
                   [Restricted_Indicator] = COALESCE(@Restricted_Indicator, [Restricted_Indicator])
			 WHERE [Server_Id] = @Server_Id

			EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @Server_Id
		END

	IF (@EnvironmentIds_Csv IS NOT NULL) BEGIN
		DELETE [EnvironmentServers] WHERE [Server_Id] = @Server_Id
		INSERT [EnvironmentServers] ([Server_Id], [Environment_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@EnvironmentIds_Csv, ',')
	END

	IF (@ServerIdsInGroup_Csv IS NOT NULL) BEGIN
		DELETE [ServerGroups] WHERE [ServerGroup_Server_Id] = @Server_Id
		INSERT [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@ServerIdsInGroup_Csv, ',')
	END

    IF (@ServerRoleIds_Csv IS NOT NULL) BEGIN
		DELETE [ServerRoleServers] WHERE [Server_Id] = @Server_Id
		INSERT [ServerRoleServers] ([Server_Id], [ServerRole_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@ServerRoleIds_Csv, ',')
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Servers_CreateOrUpdateServer] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_DeleteServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a server.'
GO


IF OBJECT_ID('[Servers_DeleteServer]') IS NOT NULL DROP PROCEDURE [Servers_DeleteServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_DeleteServer]
(
	@Server_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Server_Name SQL_VARIANT
	    SET @Server_Name = (SELECT [Server_Name] FROM [Servers] WHERE [Server_Id] = @Server_Id)

    UPDATE [ActionGroups]
       SET [Server_Id] = NULL
     WHERE [Server_Id] = @Server_Id

	UPDATE [DeploymentPlanActionGroups]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

	DELETE [EnvironmentServers]
	 WHERE [Server_Id] = @Server_Id

	DELETE [ServerGroups]
	 WHERE [ServerGroup_Server_Id] = @Server_Id
	    OR [Server_Id] = @Server_Id

	UPDATE [ConfigurationFileDeployments]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

	UPDATE [ActionGroupActions]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

    UPDATE [ActionGroupActions]
	   SET [Target_Server_Id] = NULL
	 WHERE [Target_Server_Id] = @Server_Id

    DELETE [VariableDefinitions]
     WHERE [Server_Id] = @Server_Id

	DELETE [Servers]
	 WHERE [Server_Id] = @Server_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'ENVDLS', NULL, @Server_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Servers_DeleteServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_GetServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Servers,Environments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specific server and its environments.'
GO


IF OBJECT_ID('[Servers_GetServer]') IS NOT NULL DROP PROCEDURE [Servers_GetServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_GetServer]
(
	@Server_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE [Server_Id] = @Server_Id

	SELECT *
	  FROM [Environments_Extended]
	 WHERE [Environment_Id] IN (SELECT [Environment_Id] FROM [EnvironmentServers]
	                             WHERE [Server_Id] = @Server_Id)

END
GO

GRANT EXECUTE ON [Servers_GetServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_GetServerLogs',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the logs associated with the specified server.'
GO


IF OBJECT_ID('[Servers_GetServerLogs]') IS NOT NULL DROP PROCEDURE [Servers_GetServerLogs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_GetServerLogs]
(
    @Server_Id INT,
	@LogMessage_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @LogMessage_Count IS NOT NULL
		SET ROWCOUNT @LogMessage_Count

	SELECT *
	  FROM [LogMessages_Extended]
	 WHERE [Server_Id] = @Server_Id
  ORDER BY [Logged_Date] DESC

END
GO

GRANT EXECUTE ON [Servers_GetServerLogs] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_GetServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in the system.'
GO


IF OBJECT_ID('[Servers_GetServers]') IS NOT NULL DROP PROCEDURE [Servers_GetServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_GetServers]
(
	@IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE @IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name] 

END
GO

GRANT EXECUTE ON [Servers_GetServers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_GetServersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in a specified server group.'
GO


IF OBJECT_ID('[Servers_GetServersInGroup]') IS NOT NULL DROP PROCEDURE [Servers_GetServersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_GetServersInGroup]
(
	@ServerGroup_Server_Id INT,
	@IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT S.*
	  FROM [Servers] S
	  INNER JOIN [ServerGroups] SG
	          ON SG.[Server_Id] = S.[Server_Id]
	 WHERE (SG.[ServerGroup_Server_Id] = @ServerGroup_Server_Id) AND
	       (@IncludeInactive_Indicator = 'Y' OR S.[Active_Indicator] = 'Y')
	 ORDER BY S.[Server_Name]

END
GO

GRANT EXECUTE ON [Servers_GetServersInGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_SearchServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Servers,ServerRoleServers,EnvironmentServers_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers specified by the filter, as well as their associated environments and roles.'
GO


IF OBJECT_ID('[Servers_SearchServers]') IS NOT NULL DROP PROCEDURE [Servers_SearchServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_SearchServers] (
	@Has_ServerRole_Id INT = NULL,
	@In_Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT S.* 
	  INTO #Servers
	  FROM [Servers] S
	 WHERE (@Has_ServerRole_Id IS NULL OR EXISTS(SELECT * 
                                                   FROM [ServerRoleServers] SR 
                                                  WHERE S.[Server_Id] = SR.[Server_Id] 
                                                    AND SR.[ServerRole_Id] = @Has_ServerRole_Id))
	   
       AND (@In_Environment_Id IS NULL OR EXISTS(SELECT * 
                                                   FROM [EnvironmentServers] ES
                                             CROSS APPLY dbo.Environments_GetAncestorEnvironments(ES.[Environment_Id]) AE
                                                  WHERE S.[Server_Id] = ES.[Server_Id] 
                                                    AND AE.[Environment_Id] = @In_Environment_Id))

	SELECT * FROM #Servers
	 ORDER BY [Server_Name] 
	 
	SELECT * FROM [ServerRoleServers] SR 
	 WHERE SR.[Server_Id] IN (SELECT [Server_Id] FROM #Servers)

	SELECT * FROM [EnvironmentServers_Extended] ES 
	 WHERE ES.[Server_Id] IN (SELECT [Server_Id] FROM #Servers)

END
GO

GRANT EXECUTE ON [Servers_SearchServers] TO [BuildMasterUser_Role]




GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Servers_UpdateServerStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Servers_UpdateServerStatus]') IS NOT NULL DROP PROCEDURE [Servers_UpdateServerStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Servers_UpdateServerStatus]
(
	@Server_Id INT,
	@ServerStatus_Code CHAR(5)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @Old_ServerStatus_Code CHAR(5)
        
        SELECT @Old_ServerStatus_Code = [ServerStatus_Code]
	      FROM [Servers]
	     WHERE [Server_Id] = @Server_Id

	    UPDATE [Servers]
	       SET [ServerStatus_Code] = @ServerStatus_Code
	     WHERE [Server_Id] = @Server_Id
	     
	     -- raise server error event if the status changes to ERROR from something else
	     IF (@ServerStatus_Code = 'ERROR' AND @Old_ServerStatus_Code <> 'ERROR')
	        EXEC [Events_RaiseEvent] 'SRVERR', NULL, @Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Servers_UpdateServerStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetUserConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'UserConfiguration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of user configuration settings'
GO

IF OBJECT_ID('[UserConfiguration_GetUserConfiguration]') IS NOT NULL DROP PROCEDURE [UserConfiguration_GetUserConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [UserConfiguration_GetUserConfiguration] 
(
	@User_Name USERNAME = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [UserConfiguration]
	 WHERE (@User_Name IS NULL OR [User_Name] = @User_Name)
	ORDER BY [User_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [UserConfiguration_GetUserConfiguration] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_GetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_GetValue] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_SetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

	IF (@Value_Text IS NOT NULL) 
		INSERT INTO [UserConfiguration]
			([User_Name], [Key_Name], [Value_Text])
		VALUES
			(@User_Name, @Key_Name, @Value_Text)

	--EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_SetValue] TO [BuildMasterUser_Role]
GO



GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserDirectories_CreateOrUpdateDirectory',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UserDirectory_Id',
    /* Description_Text        */ 'Creates a new user directory or updates an existing one.'
GO

IF OBJECT_ID('[UserDirectories_CreateOrUpdateDirectory]') IS NOT NULL DROP PROCEDURE [UserDirectories_CreateOrUpdateDirectory]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [UserDirectories_CreateOrUpdateDirectory]
(
	@UserDirectory_Name NVARCHAR(50),
	@UserDirectory_Configuration XML,
	@UserDirectory_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @UserDirectory_Id IS NULL
	BEGIN

		SET @UserDirectory_Id = COALESCE((SELECT MAX([UserDirectory_Id]) FROM [UserDirectories]), 0) + 1

		INSERT INTO [UserDirectories]
		(
			[UserDirectory_Id],
			[UserDirectory_Name],
			[UserDirectory_Configuration]
		)
		VALUES
		(
			@UserDirectory_Id,
			@UserDirectory_Name,
			@UserDirectory_Configuration
		)

	END
	ELSE
	BEGIN

		UPDATE [UserDirectories]
		   SET [UserDirectory_Name] = @UserDirectory_Name,
		       [UserDirectory_Configuration] = @UserDirectory_Configuration
		 WHERE [UserDirectory_Id] = @UserDirectory_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [UserDirectories_CreateOrUpdateDirectory] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserDirectories_DeleteDirectory',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UserDirectory_Id',
    /* Description_Text        */ 'Creates a new user directory or updates an existing one.'
GO

IF OBJECT_ID('[UserDirectories_DeleteDirectory]') IS NOT NULL DROP PROCEDURE [UserDirectories_DeleteDirectory]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [UserDirectories_DeleteDirectory]
(
	@UserDirectory_Id INT
)
AS
BEGIN

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE [Privileges]
     WHERE [UserDirectory_Id] = @UserDirectory_Id

	DELETE [UserDirectories]
     WHERE [UserDirectory_Id] = @UserDirectory_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [UserDirectories_DeleteDirectory] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserDirectories_GetDirectories',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'UserDirectories',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the user directories.'
GO

IF OBJECT_ID('[UserDirectories_GetDirectories]') IS NOT NULL DROP PROCEDURE [UserDirectories_GetDirectories]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [UserDirectories_GetDirectories]
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [UserDirectories]

END
GO

GRANT EXECUTE ON [UserDirectories_GetDirectories] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[dbo].[Users_CreateGroup]') IS NOT NULL DROP PROCEDURE [dbo].[Users_CreateGroup]
GO


EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user group in the system'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
(
	@Group_Name VARCHAR(50),
	@Users_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Users]
			   WHERE [User_Name] = @Group_Name)
		RAISERROR('Cannot create group "%s"; a user of the same name exists', 16, 1)

	IF NOT EXISTS(SELECT * FROM [Groups] WHERE [Group_Name] = @Group_Name)
    BEGIN
        
        INSERT INTO [Groups] ([Group_Name]) VALUES (@Group_Name)
        EXEC [Events_RaiseEvent] 'USRNWG', NULL, @Group_Name

    END

    IF @Users_XML IS NOT NULL 
    BEGIN

	    DELETE [UserGroups] WHERE [Group_Name] = @Group_Name

	    INSERT INTO [UserGroups] ([User_Name] ,[Group_Name])
	    SELECT UX.[value]('.', 'VARCHAR(250)') [User_Name], @Group_Name
	      FROM @Users_Xml.[nodes]('/users/username') AS UsersXml(UX)

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_CreateOrUpdateGroup] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user in the system or updates an existing one.'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateUser]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name VARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML,
	@ChangePasswordOnLogin_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user %s; a group of the same name exists', 16, 1)

	IF EXISTS(SELECT *
				FROM [Users]
	            WITH (UPDLOCK)
			   WHERE [User_Name] = @User_Name)
	BEGIN

		UPDATE [Users]
		   SET [Display_Name] = @Display_Name,
			   [Email_Address] = @Email_Address,
			   [ChangePasswordOnLogin_Indicator] = @ChangePasswordOnLogin_Indicator
		 WHERE [User_Name] = @User_Name

		EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	END
	ELSE
	BEGIN

		INSERT INTO [Users]
		(
			[User_Name],
			[Display_Name],
			[Email_Address],
			[ChangePasswordOnLogin_Indicator]
		)
		VALUES
		(
			@User_Name,
			@Display_Name,
			@Email_Address,
			@ChangePasswordOnLogin_Indicator
		)

		EXEC [Events_RaiseEvent] 'USRNWU', NULL, @User_Name

	END

	DELETE
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

	INSERT INTO [UserGroups]
	(
		[User_Name],
		[Group_Name]
	)
	SELECT @User_Name
		  ,GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateOrUpdateUser] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user group from the system, but does not delete the users who belong to the group'
GO

IF OBJECT_ID('[dbo].[Users_DeleteGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteGroup]
(
	@Group_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Remove group privileges */
	DELETE [Privileges]
	 WHERE [Principal_Name] = @Group_Name
	   AND [PrincipalType_Code] = 'G'

	/* Remove group users */
	DELETE [UserGroups]
	 WHERE [Group_Name] = @Group_Name

	/* Remove group */
	DELETE [Groups]
	 WHERE [Group_Name] = @Group_Name

	EXEC [Events_RaiseEvent] 'USRDLG', NULL, @Group_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteGroup] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user from the system'
GO

IF OBJECT_ID('[dbo].[Users_DeleteUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteUser]
(
	@User_Name VARCHAR(255)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	DELETE [Privileges]
	 WHERE [Principal_Name] = @User_Name
	   AND [PrincipalType_Code] = 'U'

	DELETE [UserGroups]
	 WHERE [User_Name] = @User_Name

	DELETE [Users]
	 WHERE [User_Name] = @User_Name

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteUser] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of user groups in the system'
GO

IF OBJECT_ID('[dbo].[Users_GetGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Groups]
	ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetGroups] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified user, and a list of all the groups the user belongs to'
GO

IF OBJECT_ID('[Users_GetUser]') IS NOT NULL DROP PROCEDURE [Users_GetUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_GetUser]
(
	@User_Name USERNAME
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Users]
	 WHERE [User_Name] = @User_Name

	SELECT *
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

END
GO

GRANT EXECUTE ON [Users_GetUser] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the users in the system, along with all group membership information.'
GO

IF OBJECT_ID('[Users_GetUsers]') IS NOT NULL DROP PROCEDURE [Users_GetUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_GetUsers
 AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Users]

	SELECT *
	  FROM [UserGroups]

END
GO

GRANT EXECUTE ON [Users_GetUsers] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of users in the specified group'
GO

IF OBJECT_ID('[dbo].[Users_GetUsersInGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetUsersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetUsersInGroup]
(
    @Group_Name VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT U.*
	  FROM [UserGroups] UG
	INNER JOIN [Users] U ON UG.[User_Name] = U.[User_Name]
	WHERE [Group_Name] = @Group_Name
	ORDER BY [User_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetUsersInGroup] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specific group by name, or if a wildcard search is performed, a group whose name starts with the specified name'
GO

IF OBJECT_ID('[dbo].[Users_SearchGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_SearchGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_SearchGroups]
(
    @Group_Name VARCHAR(50),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON
    
    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        SELECT *
          FROM [Groups]
         WHERE [Group_Name] LIKE @Group_Name + '%'
      ORDER BY [Group_Name]
        
    END
    ELSE
    BEGIN
	
	    SELECT *
	      FROM [Groups]
	     WHERE [Group_Name] = @Group_Name
	  ORDER BY [Group_Name]
	
	END

END
GO

GRANT EXECUTE ON [dbo].[Users_SearchGroups] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified user, or searchs for a user whose name starts with any of the specified criteria'
GO

IF OBJECT_ID('[Users_SearchUsers]') IS NOT NULL DROP PROCEDURE [Users_SearchUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SearchUsers]
(
    @User_Name USERNAME,
    @Display_Name VARCHAR(100),
    @Email_Address VARCHAR(250),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
 AS BEGIN

	SET NOCOUNT ON

    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        -- Add wildcards if specified and the arguments are not null
        IF (@User_Name IS NOT NULL)
            SET @User_Name = @User_Name + '%'
        IF (@Display_Name IS NOT NULL)
            SET @Display_Name = @Display_Name + '%'
        IF (@Email_Address IS NOT NULL)
            SET @Email_Address = @Email_Address + '%'
    
        SELECT *
	      FROM [Users]
	      WHERE [User_Name] LIKE @User_Name
	         OR [Display_Name] LIKE @Display_Name
	         OR [Email_Address] LIKE @Email_Address
    END
    ELSE
    BEGIN
        -- Perform an exact match
         SELECT *
	      FROM [Users]
	      WHERE [User_Name] = @User_Name
	        OR [Display_Name] = @Display_Name
	        OR [Email_Address] = @Email_Address
    END

	

END
GO

GRANT EXECUTE ON [Users_SearchUsers] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SetPassword',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Users_SetPassword]') IS NOT NULL
	DROP PROCEDURE [Users_SetPassword]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SetPassword]
(
	@User_Name USERNAME,
	@Password_Bytes BINARY(20),
	@Salt_Bytes BINARY(10),
	@ClearChangePasswordOnLogin_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	UPDATE [Users]
	   SET [Password_Bytes] = @Password_Bytes,
	       [Salt_Bytes] = @Salt_Bytes,
           [LegacyPassword_Indicator] = 'N',
		   [ChangePasswordOnLogin_Indicator] = CASE WHEN @ClearChangePasswordOnLogin_Indicator = 'Y' THEN 'N' ELSE [ChangePasswordOnLogin_Indicator] END
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_SetPassword] TO [BuildMasterUser_Role]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_ValidateLogin',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Validated_Indicator',
    /* Description_Text        */ 'Validates the password of a specified user'
GO

IF OBJECT_ID('Users_ValidateLogin') IS NOT NULL DROP PROCEDURE Users_ValidateLogin
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_ValidateLogin
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20),
	@Validated_Indicator YNINDICATOR = NULL OUT
) AS BEGIN

	SET NOCOUNT ON

	/* Set defaults */
	SET @Validated_Indicator = 'N'

	IF EXISTS(SELECT * FROM [Users]
	           WHERE [User_Name] = @User_Name
	             AND HASHBYTES('SHA1', [Salt_Bytes] + CAST(@Password_Text AS VARBINARY)) = [Password_Bytes])
		SET @Validated_Indicator = 'Y'


END
GO

GRANT EXECUTE ON Users_ValidateLogin TO BuildMasterUser_Role


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdatePackageVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new package variable or sets an existing release package variable''s value.'
GO

IF OBJECT_ID('[Variables_CreateOrUpdatePackageVariable]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdatePackageVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdatePackageVariable]
(
    @Build_Id INT,
	@Variable_Name VARCHAR(50),
	@ValueType_Code CHAR(1),
	@Variable_Value VARBINARY(MAX),
    @Sensitive_Indicator YNINDICATOR,
    @EvaluateVariables_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildVariables]
			   WHERE [Build_Id] = @Build_Id
                 AND [Variable_Name] = @Variable_Name)
	BEGIN

		UPDATE [BuildVariables]
		   SET [ValueType_Code] = @ValueType_Code,
		       [Variable_Value] = @Variable_Value,
               [Sensitive_Indicator] = @Sensitive_Indicator,
               [EvaluateVariables_Indicator] = @EvaluateVariables_Indicator
		 WHERE [Build_Id] = @Build_Id
           AND [Variable_Name] = @Variable_Name

	END
	ELSE
	BEGIN

		INSERT INTO [BuildVariables]
		(
			[Build_Id],
            [Variable_Name],
            [ValueType_Code],
			[Variable_Value],
            [Sensitive_Indicator],
            [EvaluateVariables_Indicator]
		)
		VALUES
		(
            @Build_Id,
            @Variable_Name,
            @ValueType_Code,
            @Variable_Value,
            @Sensitive_Indicator,
            @EvaluateVariables_Indicator
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdatePackageVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateReleaseVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new release variable or sets an existing release variable''s value.'
GO

IF OBJECT_ID('[Variables_CreateOrUpdateReleaseVariable]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateReleaseVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateReleaseVariable]
(
    @Release_Id INT,
	@Variable_Name VARCHAR(50),
	@ValueType_Code CHAR(1),
	@Variable_Value VARBINARY(MAX),
    @Sensitive_Indicator YNINDICATOR,
    @EvaluateVariables_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ReleaseVariables]
			   WHERE [Release_Id] = @Release_Id
                 AND [Variable_Name] = @Variable_Name)
	BEGIN

		UPDATE [ReleaseVariables]
		   SET [ValueType_Code] = @ValueType_Code,
		       [Variable_Value] = @Variable_Value,
               [Sensitive_Indicator] = @Sensitive_Indicator,
               [EvaluateVariables_Indicator] = @EvaluateVariables_Indicator
		 WHERE [Release_Id] = @Release_Id
           AND [Variable_Name] = @Variable_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ReleaseVariables]
		(
			[Release_Id],
            [Variable_Name],
            [ValueType_Code],
			[Variable_Value],
            [Sensitive_Indicator],
            [EvaluateVariables_Indicator]
		)
		VALUES
		(
            @Release_Id,
            @Variable_Name,
            @ValueType_Code,
            @Variable_Value,
            @Sensitive_Indicator,
            @EvaluateVariables_Indicator
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateReleaseVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Variable_Id',
    /* Description_Text        */ 'Creates a new variable, or updates an existing variable with the specified ID or matching scope.'
GO

IF OBJECT_ID('[Variables_CreateOrUpdateVariable]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariable]
(
	@Variable_Name VARCHAR(50),
	@ValueType_Code CHAR(1),
	@Variable_Value VARBINARY(MAX),
	@Environment_Id INT = NULL,
    @ServerRole_Id INT = NULL,
    @Server_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL,
    @Application_Id INT = NULL,
    @Deployable_Id INT = NULL,
    @Sensitive_Indicator YNINDICATOR = NULL,
    @EvaluateVariables_Indicator YNINDICATOR = NULL,
	@Variable_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Existing_Variable_Id INT

    SELECT @Existing_Variable_Id = [Variable_Id] 
      FROM [Variables]
     WHERE ([Variable_Id] = @Variable_Id) 
        OR  
        (   [Variable_Name] = @Variable_Name
            AND ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
            AND ([ServerRole_Id] = @ServerRole_Id OR ([ServerRole_Id] IS NULL AND @ServerRole_Id IS NULL))
            AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
            AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
            AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
            AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))
        )

	IF @Existing_Variable_Id IS NOT NULL
	BEGIN

		UPDATE [Variables]
		   SET [Variable_Name] = @Variable_Name,
		       [ValueType_Code] = @ValueType_Code,
		       [Variable_Value] = @Variable_Value,
		       [Environment_Id] = @Environment_Id,
               [ServerRole_Id] = @ServerRole_Id,
               [Server_Id] = @Server_Id,		       
		       [ApplicationGroup_Id] = @ApplicationGroup_Id,
               [Application_Id] = @Application_Id,
               [Deployable_Id] = @Deployable_Id,
               [Sensitive_Indicator] = @Sensitive_Indicator,
               [EvaluateVariables_Indicator] = @EvaluateVariables_Indicator
		 WHERE [Variable_Id] = @Existing_Variable_Id

         SET @Variable_Id = @Existing_Variable_Id

	END
	ELSE
	BEGIN

		INSERT INTO [Variables]
		(
			[Environment_Id],
			[ServerRole_Id],
            [Server_Id],
			[ApplicationGroup_Id],
            [Application_Id],
            [Deployable_Id],
			[Variable_Name],
			[ValueType_Code],
			[Variable_Value],
            [Sensitive_Indicator],
            [EvaluateVariables_Indicator]
		)
		VALUES
		(
			@Environment_Id,			
			@ServerRole_Id,
            @Server_Id,
			@ApplicationGroup_Id,
            @Application_Id,
            @Deployable_Id,
			@Variable_Name,
			@ValueType_Code,
			@Variable_Value,
            @Sensitive_Indicator,
            @EvaluateVariables_Indicator
		)

        SET @Variable_Id = SCOPE_IDENTITY()

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'VariableDeclaration_Id',
    /* Description_Text        */ 'Creates a new variable declaration or updates an existing one.'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
(
	@VariableDeclaration_Id INT OUT,
	@Variable_Name VARCHAR(50),
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
	@Scoped_PipelineStage_Name VARCHAR(50),
	@Scoped_Environment_Id INT,
	@Required_Indicator YNINDICATOR,
	@Variable_Configuration XML,
	@DefaultValue_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @VariableDeclaration_Id IS NULL
	BEGIN
		INSERT INTO [VariableDeclarations]
		(
			[Variable_Name],
			[Scope_Code],
			[Scoped_Application_Id],
			[Scoped_PipelineStage_Name],
			[Scoped_Environment_Id],
			[Required_Indicator],
			[Variable_Configuration],
			[DefaultValue_Text],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Scope_Code,
			@Scoped_Application_Id,
			@Scoped_PipelineStage_Name,
			@Scoped_Environment_Id,
			@Required_Indicator,
			@Variable_Configuration,
			@DefaultValue_Text,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Sensitive_Indicator
		)

		SET @VariableDeclaration_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [VariableDeclarations]
		   SET [Required_Indicator] = COALESCE(@Required_Indicator, [Required_Indicator]),
			   [Variable_Configuration] = COALESCE(@Variable_Configuration, [Variable_Configuration]),
			   [Scoped_PipelineStage_Name] = @Scoped_PipelineStage_Name,
			   [Scoped_Environment_Id] = @Scoped_Environment_Id,
			   [DefaultValue_Text] = @DefaultValue_Text,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDeclaration] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new variable definition or updates an existing one.'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
(
	@Variable_Name VARCHAR(50),
	@Environment_Id INT,
    @ServerRole_Id INT,
	@Server_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Deployable_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Promotion_Id INT,
	@Execution_Id INT,
	@Value_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Build_Id INT
	DECLARE @Release_Id INT
	SELECT @Build_Id = [Build_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	IF @Build_Id IS NULL
	BEGIN

		SELECT @Release_Id = [Release_Id]
		  FROM [Releases]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

	END

    DECLARE @Inferred_Scope_Code CHAR(1)
        SET @Inferred_Scope_Code = CASE
            WHEN @ServerRole_Id IS NOT NULL THEN 'L'
            WHEN @Server_Id IS NOT NULL THEN 'V'
            WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
            WHEN @Deployable_Id IS NOT NULL THEN 'D'
            WHEN @Environment_Id IS NOT NULL THEN 'E'
			WHEN @Promotion_Id IS NOT NULL THEN 'P'
            WHEN @Execution_Id IS NOT NULL THEN 'X'
            WHEN @Build_Id IS NOT NULL THEN 'B'
			WHEN @Release_Id IS NOT NULL THEN 'R'
            WHEN @Application_Id IS NOT NULL THEN 'A'
            ELSE 'S' END
            
    DECLARE @VariableDeclaration_Id INT
		SET @VariableDeclaration_Id =
              (SELECT [VariableDeclaration_Id] 
                 FROM [VariableDeclarations]
                WHERE [Variable_Name] = @Variable_Name
                  AND [Scope_Code] = @Inferred_Scope_Code
                  AND ([Scope_Code] NOT IN ('R','B','P','X') OR [Scoped_Application_Id] = @Application_Id)
                  AND ([Scope_Code] NOT IN ('P','X') OR [Scoped_Environment_Id] = @Environment_Id))

    DECLARE @Proper_Variable_Name VARCHAR(50)
    DECLARE @Declaration_Sensitive_Indicator YNINDICATOR

    SELECT @Proper_Variable_Name = [Variable_Name],
           @Declaration_Sensitive_Indicator = [Sensitive_Indicator]
      FROM [VariableDeclarations]
     WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    SET @Variable_Name = COALESCE(@Proper_Variable_Name, @Variable_Name)
    SET @Sensitive_Indicator = COALESCE(@Declaration_Sensitive_Indicator, @Sensitive_Indicator)

    DECLARE @VariableDefinition_Id INT
        SET @VariableDefinition_Id = (SELECT [VariableDefinition_Id] FROM [VariableDefinitions]
             WHERE [Variable_Name] = @Variable_Name
               AND ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
               AND ([ServerRole_Id] = @ServerRole_Id OR ([ServerRole_Id] IS NULL AND @ServerRole_Id IS NULL))
               AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
               AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
               AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))

               AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL) OR (@Build_Id IS NOT NULL) OR (@Release_Id IS NOT NULL))
               AND ([Release_Id] = @Release_Id OR ([Release_Id] IS NULL AND @Release_Id IS NULL) OR (@Build_Id IS NOT NULL))
               AND ([Build_Id] = @Build_Id OR ([Build_Id] IS NULL AND @Build_Id IS NULL))

			   AND ([Promotion_Id] = @Promotion_Id OR ([Promotion_Id] IS NULL AND @Promotion_Id IS NULL))

               AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL)) )

    IF @VariableDefinition_Id IS NULL
    BEGIN

		INSERT INTO [VariableDefinitions] 
        (
			[Variable_Name],
			[Environment_Id],
            [ServerRole_Id],
			[Server_Id],
			[ApplicationGroup_Id],
			[Application_Id],
			[Deployable_Id], 
			[Release_Id],
			[Build_Id],
			[Execution_Id],
			[Promotion_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Environment_Id,
            @ServerRole_Id,
			@Server_Id,
			@ApplicationGroup_Id,
			CASE WHEN @Build_Id IS NULL AND @Release_Id IS NULL THEN @Application_Id ELSE NULL END,
			@Deployable_Id,
			@Release_Id,
			@Build_Id,
			@Execution_Id,
			@Promotion_Id,
			@Value_Text,
			@Sensitive_Indicator
		)

	END
	ELSE
	BEGIN

		UPDATE [VariableDefinitions]
		   SET [Value_Text] = @Value_Text,
		       [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDefinition] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeletePackageVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a release package variable.'
GO

IF OBJECT_ID('[Variables_DeletePackageVariable]') IS NOT NULL DROP PROCEDURE [Variables_DeletePackageVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeletePackageVariable]
(
	@Build_Id INT,
    @Variable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [BuildVariables]
		 WHERE [Build_Id] = @Build_Id
           AND [Variable_Name] = @Variable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_DeletePackageVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteReleaseVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a release variable.'
GO

IF OBJECT_ID('[Variables_DeleteReleaseVariable]') IS NOT NULL DROP PROCEDURE [Variables_DeleteReleaseVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteReleaseVariable]
(
	@Release_Id INT,
    @Variable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [ReleaseVariables]
		 WHERE [Release_Id] = @Release_Id
           AND [Variable_Name] = @Variable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_DeleteReleaseVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable.'
GO

IF OBJECT_ID('[Variables_DeleteVariable]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariable]
(
	@Variable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [Variables]
		 WHERE [Variable_Id] = @Variable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Variables_DeleteVariable] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable declaration.'
GO


IF OBJECT_ID('[Variables_DeleteVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE [BuildScheduleVariableDefinitions]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    DELETE [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDeclaration] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable definition'
GO


IF OBJECT_ID('[Variables_DeleteVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDefinition]
(
	@VariableDefinition_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DELETE [VariableDefinitions]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDefinition] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetAllVariableDefinitions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the variable definitions in the system.'
GO


IF OBJECT_ID('[Variables_GetAllVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetAllVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetAllVariableDefinitions]
(
    @IncludeExecutionVariables_Indicator YNINDICATOR
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
      WHERE @IncludeExecutionVariables_Indicator = 'Y' OR [Execution_Id] IS NULL


END
GO

GRANT EXECUTE ON [Variables_GetAllVariableDefinitions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetPackageVariables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildVariables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the package variables for a specific release package.'
GO

IF OBJECT_ID('[Variables_GetPackageVariables]') IS NOT NULL DROP PROCEDURE [Variables_GetPackageVariables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetPackageVariables]
(
    @Build_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildVariables]
     WHERE [Build_Id] = @Build_Id

END
GO

GRANT EXECUTE ON [Variables_GetPackageVariables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetReleaseVariables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseVariables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the release variables for a specific release.'
GO

IF OBJECT_ID('[Variables_GetReleaseVariables]') IS NOT NULL DROP PROCEDURE [Variables_GetReleaseVariables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetReleaseVariables]
(
    @Release_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ReleaseVariables]
     WHERE [Release_Id] = @Release_Id

END
GO

GRANT EXECUTE ON [Variables_GetReleaseVariables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetSystemVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_System',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions for system-scoped variables.'
GO


IF OBJECT_ID('[Variables_GetSystemVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetSystemVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetSystemVariableDefinitions]
AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [VariableDefinitions_System]


END
GO

GRANT EXECUTE ON [Variables_GetSystemVariableDefinitions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a variable from a specified id.'
GO

IF OBJECT_ID('[Variables_GetVariable]') IS NOT NULL DROP PROCEDURE [Variables_GetVariable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariable]
(
	@Variable_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Variables]
	 WHERE [Variable_Id] = @Variable_Id

END
GO

GRANT EXECUTE ON [Variables_GetVariable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declaration in the system that has the specified ID.'
GO


IF OBJECT_ID('[Variables_GetVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT *
	  FROM [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclaration] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclarations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declarations in the system that are defined in the specified scope.'
GO

IF OBJECT_ID('[Variables_GetVariableDeclarations]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclarations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclarations]
(
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
	@Scoped_PiplineStage_Name VARCHAR(50),
    @Scoped_Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [VariableDeclarations]
     WHERE (@Scope_Code IS NULL OR [Scope_Code] = @Scope_Code)
       AND ([Scope_Code] NOT IN ('R','B','P','X') OR @Scoped_Application_Id IS NULL OR [Scoped_Application_Id] = @Scoped_Application_Id)
       AND ([Scope_Code] <> 'X' OR (@Scoped_Environment_Id IS NULL AND [Scoped_Environment_Id] IS NULL) OR [Scoped_Environment_Id] = @Scoped_Environment_Id OR @Scope_Code IS NULL)
       AND ([Scope_Code] <> 'P' OR @Scoped_PiplineStage_Name IS NULL OR [Scoped_PipelineStage_Name] = @Scoped_PiplineStage_Name)
     ORDER BY [Variable_Name]

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclarations] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified variable definition.'
GO


IF OBJECT_ID('[Variables_GetVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinition]
(
    @VariableDefinition_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinition] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions in the system that are defined in the specified scope'
GO


IF OBJECT_ID('[Variables_GetVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinitions]
(
    @Environment_Id INT,
    @ServerRole_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Id INT,
    @Build_Id INT,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
     WHERE ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
       AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
       AND ([ServerRole_Id] = @ServerRole_Id OR ([ServerRole_Id] IS NULL AND @ServerRole_Id IS NULL))
       AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
       AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))
       
       AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
       AND ([Release_Id] = @Release_Id OR ([Release_Id] IS NULL AND @Release_Id IS NULL))
       AND ([Build_Id] = @Build_Id OR ([Build_Id] IS NULL AND @Build_Id IS NULL))
       
       AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL))


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinitions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all variables.'
GO

IF OBJECT_ID('[Variables_GetVariables]') IS NOT NULL DROP PROCEDURE [Variables_GetVariables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariables]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Variables]

END
GO

GRANT EXECUTE ON [Variables_GetVariables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariablesAccessibleFromScope',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variables that are defined in any of the specified scopes.'
GO

IF OBJECT_ID('[Variables_GetVariablesAccessibleFromScope]') IS NOT NULL DROP PROCEDURE [Variables_GetVariablesAccessibleFromScope]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariablesAccessibleFromScope]
(
    @Environment_Id INT = NULL,
    @ServerRole_Id INT = NULL,
    @Server_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL,
    @Application_Id INT = NULL,
    @Deployable_Id INT = NULL,
    @Variable_Name VARCHAR(50) = NULL,
    @ExpandRolesAndEnvironments_Indicator YNINDICATOR = NULL,
    @IncludeSystemVariables_Indicator YNINDICATOR = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    SELECT *
      FROM [Variables]
     WHERE ([Variable_Name] = @Variable_Name OR @Variable_Name IS NULL)
       AND ([Environment_Id] = @Environment_Id
            OR (@ExpandRolesAndEnvironments_Indicator = 'Y'
                AND [Environment_Id] IN (SELECT [Environment_Id] FROM dbo.[Environments_GetAncestorEnvironments]([Environment_Id]) WHERE [Level] > 0))
            OR [ServerRole_Id] = @ServerRole_Id
            OR (@ExpandRolesAndEnvironments_Indicator = 'Y'
                AND [ServerRole_Id] IN (SELECT [ServerRole_Id] FROM [ServerRoleServers] WHERE [Server_Id] = @Server_Id))
            OR [Server_Id] = @Server_Id
            OR [ApplicationGroup_Id] = @ApplicationGroup_Id
            OR [Application_Id] = @Application_Id
            OR [Deployable_Id] = @Deployable_Id
            OR (@IncludeSystemVariables_Indicator = 'Y' AND [Environment_Id] IS NULL AND [ServerRole_Id] IS NULL AND [Server_Id] IS NULL AND [ApplicationGroup_Id] IS NULL AND [Application_Id] IS NULL AND [Deployable_Id] IS NULL))

END
GO

GRANT EXECUTE ON [Variables_GetVariablesAccessibleFromScope] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariablesForScope',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variables that are defined only in the specified scope.'
GO


IF OBJECT_ID('[Variables_GetVariablesForScope]') IS NOT NULL DROP PROCEDURE [Variables_GetVariablesForScope]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariablesForScope]
(
	@Environment_Id INT = NULL,
    @ServerRole_Id INT = NULL,
    @Server_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL,
    @Application_Id INT = NULL,
    @Deployable_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Variables]
	 WHERE ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
	   AND ([ServerRole_Id] = @ServerRole_Id OR ([ServerRole_Id] IS NULL AND @ServerRole_Id IS NULL))
       AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
       AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
       AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
       AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))

END
GO

GRANT EXECUTE ON [Variables_GetVariablesForScope] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variable_Values',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable values for the specified scope.'
GO

IF OBJECT_ID('[Variables_GetVariableValues]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableValues]
(
    @Environment_Id INT,
    @ServerRole_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
	@Promotion_Id INT,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF @Execution_Id IS NOT NULL
    BEGIN
		-- Use standard function for execution-scoped variable cascading
		SELECT *
		  FROM dbo.Variables_GetValuesForExecution(@Execution_Id, @Server_Id, @ServerRole_Id, @Deployable_Id)
    END
    ELSE
    BEGIN

		-- If deployable is specified but not application, find the application
		IF @Deployable_Id IS NOT NULL AND @Application_Id IS NULL
		BEGIN
			SET @Application_Id = (SELECT [Application_Id]
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)
		END

		-- If application is specified but not application group find the group
		IF @Application_Id IS NOT NULL AND @ApplicationGroup_Id IS NULL
		BEGIN
			SET @ApplicationGroup_Id = (SELECT [ApplicationGroup_Id]
			                              FROM [Applications]
			                             WHERE [Application_Id] = @Application_Id)
		END

		-- Make sure irrelevant release number is not considered
		IF @Release_Number IS NOT NULL AND @Application_Id IS NULL
			SET @Release_Number = NULL

		-- Make sure irrelevant build number is not considered
		IF @Build_Number IS NOT NULL AND @Release_Number IS NULL
			SET @Build_Number = NULL

		DECLARE @Release_Id INT
		DECLARE @Build_Id INT

		IF @Promotion_Id IS NOT NULL
		BEGIN

			SELECT @Release_Id = [Release_Id],
			       @Build_Id = [Build_Id],
				   @Application_Id = [Application_Id]
			  FROM [BuildPromotions_Extended]
			 WHERE [Promotion_Id] = @Promotion_Id

		END
		ELSE
		BEGIN

		 SELECT @Release_Id = [Release_Id]
		   FROM [Releases]
		  WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number

		 SELECT @Build_Id = [Build_Id]
		   FROM [Builds_Extended]
		  WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number
			AND [Build_Number] = @Build_Number
		END

		-- Determine the narrowest scope that will be considered			
		DECLARE @DesiredScope_Code CHAR(1)
		    SET @DesiredScope_Code = CASE
				WHEN @Promotion_Id IS NOT NULL THEN 'P'
				WHEN @Build_Number IS NOT NULL THEN 'B'
				WHEN @Release_Number IS NOT NULL THEN 'R'
				WHEN @Deployable_Id IS NOT NULL THEN 'D'
				WHEN @Application_Id IS NOT NULL THEN 'A'
				WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
				WHEN @Server_Id IS NOT NULL THEN 'V'
                WHEN @ServerRole_Id IS NOT NULL THEN 'L'
				WHEN @Environment_Id IS NOT NULL THEN 'E'
				ELSE 'S'
			END

		DECLARE @DesiredScope_Index INT
		    SET @DesiredScope_Index = CHARINDEX(@DesiredScope_Code, 'SELVGADRBPX')

		--Return the results by joining against all variable names in the system and filtering based on input criteria
		--The inner SELECT DISTINCTS required that the VDEF and VDEC tables need an index on the Variable_Name columns for decent performance
        SELECT [Variable_Name],
               [Value_Text],
               [Scope_Code],
               [Sensitive_Indicator]

          FROM (SELECT VN.[Variable_Name],
                       [Value_Text] = COALESCE(VDEF.[Value_Text], VDEC.[DefaultValue_Text]),
                       [Scope_Code] = COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]),
                       [Sensitive_Indicator] = COALESCE(VDEF.[Sensitive_Indicator], VDEC.[Sensitive_Indicator]),
                       [Rank] = RANK() OVER (PARTITION BY VN.[Variable_Name] ORDER BY CHARINDEX(COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]), 'SELVGADRBPX') DESC)
                  FROM (SELECT DISTINCT [Variable_Name]
						  FROM [VariableDeclarations]
                               UNION
                        SELECT DISTINCT [Variable_Name]
                          FROM [VariableDefinitions]) VN

					   LEFT JOIN [BuildPromotions] BP
					          ON BP.[Promotion_Id] = @Promotion_Id

                       LEFT JOIN [VariableDefinitions] VDEF
                              ON VN.[Variable_Name] = VDEF.[Variable_Name]
                             AND CHARINDEX(VDEF.[Scope_Code], 'SELVGADRBPX') <= @DesiredScope_Index
                             AND (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
                             AND (VDEF.[Release_Id] IS NULL OR (VDEF.[Release_Id] = @Release_Id))
                             AND (VDEF.[Build_Id] IS NULL OR (VDEF.[Build_Id] = @Build_Id))
                             AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = (
                                                                                                    SELECT TOP 1 AGA.[ApplicationGroup_Id]
                                                                                                      FROM dbo.ApplicationGroups_GetAncestorGroups(@ApplicationGroup_Id) AGA
                                                                                                     INNER JOIN [VariableDefinitions] VDI
                                                                                                             ON VDI.[ApplicationGroup_Id] = AGA.[ApplicationGroup_Id]
                                                                                                     ORDER BY AGA.[Level] ASC
                                                                                                      ))
                             AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)
                             AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
                             AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
                             AND (VDEF.[ServerRole_Id] IS NULL OR VDEF.[ServerRole_Id] = @ServerRole_Id)
							 AND (VDEF.[Promotion_Id] IS NULL OR VDEF.[Promotion_Id] = @Promotion_Id)
                       LEFT JOIN [VariableDeclarations] VDEC
                              ON VN.[Variable_Name] = VDEC.[Variable_Name]
                             AND CHARINDEX(VDEC.[Scope_Code], 'SELVGADRBPX') <= @DesiredScope_Index
                             AND (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
                             AND (VDEC.[Scoped_PipelineStage_Name] IS NULL OR (VDEC.[Scoped_Application_Id] = @Application_Id AND VDEC.[Scoped_PipelineStage_Name] = BP.[PipelineStage_Name]))
                 WHERE VDEF.[Scope_Code] = VDEC.[Scope_Code]
                    OR (VDEF.[Scope_Code] IS NULL AND VDEC.[Scope_Code] IS NOT NULL)
                    OR (VDEC.[Scope_Code] IS NULL AND VDEF.[Scope_Code] IS NOT NULL)) _
         WHERE [Rank] = 1

    END

END
GO

GRANT EXECUTE ON [Variables_GetVariableValues] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Workflows_AssignGlobalWorkflowToApplication]') IS NOT NULL DROP PROCEDURE [Workflows_AssignGlobalWorkflowToApplication]
GO



GO
IF OBJECT_ID('[Workflows_CloneWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_CloneWorkflow]
GO



GO
IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflow]
GO



GO
IF OBJECT_ID('[Workflows_DeleteApplicationWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteApplicationWorkflow]
GO



GO
IF OBJECT_ID('[Workflows_DeleteStep]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteStep]
GO



GO
IF OBJECT_ID('[Workflows_DeleteWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteWorkflow]
GO



GO
IF OBJECT_ID('[Workflows_GetGlobalWorkflows]') IS NOT NULL DROP PROCEDURE [Workflows_GetGlobalWorkflows]
GO



GO
IF OBJECT_ID('[Workflows_GetWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflow]
GO



GO
IF OBJECT_ID('[Workflows_GetWorkflows]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflows]
GO



GO
IF OBJECT_ID('[Workflows_GetWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflowStep]
GO



GO
IF OBJECT_ID('[Workflows_MakeGlobal]') IS NOT NULL DROP PROCEDURE [Workflows_MakeGlobal]
GO



GO
IF OBJECT_ID('[Workflows_MoveApproval]') IS NOT NULL DROP PROCEDURE [Workflows_MoveApproval]
GO



GO
IF OBJECT_ID('[Workflows_MoveWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_MoveWorkflowStep]
GO



GO
IF OBJECT_ID('[Workflows_SetBuildStep]') IS NOT NULL DROP PROCEDURE [Workflows_SetBuildStep]
GO



GO
--EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
--    /* StoredProc_Name         */ 'Applications_CloneApplication',
--    /* Internal_Indicator      */ 'N',
--    /* ReturnType_Name         */ 'void',
--    /* DataTableNames_Csv      */ NULL,
--    /* OutputPropertyNames_Csv */ 'New_Application_Id',
--    /* Description_Text        */ 'Clones an application, but does not copy any history'
--GO

--IF OBJECT_ID('[Applications_CloneApplication]') IS NOT NULL
--	DROP PROCEDURE [Applications_CloneApplication]
--GO

--SET QUOTED_IDENTIFIER ON
--GO

--CREATE PROCEDURE [Applications_CloneApplication]
--(
--	@Application_Id INT,
--	@LinkAllActionGroups_Indicator YNINDICATOR = 'N',
--	@LinkAllDeploymentPlans_Indicator YNINDICATOR = 'N',
--	@New_Application_Name VARCHAR(50),
--	@New_Application_Id INT = NULL OUT
--) AS
--BEGIN


--SET NOCOUNT ON

--BEGIN TRY

--	BEGIN TRANSACTION

--	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
--		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)

--	IF @LinkAllActionGroups_Indicator = 'Y' AND @LinkAllDeploymentPlans_Indicator = 'Y'
--		RAISERROR('Cannot link action groups if deployment plans are linked.', 16, 1)

--	DECLARE @Application_Name VARCHAR(50)
--	SET @Application_Name = (SELECT [Application_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

--	--==[Applications]==--
--	INSERT INTO [Applications] ([Application_Name], [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[VariableSupport_Code], [IssueTracking_AllowBuildMasterIssues_Indicator], [AlwaysShowDeployables_Indicator])
--	SELECT @New_Application_Name, [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[VariableSupport_Code], [IssueTracking_AllowBuildMasterIssues_Indicator], [AlwaysShowDeployables_Indicator]
--	  FROM [Applications]
--	 WHERE [Application_Id] = @Application_Id
--	SET @New_Application_Id = SCOPE_IDENTITY()

--	--==[Deployables]==--
--	INSERT INTO [Deployables] ([Application_Id],[Deployable_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
--	SELECT @New_Application_Id,[Deployable_Name],dbo.Session_GetUserName(),GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date]
--	 FROM [Deployables]
--	WHERE [Application_Id] = @Application_Id
	
--	--==[DeployableDependencies]==--
--	INSERT INTO [DeployableDependencies] ([Deployable_Id],[DependsOn_Deployable_Id])
--	SELECT D_NEW.[Deployable_Id],D2_NEW.[Deployable_Id]
--	  FROM [DeployableDependencies] DD
--	       INNER JOIN [Deployables] D
--	               ON D.[Deployable_Id] = DD.[Deployable_Id]
--	       INNER JOIN [Deployables] D2
--	               ON D2.[Deployable_Id] = DD.[DependsOn_Deployable_Id]
--	       INNER JOIN [Deployables] D_NEW
--	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
--	              AND D_NEW.[Application_Id] = @New_Application_Id
--	       INNER JOIN [Deployables] D2_NEW
--	               ON D2_NEW.[Deployable_Name] = D2.[Deployable_Name]
--	              AND D2_NEW.[Application_Id] = @New_Application_Id
--	 WHERE D.[Application_Id] = @Application_Id

--	--==[DeployablesImported]==--
--	INSERT INTO [DeployablesImported]([Deployable_Id],[Application_Id])
--	SELECT [Deployable_Id], @New_Application_Id
--	  FROM [DeployablesImported]
--	 WHERE [Application_Id] = @Application_Id


--     --==[Workflows]==--

--     DECLARE @AppWorkflows TABLE ([Workflow_Id] INT NOT NULL, [Workflow_Name] VARCHAR(MAX))
--      INSERT @AppWorkflows ([Workflow_Id], [Workflow_Name]) 
--      SELECT W.[Workflow_Id], W.[Workflow_Name] 
--        FROM [ApplicationWorkflows] AW
--  INNER JOIN [Workflows] W ON W.[Workflow_Id] = AW.[Workflow_Id]
--       WHERE AW.[Application_Id] = @Application_Id
--         AND W.[Global_Indicator] = 'N'

--     DECLARE @NewWorkflowIdsCsv VARCHAR(MAX)
--     DECLARE @CloneDeploymentPlans_Indicator YNINDICATOR
--     DECLARE @WorkflowCloneData_Xml VARCHAR(MAX)
--     DECLARE @Workflow_Id INT
--     DECLARE @Workflow_Name NVARCHAR(50)
     
--     SET @CloneDeploymentPlans_Indicator = CASE WHEN @LinkAllDeploymentPlans_Indicator = 'Y' THEN 'N' ELSE 'Y' END
     
--     SET @WorkflowCloneData_Xml = '<WorkflowCloneData CloneDeploymentPlans_Indicator="' + @CloneDeploymentPlans_Indicator + '" LinkAllActionGroups_Indicator="' + @LinkAllActionGroups_Indicator + '">'

--     WHILE EXISTS(SELECT * FROM @AppWorkflows)
--     BEGIN

--        SELECT TOP 1 @Workflow_Id = [Workflow_Id], @Workflow_Name = [Workflow_Name] FROM @AppWorkflows
     
--         SET @WorkflowCloneData_Xml = @WorkflowCloneData_Xml + 
--            '<Workflow>
--                <New_Workflow_Name>' + @Workflow_Name + '</New_Workflow_Name>
--                <Workflow_Id>' + CAST(@Workflow_Id AS VARCHAR(MAX)) + '</Workflow_Id>
--                <New_Application_Id>' + CAST(@New_Application_Id AS VARCHAR(MAX)) + '</New_Application_Id>
--            </Workflow>'

--        DELETE @AppWorkflows WHERE [Workflow_Id] = @Workflow_Id

--     END
     
--     SET @WorkflowCloneData_Xml =  @WorkflowCloneData_Xml + '</WorkflowCloneData>'
	  
--     EXEC [Workflows_CloneWorkflow] @WorkflowCloneData_Xml, @NewWorkflowIdsCsv 
	
--	/* Set-up Release */
--	DECLARE @New_Workflow_Id INT
--    DECLARE @New_Workflow_Name VARCHAR(MAX)
--	   SELECT TOP 1 @New_Workflow_Id = W.[Workflow_Id], @New_Workflow_Name = [Workflow_Name] 
--         FROM [ApplicationWorkflows] AW
--   INNER JOIN [Workflows] W ON W.[Workflow_Id] = AW.[Workflow_Id]
--        WHERE [Application_Id] = @New_Application_Id
	
--	DECLARE @New_Release_Number RELEASENUMBER
--	SELECT @New_Release_Number = CASE [ReleaseNumber_Scheme_Name]
--	         WHEN 'MajorMinorRevision' THEN '0.0.0'
--	         WHEN 'MajorMinor' THEN '0.0'
--	         WHEN 'DateBased' THEN '0' END
--	  FROM [Applications]
--	 WHERE [Application_Id] = @New_Application_Id
	 
--	IF @New_Workflow_Id IS NOT NULL
--		INSERT INTO [Releases]([Application_Id],[Release_Number],[Workflow_Id],[ReleaseStatus_Name],[Notes_Text],[Target_Date],[Release_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[CancelledBy_User_Name],[CancelledReason_Text],[CancelledOn_Date],[Release_Sequence],[Original_Workflow_Name])
--		VALUES (@New_Application_Id,@New_Release_Number,@New_Workflow_Id,'Active',NULL,NULL,'Setup',dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),NULL,NULL,NULL, 1, @New_Workflow_Name)

--    IF @New_Workflow_Id IS NOT NULL
--        INSERT [ReleaseDeployables] ([Application_Id], [Release_Number], [Deployable_Id], [InclusionType_Code])
--        SELECT @New_Application_Id, @New_Release_Number, [Deployable_Id], 'I' FROM [Deployables] WHERE [Application_Id] = @New_Application_Id
    

--	--==[VariableDeclarations]==--
--	PRINT 'VariableDeclarations...'
--    INSERT INTO [VariableDeclarations] 
--    (
--		[Scope_Code],
--		[Variable_Name],
--		[Scoped_Application_Id],
--		[Scoped_Environment_Id],
--		[Required_Indicator],
--		[Variable_Configuration],
--		[DefaultValue_Text],
--		[CreatedBy_User_Name],
--		[CreatedOn_Date],
--		[Sensitive_Indicator]
--	)
--    SELECT [Scope_Code],
--		   [Variable_Name],
--		   @New_Application_Id,
--		   [Scoped_Environment_Id],
--		   [Required_Indicator],
--		   [Variable_Configuration],
--		   [DefaultValue_Text],
--		   dbo.Session_GetUserName(),
--		   GETUTCDATE(),
--		   [Sensitive_Indicator]
--     FROM [VariableDeclarations] 
--	WHERE [Scoped_Application_Id] = @Application_Id

--    --==[VariableDefinitions: Deployable Variables]==--
--    PRINT 'VariableDefinitions: Deployable variables...'
--    INSERT INTO [VariableDefinitions]
--    (
--         [Variable_Name]
--        ,[Environment_Id]
--        ,[Server_Id]
--        ,[ApplicationGroup_Id]
--        ,[Application_Id]
--        ,[Deployable_Id]
--        ,[Release_Number]
--        ,[Build_Number]
--        ,[Value_Text]
--        ,[Execution_Id]
--        ,[Sensitive_Indicator]
--    )
--    SELECT 
--         [Variable_Name]
--        ,[Environment_Id]
--        ,[Server_Id]
--        ,[ApplicationGroup_Id]
--        ,VD.[Application_Id]
--        ,D_NEW.[Deployable_Id]
--        ,[Release_Number]
--        ,[Build_Number]
--        ,[Value_Text]
--        ,[Execution_Id]
--        ,[Sensitive_Indicator]
--    FROM [VariableDefinitions] VD
--      INNER JOIN [Deployables] D
--              ON D.[Deployable_Id] = VD.[Deployable_Id]
--      INNER JOIN [Deployables] D_NEW WITH (READUNCOMMITTED)
--              ON D_NEW.[Application_Id] = @New_Application_Id
--             AND D_NEW.[Deployable_Name] = D.[Deployable_Name]
--   WHERE D.[Application_Id] = @Application_Id
--     AND VD.[Scope_Code] = 'D'

--   --==[VariableDefinitions: Application Variables]==--
--    PRINT 'VariableDefinitions: Application variables...'
--    INSERT INTO [VariableDefinitions]
--    (
--         [Variable_Name]
--        ,[Environment_Id]
--        ,[Server_Id]
--        ,[ApplicationGroup_Id]
--        ,[Application_Id]
--        ,[Deployable_Id]
--        ,[Release_Number]
--        ,[Build_Number]
--        ,[Value_Text]
--        ,[Execution_Id]
--        ,[Sensitive_Indicator]
--    )
--    SELECT 
--         [Variable_Name]
--        ,[Environment_Id]
--        ,[Server_Id]
--        ,[ApplicationGroup_Id]
--        ,@New_Application_Id
--        ,[Deployable_Id]
--        ,[Release_Number]
--        ,[Build_Number]
--        ,[Value_Text]
--        ,[Execution_Id]
--        ,[Sensitive_Indicator]
--    FROM [VariableDefinitions] VD
--   WHERE VD.[Application_Id] = @Application_Id
--     AND VD.[Scope_Code] = 'A'

--	--==[BuildSchedules]==--
--	PRINT 'BuildSchedules...'
--	INSERT INTO [BuildSchedules] ([Schedule_Name],[Application_Id],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code])
--	SELECT [Schedule_Name],@New_Application_Id,[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date],[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code]
--	  FROM [BuildSchedules]
--	 WHERE [Application_Id] = @Application_Id
      
--   --==[BuildScheduleVariableDefinitions]==--
--   PRINT 'BuildScheduleVariableDefinitions...'
--	INSERT INTO [BuildScheduleVariableDefinitions]([Schedule_Id],[VariableDeclaration_Id],[Value_Text])
--	 SELECT BS_NEW.[Schedule_Id], VD_NEW.[VariableDeclaration_Id],BSVD.[Value_Text]
--	   FROM [BuildSchedules] BS
--	        INNER JOIN [BuildScheduleVariableDefinitions] BSVD
--	                ON BSVD.[Schedule_Id] = BS.[Schedule_Id]
--	        INNER JOIN [VariableDeclarations] VD
--	                ON BSVD.[VariableDeclaration_Id] = VD.[VariableDeclaration_Id]
--	        INNER JOIN [BuildSchedules] BS_NEW
--	                ON BS_NEW.[Schedule_Name] = BS.[Schedule_Name]
--	               AND BS_NEW.[Application_Id] = @New_Application_Id
--            INNER JOIN [VariableDeclarations] VD_NEW
--                    ON VD_NEW.[Scoped_Application_Id] = VD.[Scoped_Application_Id]
--                   AND VD_NEW.[Scope_Code] = VD.[Scope_Code]
--                   AND VD_NEW.[Variable_Name] = VD.[Variable_Name]
--	  WHERE BS.[Application_Id] = @Application_Id

--	--==[Privileges]==--
--	INSERT INTO [Privileges]([Principal_Name],[PrincipalType_Code],[Role_Id],[Application_Id],[Environment_Id],[PrivilegeType_Code],[UserDirectory_Id],[ApplicationGroup_Id])
--	SELECT [Principal_Name],[PrincipalType_Code],[Role_Id],@New_Application_Id,[Environment_Id],[PrivilegeType_Code],[UserDirectory_Id],[ApplicationGroup_Id]
--	  FROM [Privileges]
--	 WHERE [Application_Id] = @Application_Id

	
--	INSERT INTO [ConfigurationFiles]([Deployable_Id],[FilePath_Text],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
--	SELECT D_NEW.[Deployable_Id],[FilePath_Text],[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),CF.[ModifiedBy_User_Name],CF.[ModifiedOn_Date]
--	  FROM [ConfigurationFiles] CF
--	       INNER JOIN [Deployables] D
--			       ON D.[Deployable_Id] = CF.[Deployable_Id]
--	       INNER JOIN [Deployables] D_NEW
--	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
--	              AND D_NEW.[Application_Id] = @New_Application_Id
--	 WHERE CF.[Active_Indicator] = 'Y'
--	   AND D.[Application_Id] = @Application_Id
--	   AND @New_Release_Number IS NOT NULL

--	INSERT INTO [ConfigurationFileInstances]([ConfigurationFile_Id],[Instance_Name],[Active_Indicator],[Environment_Id],[Template_Indicator],[Template_Instance_Name],[TransformType_Code])
--	SELECT CF_NEW.[ConfigurationFile_Id],[Instance_Name],'Y',[Environment_Id],[Template_Indicator],[Template_Instance_Name],[TransformType_Code]
--	  FROM [ConfigurationFileInstances] CFI
--	       INNER JOIN [ConfigurationFiles] CF
--	               ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
--	       INNER JOIN [Deployables] D
--			       ON D.[Deployable_Id] = CF.[Deployable_Id]
--	       INNER JOIN [Deployables] D_NEW
--	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
--	              AND D_NEW.[Application_Id] = @New_Application_Id			       
--	       INNER JOIN [ConfigurationFiles] CF_NEW
--	               ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
--	              AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
--	 WHERE CFI.[Active_Indicator] = 'Y'
--	   AND D.[Application_Id] = @Application_Id


--	INSERT INTO [ConfigurationFileVersions]
--	(
--		[ConfigurationFile_Id],
--		[Instance_Name],
--		[Version_Number],
--		[VersionNotes_Text],
--		[File_Date],
--		[File_Bytes],
--		[CreatedBy_User_Name],
--		[CreatedOn_Date]
--	)
--	SELECT CF_NEW.[ConfigurationFile_Id],
--	       CFV.[Instance_Name],
--		   1,
--		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE 'Cloned from ' + @Application_Name END,
--		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE GETUTCDATE() END,
--		   [File_Bytes],
--		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE dbo.Session_GetUserName() END,
--		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE GETUTCDATE() END
--	  FROM [ConfigurationFileVersions_Extended] CFV
--		   INNER JOIN (SELECT [ConfigurationFile_Id],[Instance_Name],[Max_Version_Number] = MAX([Version_Number])
--						 FROM [ConfigurationFileVersions_Extended]
--						GROUP BY [ConfigurationFile_Id],[Instance_Name]) CFVMAX
--				   ON CFVMAX.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
--				  AND CFVMAX.[Instance_Name] = CFV.[Instance_Name]
--				  AND CFVMAX.[Max_Version_Number] = CFV.[Version_Number]
--			   INNER JOIN [ConfigurationFileInstances] CFI
--					   ON CFI.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
--					  AND CFI.[Instance_Name] = CFV.[Instance_Name]
--			   INNER JOIN [ConfigurationFiles] CF
--					   ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
--			   INNER JOIN [Deployables] D
--					   ON D.[Deployable_Id] = CF.[Deployable_Id]
--			   INNER JOIN [Deployables] D_NEW
--					   ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
--					  AND D_NEW.[Application_Id] = @New_Application_Id			       
--			   INNER JOIN [ConfigurationFiles] CF_NEW
--					   ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
--					  AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
--			   INNER JOIN [ConfigurationFileInstances] CFI_NEW
--					   ON CFI_NEW.[ConfigurationFile_Id] = CF_NEW.[ConfigurationFile_Id]
--					  AND CFI_NEW.[Instance_Name] = CFI.[Instance_Name]
--	 WHERE D.[Application_Id] = @Application_Id
	
--	/* Database Providers aren't cloned */
--	UPDATE [ActionGroupActions]
--	   SET [Active_Indicator] = 'N'
--	  FROM [ActionGroupActions] AGA
--	       INNER JOIN [ActionGroups] AG 
--		           ON AG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
--	       INNER JOIN [DeploymentPlanActionGroups] DPAG
--		           ON DPAG.[ActionGroup_Id] = AG.[ActionGroup_Id]
--           INNER JOIN [ApplicationDeploymentPlans] ADP
--		           ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
--	 WHERE CAST([Action_Configuration].query('local-name(/*[1])') AS NVARCHAR(MAX)) LIKE 'Inedo.BuildMaster.Extensibility.Actions.Database%'		           
--	   AND ADP.[Application_Id] = @New_Application_Id
--	   AND @LinkAllActionGroups_Indicator = 'N'
--	   AND @LinkAllDeploymentPlans_Indicator = 'N'


--	/* Raise Event */
--	EXEC [Events_RaiseEvent] 'APPNEW', @New_Application_Id


--	COMMIT
--END TRY BEGIN CATCH
--	IF XACT_STATE()<>0 ROLLBACK
--	EXEC [HandleError]
--END CATCH

--END
--GO

--GRANT EXECUTE ON [Applications_CloneApplication] TO [BuildMasterUser_Role]
--GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeApplicationData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire application and all associated data from the system.'
GO

IF OBJECT_ID('[Applications_PurgeApplicationData]') IS NOT NULL DROP PROCEDURE [Applications_PurgeApplicationData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeApplicationData]
(
	@Application_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	IF @Application_Name IS NULL
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)


	DECLARE @Releases TABLE ([Release_Id] INT PRIMARY KEY NOT NULL)
	INSERT INTO @Releases
	SELECT [Release_Id]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id
	 ORDER BY [Release_Sequence] DESC

	WHILE EXISTS(SELECT * FROM @Releases)
	BEGIN

		DECLARE @Release_Id INT
		SET @Release_Id = (SELECT TOP 1 [Release_Id] FROM @Releases)
		DELETE @Releases WHERE [Release_Id] = @Release_Id

		DECLARE @Release_Number RELEASENUMBER
		SET @Release_Number = (SELECT [Release_Number]
		                         FROM [Releases]
								WHERE [Release_Id] = @Release_Id)

		EXEC [Releases_PurgeReleaseData] @Application_Id, @Release_Number

	END

	DECLARE @Deployables TABLE ([Deployable_Id] INT PRIMARY KEY NOT NULL)
	INSERT INTO @Deployables
	SELECT [Deployable_Id]
	  FROM [Deployables]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @ConfigFiles TABLE ([ConfigurationFile_Id] INT PRIMARY KEY NOT NULL)
	INSERT INTO @ConfigFiles
	SELECT [ConfigurationFile_Id]
	  FROM [ConfigurationFiles]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @DatabaseConnections TABLE ([DatabaseConnection_Id] INT PRIMARY KEY NOT NULL)
	INSERT INTO @DatabaseConnections
	SELECT [DatabaseConnection_Id]
	  FROM [DatabaseConnections]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [ApplicationWorkflows]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] IN (SELECT [Schedule_Id] FROM [BuildSchedules] WHERE [Application_Id] = @Application_Id)

	DELETE
	  FROM [BuildSchedules]
	 WHERE [Application_Id] = @Application_Id


	-- BEGIN CONFIG FILES --
	DELETE
	  FROM [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] IN (SELECT [ConfigurationFile_Id] FROM @ConfigFiles)

	DELETE
	  FROM [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] IN (SELECT [ConfigurationFile_Id] FROM @ConfigFiles)

	DELETE
	  FROM [ConfigurationFileInstances]
	 WHERE [ConfigurationFile_Id] IN (SELECT [ConfigurationFile_Id] FROM @ConfigFiles)

	DELETE
	  FROM [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] IN (SELECT [ConfigurationFile_Id] FROM @ConfigFiles)
    -- END CONFIG FILES --


	-- BEGIN DATABASE CONNECTIONS --
	DELETE
	  FROM [DatabaseChangeScriptExecutions]
	 WHERE [DatabaseConnection_Id] IN (SELECT [DatabaseConnection_Id] FROM @DatabaseConnections)

	DELETE
	  FROM [DatabaseChangeScriptLegacyExecutions]
	 WHERE [DatabaseConnection_Id] IN (SELECT [DatabaseConnection_Id] FROM @DatabaseConnections)

	DELETE
	  FROM [DatabaseConnections]
	 WHERE [DatabaseConnection_Id] IN (SELECT [DatabaseConnection_Id] FROM @DatabaseConnections)
	-- END DATABASE CONNECTIONS --


	DELETE
	  FROM [EventListenerApplications]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [EventOccurenceDetails]
	 WHERE [EventOccurence_Id] IN (SELECT [EventOccurence_Id] FROM [EventOccurences] WHERE [Application_Id] = @Application_Id)

	DELETE
	  FROM [EventOccurences]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Issues]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [LogMessages]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Pipelines]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Plans]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Privileges]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [RetentionPoliciesPurgeHistory]
	 WHERE [Original_RetentionPolicy_Id] IN (SELECT [RetentionPolicy_Id] FROM [RetentionPolicies] WHERE [PurgeIn_Application_Id] = @Application_Id)

	DELETE
	  FROM [RetentionPolicies]
	 WHERE [PurgeIn_Application_Id] = @Application_Id

	DELETE
	  FROM [ScriptAssets]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id
	    OR [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DELETE
	  FROM [VariableDeclarations]
	 WHERE [Scoped_Application_Id] = @Application_Id


	-- Legacy plans should get deleted if they are anonymous and only used by this application
	DECLARE @DeploymentPlans TABLE ([DeploymentPlan_Id] INT PRIMARY KEY NOT NULL)
	INSERT INTO @DeploymentPlans
	SELECT DISTINCT ADP.[DeploymentPlan_Id]
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlans] DP
		           ON DP.[DeploymentPlan_Id] = ADP.[DeploymentPlan_Id]
	 WHERE ADP.[Application_Id] = @Application_Id
	   AND DP.[DeploymentPlan_Name] IS NULL
	   AND ADP.[DeploymentPlan_Id] NOT IN (SELECT [DeploymentPlan_Id] FROM [ApplicationDeploymentPlans] WHERE [Application_Id] <> @Application_Id)


	DELETE
	  FROM [DeployableDependencies]
	 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)
	    OR [DependsOn_Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DELETE
	  FROM [Deployables]
	 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

    DELETE
      FROM [DeployablesImported]
     WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeApplicationData] TO [BuildMasterUser_Role]



GO
IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflowStep]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks an execution as completed. If ExecutionStatus_Code is not supplied, it will be inferred.'
GO

IF OBJECT_ID('[Executions_CompleteExecution]') IS NOT NULL DROP PROCEDURE [Executions_CompleteExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteExecution]
(
	@Execution_Id INT,
	@ExecutionStatus_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @ExecutionStatus_Code IS NULL
	BEGIN
		DECLARE @Max_LogEntry_Level INT

		SELECT @Max_LogEntry_Level = MAX([LogEntry_Level])
		  FROM [ScopedExecutionLogEntries]
		 WHERE [Execution_Id] = @Execution_Id

		SET @Max_LogEntry_Level = COALESCE(@Max_LogEntry_Level, 0)
		IF @Max_LogEntry_Level < 20
			SET @ExecutionStatus_Code = 'S'
		ELSE IF @Max_LogEntry_Level < 30
			SET @ExecutionStatus_Code = 'W'
		ELSE
			SET @ExecutionStatus_Code = 'E'
	END

	UPDATE [Executions]
	   SET [ExecutionStatus_Code] = @ExecutionStatus_Code,
	       [ExecutionRunState_Code] = 'C',
	       [End_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

     UPDATE [ScopedExecutionLogs]
        SET [End_Date] = GETUTCDATE(),
            [HighestLogEntry_Level] = (SELECT MAX([LogEntry_Level])
		                                 FROM [ScopedExecutionLogEntries] L2
								        WHERE [Execution_Id] = @Execution_Id
                                          AND L2.[Scope_Sequence] = L.[Scope_Sequence])
       FROM [ScopedExecutionLogs] L
      WHERE [Execution_Id] = @Execution_Id
        AND [End_Date] IS NULL


	DECLARE @Application_Id INT

	SELECT @Application_Id = [Application_Id]
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id

     DELETE [ManualActionsInProgress]
      WHERE [Execution_Id] = @Execution_Id

	EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CompleteExecution] TO [BuildMasterUser_Role]
